import React, { Component, useState } from 'react';
import { Field, reduxForm } from 'redux-form';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import { push } from 'connected-react-router';
import { injectIntl } from 'react-intl';
import { ERROR_MESSAGES, HTTP_STATUS, STORAGE_KEY } from 'common/constants';
import { LOGIN, NAME_OF_FORMS, COMPONENTS } from 'common';
import { userLoginByPassword, openForgotPassword, openWelcomeModal, openRecommendToChangePassword, openChangePassword, settingUp, fillHeader } from 'stores/actions';
import { getHostName, isScriptCreatedSuperUser } from 'utils/simpleUtils';
import { getRightCornerRibbonHtml } from 'utils/htmlUtils';
import { isFirstUserLogin } from 'utils/utils';
import { LocationStoreMethods, clearSessionStorage, isBetaVersion } from 'utils/browserStorage';
import BetaVersionButton from 'components/BetaVersionButton';
import LoginFooter from './LoginFooter';


let LoginForm = (props) => {
    const [showPass, setShowPass] = useState(false);
    return (
        <form className={props.className} onSubmit={props.handleSubmit}>
            <div>
                <label htmlFor="email">
                    <i className="far fa-envelope" />
                </label>
                <Field
                    name="email"
                    component="input"
                    type="text"
                    placeholder={COMPONENTS.FIELDS.USER.EMAIL_FUNC(props.intl)}
                    id="email"
                />
            </div>
            <div>
                <label htmlFor="password">
                    <i className="fas fa-shield-alt" />
                </label>
                <Field
                    name="password"
                    component="input"
                    type={showPass ? 'text' : 'password'}
                    placeholder={COMPONENTS.FIELDS.USER.PASSWORD_FUNC(props.intl)}
                    id="password"
                />
                <span className="show-password" onClick={() => setShowPass(!showPass)}>
                    {showPass ? <i className="far fa-eye" /> : <i className="far fa-eye-slash" />}
                </span>
            </div>
            <span className="Login-ForgotPassword" onClick={props.forgotPassword}>
                {LOGIN.FORGOT_PASSWORD}
            </span>

            <div className="Login-Error">{props.errorMessage}</div>

            <button type="submit">{LOGIN.BUTTON}</button>
        </form>
    );
};

LoginForm.propTypes = {
    className: PropTypes.string,
    errorMessage: PropTypes.shape(),
    handleSubmit: PropTypes.func,
    forgotPassword: PropTypes.func,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

LoginForm.defaultProps = {
    className: '',
    errorMessage: null,
    handleSubmit: () => {
    },
    forgotPassword: () => {
    },
    intl: {
        formatMessage: () => {
        },
    },
};

LoginForm = reduxForm({
    form: NAME_OF_FORMS.LOGIN,
})(injectIntl(LoginForm));


class Login extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        resetError: PropTypes.func,
    };

    static defaultProps = {
        dispatch: () => { },
        resetError: () => { },
    };

    constructor(props) {
        super(props);

        this.state = {
            error: null,
        };
    }

    componentDidMount() {
        this.props.resetError();
        this.props.dispatch(push('/'));
    }

    handleLoginError(err) {
        const status = err.response && err.response.status;
        const message = err.response && err.response.data && err.response.data.error;
        console.warn('message:', message);
        console.warn('status:', status);
        if (status === HTTP_STATUS.LOCKED && message && message.startsWith(ERROR_MESSAGES.ACCOUNT_LOCKED_PREFIX)) {
            this.setState({
                error: LOGIN.ACCOUNT_LOCKED,
            });
        } else if (status === HTTP_STATUS.LOCKED && message && message.startsWith(ERROR_MESSAGES.ACCOUNT_DISABLED_PREFIX)) {
            this.setState({
                error: LOGIN.ACCOUNT_DISABLED,
            });
        } else {
            this.setState({
                error: LOGIN.ERROR_MESSAGE,
            });
        }
    }

    isUserValid(user) {
        if (!user.sign_in_count && !user.password) {
            throw new Error('User does not have sign_in_count or password');
        }

        if (isScriptCreatedSuperUser(user)) {
            return true;
        }

        if (isFirstUserLogin(user)) {
            LocationStoreMethods.setToLocalStorage(
                STORAGE_KEY.FORCE_TO_CHANGE_PASSWORD,
                user.uid,
                user.email,
                null,
            );
            this.props.dispatch(openWelcomeModal({ user }));
            return false;
        }
        if (user.isTemporaryPassword) {
            this.props.dispatch(openRecommendToChangePassword({ user }));
            return false;
        }
        if (user.needUpdatePassword) {
            this.props.dispatch(openChangePassword({
                uid: user.uid,
                email: user.email,
                token: user.token,
                force: true,
            }));
            return false;
        }
        const forceChangePasswordForUser =
            LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.FORCE_TO_CHANGE_PASSWORD, user.email, null);

        if (forceChangePasswordForUser) {
            this.props.dispatch(openChangePassword({
                uid: forceChangePasswordForUser,
                email: user.email,
                token: user.token,
                force: true,
            }));
            return false;
        }
        return true;
    }

    handleSubmit = (values) => {
        this.setState({
            error: null,
        });

        this.props.dispatch(userLoginByPassword(values))
            .then(({ user, userInfo }) => {
                fillHeader(userInfo);
                
                if (this.isUserValid(user)) {
                    this.props.dispatch(settingUp(userInfo));
                } else {
                    clearSessionStorage();
                }
            })
            .catch((err) => {
                this.handleLoginError(err);
                clearSessionStorage();
            });
    };

    forgotPassword = () => {
        this.props.dispatch(openForgotPassword());
    };

    render() {
        const hostName = getHostName();
        const isBeta = isBetaVersion();

        return (
            <div className="LoginWrap">
                <div className="RPP-Login">
                    <div className="LoginInner">
                        {isBeta && getRightCornerRibbonHtml('Beta Version', 'BETA')}
                        <img
                            src={`${hostName}/logo/company-logo.png`}
                            className="Login-Brand"
                            alt=":("
                        />

                        <LoginForm
                            className="Login-Form"
                            onSubmit={this.handleSubmit}
                            forgotPassword={this.forgotPassword}
                            errorMessage={this.state.error}
                        />
                        <LoginFooter />
                    </div>
                    <div className="Beta-Login">
                        <BetaVersionButton />
                    </div>
                </div>
            </div>
        );
    }
}

export default connect()(Login);
