import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { isCurrentPageSitePage } from 'components/NavBar/navBarHelpers';
import { DB_NAMES, COMPONENTS } from 'common/ui-labels';
import { SITE_MODE } from 'common/constants';
import { sortByField, addValueLabelToValue } from 'utils';
import { selectFloorplan, selectLocation, selectSublocation } from 'stores/actions';
import SmallSelect from 'components/SmallSelect';

function clearSiteViewMode(mode) {
    if (mode && [SITE_MODE.NEAR_MISS, SITE_MODE.PLAYBACK, SITE_MODE.HEATMAP, SITE_MODE.SPAGHETTI].includes(mode.name)) {
        mode.onReturn();
    }
}

function getFloorplansBySublocation(selectedSublocation, floorplans) {
    return selectedSublocation ? floorplans.reduce((memo, floorplan) => {
        if (floorplan.sublocation_uid === selectedSublocation.uid) {
            if (!floorplan.display_name) {
                const arrUrl = floorplan.url.split('/');
                floorplan.display_name = arrUrl[arrUrl.length - 1];
            }
            memo.push(floorplan);
        }
        return memo;
    }, []) : [];
}

function getSublocationsByLocation(selectedLocation, sublocations) {
    return selectedLocation ? sublocations.filter(sublocation =>
        sublocation.location_uid === selectedLocation.uid) : [];
}


function SitePageSubmenu(props) {
    const {
        item, id, intl, selectedLocation, selectedSublocation, selectedFloorplan, locations, sublocations, floorplans, mode,
    } = props;
    
    if (!item.expandable || item.submenu || !isCurrentPageSitePage(item.to)) {
        return null;
    }
    
    const _sublocations = getSublocationsByLocation(selectedLocation, sublocations);
    const _floorplans = getFloorplansBySublocation(selectedSublocation, floorplans);
    const floorplanUID = selectedFloorplan && selectedFloorplan.uid;

    const siteSelectorOptions = sortByField(locations, 'name').map(addValueLabelToValue);
    const areaSelectorOptions = sortByField(_sublocations, 'name').map(addValueLabelToValue);
    const floorplanSelectorOptions = sortByField(_floorplans, 'display_name').map(el => ({ label: el.display_name, value: el.uid }));

    return (
        <Fragment>
            <input
                type="checkbox"
                name="radio"
                id={`menu_bar${id}`}
                className="submenu-toggle"
                checked={props.getNavBarMenuOpenCloseStatus(item.to)}
                onChange={(e) => {
                    props.setNavBarMenuOpenCloseStatus(item.to, e.target.checked);
                }}
            />
            <label className="dropdown-menu-arrow" htmlFor={`menu_bar${id}`}>
                <i className="fa fa-sort-down" />
            </label>
            <div className="expandable expandable-for-selects">
                <div className="selector-wrapper">
                    <div className="selector-label">{COMPONENTS.FIELDS.SITE.SITE}</div>
                    <SmallSelect
                        className="site-selector"
                        onChange={(val) => {
                            props.selectLocationSiteView(val.value, locations, floorplans, sublocations, mode);
                        }}
                        value={selectedLocation ? selectedLocation.uid : ''}
                        options={siteSelectorOptions}
                    />
                </div>
                <div className="selector-wrapper">
                    <div className="selector-label">{COMPONENTS.FIELDS.SITE.AREA}</div>
                    <SmallSelect
                        className="area-selector"
                        onChange={(val) => {
                            props.selectSublocationSiteView(val.value, sublocations, floorplans, mode);
                        }}
                        disabled={!selectedSublocation}
                        value={selectedSublocation ? selectedSublocation.uid : ''}
                        placeholder={DB_NAMES.NONE(intl)}
                        options={areaSelectorOptions}
                    />
                </div>
                <div className="selector-wrapper">
                    <div className="selector-label">{COMPONENTS.FIELDS.SITE.FLOORPLAN}</div>
                    <SmallSelect
                        className="floorplan-selector"
                        onChange={(val) => {
                            props.selectFloorplanSiteView(val.value, floorplans);
                        }}
                        disabled={!floorplanUID}
                        value={floorplanUID || ''}
                        placeholder={DB_NAMES.NONE(intl)}
                        options={floorplanSelectorOptions}
                    />
                </div>
            </div>
        </Fragment>
    );
}

SitePageSubmenu.propTypes = {
    item: PropTypes.shape({
        to: PropTypes.string,
        submenu: PropTypes.arrayOf(PropTypes.shape({})),
    }),
    id: PropTypes.number,
    selectedLocation: PropTypes.shape({}),
    locations: PropTypes.arrayOf(PropTypes.shape({})),
    selectedSublocation: PropTypes.shape({}),
    selectedFloorplan: PropTypes.shape({}),
    sublocations: PropTypes.arrayOf(PropTypes.shape({})),
    floorplans: PropTypes.arrayOf(PropTypes.shape({})),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    mode: PropTypes.shape({}),
    getNavBarMenuOpenCloseStatus: PropTypes.func,
    setNavBarMenuOpenCloseStatus: PropTypes.func,
    selectLocationSiteView: PropTypes.func,
    selectSublocationSiteView: PropTypes.func,
    selectFloorplanSiteView: PropTypes.func,
};

SitePageSubmenu.defaultProps = {
    item: {
        to: '',
        submenu: [],
    },
    id: 0,
    selectedLocation: null,
    locations: [],
    selectedSublocation: null,
    selectedFloorplan: null,
    sublocations: [],
    floorplans: [],
    intl: {
        formatMessage: () => {},
    },
    mode: null,
    getNavBarMenuOpenCloseStatus: () => {},
    setNavBarMenuOpenCloseStatus: () => {},
    selectLocationSiteView: () => {},
    selectSublocationSiteView: () => {},
    selectFloorplanSiteView: () => {},
};

const mapDispatchToProps = dispatch => ({
    selectFloorplanSiteView: (floorplanUid, floorplans) => {
        const defaultFloorplan = floorplans.find(f => f.isFloorplanUidEqualTo(floorplanUid));
        dispatch(selectFloorplan(defaultFloorplan));
    },
    selectSublocationSiteView: (sublocationUid, sublocations, floorplans, mode) => {
        const defaultSublocation = sublocations.find(s => s.isSublocationUidEqualTo(sublocationUid));
        const filteredFloorplans = floorplans.filter(floorplan => defaultSublocation && defaultSublocation.uid === floorplan.sublocation_uid);
    
        dispatch(selectSublocation(defaultSublocation));
        dispatch(selectFloorplan(filteredFloorplans[0]));
        clearSiteViewMode(mode);
    },
    selectLocationSiteView: (locationUid, locations, floorplans, sublocations, mode) => {
        const location = locations.find(l => l.isLocationUidEqualTo(locationUid));
        const filteredSublocations = sublocations.filter(sublocation => location && sublocation.location_uid === location.uid);
        const filteredFloorplans = floorplans.filter(floorplan => filteredSublocations[0] && filteredSublocations[0].isSublocationUidEqualTo(floorplan.sublocation_uid));
    
        dispatch(selectLocation(location));
        dispatch(selectSublocation(filteredSublocations[0]));
        dispatch(selectFloorplan(filteredFloorplans[0]));
        clearSiteViewMode(mode);
    },
});

const mapStateToProps = state => ({
    mode: state.siteView.mode,
    locations: state.entities.locations,
    sublocations: state.entities.sublocations,
    floorplans: state.entities.floorplans,
    selectedLocation: state.siteView.selectedLocation,
    selectedSublocation: state.siteView.selectedSublocation,
    selectedFloorplan: state.siteView.selectedFloorplan,
});

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(SitePageSubmenu));
