import React, { Component } from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import _isEqual from 'lodash/isEqual';
import { getLocalStorageValue } from 'utils';
import { STORAGE_KEY } from 'common/constants';
import { isMenuOpened } from '../navBarHelpers';
import MenuItem from './MenuItem';


class NavigationMenu extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        expandSidebar: PropTypes.func,
        menuItems: PropTypes.arrayOf(PropTypes.object),
        currentPathname: PropTypes.string,
        monitorLink: PropTypes.string,
    };

    static defaultProps = {
        dispatch: () => {},
        expandSidebar: () => {},
        menuItems: [{}],
        currentPathname: '',
        monitorLink: null,
    };

    constructor(props) {
        super(props);

        this.state = {
            navBarMenuOpenCloseStatus: null,
        };
        this.navMenu = React.createRef();
    }

    static getDerivedStateFromProps(props, state) {
        const { navBarMenuOpenCloseStatus } = state;
        const { menuItems } = props;

        if (navBarMenuOpenCloseStatus === null) {
            const _navBarMenuOpenCloseStatus = new Map();

            const lastPagePath = getLocalStorageValue(STORAGE_KEY.LAST_PAGE);

            if (lastPagePath && lastPagePath.length > 0) {
                menuItems
                    .filter(menuItem => (menuItem.submenu || menuItem.expandable))
                    .map(menuItem =>
                        _navBarMenuOpenCloseStatus.set(menuItem.to, isMenuOpened(menuItem.to, lastPagePath)));
            }
            return { navBarMenuOpenCloseStatus: _navBarMenuOpenCloseStatus };
        }

        return null;
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutsideNavMenu);
    }
    
    shouldComponentUpdate(nextProps, nextState) {
        const currentArrayFromMap = Array.from(this.state.navBarMenuOpenCloseStatus);
        const nextArrayFromMap = Array.from(nextState.navBarMenuOpenCloseStatus);
        
        return !_isEqual(nextProps.currentPathname, this.props.currentPathname)
            || !_isEqual(currentArrayFromMap, nextArrayFromMap);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutsideNavMenu);
    }

    setNavBarMenuOpenCloseStatus = (key, value = true) => {
        const { navBarMenuOpenCloseStatus: oldNavBarMenuOpenCloseStatus } = this.state;
        const navBarMenuOpenCloseStatus = new Map(oldNavBarMenuOpenCloseStatus);

        // set map to the initial state
        navBarMenuOpenCloseStatus.forEach((val, index, map) => map.set(index, false));

        navBarMenuOpenCloseStatus.set(key, value);
        this.setState({ navBarMenuOpenCloseStatus });
    };

    getNavBarMenuOpenCloseStatus = path => this.state.navBarMenuOpenCloseStatus.get(path);

    handleMenuItemClick = (path) => {
        this.setNavBarMenuOpenCloseStatus(path);
        setTimeout(this.props.expandSidebar, 500);
    };

    handleClickOutsideNavMenu = (event) => {
        const navMenu = this.navMenu.current; // from ref

        if (navMenu && !navMenu.contains(event.target)) {
            this.closeAllNavBarMenus();
        }
    };

    closeAllNavBarMenus = () => {
        const { navBarMenuOpenCloseStatus: oldNavBarMenuOpenCloseStatus } = this.state;
        const navBarMenuOpenCloseStatus = new Map(oldNavBarMenuOpenCloseStatus);

        // set map to the initial state
        navBarMenuOpenCloseStatus.forEach((val, index, map) => map.set(index, false));

        this.setState({ navBarMenuOpenCloseStatus });
    };

    render() {
        const { menuItems, currentPathname, monitorLink, dispatch } = this.props;

        return (
            <div
                className="menu page-nav-menu"
                ref={this.navMenu}
            >
                {menuItems.map((item, index) => (
                    <MenuItem
                        currentPathname={currentPathname}
                        monitorLink={monitorLink}
                        dispatch={dispatch}
                        id={index}
                        key={index}
                        item={item}
                        closeAllNavBarMenus={this.closeAllNavBarMenus}
                        getNavBarMenuOpenCloseStatus={this.getNavBarMenuOpenCloseStatus}
                        setNavBarMenuOpenCloseStatus={this.setNavBarMenuOpenCloseStatus}
                        handleMenuItemClick={this.handleMenuItemClick}
                    />
                ))}
            </div>
        );
    }
}

const mapStateToProps = state => ({
    monitorLink: state.appContext.monitorLink,
    menuItems: state.user.scope.navButtons,
    currentPathname: state.router.location.pathname,
});

export default connect(mapStateToProps)(injectIntl(NavigationMenu));
