import React, { PureComponent } from 'react';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';

import LiveButton from 'components/LiveButton';
import { RIGHT_SIDE_BAR_TABLES } from 'common/ui-labels';
import { getLocalStorageValue } from 'utils/browserStorage';
import { stringToBoolean, isProjectScopeUser } from 'utils/utils';
import { CUSTOM_LOCAL_STORE_KEYS } from 'common/constants';
import { clearPlayback, stopLiveMode, startLiveMode } from 'stores/actions';
import { Alarm } from 'stores/classes';
import AlarmNotifications from './AlarmNotifications';


class TopButtons extends PureComponent {
    static propTypes = {
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        dispatch: PropTypes.func,
        alarmsLength: PropTypes.number,
        isSidebarCollapsed: PropTypes.bool,
        isRealTimeOn: PropTypes.bool,
        access: PropTypes.shape({}),
        playback: PropTypes.shape({
            isOpenPlayBackControl: PropTypes.bool,
        }),
    };

    static defaultProps = {
        intl: {
            formatMessage: () => {},
        },
        dispatch: () => {},
        alarmsLength: 0,
        isSidebarCollapsed: false,
        isRealTimeOn: true,
        access: {},
        playback: {
            isOpenPlayBackControl: false,
        },
    };

    constructor(props) {
        super(props);

        this.state = {
            isAlarmsListShowed: false,
        };
    }

    hideAlarmsList = () => {
        this.setState({ isAlarmsListShowed: false });
    };

    showAlarmsList = (value = true) => {
        this.setState({ isAlarmsListShowed: value });
    };

    handleAlarmsClick = () => {
        const { isAlarmsListShowed } = this.state;
        this.showAlarmsList(!isAlarmsListShowed);
    };

    stopStartLiveMode = () => {
        const { playback, dispatch, isRealTimeOn } = this.props;

        if (isRealTimeOn) {
            dispatch(stopLiveMode());
        } else if (playback.isOpenPlayBackControl) {
            dispatch(clearPlayback());
        } else {
            dispatch(startLiveMode());
        }
    };

    render() {
        const {
            intl,
            isSidebarCollapsed,
            isRealTimeOn,
            access,
            alarmsLength,
        } = this.props;

        const { isAlarmsListShowed } = this.state;

        const alarmsLocalization = RIGHT_SIDE_BAR_TABLES.TITLES.ALARMS_FUNC(intl);

        const showLiveButton = stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.SHOW_EXTENDS));

        const areThereActiveAlarms = alarmsLength > 0;

        return (
            <div className="navbar-top-buttons-row">
                { showLiveButton && <LiveButton
                    isRealTimeOn={isRealTimeOn}
                    stopStartLiveMode={this.stopStartLiveMode}
                    access={access}
                /> }

                {
                    isProjectScopeUser(access.currentRole) && areThereActiveAlarms &&
                    <React.Fragment>
                        <AlarmNotifications
                            isAlarmsListShowed={isAlarmsListShowed}
                            showAlarmsList={this.showAlarmsList}
                            hideAlarmsList={this.hideAlarmsList}
                        />
                        <button
                            data-id="alarms-button"
                            title={isSidebarCollapsed ? alarmsLocalization : ''}
                            className="alarms-button btn"
                            onClick={this.handleAlarmsClick}
                        >
                            <span className="alarms-icon">
                                <i className="fas fa-bell" />
                            </span>
                            <span className="alarms-text">
                                {alarmsLocalization}
                            </span>
                            {
                                areThereActiveAlarms &&
                                <span className="alarms-counter">
                                    {alarmsLength}
                                </span>
                            }
                        </button>
                    </React.Fragment>
                }
            </div>
        );
    }
}

const mapStateToProps = state => ({
    access: state.user.access,
    isRealTimeOn: state.worker.isRealTimeOn,
    playback: state.playback,
    alarmsLength: Alarm.getAllAlarms(state).length,
});

export default connect(mapStateToProps)(injectIntl(TopButtons));
