import React from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import Color from 'color';
import { NAME_OF_FORMS } from 'common/constants';
import { NAME_OF_GEAR_BUTTONS } from 'common/ui-labels';
import BetaVersionButton from 'components/BetaVersionButton';
import { getUserByEmail } from 'utils';
import { getTitleText, clearSelectionUserMenuItemButtons, getCompanyColor, getContrastHoverTextColor, resetCssVariables } from './navBarHelpers';
import cssVariables from '../../../styles/variables.module.scss';


class UserMenu extends React.Component {
    static propTypes = {
        isCompanyEditModalOpen: PropTypes.bool,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        dispatch: PropTypes.func,
        userMenuFunc: PropTypes.func,
        isSidebarCollapsed: PropTypes.bool,
        user: PropTypes.shape({}),
        usersMap: PropTypes.shape({}),
        company: PropTypes.shape({}),
        currentUser: PropTypes.shape({}),
        editingCompanyParams: PropTypes.shape({}),
    };

    static defaultProps = {
        isCompanyEditModalOpen: false,
        intl: {
            formatMessage: () => {},
        },
        dispatch: () => {},
        userMenuFunc: () => {},
        isSidebarCollapsed: false,
        user: {},
        usersMap: {},
        company: {},
        currentUser: {},
        editingCompanyParams: {},
    };

    constructor(props) {
        super(props);

        this.state = {
            userMenuOpenedIndex: 0,
            isUserMenuSubmenuShowed: false,
        };

        this.userMenu = React.createRef();
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutsideUserMenu);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutsideUserMenu);
    }

    shouldComponentUpdate(nextProps, nextState) {
        const colorChanged = this.props.company?.color !== nextProps.company?.color;
        const editedColorChanged = this.props.editingCompanyParams !== nextProps.editingCompanyParams;
        const modalOpened = this.props.isCompanyEditModalOpen !== nextProps.isCompanyEditModalOpen;
        const userChanged = this.props.user !== nextProps.user || this.props.usersMap !== nextProps.usersMap;
        const sideBarChanged = this.props.isSidebarCollapsed !== nextProps.isSidebarCollapsed;
        const menuIndexChanged = this.state.userMenuOpenedIndex !== nextState.userMenuOpenedIndex;
        const menuStateChanged = this.state.isUserMenuSubmenuShowed !== nextState.isUserMenuSubmenuShowed;

        return colorChanged || editedColorChanged || modalOpened || userChanged || sideBarChanged || menuIndexChanged || menuStateChanged;
    }

    handleUserMenuSubmenuClick = (submenuItem) => {
        const { dispatch } = this.props;

        if (submenuItem.label === NAME_OF_GEAR_BUTTONS.HELP_USER_GUIDE_PDF) {
            submenuItem.onClick();
        } else {
            dispatch(submenuItem.onClick());
        }
        this.hideUserSubmenu();
    };

    hideUserSubmenu = () => {
        this.setState({ isUserMenuSubmenuShowed: false });
        clearSelectionUserMenuItemButtons();
    };

    selectUserMenuItemButton = (isUserMenuSubmenuShowed, currentTarget) => {
        clearSelectionUserMenuItemButtons();
        currentTarget.classList.toggle('selected', isUserMenuSubmenuShowed);
    };

    showUserSubmenu = (isUserMenuSubmenuShowed, currentTarget) => {
        this.setState({ isUserMenuSubmenuShowed });
        this.selectUserMenuItemButton(isUserMenuSubmenuShowed, currentTarget);
    };

    handleClickOutsideUserMenu = (event) => {
        const userMenu = this.userMenu.current; // from ref

        if (userMenu && !userMenu.contains(event.target)) {
            this.hideUserSubmenu();
        }
    };

    handleUserMenuClick = (index, event) => {
        event.stopPropagation();
        const { isUserMenuSubmenuShowed, userMenuOpenedIndex } = this.state;
        const isUserSubmenuShowed = (userMenuOpenedIndex !== index) || !isUserMenuSubmenuShowed;

        this.showUserSubmenu(isUserSubmenuShowed, event.currentTarget);
        this.setState({ userMenuOpenedIndex: index });
    };

    render() {
        const { intl, userMenuFunc, user, usersMap, company, editingCompanyParams, isCompanyEditModalOpen } = this.props;
        const { userMenuOpenedIndex, isUserMenuSubmenuShowed } = this.state;
        const currentUser = getUserByEmail(user, usersMap);
        const userMenu = userMenuFunc(currentUser);

        const rootStyle = document.documentElement.style;
        rootStyle.setProperty('--userMenuItemsCount', userMenu.length);

        const initialCssVariables = {
            mainColor: cssVariables.mainColor,
            mainColorLight: cssVariables.mainColorLight,
            contrastTextColor: cssVariables.contrastTextColor,
            defaultBackgroundColor: cssVariables.defaultBackgroundColor,
        };

        const { menuRowHeight } = cssVariables;

        const color = getCompanyColor(company, editingCompanyParams, isCompanyEditModalOpen);
        if (color) {
            const mainColorObj = Color(color);
            const lightenMainColor = mainColorObj.lighten(0.17).rgbNumber().toString(16).padStart(6, '0');
            const lightenMainColorHexString = `#${lightenMainColor}`;

            const contrastTextColor = getContrastHoverTextColor(mainColorObj, cssVariables);

            rootStyle.setProperty('--mainColor', color);
            rootStyle.setProperty('--mainColorLight', lightenMainColorHexString);
            rootStyle.setProperty('--contrastTextColor', contrastTextColor);
        } else {
            resetCssVariables(rootStyle, initialCssVariables);
        }

        return (
            <div
                className="menu user-menu"
                ref={this.userMenu}
            >
                <div className="Beta-User-Menu cursor-pointer text-center">
                    <BetaVersionButton />
                </div>
                {userMenu.map((userMenuItem, index) => (
                    <button
                        key={index}
                        className="user-menu-item-button"
                        data-id={`user-menu-item-button-${index}`}
                        data-cy={userMenuItem.dataId}
                        onClick={event => this.handleUserMenuClick(index, event)}
                        title={getTitleText(userMenuItem.title, intl)}
                    >
                        <span className="user-menu-item">
                            <span className="icon-wrapper">
                                <span className="icon">
                                    <i className={userMenuItem.iconClass} />
                                </span>
                                <span className="icon-caret">
                                    <i className="fa fa-caret-right" />
                                </span>
                            </span>
                            <span className="user-menu-item-name">
                                <span className="multiple-lines-ellipsis">{userMenuItem.label}</span>
                            </span>
                        </span>
                        <span className="user-menu-item-arrow">
                            <i className="fa fa-caret-right" />
                        </span>
                    </button>
                ))}

                {
                    isUserMenuSubmenuShowed &&
                    <div
                        className="submenu"
                        style={{ marginTop: `${userMenuOpenedIndex * parseInt(menuRowHeight) - 2}px` }}
                    >
                        {userMenu[userMenuOpenedIndex].submenu.map((submenuItem, index) => (
                            <div className="submenu-item" key={index}>
                                <button
                                    className="user-menu-submenu-item-button"
                                    data-id={`user-menu-submenu-item-button-${index}`}
                                    data-cy={submenuItem.dataId}
                                    onClick={() => this.handleUserMenuSubmenuClick(submenuItem)}
                                >
                                    <span className="icon-wrapper">
                                        <span className="icon">
                                            <i className={submenuItem.iconClass} />
                                        </span>
                                        {submenuItem.subIconClass &&
                                        <span className="sub-icon">
                                            <i className={submenuItem.subIconClass} />
                                        </span>}
                                    </span>
                                    <span className="user-menu-item-name">
                                        <span className="multiple-lines-ellipsis">{submenuItem.label}</span>
                                    </span>
                                </button>
                            </div>
                        ))}
                    </div>
                }
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    const companyEditForm = state.form[NAME_OF_FORMS.COMPANY_EDIT];

    return ({
        company: state.appContext.company,
        userMenuFunc: state.user.scope.userMenu,
        user: state.user,
        usersMap: state.entities.users.byUid,
        editingCompanyParams: companyEditForm && companyEditForm.values,
        isCompanyEditModalOpen: state.modals.companyModals.companyEdit.openCompanyScope,
    });
};

export default connect(mapStateToProps)(injectIntl(UserMenu));
