import React from 'react';
import { connect } from 'react-redux';
import { createSelector } from 'reselect';
import { LayerGroup } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import _Get from 'lodash/get';
import _isEqual from 'lodash/isEqual';
import _Throttle from 'lodash/throttle';
import { positionOffset, isAnchorLive, isAnchor, isBridge, getAnchorClass } from 'utils';
import { TIMINGS } from 'common/constants';
import { selectLeafletItemForPopup } from 'stores/actions';
import { Node, WinAnchor } from 'stores/classes';
import { AnchorPopUp } from 'components/leaflet/popups'


class LeafletAnchorsLayer extends LayerGroup {
    static propTypes = {
        winAnchors: PropTypes.arrayOf(PropTypes.shape({})),
        sublocation: PropTypes.shape({}),
        selectedAnchor: PropTypes.shape({}),
    };
    
    static defaultProps = {
        winAnchors: [{}],
        sublocation: { offset_x: 0, offset_y: 0 },
        selectedAnchor: null,
    };

    constructor(props) {
        super(props);

        this.state = {
            firstRender: false,
        };
    }

    shouldComponentUpdate(nextProps) {
        if (!this.state.firstRender) {
            this.setState({ firstRender: true });
            return true;
        }
        return !_isEqual(nextProps.winAnchors, this.props.winAnchors) || !_isEqual(nextProps.selectedAnchor, this.props.selectedAnchor);
    }

    componentDidUpdate() {
        Object.keys(this.refs).forEach((key) => {
            const keyRef = this.refs[key];
            if (keyRef && keyRef.leafletElement && keyRef.leafletElement._icon && keyRef.leafletElement._icon.className) {
                keyRef.leafletElement._icon.className = keyRef.props.className;
            }
        });
    }

    onAnchorClick = (anchor) => {
        this.dispatchSelectedItem(anchor);
    };
    
    onClosePopup = () => {
        this.dispatchSelectedItem(null);
    };
    
    dispatchSelectedItem = (item) => {
        this.props.dispatch(selectLeafletItemForPopup(item));
    };

    renderAnchors = () => (this.props.winAnchors.map((anchor) => {
        const anchorPosition = positionOffset(anchor.position, this.props.sublocation);

        if (!anchorPosition) {
            return null;
        }

        const anchorClassName = getAnchorClass(anchor);
        const isAlive = isAnchorLive(anchor);

        return (
            <DivIcon
                key={anchor.uid}
                ref={anchor.uid}
                onClick={this.onAnchorClick.bind(this, anchor)}
                className="div-icon"
                position={anchorPosition}
            >
                <div className={anchorClassName} />
                {!isAlive && <i className="unresponsive-img unresponsive-sign" />}
            </DivIcon>);
    }));

    render() {
        return (
            <div>
                {this.renderAnchors()}
                <AnchorPopUp
                    anchor={this.props.selectedAnchor}
                    sublocation={this.props.sublocation}
                    intl={this.props.intl}
                    onClose={this.onClosePopup}
                />
            </div>
        );
    }
}

const getSelectedSublocation = state => state.siteView.selectedSublocation;
const getWinAnchors = state => WinAnchor.getAllWinAnchors(state);
const getFilter = state => state.siteViewFilter.objectFilter;
const getSelectedLeafletElement = state => state.siteView.selectedLeafletElement;

const getVisibleAnchors = (sublocation, winAnchors, leafletFilter, nodes, playback) => {
    if (!sublocation || playback.isOpenPlayBackControl) return [];
    const {
        showAnchor: areFilteredAnchorsShown,
        showBridges: areFilteredBridgesShown,
    } = leafletFilter;
    const anchors = Object.values(winAnchors);
    
    return anchors.reduce((memo, _anchor) => {
        const anchor = _anchor.getProperties();
        const isAnchorVisible = areFilteredAnchorsShown && isAnchor(anchor.type);
        const isBridgeVisible = areFilteredBridgesShown && isBridge(anchor.type);
        const isInSublocation = anchor.position && anchor.position.sublocation_uid === sublocation.uid;
        const mac_address = _Get(anchor, 'node.mac_address', null);
        const node = mac_address && nodes[mac_address.toUpperCase()];

        if ((isAnchorVisible || isBridgeVisible) && isInSublocation && node) {
            const parent_mac_address = node.parent_mac_address;
            memo.push({ ...anchor, node: { ...anchor.node, parent_mac_address  } });
        }

        return memo;
    }, []);
};

const getSelectedLeafletAnchor = (selectedLeafletElement, winAnchors) => (selectedLeafletElement ? winAnchors[selectedLeafletElement.uid] : null);

const selectedLeafletAnchor = createSelector([
    getSelectedLeafletElement, getWinAnchors,
], (selectedLeafletElement, winAnchors) => getSelectedLeafletAnchor(selectedLeafletElement, winAnchors));

const mapStateToProps = (state) => {
    const sublocation = getSelectedSublocation(state);
    const wins = getWinAnchors(state);
    const leafletFilter = getFilter(state);
    const selectedAnchor = selectedLeafletAnchor(state);
    const nodesMap = Node.getAllNodes(state);
    const playback = state.playback;

    const winAnchors = getVisibleAnchors(sublocation, wins, leafletFilter, nodesMap, playback);
    return {
        selectedAnchor: selectedAnchor ? selectedAnchor.getProperties() : null,
        winAnchors,
        appContext: state.appContext,
    };
};

export default connect(_Throttle(mapStateToProps, TIMINGS.THROTTLE_TIME))(injectIntl(LeafletAnchorsLayer));
