import React from 'react';
import { connect } from 'react-redux';
import { LayerGroup, Circle } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import PropTypes from 'prop-types';
import _isEqual from 'lodash/isEqual';
import { zoneFullActive, showBuffer, isBufferAlarmed, isBufferSelected } from 'utils/zoneUtils';
import { positionOffset } from 'utils/formatters';
import { zoneSeverityColor } from 'utils/simpleUtils';
import { Alarm } from 'stores/classes';
import { Zone } from 'stores/classes/Selectors';


class LeafletBufferLayer extends LayerGroup {
    static propTypes = {
        buffers: PropTypes.arrayOf(PropTypes.shape()),
        sublocation: PropTypes.shape(),
        playbackSpeed: PropTypes.number,
        isShowingZoneLabels: PropTypes.bool,
    };
    
    static defaultProps = {
        buffers: [],
        sublocation: { offset_x: 0, offset_y: 0 },
        playbackSpeed: 1,
        isShowingZoneLabels: true,
    };
    
    constructor(props) {
        super(props);
        
        this.state = {
            firstRender: false,
        };
    }
    
    shouldComponentUpdate(nextProps) {
        if (!this.state.firstRender) {
            this.setState({ firstRender: true });
            return true;
        }
        
        const buffersChanged = !_isEqual(nextProps.buffers, this.props.buffers);
        const sublocationChanged = !_isEqual(nextProps.sublocation, this.props.sublocation);
        const showLabelsChanged = nextProps.isShowingZoneLabels !== this.props.isShowingZoneLabels;
        const playbackSpeedChanged = nextProps.playbackSpeed !== this.props.playbackSpeed;
        
        return buffersChanged || sublocationChanged || showLabelsChanged || playbackSpeedChanged;
    }

    getBuffersLabels = () => (
        this.props.buffers.reduce((memo, buffer) => {
            if (buffer.dynamic && buffer.position) {
                const center = {
                    x: buffer.position.x + (buffer.zone_geometry.radius * 1000),
                    y: buffer.position.y,
                };
                const zonePosition = positionOffset(center, this.props.sublocation);
                const className = this.props.playbackSpeed ? `Leaflet-Transition-Speed${this.props.playbackSpeed} Leaflet-ZoneLabel` : 'Leaflet-ZoneLabel';
    
                memo.push(<DivIcon
                    key={`${buffer.uid}+1`}
                    ref={`${buffer.uid}+1`}
                    className={className}
                    position={zonePosition}
                >
                    <div className="Leaflet-ZoneTitle">
                        {buffer.name}
                    </div>
                </DivIcon>);
            }
            return memo;
        }, [])
    );
    
    makeCircle = ({ buffer, dashOpacity, dashColor, dashWeight, dashArray, fillColor }) => {
        const zonePosition = positionOffset(buffer.position, this.props.sublocation);
        if (!zonePosition) {
            return null;
        }
        const center = [zonePosition.lat, zonePosition.lng];
        const radius = buffer.zone_geometry.radius * 10; // convert m to suitable format
        const className = (this.props.playbackSpeed ? `Leaflet-Transition-Speed${this.props.playbackSpeed}` : '');
    
        return {
            dashOpacity,
            dashColor,
            dashWeight,
            dashArray,
            fillColor,
            center,
            radius,
            className,
            uid: buffer.uid,
        };
    };
    
    componentDidUpdate() {
        Object.keys(this.refs).forEach((key) => {
            const keyRef = this.refs[key];
            if (keyRef && keyRef.leafletElement && keyRef.leafletElement._path && keyRef.leafletElement._path.attributes.class) {
                keyRef.leafletElement._path.attributes.class.nodeValue = keyRef.props.className;
            } else if (keyRef && keyRef.leafletElement && keyRef.leafletElement._icon) {
                keyRef.leafletElement._icon.className = keyRef.props.className;
            }
        });
    }
   
    render() {
        const { buffers } = this.props;
        
        const circles = buffers.sort((a, b) => a.active < b.active).reduce((result, buffer) => {
            const fillColor = zoneSeverityColor(buffer.zone_alarm_level);
            const dashOpacity = buffer.isSelected ? 1 : 0.3;
            const dashArray = zoneFullActive(buffer) ? '1' : '5 10';
            const dashColor = buffer.isAlarmed ? 'red' : (buffer.isSelected ? 'yellow' : 'black');
            const dashWeight = (buffer.isSelected || buffer.isAlarmed) ? 5 : 2;
            if (buffer.dynamic) {
                const circle = this.makeCircle({ buffer, dashOpacity, dashColor, dashWeight, dashArray, fillColor });
                if (circle) {
                    result.push(circle);
                }
            }
            return result;
        }, []);
    
        return (
            <div>
                {circles.map(circle => (
                    <Circle
                        ref={circle.uid}
                        className={circle.className}
                        key={circle.uid}
                        opacity={circle.dashOpacity}
                        fillOpacity={0.2}
                        fillColor={circle.fillColor}
                        color={circle.dashColor}
                        weight={circle.dashWeight}
                        dashArray={circle.dashArray}
                        center={circle.center}
                        radius={circle.radius}
                    />))}
                
                {this.props.isShowingZoneLabels && this.getBuffersLabels()}
            </div>
        );
    }
}

const getLayerBuffers = (state, sublocation, visibleTrackables) => {
    const alarms = Alarm.getAllAlarms(state).filter(alarm => !alarm.acknowledged);
    const { playback, siteViewFilter, siteView: { selectedItems } } = state;
    
    return Zone.getBuffersBySublocation(sublocation, (playback && playback.isOpenPlayBackControl))
        .reduce((memo, buffer) => {
            const bufferVisible = showBuffer(buffer, siteViewFilter);
            const trackable = visibleTrackables.find(tko => tko.uid === buffer.trackable_object_uid);
            if (bufferVisible && trackable) {
                memo.push({
                    ...buffer,
                    isSelected: isBufferSelected(selectedItems, buffer),
                    isAlarmed: isBufferAlarmed(alarms, buffer),
                });
            }
            return memo;
        }, []);
};

const mapStateToProps = (state, props) => {
    const sublocation = props.sublocation || state.siteView.selectedSublocation;
    return {
        sublocation,
        buffers: getLayerBuffers(state, sublocation, props.visibleTrackables),
        playbackSpeed: state.playback.playSpeed,
        isShowingZoneLabels: state.siteViewFilter.layerFilter.isShowingZoneLabels,
    };
};

export default connect(mapStateToProps)(LeafletBufferLayer);
