import React from 'react';
import PropTypes from 'prop-types';
import L from 'leaflet';
import { connect } from 'react-redux';
import { LayerGroup } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import { injectIntl } from 'react-intl';
import isEqual from 'lodash/isEqual';
import _Get from 'lodash/get';
import { setClusterPopUp } from 'stores/actions';
import { positionOffset } from 'utils/formatters';
import { hasAlarms, getClusterMarkers } from 'utils/utils';
import TrackablesCluster from 'stores/classes/TrackablesCluster';
import { FILTER } from 'common/constants';
import { ClusterPopUp } from 'components/leaflet/popups';
import MarkerClusterGroup from './MarkerClusterGroup';


function isVdaCluster(clusterUids, trackablesMap) {
    if (clusterUids.length !== 2) return false;

    const clusterTko = trackablesMap[clusterUids[0]];
    return clusterTko && clusterTko.paired;
}

class LeafletClusterLayer extends LayerGroup {
    static propTypes = {
        currentClusterTrackables: PropTypes.arrayOf(PropTypes.shape()),
        sublocation: PropTypes.shape(),
        trackables: PropTypes.arrayOf(PropTypes.shape()),
        clusterTrackables: PropTypes.shape({}),
        trackablesMap: PropTypes.shape({}),
        selectedCluster: PropTypes.shape({
            position: PropTypes.shape({
                lat: PropTypes.number,
                lng: PropTypes.number,
            }),
            trackablesUids: PropTypes.string,
        }),
        trackablesGroupsInCluster: PropTypes.arrayOf(PropTypes.shape({
            name: PropTypes.string,
            value: PropTypes.number,
        })),
        alarmsInCluster: PropTypes.arrayOf(PropTypes.shape()),
        unresponsiveTrackablesCount: PropTypes.number,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        setClusterTrackables: PropTypes.func,
    };
    
    static defaultProps = {
        currentClusterTrackables: [],
        sublocation: { offset_x: 0, offset_y: 0 },
        trackables: [],
        trackablesMap: {},
        clusterTrackables: {},
        selectedCluster: {
            position: {
                lat: 0,
                lng: 0,
            },
            trackablesUids: '',
        },
        trackablesGroupsInCluster: [],
        alarmsInCluster: [],
        unresponsiveTrackablesCount: 0,
        intl: {
            formatMessage: () => {
            },
        },
        setClusterTrackables: () => {},
    };
    
    componentDidUpdate() {
        const clusterTrackablesFromDOM = {};
        const clusters = document.getElementsByClassName('cluster-counter');
        for (let i = 0; i < clusters.length; i += 1) {
            const clusterTrackables = clusters[i].getAttribute('data-uids').split(' ');
            clusterTrackables.forEach((uid) => {
                clusterTrackablesFromDOM[uid] = uid;
            });
        }
        if (!isEqual(clusterTrackablesFromDOM, this.props.clusterTrackables)) {
            this.props.setClusterTrackables(clusterTrackablesFromDOM);
        }
    }
    
    createClusterCustomIcon = (cluster) => {
        const { alarms, trackablesMap } = this.props;
        let clusterHasAlarm = false;

        const uids = (getClusterMarkers(cluster) || []).reduce((memo, clusterTko) => {
            if (clusterTko && clusterTko.options && clusterTko.options['data-uid']) {
                if (memo.indexOf(clusterTko.options['data-uid']) === -1) {
                    memo.push(clusterTko.options['data-uid']);
                }
            }
            return memo;
        }, []);

        alarms.forEach((alarm) => {
            if (uids.indexOf(alarm.obj_uid) !== -1) {
                clusterHasAlarm = true;
            }
        });
        cluster.uids = uids.join(' ');

        if (isVdaCluster(uids, trackablesMap)) {
            return L.divIcon({
                html: '<div style={{ display: "none" }}></div>',
                className: 'hidden',
            });
        }

        return L.divIcon({
            html: `<div class='cluster-counter' data-uids='${cluster.uids}'>
                        ${cluster._childCount || '?'}
                   </div>`,
            className: clusterHasAlarm ? 'Marker-Cluster-Alarmed' : 'Marker-Cluster-Custom',
            iconSize: L.point(40, 40, true),
        });
    };
    
    mapClusterTrackables = () => {
        const { trackables, sublocation } = this.props;

        return trackables.reduce((result, trackable) => {
            const trackablePosition = positionOffset(trackable.position, sublocation);
            if (trackablePosition) {
                result.push(<DivIcon
                    key={trackable.uid}
                    ref={trackable.uid}
                    className={trackable.uid}
                    data-uid={trackable.uid}
                    position={trackablePosition}
                >
                    <div data-uid={trackable.uid} />
                </DivIcon>);
            }
            return result;
        }, []);
    };
    
    dispatchSelectedItem = (item) => {
        this.props.dispatch(setClusterPopUp(item));
    };
    
    onClosePopup = () => {
        this.dispatchSelectedItem(null);
    };
    
    renderPopup = () => {
        const {
            selectedCluster, trackablesGroupsInCluster, alarmsInCluster, unresponsiveTrackablesCount, intl, selectedClusterTrackables,
        } = this.props;

        if (!selectedCluster || !trackablesGroupsInCluster) return null;
        const trackableUids = selectedCluster.trackablesUids && selectedCluster.trackablesUids.split(' ');
        const trackablesCluster = new TrackablesCluster(
            selectedClusterTrackables,
            trackableUids,
            unresponsiveTrackablesCount,
            trackablesGroupsInCluster,
            alarmsInCluster,
        );

        return <ClusterPopUp trackablesCluster={trackablesCluster} popupPosition={selectedCluster.position} onClosePopup={this.onClosePopup} intl={intl} />
    };
    
    render() {
        const clusterTrackables = this.mapClusterTrackables();
        if (!clusterTrackables || (clusterTrackables && clusterTrackables.length === 0)) {
            return null;
        }
        if (clusterTrackables.length < 2) {
            return null;
        }
        return (
            <div>
                <MarkerClusterGroup
                    spiderfyOnMaxZoom={false}
                    zoomToBoundsOnClick={false}
                    maxClusterRadius={14}
                    markers={clusterTrackables}
                    iconCreateFunction={this.createClusterCustomIcon}
                    onClusterClick={(event) => {
                        const trackablesUids = event.uids;
                        const position = event._cLatLng;

                        this.props.dispatch(setClusterPopUp({ position, trackablesUids }));
                    }}
                />
                {
                    this.renderPopup()
                }
            </div>
        );
    }
}

const mapStateToProps = (state, props) => {
    const groups = state.entities.groups.byUid;
    const trackables = Object.values(state.entities.trackables.byUid);
    const nodes = state.entities.nodes.byMac;

    const isPlaybackOpen = state.playback.isOpenPlayBackControl;
    const alarms = isPlaybackOpen ? state.playback.alarms : state.entities.alarms;
    
    const { selectedCluster } = state.siteView;
    
    let trackablesGroupsInCluster = [];
    let alarmsInCluster = [];
    let currentClusterTrackables = [];
    let unresponsiveTrackablesCount = 0;
    const selectedClusterTrackables = [];
    const groupsWithCount = {};

    if (selectedCluster) {
        const uids = selectedCluster.trackablesUids ? selectedCluster.trackablesUids.split(' ') : [];

        uids.forEach((tUid) => {
            const clustersTrackable = props.visibleTrackables.find(tko => tko.uid === tUid);
            const mac_address = _Get(clustersTrackable, 'node.mac_address', '');
            const node = nodes[mac_address.toUpperCase()];
            if (node) {
                Object.assign(clustersTrackable.node, { battery: node.battery, last_heard: node.last_heard, charging: node.charging });
            }
            
            if (clustersTrackable) {
                selectedClusterTrackables.push(clustersTrackable);
            }
        });

        currentClusterTrackables = trackables.reduce((memo, clusterTrackable) => {
            if (uids.some(y => y === clusterTrackable.uid)) {
                if (!clusterTrackable.isResponsive()) {
                    unresponsiveTrackablesCount += 1;
                }
                
                memo.push(clusterTrackable);
                
                if (clusterTrackable.groups) {
                    clusterTrackable.groups.forEach((y) => {
                        if (!groupsWithCount[y]) {
                            groupsWithCount[y] = 0;
                        }
                        groupsWithCount[y] += 1;
                    });
                } else {
                    if (!groupsWithCount[FILTER.NA]) {
                        groupsWithCount[FILTER.NA] = 0;
                    }
                    groupsWithCount[FILTER.NA] += 1;
                }
            }
            return memo;
        }, []);
    
        trackablesGroupsInCluster = Object.keys(groupsWithCount).map((key) => {
            if (key === FILTER.NA) {
                return {
                    name: key,
                    value: groupsWithCount[key],
                };
            }
            const group = groups[key];
            if (group && group.name) {
                return {
                    name: group.name,
                    value: groupsWithCount[key],
                };
            }
            return {
                name: '??',
                value: groupsWithCount[key],
            };
        });

        alarmsInCluster = selectedClusterTrackables.reduce((memo, tko) => (hasAlarms(tko) ? memo.concat(tko.alarms) : memo), []);
    }

    return {
        nodes,
        alarms,
        selectedCluster,
        alarmsInCluster,
        currentClusterTrackables,
        trackablesGroupsInCluster,
        unresponsiveTrackablesCount,
        trackables: props.visibleTrackables,
        trackablesMap: state.entities.trackables.byUid,
        selectedClusterTrackables,
    };
};

export default connect(mapStateToProps)(injectIntl(LeafletClusterLayer));
