import React from 'react';
import L from 'leaflet';
import _Has from 'lodash/has';
import { connect } from 'react-redux';
import { LayerGroup, Polygon, FeatureGroup } from 'react-leaflet';
import PropTypes from 'prop-types';

import EditControl from 'components/leaflet/EditControl';
import LeafletZoneCoordinateLayer from 'components/leaflet/LeafletZoneCoordinateLayer';
import {
    isStopZone, rotateGeometry, formatZoneGeometry, setWidthToGeometry,
    getStopSignGeometry, getDirectionGeometry, getRectangleZoneCenter, getStopSignHandGeometry,
    calculateAngleByTwoPoints, geometryFromDegreesToMillimeters,
} from 'utils/zoneUtils';
import { degreeToLength, zonePositions } from 'utils/formatters';
import { getUUID } from 'utils/simpleUtils';
import { setZoneWidth, addCreatedZoneGeometry } from 'stores/actions';
import { GEOMETRY_RESTRICTION_TYPES, SITE_MODE } from 'common/constants';
import { editToolbarExtend, editPolyVerticesEditExtend } from 'common/leaflet-methods';
import Position from 'stores/classes/Position';


class LeafletEditModeZoneLayer extends LayerGroup {
    static propTypes = {
        mode: PropTypes.shape(),
        width: PropTypes.number,
        error: PropTypes.any,
        rotation: PropTypes.number,
        sublocation: PropTypes.shape(),
        creatingZone: PropTypes.shape(),
        zone_type_name: PropTypes.string,
        changeGeometry: PropTypes.func,
        isAdminPageZoneCreateMode: PropTypes.bool,
        coordinateMode: PropTypes.bool,
        isRectangleRestriction: PropTypes.bool,
    };
    
    static defaultProps = {
        mode: {},
        width: 0,
        error: {},
        rotation: 0,
        creatingZone: {},
        zone_type_name: '',
        changeGeometry: () => {},
        isAdminPageZoneCreateMode: false,
        isRectangleRestriction: false,
        coordinateMode: false,
        sublocation: { offset_x: 0, offset_y: 0 },
    };

    constructor() {
        super();
        L.EditToolbar.Edit = L.EditToolbar.Edit.extend(editToolbarExtend());
        L.Edit.PolyVerticesEdit = L.Edit.PolyVerticesEdit.extend(editPolyVerticesEditExtend);
    }

    shouldComponentUpdate(nextProps) {
        if (this.props.coordinateMode !== nextProps.coordinateMode) {
            L.EditToolbar.Edit = L.EditToolbar.Edit.extend(editToolbarExtend());
            L.Edit.PolyVerticesEdit = L.Edit.PolyVerticesEdit.extend(editPolyVerticesEditExtend);
        }
        return true;
    }

    getEditedZoneGeometry = ({ mode, creatingZone, width, rotation }) => {
        const positions = [];
        let directionalArrow = {};
        if (mode.type === GEOMETRY_RESTRICTION_TYPES.DIRECTIONAL) {
            const { geometry, zoneCenter } = setWidthToGeometry(creatingZone, width);
            const zoneGeometryInDegrees = rotateGeometry({ zoneGeometryInDegrees: geometry }, rotation);
            positions.push(...zoneGeometryInDegrees);
            if (zoneCenter) {
                const directionGeometry = getDirectionGeometry(zoneCenter);
                directionalArrow = {
                    dashArray: '1',
                    dashColor: 'black',
                    dashOpacity: 1,
                    dashWeight: 1,
                    fillOpacity: 1,
                    color: 'orange',
                    positions: rotateGeometry({ zoneGeometryInDegrees: directionGeometry }, Number(rotation), zoneCenter),
                };
            }
        } else {
            positions.push(...rotateGeometry(creatingZone, rotation));
        }
        const zoneObject = {
            dashArray: '1',
            dashColor: 'black',
            dashOpacity: 0.9,
            dashWeight: 2,
            fillColor: '#001cc9',
            positions,
        };
    
        directionalArrow.positions = zonePositions({
            zone_geometry: {
                positions: directionalArrow.positions,
            },
        }, this.props.sublocation);
    
        zoneObject.positions = zonePositions({
            zone_geometry: {
                positions: zoneObject.positions,
            },
        }, this.props.sublocation);
        
        return { zoneObject, directionalArrow };
    };
    
    getStopSign = (zone, position, zone_type_name) => {
        if (!position || !zone_type_name || zone_type_name !== 'stop_zone') {
            return [];
        }
        const geometry = [];
        const stopSignGeometry = getStopSignGeometry(position);
        const stopHandGeometry = getStopSignHandGeometry(position);
        geometry.push({
            dashArray: 1,
            dashColor: 'red',
            dashOpacity: 1,
            weight: 2,
            fillOpacity: 1,
            color: 'white',
            fillColor: 'red',
            positions: stopSignGeometry,
        });
        geometry.push({
            dashArray: 1,
            dashColor: 'white',
            dashOpacity: 1,
            weight: 0.1,
            fillOpacity: 1,
            color: 'white',
            fillColor: 'white',
            positions: stopHandGeometry,
        });
    
        return geometry;
    };
    
    handleEditDirectionalZone = ({ points, rotation }) => {
        const {
            width, isAdminPageZoneCreateMode, changeWidth, dispatch,
        } = this.props;
        
        if (Array.isArray(points) && _Has(points[0], 'lat') && _Has(points[0], 'lng')) {
            const point_0 = {
                lat: (points[0].lat + points[1].lat) / 2,
                lng: (points[0].lng + points[1].lng) / 2,
            };
            const point_1 = {
                lat: (points[3].lat + points[2].lat) / 2,
                lng: (points[3].lng + points[2].lng) / 2,
            };
    
            const point_2 = {
                lat: (points[1].lat + points[2].lat) / 2,
                lng: (points[1].lng + points[2].lng) / 2,
            };
            const point_3 = {
                lat: (points[3].lat + points[0].lat) / 2,
                lng: (points[3].lng + points[0].lng) / 2,
            };
    
            const firstMedian = [point_0, point_1];
            const secondMedian = [point_2, point_3];
            const firstMedianAngle = calculateAngleByTwoPoints(point_0.lng, point_1.lng, point_0.lat, point_1.lat);
            let newWidth = width;
    
            if (Math.round((rotation - firstMedianAngle) / 90) % 2 === 0) {
                points = [...firstMedian];
                const len1 = secondMedian[0].lng - secondMedian[1].lng;
                const len2 = secondMedian[0].lat - secondMedian[1].lat;
                newWidth = Math.round(degreeToLength(Math.sqrt((len1 ** 2) + (len2 ** 2)))) / 10;
            } else {
                const len1 = firstMedian[0].lng - firstMedian[1].lng;
                const len2 = firstMedian[0].lat - firstMedian[1].lat;
                points = [...secondMedian];
                newWidth = Math.round(degreeToLength(Math.sqrt((len1 ** 2) + (len2 ** 2)))) / 10;
            }
            if (isAdminPageZoneCreateMode) {
                changeWidth(newWidth);
            } else {
                dispatch(setZoneWidth(newWidth));
            }
        }
        
        return points;
    };
    
    switchOnLeafletEditMode = () => {
        const editModeLeafletButtons = document.getElementsByClassName('leaflet-draw-edit-edit');
        if (editModeLeafletButtons) {
            for (let i = 0; i < editModeLeafletButtons.length; i += 1) {
                editModeLeafletButtons[i].click();
            }
        }
    };
    
    _onEditPath = (e) => {
        const {
            mode, dispatch, isAdminPageZoneCreateMode, rotation, sublocation, changeGeometry,
        } = this.props;
        
        let points = Array.isArray(e.poly._latlngs[0]) ? e.poly._latlngs[0] : e.poly._latlngs;
        
        if (mode.type === GEOMETRY_RESTRICTION_TYPES.DIRECTIONAL && points) {
            points = this.handleEditDirectionalZone({ points, rotation });
        }
    
        const sublocation_offset = { offset_x: -sublocation.offset_x, offset_y: -sublocation.offset_y };
        const newZoneLatLng = points.map(point => new Position(point, sublocation_offset).latlngObj);
        const zoneGeometryInDegrees = rotateGeometry(formatZoneGeometry(newZoneLatLng), -rotation);
        const zoneGeometryInMillimeters = geometryFromDegreesToMillimeters(zoneGeometryInDegrees);
        
        if (isAdminPageZoneCreateMode) {
            changeGeometry({ zoneGeometryInDegrees, zoneGeometryInMillimeters });
        } else {
            dispatch(addCreatedZoneGeometry({ zoneGeometryInDegrees, zoneGeometryInMillimeters }));
        }
    };
    
    render() {
        const {
            mode, creatingZone, rotation, width, isRectangleRestriction, zone_type_name, error, coordinateMode,
        } = this.props;
        
        if (!mode || mode.name !== SITE_MODE.EDIT_ZONE) {
            return null;
        }
        const isStopZoneEditing = isStopZone({ zone_type_name });
        const { zoneObject, directionalArrow } = this.getEditedZoneGeometry({ mode, width, rotation, creatingZone });
        const zoneCenter = isStopZoneEditing ? getRectangleZoneCenter(zoneObject.positions) : [];
        const stopSign = isStopZoneEditing ? this.getStopSign(creatingZone, zoneCenter, zone_type_name) : [];
        
        this.switchOnLeafletEditMode();

        return (
            <div>
                <FeatureGroup ref="fg">
                    <div>
                        {
                            !coordinateMode && <EditControl /* added something styles, that don't show buttons of edit and trash */
                                position="bottomleft"
                                onEdited={this._onEditPath}
                                onEditVertex={this._onEditPath}
                                edit={{
                                    shapeOptions: {
                                        color: 'red',
                                        opacity: 0.1,
                                    },
                                    remove: true,
                                }}
                            />
                        }
                        <Polygon
                            key={getUUID()}
                            opacity={zoneObject.dashOpacity}
                            fillOpacity={0.2}
                            originalShape={isRectangleRestriction ? GEOMETRY_RESTRICTION_TYPES.RECTANGLE : ''}
                            fillColor={zoneObject.fillColor}
                            color="red"
                            weight={zoneObject.dashWeight}
                            dashArray={zoneObject.dashArray}
                            positions={zoneObject.positions}
                        />
                    </div>
                </FeatureGroup>
                
                {directionalArrow.positions && <Polygon
                    opacity={directionalArrow.dashOpacity}
                    fillOpacity={0.2}
                    fillColor={directionalArrow.fillColor}
                    color="orange"
                    weight={directionalArrow.dashWeight}
                    dashArray={directionalArrow.dashArray}
                    positions={directionalArrow.positions}
                />}
                
                {isStopZoneEditing && stopSign.map((polygon, index) => (
                    <Polygon
                        key={index}
                        opacity={polygon.dashOpacity || 1}
                        fillOpacity={polygon.fillOpacity || 1}
                        fillColor={polygon.fillColor || 'red'}
                        color={polygon.dashColor || 'red'}
                        weight={polygon.dashWeight || 1}
                        dashArray={polygon.dashArray || 1}
                        positions={polygon.positions}
                    />
                ))}
                <LeafletZoneCoordinateLayer error={error} />
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    let isRectangleRestriction;
    const restriction = state.siteView.geometry.restriction;
    const zone_type_name = state.modals.zoneModals.zoneChooseGeometry.zone_type_name || '';
    const isAdminPageZoneCreateMode = state.modals.zoneModals.zoneCreateAdminPage.open;
    if (restriction && Array.isArray(restriction)) {
        isRectangleRestriction = restriction.indexOf(GEOMETRY_RESTRICTION_TYPES.RECTANGLE) !== -1;
    }
    return {
        zone_type_name,
        isRectangleRestriction,
        isAdminPageZoneCreateMode,
    };
};

export default connect(mapStateToProps)(LeafletEditModeZoneLayer);
