import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { LayerGroup, GeoJSON } from 'react-leaflet';
import { createSelector } from 'reselect';
import { createGrid } from '../../utils/formatters';

class LeafletGrids extends LayerGroup {
    static propTypes = {
        step: PropTypes.number,
        lines: PropTypes.arrayOf(PropTypes.array),
        isShowingGrid1m: PropTypes.bool,
        isShowingGrid5m: PropTypes.bool,
    };
    
    static defaultProps = {
        step: 1,
        lines: [],
        isShowingGrid1m: false,
        isShowingGrid5m: false,
    };

    calculateLines = (grid) => {
        let geometries = [];
        const axesGeometries = [];
        if (Array.isArray(grid)) {
            geometries = grid.map((rectKey) => {
                const rect = rectKey;
                const isAxis = (rect[0][0] && rect[0][1]) || (rect[1][0] && rect[1][1]);
                if (!isAxis) {
                    axesGeometries.push({
                        type: 'LineString',
                        coordinates: rectKey,
                    });
                }
                return {
                    type: 'LineString',
                    coordinates: rectKey,
                };
            });
        }
        const data = {
            type: 'Feature',
            geometry: {
                type: 'GeometryCollection',
                geometries,
            },
        };
        const dataAxes = {
            type: 'Feature',
            geometry: {
                type: 'GeometryCollection',
                geometries: axesGeometries,
            },
        };

        const geometriesLength = data.geometry.geometries.length;

        return ([
            (<GeoJSON
                data={data}
                key={geometriesLength}
                style={() => ({
                    color: '#336ad7',
                    weight: 0.6,
                })}
            />),
            (<GeoJSON
                data={dataAxes}
                key={`dataAxes${geometriesLength}`}
                style={() => ({
                    color: 'green',
                    weight: 2.5,
                })}
            />),
        ]);
    };

    render() {
        const { lines, isShowingGrid1m, isShowingGrid5m } = this.props;
        return (
            <div>
                {(isShowingGrid1m || isShowingGrid5m) && this.calculateLines(lines)}
            </div>
        );
    }
}

const getFloorPlanWidth = (state, props) => {
    const { selectedFloorplan } = state.siteView;
    const { floorplan } = props;
    if (floorplan && floorplan.width) {
        return floorplan.width * 2;
    }
    if (selectedFloorplan && selectedFloorplan.width) {
        return selectedFloorplan.width * 2;
    }
    return undefined;
};

const getFloorPlanHeight = (state, props) => {
    const { selectedFloorplan } = state.siteView;
    const { floorplan } = props;
    if (floorplan && floorplan.height) {
        return floorplan.height * 2;
    }
    
    if (selectedFloorplan && selectedFloorplan.height) {
        return selectedFloorplan.height * 2;
    }
    return undefined;
};

const getStep = (state, props) => (props && props.step);

const getZoom = (state, props) => (props.zoom || state.siteViewFilter.leaflet.zoom);

const isGridShowing = (state, props, step) => {
    const varTemplate = `isShowingGrid${step}m`;
    return (props[varTemplate] || state.siteViewFilter.layerFilter[varTemplate]) && props.step === step;
};

const getGridShowingState1m = (state, props) => isGridShowing(state, props, 1);
const getGridShowingState5m = (state, props) => isGridShowing(state, props, 5);

const makeGetGridLines = () => createSelector(
    [getFloorPlanWidth, getFloorPlanHeight, getStep, getZoom],
    (width, height, step, zoom) => createGrid(zoom, step, width, height),
);

const makeGetGrids = () => createSelector(
    [makeGetGridLines(), getGridShowingState1m, getGridShowingState5m, getZoom],
    (lines, isShowingGrid1m, isShowingGrid5m, zoom) => ({ lines, zoom, isShowingGrid1m, isShowingGrid5m }),
);

const makeMapStateToProps = () => {
    const getGrids = makeGetGrids();
    return (state, props) => getGrids(state, props);
};

export default connect(makeMapStateToProps)(LeafletGrids);
