import React, { PureComponent, Fragment } from 'react';
import { connect } from 'react-redux';
import Control from 'react-leaflet-control';
import PropTypes from 'prop-types';
import _isUndefined from 'lodash/isUndefined';
import _isNil from 'lodash/isNil';

import { ZonePrototype } from 'stores/classes';
import { setLayerFilter } from 'stores/actions/siteViewActions';
import { LEAFLET_VIEW_FILTER } from 'common/ui-labels';
import { formatDbNameToHumanReadable } from 'utils/htmlUtils';

class LeafletLayerFilter extends PureComponent {
    static propTypes = {
        layerFilter: PropTypes.shape({
            isOpenViewFilter: PropTypes.bool,
            isOpenZoneFilter: PropTypes.bool,

            isShowingActiveZonesOnly: PropTypes.bool,
            isShowingInactiveZonesOnly: PropTypes.bool,

            isShowingAllZones: PropTypes.bool,
            isShowingAlarmZones: PropTypes.bool,
            isShowingInfoZones: PropTypes.bool,
            isShowingInactiveZones: PropTypes.bool,

            isShowingActiveAlarmZones: PropTypes.bool,
            isShowingInactiveAlarmZones: PropTypes.bool,
            isShowingActiveInfoZones: PropTypes.bool,
            isShowingInactiveInfoZones: PropTypes.bool,

            isShowingTrackableLabels: PropTypes.bool,
            isShowingZoneLabels: PropTypes.bool,
            isShowingAnchors: PropTypes.bool,
            isShowingScale: PropTypes.bool,
        }),
        zonesPrototypes: PropTypes.arrayOf(PropTypes.shape()),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        dispatch: PropTypes.func,
        position: PropTypes.string,
    };
    
    static defaultProps = {
        layerFilter: {
            isOpenViewFilter: false,
            isOpenZoneFilter: false,

            isShowingActiveZonesOnly: true,
            isShowingInactiveZonesOnly: true,

            isShowingAllZones: true,
            isShowingAlarmZones: true,
            isShowingInfoZones: true,
            isShowingCustomZones: true,
            isShowingInactiveZones: true,

            isShowingActiveAlarmZones: true,
            isShowingInactiveAlarmZones: true,
            isShowingActiveInfoZones: true,
            isShowingInactiveInfoZones: true,

            isShowingTrackableLabels: true,
            isShowingZoneLabels: true,
            isShowingAnchors: true,
            isShowingScale: true,
        },
        zonesPrototypes: [],
        intl: {
            formatMessage: () => {
            },
        },
        dispatch: () => {},
        position: 'topright',
    };

    constructor(props) {
        super(props);

        this.filterCheckboxes = [];

        this.layerFilter = this.getLayerFilters(props.zonesPrototypes, props.intl);

        this.ulRef = null;
    }

    componentWillUnmount() {
        this.ulRef && this.ulRef.removeEventListener('wheel', this.handleWheel);
    }

    handleWheel = (e) => {
        e.stopPropagation();
    }

    getLayerFilters = (zonesPrototypes, intl) => {
        const areCustomZonesAvailable = zonesPrototypes.length > 0;
        return {
            label: LEAFLET_VIEW_FILTER.TITLES.VIEW_FUNC(intl),
            nameOfVarOnLeaflet: 'isOpenViewFilter',
            children: [
                {
                    label: LEAFLET_VIEW_FILTER.FILTERS.TRACKABLE_LABELS_FUNC(intl),
                    nameOfVarOnLeaflet: 'isShowingTrackableLabels',
                },
                {
                    label: LEAFLET_VIEW_FILTER.FILTERS.ZONE_LABELS_FUNC(intl),
                    nameOfVarOnLeaflet: 'isShowingZoneLabels',
                },
                {
                    label: LEAFLET_VIEW_FILTER.FILTERS.ZONES_FUNC(intl),
                    nameOfVarOnLeaflet: 'isShowingAllZones',
                    children: [
                        {
                            label: LEAFLET_VIEW_FILTER.FILTERS.ACTIVE_ONLY_FUNC(intl),
                            nameOfVarOnLeaflet: 'isShowingActiveZonesOnly',
                            parent: 'isShowingAllZones',
                        },
                        {
                            label: LEAFLET_VIEW_FILTER.FILTERS.INACTIVE_ONLY_FUNC(intl),
                            nameOfVarOnLeaflet: 'isShowingInactiveZonesOnly',
                            parent: 'isShowingAllZones',
                        },
                        {
                            label: LEAFLET_VIEW_FILTER.FILTERS.SAFETY_ZONES_FUNC(intl),
                            nameOfVarOnLeaflet: 'isShowingAlarmZones',
                            parent: 'isShowingAllZones',
                        },
                        {
                            label: LEAFLET_VIEW_FILTER.FILTERS.INFO_ZONES_FUNC(intl),
                            nameOfVarOnLeaflet: 'isShowingInfoZones',
                            parent: 'isShowingAllZones',
                        },
                        ...(this.getCustomZoneFilters(intl)),
                    ],
                },
                {
                    label: LEAFLET_VIEW_FILTER.FILTERS.SCALING_FUNC(intl),
                    nameOfVarOnLeaflet: 'isShowingScale',
                },
            ],
        };
    };

    getCustomZoneFilters = (intl) => {
        const { zonesPrototypes } = this.props;

        return zonesPrototypes.map((zp) => {
            const { display_name, name: zoneTypeName } = zp.zone_type;
            const currentZoneFilterName = `isShowing_${zoneTypeName}`;

            return ({
                label: formatDbNameToHumanReadable(intl, zoneTypeName.toUpperCase(), display_name),
                nameOfVarOnLeaflet: currentZoneFilterName,
                parent: 'isShowingAllZones',
            });
        });
    };

    findItem = (elem, elemName) => {
        for (let i = 0; i < elem.children.length; i++) {
            if (elem.children[i].nameOfVarOnLeaflet === elemName) {
                return elem.children[i];
            } else if (elem.children[i].children) {
                const foundItem = this.findItem(elem.children[i], elemName);
                if (foundItem) {
                    return foundItem;
                }
            }
        }

        return false;
    };

    changeViewFilter = (args) => {
        const changedParams = {};
        Object.keys(args).forEach((key) => {
            if (args[key] !== this.props.layerFilter[key] && args[key] !== undefined) {
                changedParams[key] = args[key];
            }
        });

        this.props.dispatch(setLayerFilter(changedParams));
    };

    getFilterCheckboxElement = checkboxName => this.filterCheckboxes[checkboxName];

    renderList = filters => filters.map((_filter) => {
        const isDelimiterNeeded = _filter.nameOfVarOnLeaflet === 'isShowingInactiveZonesOnly';
        const delimiterClass = (isDelimiterNeeded) ? 'delimiter-bottom-small' : '';
        return (<li key={_filter.label} className={`filter-menu-item ViewFilter-List-Elem ${delimiterClass}`}>
            {this.renderFilterMenu(_filter)}
        </li>);
    });

    isChecked = value => value || (value === null);

    isIndeterminate = (filter) => {
        const parentElem = this.getFilterCheckboxElement(filter);
        if (!_isNil(parentElem)) {
            const { layerFilter } = this.props;

            parentElem.indeterminate = layerFilter[filter] === null;
        }
    };

    renderFilterMenu = (filter) => {
        const { layerFilter } = this.props;

        if (_isUndefined(filter)) {
            return null;
        }

        return (<Fragment>
            <input
                ref={(ref) => {
                    this.filterCheckboxes[filter.nameOfVarOnLeaflet] = ref;
                    this.isIndeterminate(filter.nameOfVarOnLeaflet);
                }}
                data-name={filter.nameOfVarOnLeaflet}
                data-parent-name={filter.parent}
                type="checkbox"
                id={filter.nameOfVarOnLeaflet}
                checked={this.isChecked(layerFilter[filter.nameOfVarOnLeaflet])}
                onChange={() => {
                    this.changeViewFilter({
                        [filter.nameOfVarOnLeaflet]: !layerFilter[filter.nameOfVarOnLeaflet],
                    });
                }}
            />
            <label
                htmlFor={filter.nameOfVarOnLeaflet}
            >
                {filter.label} {filter.children && <i className="fa fa-caret-left margin-l-s" />}
            </label>
            {
                filter.children && <ul
                    className="filter-menu-dropdown ViewFilter-List ViewFilter-List-Overflow-Y"
                    ref={(myRef) => {
                        if (!this.ulRef) {
                            this.ulRef = myRef;
                            this.ulRef.addEventListener('wheel', this.handleWheel);
                        }
                    }}
                >
                    {this.renderList(filter.children)}
                </ul>
            }
        </Fragment>);
    };
    
    render() {
        const rootFilterElement = this.layerFilter;

        return (
            <Control position={this.props.position}>
                <div className="Leaflet-ViewFilter">
                    <ul className="filter-menu-dropdown ViewFilter-List">
                        <li className="filter-menu-item ViewFilter-List-Elem ViewFilter-Button">
                            <button className="text-center">
                                {rootFilterElement.label}
                            </button>
                            <ul className="filter-menu-dropdown ViewFilter-List ViewFilter-RootList">
                                {
                                    this.layerFilter.children.map(filter => (
                                        <li
                                            className="filter-menu-item ViewFilter-List-Elem"
                                            key={filter.label}
                                        >
                                            {this.renderFilterMenu(filter)}
                                        </li>
                                    ))
                                }
                            </ul>
                        </li>
                    </ul>
                </div>
            </Control>
        );
    }
}

const mapStateToProps = state => ({
    layerFilter: state.siteViewFilter.layerFilter,
    zonesPrototypes: ZonePrototype.getAllZonePrototypes(state),
});

export default connect(mapStateToProps)(LeafletLayerFilter);
