import React, { Fragment } from 'react';
import { connect } from 'react-redux';
import { LayerGroup, GeoJSON } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import PropTypes from 'prop-types';

import Position from 'stores/classes/Position';

import { createLabels } from '../../utils/formatters';
import { DEFAULT_VALUES, MAX_MAP_BOUNDS_LATLNG_OBJECT } from '../../common/constants';

class LeafletScaling extends LayerGroup {
    static propTypes = {
        zoom: PropTypes.number,
        isShowingScale: PropTypes.bool,
        bounds: PropTypes.shape(),
        floorplan: PropTypes.shape(),
        floorplanSize: PropTypes.shape(),
    };
    
    static defaultProps = {
        zoom: DEFAULT_VALUES.MAP_ZOOM.DEFAULT,
        isShowingScale: true,
        bounds: {},
        floorplan: {},
        floorplanSize: {},
    };

    getLineString = (swPosition, nePosition) => {
        const swPositionObj = new Position(swPosition);
        const nePositionObj = new Position(nePosition);
        const coordinates = [swPositionObj.latlng, nePositionObj.latlng];

        return { type: 'LineString', coordinates };
    };

    getGeometryCollection = (geometries) => {
        const geometry = { type: 'GeometryCollection', geometries };
        return { type: 'Feature', geometry };
    };

    renderGridLines = (grid, step) => {
        const bounds = Object.assign({}, MAX_MAP_BOUNDS_LATLNG_OBJECT);
        const { _southWest: SW, _northEast: NE } = bounds;

        const geometries = grid.reduce((memo, rectKey) => {
            const horizontalLine = this.getLineString([SW.lat * 100, rectKey], [NE.lat * 100, rectKey]);
            memo.push(horizontalLine);

            const verticalLine = this.getLineString([rectKey, SW.lng * 100], [rectKey, NE.lng * 100]);
            memo.push(verticalLine);

            return memo;
        }, []);

        const dataGrid = this.getGeometryCollection(geometries);

        const horizontalLine = this.getLineString([0, SW.lng * 100], [0, NE.lng * 100]);
        const verticalLine = this.getLineString([SW.lat * 100, 0], [NE.lat * 100, 0]);

        const dataAxes = this.getGeometryCollection([horizontalLine, verticalLine]);

        const geometriesLength = dataGrid.geometry.geometries.length;
        const scalingLinesKey = `key_${NE.lat}_${step}_${geometriesLength}`;

        return ([
            (<GeoJSON
                data={dataGrid}
                key={scalingLinesKey}
                style={() => ({
                    color: 'darkblue',
                    weight: 0.3,
                })}
            />),
            (<GeoJSON
                data={dataAxes}
                key={`dataAxes${geometriesLength}`}
                style={() => ({
                    color: 'green',
                    weight: 1.5,
                })}
            />),
        ]);
    };

    renderLabels = (grid, bounds, step) => {
        const labelIndex = grid.length / 2;
        return grid && grid.map((rectKey, index) => {
            const label = ((index < labelIndex) ? -index : (index - labelIndex) + 1) * step;

            const verticalPosition = new Position([rectKey, bounds._southWest.lng]);
            const horizontalPosition = new Position([bounds._southWest.lat, rectKey]);

            return (
                <div key={index}>
                    <DivIcon position={verticalPosition.latlng}>
                        <div style={{ fontSize: '2em', marginLeft: '20px', color: 'grey' }}>{label}</div>
                    </DivIcon>
                    <DivIcon position={horizontalPosition.latlng}>
                        <div style={{ fontSize: '2em', marginTop: '-30px', color: 'grey' }}>{label}</div>
                    </DivIcon>
                </div>
            );
        });
    };

    renderScaling = (grid, bounds, step) => (<Fragment>
        { this.renderLabels(grid, bounds, step) }
        { this.renderGridLines(grid, step) }
    </Fragment>);

    render() {
        const {
            zoom, bounds, isShowingScale, floorplan, floorplanSize,
        } = this.props;
        
        if (!bounds || Object.keys(bounds).length === 0) return null;
        
        let step;
        switch (zoom) {
            case 12:
                step = 300;
                break;
            case 13:
                step = 200;
                break;
            case 14:
                step = 100;
                break;
            case 15:
                step = 50;
                break;
            case 16:
                step = 20;
                break;
            case 17:
                step = 10;
                break;
            case 18:
            case 19:
                step = 5;
                break;
            default:
                if (zoom > 19) {
                    step = 1;
                } else if (zoom < 12) {
                    step = 500;
                }
        }
        
        let width = 0;
        let height = 0;
        
        if (floorplan && floorplan.width && floorplan.height) {
            width = floorplan.width;
            height = floorplan.height;
        }
        if (floorplanSize && floorplanSize.width && floorplanSize.height) {
            width = floorplanSize.width;
            height = floorplanSize.height;
        }
        
        const grid = createLabels(zoom, step, width, height, floorplan.image_scale);

        return (
            <div>
                {isShowingScale && grid && this.renderScaling(grid, bounds, step)}
            </div>
        );
    }
}

const mapStateToProps = (state, props) => ({
    zoom: props.zoom ? Math.round(props.zoom) : Math.round(state.siteViewFilter.leaflet.zoom),
    isShowingScale: props.isShowingScale || state.siteViewFilter.layerFilter.isShowingScale,
    bounds: props.bounds || state.siteViewFilter.leaflet.bounds,
});

export default connect(mapStateToProps)(LeafletScaling);
