import React from 'react';
import { connect } from 'react-redux';
import { LayerGroup } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import _Get from 'lodash/get';
import { positionOffset } from 'utils/formatters';
import { calculateLeafletElementClassName, calculateTrackableClassName } from 'utils/htmlUtils';
import { isVehicle } from 'utils/utils';
import { getNodeInfoByMac } from 'utils/trackableUtils';
import { _getHostName } from 'utils/browserStorage';
import { selectLeafletItemForPopup } from 'stores/actions';
import { Group } from 'stores/classes';
import { DEFAULT_VALUES } from 'common/constants';
import { TrackablePopUp, VdaPopUp } from 'components/leaflet/popups';

class LeafletTrackableLayer extends LayerGroup {
    static propTypes = {
        visibleTrackables: PropTypes.arrayOf(PropTypes.shape()),
        clusterTrackables: PropTypes.shape({}),
        nodes: PropTypes.shape({}),
        sublocation: PropTypes.shape(),
        selectedTrackableForPopup: PropTypes.shape(),
        playbackSpeed: PropTypes.number,
        isShowingTrackableLabels: PropTypes.bool,
    };

    static defaultProps = {
        visibleTrackables: [],
        clusterTrackables: {},
        nodes: {},
        sublocation: { offset_x: 0, offset_y: 0 },
        selectedTrackableForPopup: null,
        playbackSpeed: 1,
        isShowingTrackableLabels: true,
    };

    onTrackableClick = (trackable) => {
        this.dispatchSelectedItem(trackable);
    };

    onClosePopup = () => {
        this.dispatchSelectedItem(null);
    };

    dispatchSelectedItem = (item) => {
        this.props.dispatch(selectLeafletItemForPopup(item));
    };

    getPairedTrackablesDisplayState = (tko, visibleTrackables) => {
        const isFirstTkoResponsive = tko && tko.isResponsive();
        const pairedTko = visibleTrackables.find((x) => x.uid === tko.paired);
        const isPairedTkoResponsive = pairedTko && pairedTko.isResponsive();

        const arePairedTkosVisible = tko.paired
            && visibleTrackables.some((x) => x.uid === tko.paired)
            && visibleTrackables.some((x) => x.uid === tko.uid);

        return !!(isFirstTkoResponsive && isPairedTkoResponsive && arePairedTkosVisible);
    };

    renderTrackables = (isPlaybackOpen) => {
        const {
            visibleTrackables,
            clusterTrackables,
            sublocation,
            groups,
            playbackSpeed,
            isShowingTrackableLabels,
        } = this.props;

        return visibleTrackables.reduce((memo, trackable) => {
            const trackablePosition = positionOffset(trackable.position, sublocation);
            if (!clusterTrackables[trackable.uid]) {
                if (trackablePosition) {
                    const trackableColor = trackable.groups ? Group.getColorOfGroup(trackable.groups[0], groups) : DEFAULT_VALUES.TRACKABLE_COLOR;
                    const arePairedTrackablesShowing = this.getPairedTrackablesDisplayState(trackable, visibleTrackables);

                    const isStationaryTag = (_trackable) => _Get(_trackable, 'tag_config.stationary', false);

                    const trackableClassName = calculateTrackableClassName(trackable, isStationaryTag);
                    const leafletElementClassName = calculateLeafletElementClassName(playbackSpeed, arePairedTrackablesShowing ? 'paired-trackable' : undefined);

                    const trackableObjectStyle = isStationaryTag(trackable)
                        ? { borderColor: `transparent transparent ${trackableColor} transparent` }
                        : { backgroundColor: trackableColor };

                    if (arePairedTrackablesShowing) {
                        const isCategoryVehicle = isVehicle(trackable.category);
                        if (isCategoryVehicle) {
                            memo.push(<DivIcon
                                key={trackable.uid}
                                ref={trackable.uid}
                                onClick={this.onTrackableClick.bind(this, trackable)}
                                position={trackablePosition}
                                className={`${leafletElementClassName} ${(trackable.isAlarmed) ? 'alarmed' : ''}`}
                            >
                                <i className="fa fa-truck" aria-hidden="true" />
                            </DivIcon>);
                        }
                    } else {
                        memo.push(<DivIcon
                            key={trackable.uid}
                            ref={trackable.uid}
                            onClick={this.onTrackableClick.bind(this, trackable)}
                            position={trackablePosition}
                            className={leafletElementClassName}
                        >
                            <div className={trackableClassName} style={trackableObjectStyle}>
                                {
                                    isShowingTrackableLabels
                                    && (
                                        <span className="trackable-name">
                                            {trackable.getLeafletTitle()}
                                        </span>
                                    )
                                }
                                {
                                    !trackable.isResponsive(isPlaybackOpen)
                                    && (
                                        <i className="unresponsive-img unresponsive-sign" />
                                    )
                                }
                            </div>
                        </DivIcon>);
                    }
                }
            } else {
                // Do Not Filter Out. When cluster trackables are not in render list, browser could not remove them and
                // this leads to a memory leak
                memo.push(<DivIcon
                    key={trackable.uid}
                    ref={trackable.uid}
                    position={trackablePosition}
                    className="Hidden-Trackable-Marker"
                >
                    <div className="Hidden-Trackable-Marker" />
                </DivIcon>);
            }
            return memo;
        }, []);
    };

    renderPopup = () => {
        const { selectedTrackableForPopup, visibleTrackables, sublocation, playbackSpeed, intl, nodes, trackableTemplates } = this.props;
        const visiblePairedTrackable = visibleTrackables.find((tko) => tko.uid === selectedTrackableForPopup.paired);
        if (visiblePairedTrackable) {
            const mac_address = _Get(visiblePairedTrackable, 'node.mac_address', null);
            const nodeInfo = getNodeInfoByMac(mac_address, nodes);
            const pairedTrackable = visiblePairedTrackable.getProperties();
            if (nodeInfo) {
                Object.assign(pairedTrackable.node, nodeInfo);
            }
            const pairedTrackables = [selectedTrackableForPopup, pairedTrackable];
            return (
                <VdaPopUp
                    selectedClusterTrackables={pairedTrackables}
                    sublocation={sublocation}
                    onClose={this.onClosePopup}
                    playbackSpeed={playbackSpeed}
                    intl={intl}
                />
            );
        }

        return (
            <TrackablePopUp
                trackable={selectedTrackableForPopup}
                sublocation={sublocation}
                onClose={this.onClosePopup}
                playbackSpeed={playbackSpeed}
                trackableTemplates={trackableTemplates}
                intl={intl}
            />
        );
    };

    componentDidUpdate() {
        Object.keys(this.refs).forEach((key) => {
            const keyRef = this.refs[key];
            if (keyRef && keyRef.leafletElement && keyRef.leafletElement._path && keyRef.leafletElement._path.attributes.class) {
                // for trackable transition playback
                keyRef.leafletElement._path.attributes.class.nodeValue = keyRef.props.className;
            } else if (keyRef && keyRef.leafletElement && keyRef.leafletElement._icon) {
                // for label transition playback
                keyRef.leafletElement._icon.className = keyRef.props.className;
            }
        });
    }

    render() {
        return (
            <div>
                {this.renderTrackables(this.props.isPlaybackOpen)}
                {this.props.selectedTrackableForPopup && this.renderPopup()}
            </div>
        );
    }
}

const mapStateToProps = (state, props) => {
    const nodes = state.entities.nodes.byMac;
    const { selectedLeafletElement } = state.siteView;

    const selectedTrackable = selectedLeafletElement ? props.visibleTrackables.find((x) => x.uid === selectedLeafletElement.uid) : null;
    const selectedTrackableCopy = selectedTrackable ? selectedTrackable.getProperties() : null;

    const mac_address = _Get(selectedTrackableCopy, 'node.mac_address', null);
    const nodeInfo = getNodeInfoByMac(mac_address, nodes);

    if (nodeInfo) {
        Object.assign(selectedTrackableCopy.node, nodeInfo);
    }

    return {
        groups: state.entities.groups,
        host: _getHostName(state.appContext),
        isPlaybackOpen: state.playback.isOpenPlayBackControl,
        isShowingTrackableLabels: state.siteViewFilter.layerFilter.isShowingTrackableLabels,
        nodes,
        playbackSpeed: state.playback.playSpeed,
        selectedTrackableForPopup: selectedTrackableCopy,
        trackableTemplates: state.entities.trackableTemplates,
        visibleTrackables: props.visibleTrackables,
    };
};

export default connect(mapStateToProps)(injectIntl(LeafletTrackableLayer));
