import React from 'react';
import { connect } from 'react-redux';
import { Polygon, LayerGroup } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import PropTypes from 'prop-types';
import _isEqual from 'lodash/isEqual';
import {
    zonePositions, rotateGeometry, zoneFullActive, positionOffset, getDirectionGeometry, getRectangleZoneCenter,
    getStopSignGeometry, getStopSignHandGeometry,
} from 'utils';
import { userHasPrivileges } from 'utils/utils';
import { zoneSeverityColor } from 'utils/simpleUtils';
import { ZonesPopUp } from 'components/leaflet/popups';
import { openZoneEditGeoSitePage, selectLeafletItemForPopup } from 'stores/actions';
import { USER_ROLES } from 'common/user-roles';


class LeafletZoneLayer extends LayerGroup {
    static propTypes = {
        creatingZone: PropTypes.shape({}),
        zonesPrototypes: PropTypes.arrayOf(PropTypes.shape({})),
        mouseClickPosition: PropTypes.shape({}),
        visibleZones: PropTypes.arrayOf(PropTypes.shape({})),
        selectedZones: PropTypes.arrayOf(PropTypes.shape({})),
        sublocation: PropTypes.shape({}),
        handleSiteViewZoneSelection: PropTypes.func,
        currentRole: PropTypes.string,
        isShowingZoneLabels: PropTypes.bool,
        withAlarm: PropTypes.bool,
    };
    
    static defaultProps = {
        creatingZone: {},
        zonesPrototypes: [],
        mouseClickPosition: null,
        visibleZones: [],
        selectedZones: [],
        sublocation: { offset_x: 0, offset_y: 0 },
        handleSiteViewZoneSelection: () => {},
        currentRole: '',
        isShowingZoneLabels: true,
        withAlarm: true,
    };
    
    constructor(props) {
        super(props);
        
        this.state = {
            firstLoad: true,
        };
    }
    
    shouldComponentUpdate(nextProps) {
        const zonesChanged = !_isEqual(nextProps.visibleZones, this.props.visibleZones);
        const creatingZoneChanged = !_isEqual(nextProps.creatingZone, this.props.creatingZone);
        const sublocationChanged = !_isEqual(nextProps.sublocation, this.props.sublocation);
        const showLabelsChanged = nextProps.isShowingZoneLabels !== this.props.isShowingZoneLabels;
        const selectedZoneChanged = !_isEqual(nextProps.selectedZones, this.props.selectedZones);
        const popUpPositionChanged = !_isEqual(nextProps.mouseClickPosition, this.props.mouseClickPosition);

        // Need to fix leaflet problem with render zone label WU-1908
        if (this.state.firstLoad) {
            this.setState({ firstLoad: false });
            this.forceUpdate();
        }
        
        
        return zonesChanged || creatingZoneChanged || sublocationChanged || showLabelsChanged || selectedZoneChanged || popUpPositionChanged;
    }

    selectZone = (zone) => {
        const { handleSiteViewZoneSelection } = this.props;
        handleSiteViewZoneSelection(zone);
    };

    openZoneEditModal = (zone) => {
        if (userHasPrivileges(this.props.currentRole, USER_ROLES.PROJECT_OPERATIONS_ADMIN)) {
            this.props.dispatch(selectLeafletItemForPopup(null));
            this.props.dispatch(openZoneEditGeoSitePage(zone));
        }
    };

    onPopupClose = () => this.props.dispatch(selectLeafletItemForPopup(null));
    
    getZonesLabels = () => (
        this.props.visibleZones.reduce((memo, zone) => {
            if (zone.zone_geometry && zone.zone_geometry.positions) {
                const position = zone.zone_geometry.positions.reduce((accumulator, current) => {
                    if (!accumulator.x || (current.x > accumulator.x && current.y > accumulator.y)) {
                        accumulator.x = current.x;
                        accumulator.y = current.y;
                    }
                    return accumulator;
                }, {});
                const translatedPosition = positionOffset(position, this.props.sublocation);

                if (translatedPosition) {
                    memo.push(<DivIcon
                        key={`${zone.uid}`}
                        onClick={this.selectZone.bind(this, zone)}
                        position={translatedPosition}
                        className="Hidden-Icon"
                    >
                        <span className="Leaflet-ZoneTitle">
                            {zone.name}
                        </span>
                    </DivIcon>);
                }
            }
            return memo;
        }, [])
    );
    
    showZoneDirection = (zone, createdZonePrototype) => (zone.user_params && !isNaN(zone.user_params.direction) && createdZonePrototype
            && createdZonePrototype.zone_geometry_restriction && createdZonePrototype.zone_geometry_restriction
        && createdZonePrototype.zone_geometry_restriction.indexOf('directional') !== -1);
    
    showStopSign = (zone, createdZonePrototype) => (createdZonePrototype && createdZonePrototype.zone_type
        && createdZonePrototype.zone_type.name === 'stop_zone');

    makePolygon = ({ zone, dashOpacity, dashColor, dashWeight, dashArray, fillColor, zonesPrototypes }) => {
        const polygonsObjects = [];
        const stopSigns = [];
        const zonePrototype = zonesPrototypes.find(zone_prototype => zone_prototype.isZoneTypeIdEqualTo(zone.zone_type_id));
        const positions = zonePositions(zone, this.props.sublocation);
        const zoneCenter = getRectangleZoneCenter(positions);
        const { color, fillOpacity } = zonePrototype ? zonePrototype.getZoneColorProps() : {};
        
        polygonsObjects.push({
            dashOpacity,
            dashColor,
            dashWeight,
            dashArray,
            fillColor: color || fillColor,
            fillOpacity: fillOpacity || 0.2,
            positions,
            zone,
        });
        
        if (this.showZoneDirection(zone, zonePrototype)) {
            const directionGeometry = getDirectionGeometry(zoneCenter);
            polygonsObjects.push({
                dashArray: '1',
                dashColor: 'black',
                dashOpacity: 1,
                dashWeight: 1,
                fillOpacity: 0.2,
                color: 'orange',
                positions: rotateGeometry({ zoneGeometryInDegrees: directionGeometry }, zone.user_params.direction, zoneCenter),
            });
        }
        
        if (this.showStopSign(zone, zonePrototype)) {
            const stopSignGeometry = getStopSignGeometry(zoneCenter);
            const stopHandGeometry = getStopSignHandGeometry(zoneCenter);
            stopSigns.push({
                dashArray: '1',
                dashColor: 'red',
                dashOpacity: 1,
                weight: 2,
                fillOpacity: 1,
                color: 'white',
                fillColor: 'red',
                positions: stopSignGeometry,
            });
            stopSigns.push({
                dashArray: '1',
                dashColor: 'white',
                dashOpacity: 1,
                weight: 0.1,
                fillOpacity: 1,
                color: 'white',
                fillColor: 'white',
                positions: stopHandGeometry,
            });
        }
        return { polygonsObjects, stopSigns };
    };
    
    render() {
        const { visibleZones, zonesPrototypes, withAlarm } = this.props;
        
        const polygons = [];
        const signs = [];

        visibleZones.sort((a, b) => a.active < b.active).forEach((zone) => {
            const fillColor = zoneSeverityColor(zone.zone_alarm_level);
            const dashOpacity = zone.isSelected ? 1 : 0.3;
            const dashArray = zoneFullActive(zone) ? '1' : '5 10';
            const dashColor = zone.isAlarmed && withAlarm ? 'red' : (zone.isSelected ? 'yellow' : 'black');
            const dashWeight = (zone.isSelected || (zone.isAlarmed && withAlarm)) ? 5 : 2;
            if (!zone.dynamic) {
                const { polygonsObjects, stopSigns } = this.makePolygon({ zone, dashOpacity, dashColor, dashWeight, dashArray, fillColor, zonesPrototypes });
                polygons.push(...polygonsObjects);
                signs.push(...stopSigns);
            }
        });
        
        return (
            <div>
                {polygons.map((polygon, index) => (
                    <Polygon
                        key={index}
                        opacity={polygon.dashOpacity}
                        fillOpacity={polygon.fillOpacity || 0.2}
                        fillColor={polygon.fillColor}
                        color={polygon.dashColor}
                        weight={polygon.dashWeight}
                        dashArray={polygon.dashArray}
                        positions={polygon.positions}
                    />
                ))}
                
                {signs.map((polygon, index) => (
                    <Polygon
                        key={index}
                        opacity={polygon.dashOpacity}
                        fillOpacity={polygon.fillOpacity}
                        fillColor={polygon.fillColor}
                        color={polygon.color}
                        weight={polygon.weight}
                        dashArray={polygon.dashArray}
                        positions={polygon.positions}
                    />
                ))}
                
                {this.props.isShowingZoneLabels && this.getZonesLabels()}
                <ZonesPopUp
                    zones={this.props.selectedZones}
                    position={this.props.mouseClickPosition}
                    handleZoneClick={this.openZoneEditModal}
                    onClose={this.onPopupClose}
                />
            </div>
        );
    }
}

const mapStateToProps = (state, props) => {
    const sublocation = props.sublocation || state.siteView.selectedSublocation;
    const { isShowingZoneLabels } = state.siteViewFilter.layerFilter;
    const { selectedLeafletElement } = state.siteView;
    const selectedZones = Array.isArray(selectedLeafletElement) ? selectedLeafletElement.filter(el => state.entities.zones.byUid[el.uid]) : [];

    return {
        sublocation,
        creatingZone: state.siteView.geometry,
        zonesPrototypes: state.entities.zonesPrototypes,
        selectedZones,
        isShowingZoneLabels,
        currentRole: state.user?.access?.currentRole,
    };
};

export default connect(mapStateToProps)(LeafletZoneLayer);
