import React from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { LayerGroup, FeatureGroup } from 'react-leaflet';
import { EditControl } from 'react-leaflet-draw';
import PropTypes from 'prop-types';
import L from 'leaflet';

import { rotateLineByCenter, formatZoneGeometry, getVerticesLimit } from 'utils/zoneUtils';
import { openZoneVerticesLimitModal, addCreatedZoneGeometry, rotateZoneGeometry } from 'stores/actions';
import Position from 'stores/classes/Position';
import { MODAL_ACTIONS, ZONES_CONSTANTS } from 'common';
import { drawPolygonExtend, drawPolylineExtend } from 'common/leaflet-methods';


L.Draw.Polyline = L.Draw.Polyline.extend(drawPolylineExtend);
L.Draw.Polygon = L.Draw.Polyline.extend(drawPolygonExtend);

class ZoneDrawingControl extends LayerGroup {
    static propTypes = {
        sublocation: PropTypes.shape(),
        project: PropTypes.shape({}),
        isPolygonCreated: PropTypes.bool,
        isDirectionCreated: PropTypes.bool,
        isAdminPageZoneCreateMode: PropTypes.bool,
        changeGeometry: PropTypes.func,
        changeRotation: PropTypes.func,
        drawControlMounted: PropTypes.func,
    };
    
    static defaultProps = {
        sublocation: {},
        project: {},
        isPolygonCreated: false,
        isDirectionCreated: false,
        isAdminPageZoneCreateMode: false,
        changeGeometry: () => {},
        changeRotation: () => {},
        drawControlMounted: () => {},
    };
    
    constructor(props) {
        super(props);
        
        this.state = {
            disableFinishButton: true,
        };
    }
    
    getCreatingZoneGeometry = (zoneInLatLngs) => {
        let rotation;
        // if the zone is directional then geometry is line with two points
        if (this.props.isDirectionCreated && zoneInLatLngs.length === 2) {
            const { newLine, angle } = rotateLineByCenter(zoneInLatLngs);
            zoneInLatLngs = newLine;
            rotation = angle;
        }
        const formattedZoneGeometry = formatZoneGeometry(zoneInLatLngs);
        
        return {
            createdZone: { ...formattedZoneGeometry },
            rotation,
        };
    };
    
    _onCreate = (e) => {
        const {
            isPolygonCreated, isAdminPageZoneCreateMode, isDirectionCreated, changeGeometry, changeRotation, sublocation,
        } = this.props;
        const points = Array.isArray(e.layer._latlngs[0]) ? e.layer._latlngs[0] : e.layer._latlngs;
        const sublocation_offset = { offset_x: -sublocation.offset_x, offset_y: -sublocation.offset_y };
        const newZoneLatLngs = points.map(point => new Position(point, sublocation_offset).latlngObj);
        
        if (isPolygonCreated) {
            newZoneLatLngs.push(newZoneLatLngs[0]);
        }
        const { createdZone, rotation } = this.getCreatingZoneGeometry(newZoneLatLngs);
        
        if (isAdminPageZoneCreateMode) {
            changeGeometry(createdZone);
        } else {
            this.props.dispatch(addCreatedZoneGeometry(createdZone));
        }
        
        if (isDirectionCreated) {
            if (isAdminPageZoneCreateMode) {
                changeRotation(rotation);
            } else {
                this.props.dispatch(rotateZoneGeometry(rotation));
            }
        }

        this.props.dispatch({ type: MODAL_ACTIONS.MODAL_ZONE_POLYGON_STOP_CREATE });
        this.props.dispatch({ type: MODAL_ACTIONS.MODAL_ZONE_DIRECTION_STOP_CREATE });
    };
    
    finishPolygonDraw = () => {
        const finishPolygonDraw = document.querySelector('[title="Finish drawing"]');
        finishPolygonDraw.click();
    };
    
    undoLastPoint = () => {
        const undoLastPointDraw = document.querySelector('[title="Delete last point drawn"]');
        undoLastPointDraw.click();
    };
    
    cancelPolygonDraw = () => {
        const cancelDraw = document.querySelector('[title="Cancel drawing"]');
        if (cancelDraw) {
            cancelDraw.click();
        }
        const startDraw = document.getElementsByClassName('leaflet-draw-draw-polygon');
        if (startDraw.length > 1) {
            startDraw[1].click();
        } else if (startDraw.length === 1) {
            startDraw[0].click();
        }
    };
    
    handlePolygonDrawEvent = (verticesNumber) => {
        const maxVertices = getVerticesLimit(this.props.project);
        
        switch (verticesNumber) {
            case ZONES_CONSTANTS.MIN_VERTICES: {
                this.setState({
                    disableFinishButton: false,
                });
                break;
            }
            case maxVertices: {
                this.props.dispatch(openZoneVerticesLimitModal({
                    finish: this.finishPolygonDraw,
                    cancel: this.cancelPolygonDraw,
                    undo: this.undoLastPoint,
                }));
                break;
            }
            default: {
                return undefined;
            }
        }
    };
    
    handleDrawVertex = (event) => {
        const verticesNumber = Object.values(event.layers._layers).length;
        if (this.props.isDirectionCreated && verticesNumber === ZONES_CONSTANTS.LINE_VERTICES) {
            this.finishPolygonDraw();
        } else if (this.props.isPolygonCreated) {
            this.handlePolygonDrawEvent(verticesNumber);
        }
    };
    
    shouldComponentUpdate(nextProps, nextState) {
        if (!nextProps.isPolygonCreated && !this.state.disableFinishButton) {
            this.setState({
                disableFinishButton: true,
            });
        }
        return (this.props !== nextProps || this.state !== nextState);
    }
    
    componentDidMount() {
        if (this.refs.editcontrol && this.refs.editcontrol.__proto__.componentWillUnmount) { // remove handler for remove uncaught error
            this.refs.editcontrol.__proto__.componentWillUnmount = null;
            this.refs.editcontrol.__proto__.componentDidUpdate = null;
        }
        if (this.props.drawControlMounted) {
            this.props.drawControlMounted();
        }
    }
    
    render() {
        return (
            <div>
                <FeatureGroup ref="fg">
                    <EditControl /* added something styles, that don't show buttons of edit and trash */
                        ref="editcontrol"
                        position="bottomleft"
                        onCreated={this._onCreate}
                        onDrawVertex={this.handleDrawVertex}
                        draw={{
                            polyline: {
                                shapeOptions: {
                                    color: 'orange',
                                },
                            },
                            marker: false,
                            polygon: {
                                shapeOptions: {
                                    color: 'orange',
                                    opacity: 0.3,
                                },
                                allowIntersection: false,
                            },
                            circle: false,
                            rectangle: {
                                shapeOptions: {
                                    color: 'orange',
                                },
                            },
                        }}
                    />
                </FeatureGroup>
            </div>
        );
    }
}

const mapStateToProps = state => ({
    project: state.appContext.project,
    isPolygonCreated: state.modals.zoneModals.zoneChooseGeometry.polygon_create,
    isDirectionCreated: state.modals.zoneModals.zoneChooseGeometry.direction_create,
    isAdminPageZoneCreateMode: state.modals.zoneModals.zoneCreateAdminPage.open,
});

export default connect(mapStateToProps)(injectIntl(ZoneDrawingControl));
