import React from 'react';
import PropTypes from 'prop-types';
import { BootstrapTable, TableHeaderColumn } from 'react-bootstrap-table';
import { TRACKABLE_POP_UP } from 'common/ui-labels';
import { getDeviceTypeClass, getKeyValueHtml, getKeyValuesHtml, getLastReported, getLeftCornerRibbonHtml,
    getNodeBatteryLevel, getTrackableStatus, hasAlarms, isAssetTag, macToUpperCase
} from 'utils';
import { PopUpWrapper } from 'components/leaflet/PopUpWrapper';


export function ClusterPopUp(props) {
    const { trackablesCluster, popupPosition, onClosePopup, intl } = props;

    if (!trackablesCluster || !popupPosition) return null;

    const popupClass = ['RPP-BlockBody trackable-cluster-popup'];
    const { unresponsiveTrackablesCount, selectedClusterTrackables } = trackablesCluster;

    const getTableData = selectedClusterTrackables.map((tko, i) => ({
        id: i,
        uid: tko.uid,
        trackableName: tko.name,
        trackableStatus: getTrackableStatus(tko, intl),
        isResponsive: tko.isResponsive(),
        hasAlarm: hasAlarms(tko),
    }));

    const createCustomToolBar = props => (<div className="col-md-12">{ props.components.searchPanel }</div>);

    const options = {
        defaultSortName: 'trackableName',
        toolBar: createCustomToolBar,
    };

    const expandComponent = (row) => {
        const expandedTrackable = selectedClusterTrackables.find(to => (to.uid === row.uid));

        if (expandedTrackable && expandedTrackable.node) {
            const { node } = expandedTrackable;

            const info = {
                [TRACKABLE_POP_UP.BATTERY_LEVEL(intl)]: getNodeBatteryLevel(node, isAssetTag(expandedTrackable.category)),
                [TRACKABLE_POP_UP.TAG_MAC(intl)]: macToUpperCase(node.mac_address),
                [TRACKABLE_POP_UP.REPORTED(intl)]: getLastReported(node.last_heard, intl),
            };

            const trackableGroupsString = expandedTrackable.groupList.join(', ');
            if (trackableGroupsString.length > 0) {
                info[TRACKABLE_POP_UP.GROUPS(intl)] = trackableGroupsString;
            }

            if (hasAlarms(expandedTrackable)) {
                info[TRACKABLE_POP_UP.CURRENT_ALARMS(intl)] = expandedTrackable.alarms.map(to => to.additional_info.text);
            }

            return (<div className="RPP-PopUpRowsWrapper expanded-rows">
                {Object.keys(info).map((key, i) => {
                    if (key === TRACKABLE_POP_UP.CURRENT_ALARMS(intl)) {
                        if (info[key]) {
                            return getKeyValuesHtml(key, info[key], i);
                        }
                        return <div key={i} />;
                    }
                    return getKeyValueHtml(key, info[key], i);
                })}
            </div>);
        }
        return '';
    };

    const expandColumnComponent = props => (
        <div>
            <i className={`fa fa-angle-${props.isExpanded ? 'up' : 'down'}`} />
        </div>
    );

    const columnClassNameFormat = (fieldValue, row) => {
        const { isResponsive, hasAlarm } = row;
        const responsiveState = isResponsive ? 'success' : 'danger';
        return `padding-l-m text-${hasAlarm ? 'alarmed' : responsiveState}`;
    };

    return (
        <PopUpWrapper
            position={popupPosition}
            closeButton={false}
            onClose={onClosePopup}
            className={popupClass.join(' ')}
        >
            <div className="RPP-BlockBody-Wrapper">
                <div className="RPP-BlockHeader">
                    {getLeftCornerRibbonHtml('Cluster', getDeviceTypeClass(trackablesCluster))}
                    <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                        <strong>{selectedClusterTrackables.length}</strong> {TRACKABLE_POP_UP.TRACKABLE_OR_TRACKABLES_FUNC(intl)}
                    </h4>
                    {unresponsiveTrackablesCount > 0 && <div className="unresponsive-message text-danger">
                        <strong>{unresponsiveTrackablesCount}</strong> {TRACKABLE_POP_UP.NOT_RESPONDING(intl)}!
                    </div>}
                    <button
                        type="button"
                        className="RPP-BtnPopUp"
                        onClick={onClosePopup}
                    >
                        <i className="fa fa-times" />
                    </button>
                </div>
                <div className="RPP-PopUpRowsWrapper">
                    <BootstrapTable
                        data={getTableData}
                        options={options}
                        tableHeaderClass="hidden"
                        tableBodyClass="lineless-table popup-info"
                        trClassName="cursor-pointer"
                        bordered={false}
                        expandComponent={expandComponent}
                        expandableRow={() => true}
                        expandColumnOptions={{
                            expandColumnVisible: true,
                            expandColumnComponent,
                            columnWidth: 20,
                        }}
                        search
                        hover
                    >
                        <TableHeaderColumn
                            dataField="id"
                            searchable={false}
                            hidden
                            isKey
                        />
                        <TableHeaderColumn
                            columnClassName="text-ellipsis"
                            dataField="trackableName"
                            tdAttr={{ 'data-id': 'trackableName' }}
                        />
                        <TableHeaderColumn
                            columnClassName={columnClassNameFormat}
                            dataField="trackableStatus"
                            tdAttr={{ 'data-id': 'trackableStatus' }}
                        />
                    </BootstrapTable>
                </div>
            </div>
        </PopUpWrapper>
    );
}


ClusterPopUp.propTypes = {
    trackablesCluster: PropTypes.shape({}),
    popupPosition: PropTypes.shape({}),
    onClosePopup: PropTypes.func,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func
    })
}

ClusterPopUp.defaultProps = {
    trackablesCluster: null,
    popupPosition: null,
    onClosePopup: () => {},
    intl: {
        formatMessage: () => {}
    }
}
