import React from 'react';
import PropTypes from 'prop-types';
import _Get from 'lodash/get';
import {
    getDeviceTypeClass,
    getKeyValueHtml,
    getKeyValuesHtml,
    getLastReported,
    getLeftCornerRibbonHtml,
    getNodeBatteryLevel,
    getTrackableType,
    isAssetTag,
    macToUpperCase,
    positionOffset,
    userHasPrivileges,
} from 'utils';
import { TrackableTemplate } from 'stores/classes';
import store from 'stores/LocationStore';
import { COMPONENTS, TRACKABLE_POP_UP } from 'common/ui-labels';
import { USER_ROLES } from 'common/user-roles';
import { PopUpWrapper } from 'components/leaflet/PopUpWrapper';

export function TrackablePopUp(props) {
    const { trackable, sublocation, intl, onClose, playbackSpeed, trackableTemplates } = props;
    const user = store.getCurrentUser();
    const { isOpenPlayBackControl } = store.getPlayback();
    const hasPrivileges = userHasPrivileges(user?.access?.role || user?.access?.currentRole, USER_ROLES.PROJECT_MAINTAINER);

    if (!hasPrivileges || !trackable || !trackable.uid || !trackable.position || !sublocation) return null;

    const trackablePosition = positionOffset(trackable.position, sublocation);
    const info = {
        [COMPONENTS.FIELDS.NODE.BATTERY_FUNC(intl)]: getNodeBatteryLevel(trackable.node, isAssetTag(trackable.category)),
        [TRACKABLE_POP_UP.TAG_MAC(intl)]: macToUpperCase(_Get(trackable, 'node.mac_address')),
        [TRACKABLE_POP_UP.REPORTED(intl)]: getLastReported(_Get(trackable, 'node.last_heard'), intl),
    };

    const trackableGroupsString = trackable.groupList.join(', ');
    if (trackableGroupsString.length > 0) {
        info[TRACKABLE_POP_UP.GROUPS(intl)] = trackableGroupsString;
    }

    const popupClass = ['RPP-BlockBody', `Leaflet-Transition-Speed${playbackSpeed}`];
    const { alarms } = trackable;
    if (alarms && alarms.length > 0) {
        popupClass.push('alarming');
        info[TRACKABLE_POP_UP.CURRENT_ALARMS(intl)] = alarms.map((alarm) => alarm.additional_info.text);
    }

    const templateProperties = TrackableTemplate.getTemplatePropertyByCategory(trackable.category, trackableTemplates);

    if (templateProperties?.length) {
        templateProperties.forEach((item) => {
            if (trackable?.user_params && trackable?.user_params[item.name]) {
                info[item.label] = trackable.user_params[item.name];
            }
        });
    }

    const cornerRibbonTextLabel = getTrackableType(intl, trackable);

    return (
        <PopUpWrapper
            position={trackablePosition}
            closeButton={false}
            onClose={onClose}
            className={popupClass.join(' ')}
        >
            <div className="RPP-BlockBody-Wrapper">
                <div className="RPP-BlockHeader">
                    {getLeftCornerRibbonHtml(cornerRibbonTextLabel, getDeviceTypeClass(trackable))}
                    <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                        {trackable.name}
                    </h4>
                    {!trackable.isResponsive(isOpenPlayBackControl) &&
                        <div className="unresponsive-message text-danger">
                            {TRACKABLE_POP_UP.TRACKABLE_NOT_RESPONDING(intl)}!
                        </div>}
                    <button
                        type="button"
                        className="RPP-BtnPopUp"
                        onClick={onClose}
                    >
                        <i className="fa fa-times" />
                    </button>
                </div>
                <div className="RPP-PopUpRowsWrapper">
                    {Object.keys(info).map((key, i) => {
                        if (key === TRACKABLE_POP_UP.CURRENT_ALARMS(intl)) {
                            if (info[key]) {
                                return getKeyValuesHtml(key, info[key], i);
                            }
                            return <div key={i} />;
                        }
                        return getKeyValueHtml(key, info[key], i);
                    })}
                </div>
            </div>
        </PopUpWrapper>
    );
}

TrackablePopUp.propTypes = {
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    onClose: PropTypes.func,
    playbackSpeed: PropTypes.number,
    sublocation: PropTypes.shape({}),
    trackable: PropTypes.shape({
        alarms: PropTypes.arrayOf(PropTypes.shape({})),
        category: PropTypes.string,
        groupList: PropTypes.string,
        isResponsive: PropTypes.func,
        name: PropTypes.string,
        node: PropTypes.shape({}),
        position: PropTypes.string,
        uid: PropTypes.string,
        user_params: PropTypes.shape({}),
    }),
    trackableTemplates: PropTypes.arrayOf(PropTypes.shape({
        category: PropTypes.string,
        variables: PropTypes.shape({
            label: PropTypes.string,
            name: PropTypes.string,
        }),
    })),
};

TrackablePopUp.defaultProps = {
    intl: {
        formatMessage: () => {},
    },
    onClose: () => {},
    playbackSpeed: 1,
    sublocation: null,
    trackable: null,
    trackableTemplates: [],
};
