import React from 'react';
import PropTypes from 'prop-types';
import { COMPONENTS, DB_NAMES, TRACKABLE_POP_UP } from 'common/ui-labels';
import { getDeviceTypeClass, getKeyValueHtml, getKeyValuesHtml, getLeftCornerRibbonHtml, getNodeBatteryLevel, isPerson, isVehicle, positionOffset } from 'utils';
import { PopUpWrapper } from 'components/leaflet/PopUpWrapper';


const getDriverVehicle = (trackables) => {
    const driverTko = trackables.find(tko => isPerson(tko.category)) || trackables[0];
    const vehicleTko = trackables.find(tko => isVehicle(tko.category)) || trackables[1];

    if (driverTko && vehicleTko) {
        return { driverTko, vehicleTko };
    }

    return { driverTko: trackables[0], vehicleTko: trackables[1] };
};

export function VdaPopUp(props) {
    const { selectedClusterTrackables, sublocation, onClose, playbackSpeed, intl } = props;
    const { driverTko, vehicleTko } = getDriverVehicle(selectedClusterTrackables);
    const { BATTERY_FUNC, GROUPS_FUNC } = COMPONENTS.FIELDS.NODE;

    if (!vehicleTko || !driverTko) return null;

    const driverGroups = driverTko.groupList.join(', ');
    const vehicleGroups = vehicleTko.groupList.join(', ');

    const info = {
        [BATTERY_FUNC(intl)]: getNodeBatteryLevel(driverTko.node),
        ...(driverGroups && { [GROUPS_FUNC(intl)]: driverGroups }),
        [DB_NAMES.VEHICLE(intl)]: vehicleTko.name,
        ...(vehicleGroups && { [TRACKABLE_POP_UP.VEHICLE_GROUP(intl)]: vehicleGroups }),
    };

    const vehicleAlarms = vehicleTko.alarms;
    const driverAlarms = driverTko.alarms;
    const isVehicleAlarmed = vehicleAlarms && vehicleAlarms.length > 0;
    const isDriverAlarmed = driverAlarms && driverAlarms.length > 0;
    const isVDAAlarmed = isDriverAlarmed || isVehicleAlarmed;

    const popupClass = ['RPP-BlockBody', `Leaflet-Transition-Speed${playbackSpeed}`];
    const popupPosition = positionOffset(vehicleTko.position, sublocation);

    if (isVDAAlarmed) {
        popupClass.push('alarming');
        if (isVehicleAlarmed) {
            info[TRACKABLE_POP_UP.VDA.VEHICLE_ALARMS(intl)] = vehicleAlarms.map(alarm => alarm.additional_info.text);
        }
        if (isDriverAlarmed) {
            info[TRACKABLE_POP_UP.VDA.DRIVER_ALARMS(intl)] = driverAlarms.map(alarm => alarm.additional_info.text);
        }
    }

    return (
        <PopUpWrapper
            position={popupPosition}
            closeButton={false}
            onClose={onClose}
            className={popupClass.join(' ')}
        >
            <div className="RPP-BlockBody-Wrapper">
                <div className="RPP-BlockHeader">
                    {getLeftCornerRibbonHtml(TRACKABLE_POP_UP.DRIVER(intl), getDeviceTypeClass(driverTko))}
                    <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                        { driverTko.name }
                    </h4>
                    <button
                        type="button"
                        className="RPP-BtnPopUp"
                        onClick={onClose}
                    >
                        <i className="fa fa-times" />
                    </button>
                </div>
                <div className="RPP-PopUpRowsWrapper">
                    {Object.keys(info).map((key, i) => {
                        if (key === TRACKABLE_POP_UP.VDA.DRIVER_ALARMS(intl) || key === TRACKABLE_POP_UP.VDA.VEHICLE_ALARMS(intl)) {
                            if (info[key]) {
                                return getKeyValuesHtml(key, info[key], i);
                            }
                            return <div key={i} />;
                        }
                        return getKeyValueHtml(key, info[key], i);
                    })}
                </div>
            </div>
        </PopUpWrapper>
    );
}

VdaPopUp.propTypes = {
    selectedClusterTrackables: PropTypes.arrayOf(PropTypes.shape({})),
    sublocation: PropTypes.shape({}),
    onClose: PropTypes.func,
    playbackSpeed: PropTypes.number,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    })
}

VdaPopUp.defaultProps = {
    selectedClusterTrackables: [],
    sublocation: null,
    onClose: () => {},
    playbackSpeed: 1,
    intl: {
        formatMessage: () => {},
    }
}
