import React from 'react';
import PropTypes from 'prop-types';
import { FormSection, Field } from 'redux-form';
import _isEmpty from 'lodash/isEmpty';
import { injectIntl } from 'react-intl';

import { formatDbNameToHumanReadable } from 'utils/htmlUtils';

import RenderField from '../../RenderField';
import RenderCustomSelectField from '../../RenderCustomSelectField';

const renderBigField = RenderField(6, 5);
const renderBigSelectField = RenderCustomSelectField(6, 5);
const renderAverageSelectField = RenderCustomSelectField(5, 7, 'Form-Label-Average');
const renderAverageField = RenderField(5, 7, 'Form-Label-Average');

const requiredFunc = value => ((value === null || value === undefined || value.length === 0) ? 'Required' : undefined);
const isLessThanMinimum = min => value => ((Number(value) < min) ? `Must be greater than or equal to ${min}` : undefined);
const isGreaterThanMaximum = max => value => ((Number(value) > max) ? `Must be less than ${max}` : undefined);
const numberTypeCheck = (step, disabled) => (value) => {
    const isStepInteger = Number.isInteger(step);
    const isValueInteger = Number.isInteger(value);
    // remove value type validation for disabled custom fields
    if (disabled) {
        return undefined;
    }
    if (!isNaN(value)) {
        return (!isValueInteger && isStepInteger)
            ? 'Number must be an integer'
            : undefined;
    } 
    return 'Value must be a number';
};
const decimalNumbersLimit = limit => (value) => {
    const hasDecimals = value % 1;
    return (hasDecimals && value.toString().split('.')[1].length > limit)
        ? `Maximum ${limit} decimals is allowed`
        : undefined;
};

const getUserParams = (user_params, optionName, options) => {
    if (!user_params[optionName] && !_isEmpty(options)) {
        return options[0].value;
    }
    return user_params[optionName];
};

const getLabel = (label, required) => {
    if (required) {
        return `${label}*`;
    }
    return label;
};

class CustomZoneOptions extends React.Component {
    static propTypes = {
        zone_type_name: PropTypes.string,
        zone_id: PropTypes.number,
        isBigField: PropTypes.bool,
        zonePrototype: PropTypes.shape(),
        formState: PropTypes.shape(),
        zones: PropTypes.arrayOf(PropTypes.shape()),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };

    static defaultProps = {
        zone_type_name: '',
        zone_id: 0,
        isBigField: true,
        zonePrototype: {},
        formState: {
            user_params: null,
        },
        zones: PropTypes.arrayOf(PropTypes.shape()),
        intl: {
            formatMessage: () => {
            },
        },
    };

    shouldComponentUpdate(nextProps) {
        return (this.props.zone_type_name !== nextProps.zone_type_name);
    }

    render() {
        const { zonePrototype, isBigField, formState: { user_params }, intl } = this.props;

        if (zonePrototype && zonePrototype.variables) {
            const additionalOptions = zonePrototype.getEditModalCustomOptions(this.props.zone_id, this.props.zones);
            return (
                <FormSection name="user_params">
                    {zonePrototype.variables.map((variable, idx) => {
                        const {
                            label, name, htmlType, type, min, max, step, disabled, options, required,
                        } = variable;

                        const labelIntl = formatDbNameToHumanReadable(intl, name, label);
                        const minimum = isLessThanMinimum(min);
                        const maximum = isGreaterThanMaximum(max);
                        const numberTypeValidation = numberTypeCheck(step, disabled);
                        const maximum2Decimals = decimalNumbersLimit(2);

                        if (htmlType === 'select') {
                            user_params[name] = getUserParams(user_params, name, options);
                            return (
                                <div key={idx} className="Form-Line row" style={{ display: disabled ? 'none' : 'block' }}>
                                    <Field
                                        className="Form-Input"
                                        parse={value => (type === 'number' ? Number(value) : value)}
                                        name={name}
                                        label={getLabel(labelIntl, required)}
                                        component={isBigField ? renderBigSelectField : renderAverageSelectField}
                                        validate={[requiredFunc]}
                                    >
                                        {[...options, ...additionalOptions].map((option, index) => (<option key={index} value={option.value}>{option.label}</option>))}
                                    </Field>
                                </div>
                            );
                        }
                        return (
                            <div key={idx} className="Form-Line row" style={{ display: disabled ? 'none' : 'block' }}>
                                <Field
                                    className="Form-Input"
                                    parse={value => (type === 'number' ? Number(value) : value)}
                                    name={name}
                                    min={min}
                                    max={max}
                                    step={step}
                                    component={isBigField ? renderBigField : renderAverageField}
                                    validate={[requiredFunc, minimum, maximum, numberTypeValidation, maximum2Decimals]}
                                    label={`${labelIntl}*`}
                                    type={htmlType}
                                />
                            </div>
                        );
                    })}
                </FormSection>
            );
        }

        return (
            <section />
        );
    }
}

export default injectIntl(CustomZoneOptions);
