import React from 'react';
import { Map, LayersControl, Polygon } from 'react-leaflet';

import PropTypes from 'prop-types';

import LeafletGrids from '../../../../components/leaflet/LeafletGrids';
import LeafletScaling from '../../../../components/leaflet/LeafletScaling';
import LeafletFloorplan from '../../../../components/leaflet/LeafletFloorplan';
import { DEFAULT_VALUES, ZONE_DETAILS_MODAL } from '../../../../common';
import {
    getDirectionGeometry, getRectangleZoneCenter, getStopSignGeometry, getStopSignHandGeometry,
    rotateGeometry, MAP_BOUNDS,
} from '../../../../utils';

const { Overlay } = LayersControl;

const showZoneDirection = zonePrototype => (zonePrototype && zonePrototype.zone_geometry_restriction &&
    zonePrototype.zone_geometry_restriction.indexOf('directional') !== -1);

const showStopSign = zonePrototype => (zonePrototype && zonePrototype.zone_type && zonePrototype.zone_type.name === 'stop_zone');

const getStopSigns = (zonePrototype, zoneCenter) => {
    const signs = [];
    if (showStopSign(zonePrototype)) {
        const stopSignGeometry = getStopSignGeometry(zoneCenter);
        const stopHandGeometry = getStopSignHandGeometry(zoneCenter);
        signs.push({
            dashArray: '1',
            dashColor: 'red',
            dashOpacity: 1,
            weight: 2,
            fillOpacity: 1,
            color: 'white',
            fillColor: 'red',
            positions: stopSignGeometry,
        });
        signs.push({
            dashArray: '1',
            dashColor: 'white',
            dashOpacity: 1,
            weight: 0.1,
            fillOpacity: 1,
            color: 'white',
            fillColor: 'white',
            positions: stopHandGeometry,
        });
    }
    return signs;
};

const getDirections = (zonePrototype, zoneCenter, direction) => {
    const directions = [];
    if (showZoneDirection(zonePrototype)) {
        const directionGeometry = getDirectionGeometry(zoneCenter);
        directions.push({
            dashArray: '1',
            dashColor: 'black',
            dashOpacity: 1,
            dashWeight: 1,
            fillOpacity: 0.2,
            color: 'orange',
            positions: rotateGeometry({ zoneGeometryInDegrees: directionGeometry }, direction, zoneCenter),
        });
    }
    return directions;
};

const FloorplanSelected = ({
    setLeafletToState, onMove, boundsForScale, zoom, selectedFloorplan, zoneSublocation, zone, selectFloorplan,
    unSelectFloorplan, leafletClass, mapStyle, zonePrototype, direction, intl,
}) => {
    if (!selectedFloorplan) {
        return null;
    }
    const zoneCenter = getRectangleZoneCenter(zone.positions);
    const directions = getDirections(zonePrototype, zoneCenter, direction);
    const signs = getStopSigns(zonePrototype, zoneCenter);

    return (
        <div className="Zone-Details-Body">
            <div
                className={leafletClass}
                title={ZONE_DETAILS_MODAL.DOUBLE_CLICK_COLLAPSE(intl)}
                onDoubleClick={() => {
                    if (selectedFloorplan) {
                        unSelectFloorplan();
                    } else {
                        selectFloorplan(selectedFloorplan);
                    }
                }}
            >
                <Map
                    center={[0, 0]}
                    className="zone-modal"
                    ref={leaflet => setLeafletToState(leaflet)}
                    bounds={zone.positions || MAP_BOUNDS(zoneSublocation, selectedFloorplan)}
                    style={mapStyle}
                    zoomAnimation={false}
                    zoomSnap={0}
                    scrollWheelZoom="center"
                    onMoveend={onMove}
                    minZoom={DEFAULT_VALUES.MAP_ZOOM.MIN}
                    doubleClickZoom={false}
                    maxZoom={DEFAULT_VALUES.MAP_ZOOM.MAX}
                    zoom={DEFAULT_VALUES.MAP_ZOOM.DEFAULT}
                >
                    
                    <LayersControl position="topright">
                        <Overlay checked name="Floorplan">
                            <LeafletFloorplan floorplan={selectedFloorplan} />
                        </Overlay>
                        
                        <Overlay checked name="Zones">
                            <Polygon
                                opacity={zone.dashOpacity}
                                fillOpacity={0.2}
                                fillColor={zone.fillColor}
                                color={zone.dashColor}
                                weight={zone.dashWeight}
                                dashArray={zone.dashArray}
                                positions={zone.positions}
                            />
                            {directions.map((polygon, index) => (
                                <Polygon
                                    key={index}
                                    opacity={polygon.dashOpacity}
                                    fillOpacity={polygon.fillOpacity || 0.2}
                                    fillColor={polygon.fillColor}
                                    color={polygon.dashColor}
                                    weight={polygon.dashWeight}
                                    dashArray={polygon.dashArray}
                                    positions={polygon.positions}
                                />
                            ))}
                            
                            {signs.map((polygon, index) => (
                                <Polygon
                                    key={index}
                                    opacity={polygon.dashOpacity}
                                    fillOpacity={polygon.fillOpacity}
                                    fillColor={polygon.fillColor}
                                    color={polygon.color}
                                    weight={polygon.weight}
                                    dashArray={polygon.dashArray}
                                    positions={polygon.positions}
                                />
                            ))}
                        </Overlay>
                        
                        <Overlay checked name="Grid-5m">
                            <LeafletGrids
                                zoom={14}
                                step={5}
                                floorplan={selectedFloorplan}
                                isShowingGrid5m
                                floorplanSize={{ width: 200, height: 200 }}
                                sublocation={zoneSublocation}
                            />
                        </Overlay>

                        {!!boundsForScale && Object.keys(boundsForScale).length >= 2 &&
                        <LeafletScaling
                            zoom={zoom}
                            floorplanSize={{ width: window.screen.width, height: window.screen.height }}
                            isShowingScale
                            bounds={boundsForScale}
                        />
                        }
                    </LayersControl>
                </Map>
            
            </div>
        </div>);
};

FloorplanSelected.propTypes = {
    setLeafletToState: PropTypes.func,
    onMove: PropTypes.func,
    boundsForScale: PropTypes.shape({}),
    zoom: PropTypes.number,
    selectedFloorplan: PropTypes.shape({}),
    zoneSublocation: PropTypes.shape({}),
    zone: PropTypes.shape({}),
    selectFloorplan: PropTypes.func,
    unSelectFloorplan: PropTypes.func,
    leafletClass: PropTypes.string,
    mapStyle: PropTypes.shape({}),
    zonePrototype: PropTypes.shape(),
    direction: PropTypes.number,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

FloorplanSelected.defaultProps = {
    setLeafletToState: () => {},
    onMove: () => {},
    boundsForScale: {},
    zoom: DEFAULT_VALUES.MAP_ZOOM.DEFAULT,
    selectedFloorplan: {},
    zoneSublocation: {},
    zone: {},
    selectFloorplan: () => {},
    unSelectFloorplan: () => {},
    leafletClass: '',
    mapStyle: {},
    zonePrototype: {},
    direction: 0,
    intl: {
        formatMessage: () => {
        },
    },
};

export default FloorplanSelected;
