import React from 'react';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import PropTypes from 'prop-types';
import _Has from 'lodash/has';
import { injectIntl } from 'react-intl';

import SelectComponent from 'components/modals/SelectComponent';
import { NAME_OF_FORMS, TRACKABLE_TAG_ASSOCIATION, COMPONENTS } from 'common';
import { sortByField, isOpenLllValidationModal } from 'utils';
import {
    closeTrackableAssignModal,
    createTrackableAssociations,
    openAssignLowLatencyLinkValidateModal,
} from 'stores/actions';
import { Group, Trackable } from 'stores/classes';
import { Node } from 'stores/classes/Selectors';


let TrackableAssignForm = (props) => {
    const {
        close, submit, className, notAssociatedNode, formState, intl, submitSucceeded,
    } = props;

    const trackableAssignFormState = formState && formState[NAME_OF_FORMS.TRACKABLE_ASSIGN];
    const { node } = _Has(trackableAssignFormState, 'values.node') && trackableAssignFormState.values;

    return (
        <form className={className}>
            <div className="Form-Line row">
                <div htmlFor="node" className="Form-Label col-md-3">{COMPONENTS.FIELDS.NODE.TAG}</div>
                <div className="col-md-9">
                    <Field
                        className="form-control input-sm "
                        name="node"
                        data-id="trackable-assign-node"
                        style={{ fontSize: '14px' }}
                        props={{ disabled: false }}
                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                        noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                        options={notAssociatedNode}
                        component={SelectComponent}
                    />
                </div>
            </div>
            <div className="Form-Buttons">
                <button
                    type="button"
                    data-id="trackable-assign-assign"
                    onClick={submit}
                    disabled={submitSucceeded || !node}
                >
                    {COMPONENTS.BUTTONS.ASSIGN}
                </button>
                <button
                    type="button"
                    data-id="trackable-assign-cancel"
                    onClick={close}
                >
                    {COMPONENTS.BUTTONS.CANCEL}
                </button>
            </div>
        </form>
    );
};

TrackableAssignForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    notAssociatedNode: PropTypes.arrayOf(PropTypes.shape({})),
    formState: PropTypes.shape(),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    submitSucceeded: PropTypes.bool,
};

TrackableAssignForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    className: '',
    notAssociatedNode: [],
    formState: {},
    intl: {
        formatMessage: () => {},
    },
    submitSucceeded: false,
};

TrackableAssignForm = reduxForm({
    form: NAME_OF_FORMS.TRACKABLE_ASSIGN,
})(TrackableAssignForm);


const mapStateToForm = (state) => {
    const project = state.appContext.project;
    const notAssociatedNode = sortByField(Node.getNotAssociatedNode(state, project), 'mac_address');
    const formState = state.form;

    return {
        notAssociatedNode,
        formState,
    };
};
TrackableAssignForm = connect(mapStateToForm)(injectIntl(TrackableAssignForm));


class TrackableAssignModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        trackableAssign: PropTypes.shape({
            params: PropTypes.shape({
                name: PropTypes.string,
                uid: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        groups: PropTypes.arrayOf(PropTypes.shape),
        style: PropTypes.shape({
            content: PropTypes.object,
            overlay: PropTypes.object,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        trackableAssign: {
            params: {
                name: '',
                uid: '',
            },
            open: false,
        },
        groups: [],
        style: {
            content: {},
            overlay: {},
        },
    };
    
    close = () => {
        this.props.dispatch(closeTrackableAssignModal());
    };
    
    submit = (value) => {
        if (value.node) {
            const node = Node.getNodeByMacAddress(value.node.mac_address);
            const trackable = this.props.trackableAssign.params;

            const isOpen = isOpenLllValidationModal({ trackable: new Trackable(trackable), node });

            if (isOpen) {
                this.props.dispatch(openAssignLowLatencyLinkValidateModal({
                    mac_address: value.node.mac_address,
                    trackable_uid: trackable.uid,
                }));
            } else {
                this.props.dispatch(createTrackableAssociations({
                    mac_address: value.node.mac_address,
                    trackable_uid: trackable.uid,
                }));
            }
        }
        this.props.dispatch(closeTrackableAssignModal());
    };
    
    render() {
        const { style } = this.props;
        const { params } = this.props.trackableAssign;

        return (
            <Modal
                contentLabel="trackableAssign"
                isOpen={this.props.trackableAssign.open}
                onRequestClose={this.close}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header2-Grey">
                        {TRACKABLE_TAG_ASSOCIATION.FORM.ASSIGN_TRACKABLE_FUNC(params && params.name ? params.name : '')}
                    </span>
                    <TrackableAssignForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    trackableAssign: state.modals.assignModals.trackableAssign,
    groups: Group.getAllGroupsArray(state),
});

export default connect(mapStateToProps)(TrackableAssignModal);
