import React from 'react';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';

import * as api from 'api';
import RenderField from 'components/modals/RenderField';
import { NAME_OF_FORMS, CHANGE_PASSWORD_MODAL, DEFAULT_VALUES, STORAGE_KEY, ACTIONS } from 'common';
import { CHANGE_PASSWORD_POPUP, COMPONENTS } from 'common/ui-labels';
import {
    closeChangePassword, changePassword, openChangePassword,
    openEditProfile, openStatusNotificationPopup, closeStatusNotificationPopup,
    settingUp,
    userLoginByPassword,
} from 'stores/actions';
import { makeCopy } from 'utils/simpleUtils';
import { decodeToken, sleep } from 'utils/utils';
import { LocationStoreMethods, clearSessionStorage } from 'utils/browserStorage';

import { getValidatePasswordError } from '../user/utils';

const validate = (values, props) => {
    const errors = {};
    if (!values.oldPassword) {
        errors.oldPassword = COMPONENTS.VALIDATION.REQUIRED;
    }

    const passwordError = getValidatePasswordError(values.newPassword, props.intl);
    if (passwordError) {
        errors.newPassword = passwordError;
    } else if (!values.newPassword.localeCompare(values.oldPassword)) {
        errors.newPassword = CHANGE_PASSWORD_MODAL.ERROR.CURRENT_PASSWORD_NOT_EQUAL_TO_NEW_ONE;
    }
    
    if (!values.confirmNewPassword) {
        errors.confirmNewPassword = COMPONENTS.VALIDATION.REQUIRED;
    } else if (values.confirmNewPassword.length < 8) {
        errors.confirmNewPassword = COMPONENTS.VALIDATION.MIN_LENGTH_FUNC(8);
    }
    
    if (values.newPassword !== values.confirmNewPassword) {
        errors.confirmNewPassword = CHANGE_PASSWORD_MODAL.ERROR.REQUIRE_EQUALS_PASSWORD;
    }
    
    return errors;
};

const asyncValidate = (values, dispatch, props, field) => {
    if (field === 'oldPassword') {
        return api.validateUser({ password: values.oldPassword, uid: values.uid, isOld: true }).then((result) => {
            if (!result.valid) {
                // eslint-disable-next-line
                throw { oldPassword: CHANGE_PASSWORD_MODAL.ERROR.CURRENT_PASSWORD_NOT_VALID }; // 'Current password is not valid'};
            }
        });
    }
    return api.validateUser({ password: values.newPassword, uid: values.uid, isOld: false }).then((result) => {
        if (!result.valid) {
            // eslint-disable-next-line
            throw { newPassword: CHANGE_PASSWORD_MODAL.ERROR.CURRENT_PASSWORD_CANT_BE_SAME_AS_LAST_10 }; // 'Current password is not valid'};
        }
    });
};

const renderField = RenderField(5, 7);

const showPassword = show => (show ? 'text' : 'password');

let ChangePasswordForm = (props) => {
    const {
        close, submit, className, invalid, formState, intl,
    } = props;
    
    const changePasswordFormState = formState[NAME_OF_FORMS.CHANGE_PASSWORD] || {};
    const isPasswordShow = changePasswordFormState.values ? changePasswordFormState.values.showPassword : false;
    
    return (
        <form className={className}>
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="oldPassword"
                    dataId="change-password-old"
                    component={renderField}
                    label={CHANGE_PASSWORD_MODAL.FORM.CURRENT_PASSWORD_FUNC(intl)}
                    type={showPassword(isPasswordShow)}
                />
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="newPassword"
                    component={renderField}
                    dataId="change-password-new"
                    label={CHANGE_PASSWORD_MODAL.FORM.NEW_PASSWORD_FUNC(intl)}
                    type={showPassword(isPasswordShow)}
                />
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="confirmNewPassword"
                    dataId="change-password-confirm"
                    component={renderField}
                    label={CHANGE_PASSWORD_MODAL.FORM.CONFIRM_PASSWORD_FUNC(intl)}
                    type={showPassword(isPasswordShow)}
                />
            </div>
            
            <div className="Form-Line row" style={{ marginTop: '20px' }}>
                <div className="col-md-7">
                    <Field
                        name="showPassword"
                        data-id="change-password-password"
                        id="password"
                        component="input"
                        type="checkbox"
                        style={{ marginRight: '10px', verticalAlign: 'middle' }}
                    />
                    <span style={{ verticalAlign: 'bottom' }}>{COMPONENTS.FIELDS.USER.SHOW_PASSWORD}</span>
                </div>
            </div>
            
            <div className="Form-Buttons">
                <button
                    type="button"
                    data-id="change-password-save"
                    onClick={submit}
                    disabled={invalid}
                >
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button type="button" data-id="change-password-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

ChangePasswordForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    invalid: PropTypes.bool,
    formState: PropTypes.shape(),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

ChangePasswordForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    formState: {},
    className: '',
    invalid: false,
    intl: {
        formatMessage: () => {
        },
    },
};


ChangePasswordForm = reduxForm({
    form: NAME_OF_FORMS.CHANGE_PASSWORD,
    validate,
    asyncValidate,
    asyncBlurFields: ['oldPassword', 'newPassword'],
    shouldAsyncValidate: (params) => {
        if (!params.syncValidationPasses) {
            return false;
        }
        return params.trigger === 'blur';
    },
})(ChangePasswordForm);

const mapStateToForm = state => ({
    initialValues: state.modals.authModals.changePassword,
    formState: state.form,
});
ChangePasswordForm = connect(mapStateToForm)(injectIntl(ChangePasswordForm));

async function submitMethod(dispatch, changePasswordModal, values, intl) {
    const decodedToken = decodeToken(changePasswordModal.token);

    await dispatch(changePassword({
        decodedToken,
        uid: values.uid,
        oldPassword: values.oldPassword,
        newPassword: values.newPassword,
    }));
    dispatch(closeChangePassword());
    if (changePasswordModal.openEditProfile) {
        dispatch(openEditProfile(changePasswordModal.openEditProfile));
    } else {
        dispatch(openStatusNotificationPopup({
            title: CHANGE_PASSWORD_POPUP.TITLE(intl),
            message: CHANGE_PASSWORD_POPUP.MESSAGE(intl),
            status: DEFAULT_VALUES.SUCCESS_STATUS,
        }));
        await sleep(1000);
        const { userInfo } = await dispatch(userLoginByPassword({
            email: decodedToken.user.email,
            password: values.newPassword,
        })).catch((err) => {
            console.error('err', err);
        });
        dispatch(settingUp(userInfo));
        dispatch(closeStatusNotificationPopup());
    }
}

class ChangePasswordModal extends React.PureComponent {
    static propTypes = {
        dispatch: PropTypes.func,
        changePasswordModal: PropTypes.shape({
            isSystemUser: PropTypes.bool,
            openEditProfile: PropTypes.object,
            uid: PropTypes.string,
            email: PropTypes.string,
            token: PropTypes.string,
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({
            content: PropTypes.object,
            overlay: PropTypes.object,
        }),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        changePasswordModal: {
            isSystemUser: false,
            openEditProfile: {},
            uid: '',
            email: '',
            open: false,
        },
        style: {
            content: {},
            overlay: {},
        },
        intl: {
            formatMessage: () => {
            },
        },
    };

    requestClose = () => {
        this.props.dispatch(closeChangePassword());

        if (this.props.changePasswordModal.openEditProfile) {
            this.props.dispatch(openEditProfile(this.props.changePasswordModal.openEditProfile));
        } else {
            this.props.dispatch(openChangePassword({
                force: true,
                email: this.props.changePasswordModal.email,
                token: this.props.changePasswordModal.token,
                uid: this.props.changePasswordModal.uid,
            }));
        }
    };

    closeButton = () => {
        this.props.dispatch(closeChangePassword());

        if (this.props.changePasswordModal.openEditProfile) {
            this.props.dispatch(openEditProfile(this.props.changePasswordModal.openEditProfile));
        } else {
            LocationStoreMethods.setToLocalStorage(
                STORAGE_KEY.FORCE_TO_CHANGE_PASSWORD,
                this.props.changePasswordModal.uid,
                this.props.changePasswordModal.email,
                null,
            );
            clearSessionStorage();
            this.props.dispatch({ type: ACTIONS.USER_LOGOUT });
        }
    };

    submit = (values) => {
        const { changePasswordModal, dispatch, intl } = this.props;
        submitMethod(dispatch, changePasswordModal, values, intl);
    };
    
    render() {
        const { changePasswordModal } = this.props;

        const style = makeCopy(this.props.style);
        style.content.width = '545px';
        
        return (
            <Modal
                contentLabel="EditProfile"
                isOpen={changePasswordModal.open}
                onRequestClose={this.requestClose}
                shouldCloseOnOverlayClick={false}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header">
                        {changePasswordModal.force
                            ? CHANGE_PASSWORD_MODAL.HEADER_FORCED
                            : CHANGE_PASSWORD_MODAL.HEADER}
                    </span>
                    
                    <ChangePasswordForm
                        className="Modal-Form"
                        close={this.closeButton}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    changePasswordModal: state.modals.authModals.changePassword,
});

export default connect(mapStateToProps)(injectIntl(ChangePasswordModal));
