import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, FieldArray, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';

import { NAME_OF_FORMS } from 'common/constants';
import { PROJECT_MODALS, COMPONENTS } from 'common/ui-labels';
import RenderSelectField from 'components/modals/RenderSelectField';
import { getTimeZones, getTimeZoneOffset } from 'utils/timeUtils';
import { formatTimeZoneOffset } from 'utils/formatters';
import { openProjectsSettingsModalBack, closeProjectsSettingsModal } from 'stores/actions/modalActions/authActions';
import { editProject } from 'stores/actions/entityActions/projectActions';
import store from 'stores/LocationStore';


const validate = (values, props) => {
    const errors = {};
    if (values.projects) {
        values.projects.forEach((project) => {
            if (Object.keys(values).indexOf(project.uid) === -1) {
                errors[project.uid] = PROJECT_MODALS.PROJECT_SETTINGS.ERRORS.PROJECT_SHOULD_SPECIFY(props.intl);
            }
        });
    }
    return errors;
};

const renderProjects = ({ projects }) => {
    const timezones = getTimeZones();
    return projects.map(project => (
        <div className="Form-Line row">
            <div className="Form-Label col-md-4">{project.name}</div>
            <div className="col-md-8">
                <Field
                    className="form-control input-sm "
                    name={project.uid}
                    dataId="project-edit-timezone"
                    style={{ fontSize: '14px' }}
                    component={RenderSelectField}
                >
                    <option />
                    {
                        timezones.map(timezone => (
                            <option value={timezone} key={timezone}>
                                {formatTimeZoneOffset(timezone)}
                            </option>
                        ))
                    }
                </Field>
            </div>
        </div>
    ));
};

let ProjectSettingsForm = (props) => {
    const {
        close, submit, className, projects, invalid,
    } = props;

    return (
        <form className={className}>
            <FieldArray name="members" component={renderProjects} projects={projects} />
            
            <div className="Form-Buttons">
                <button type="button" onClick={submit} disabled={invalid} data-id="project-edit-save">{COMPONENTS.BUTTONS.SAVE}</button>
                <button type="button" data-id="project-edit-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

ProjectSettingsForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    projects: PropTypes.arrayOf(PropTypes.shape({})),
    className: PropTypes.string,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

ProjectSettingsForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    invalid: false,
    projects: [],
    className: '',
    intl: {
        formatMessage: () => {
        },
    },
};

ProjectSettingsForm = reduxForm({
    form: NAME_OF_FORMS.PROJECTS_SETTINGS,
    validate,
})(ProjectSettingsForm);

const mapStateToForm = state => ({
    initialValues: state.modals.authModals.projectsSettings,
});
ProjectSettingsForm = connect(mapStateToForm)(injectIntl(ProjectSettingsForm));

class ProjectsSettingsModal extends React.PureComponent {
    static propTypes = {
        dispatch: PropTypes.func,
        projectsSettings: PropTypes.shape({
            open: PropTypes.bool,
            projects: PropTypes.arrayOf(PropTypes.shape({})),
        }),
        style: PropTypes.shape(),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        projectsSettings: {
            open: false,
            projects: [],
        },
        style: {},
    };
    
    close = () => {
        this.props.dispatch(openProjectsSettingsModalBack());
    };
    
    submit = (params) => {
        params.projects.forEach((project) => {
            const editedProject = {
                ...project,
            };
            const formFields = Object.keys(params);
            const formValues = Object.values(params);
            const timezone = formValues[formFields.indexOf(project.uid)];
            editedProject.offset = getTimeZoneOffset(store.getCurrentProject(), timezone); // in minutes
            editedProject.timezone = timezone;
            
            this.props.dispatch(editProject(editedProject));
        });
        this.props.dispatch(closeProjectsSettingsModal());
    };
    
    render() {
        const { projectsSettings, style } = this.props;
        
        return (
            <Modal
                contentLabel="Welcome"
                isOpen={projectsSettings.open}
                onRequestClose={this.close}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <div className="Modal-Header">
                        {PROJECT_MODALS.PROJECT_SETTINGS.HEADER}
                    </div>
                    
                    <div className="Modal-Text">
                        {PROJECT_MODALS.PROJECT_SETTINGS.TEXT}
                    </div>
    
                    <ProjectSettingsForm className="Modal-Form" projects={projectsSettings.projects} close={this.close} onSubmit={this.submit} />

                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    projectsSettings: state.modals.authModals.projectsSettings,
});

export default connect(mapStateToProps)(ProjectsSettingsModal);
