import React from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { setProjectUidToHeader, removeProjectFromHeader } from 'api';

import { NAME_OF_FORMS, NODE_RELEASE_ASSIGN_MODAL, COMPONENTS } from '../../common';
import { sortByField } from './../../utils';
import { closeReleaseAssignModal, releaseNodes, assignNodes, closeTrackableAssociations } from '../../stores/actions';

import { Association } from '../../stores/classes';
import { Trackable } from '../../stores/classes/Selectors';

const renderSelectField = ({
    input, children, dataId,
}) => (
    <div className="col-md-6" data-id={dataId}>
        <select {...input} className="form-control input-lg">
            {children}
        </select>
    </div>
);

renderSelectField.propTypes = {
    input: PropTypes.shape({}),
    children: PropTypes.oneOfType([
        PropTypes.arrayOf(PropTypes.node),
        PropTypes.node,
    ]),
    dataId: PropTypes.string,
};

renderSelectField.defaultProps = {
    input: {},
    children: [],
    dataId: '',
};

let NodeReleaseAssignForm = (props) => {
    const {
        close, submit, params, className, assigned_to, mac_address,
    } = props;
    const isProjectScope = props.currentProject ? props.currentProject.uid : null;
    const isRelease = !!(params.project_uid && params.company_uid);
    const isAssigned = !!(assigned_to && assigned_to.obj_uid);
    const trackable = isAssigned ? Trackable.getTrackableByUid(assigned_to && assigned_to.obj_uid) : '';
    let modalDetails = '';
    let modalMessage = '';
    let buttonName = '';
    
    if (isProjectScope && isRelease) {
        // Release tag in project scope
        modalMessage = isAssigned
            ? NODE_RELEASE_ASSIGN_MODAL.MESSAGE.RELEASE_ASSIGNED_FUNC(mac_address, trackable.name)
            : NODE_RELEASE_ASSIGN_MODAL.MESSAGE.RELEASE_NOT_ASSIGNED_PROJECT_SCOPE;
        modalDetails = isAssigned ? NODE_RELEASE_ASSIGN_MODAL.DETAIL.RELEASE_ASSIGNED : '';
        buttonName = COMPONENTS.BUTTONS.RELEASE;
    } else if (!isProjectScope && isRelease) {
        // Release tag in company scope
        modalMessage = isAssigned
            ? NODE_RELEASE_ASSIGN_MODAL.MESSAGE.RELEASE_ASSIGNED_FUNC(mac_address, trackable.name)
            : NODE_RELEASE_ASSIGN_MODAL.MESSAGE.RELEASE_NOT_ASSIGNED_COMPANY_SCOPE(params.project_name);
        modalDetails = isAssigned ? NODE_RELEASE_ASSIGN_MODAL.DETAIL.RELEASE_ASSIGNED : '';
        buttonName = COMPONENTS.BUTTONS.RELEASE;
    } else if (isProjectScope && !isRelease) {
        // Assign tag in project scope
        modalMessage = NODE_RELEASE_ASSIGN_MODAL.MESSAGE.ASSIGN_PROJECT_SCOPE;
        buttonName = COMPONENTS.BUTTONS.ASSIGN;
    } else if (!isProjectScope && !isRelease) {
        // Assign tag in company scope
        modalMessage = NODE_RELEASE_ASSIGN_MODAL.MESSAGE.ASSIGN_COMPANY_SCOPE;
        buttonName = COMPONENTS.BUTTONS.ASSIGN;
    }
    
    if (isProjectScope || isRelease) {
        return (
            <form className={className}>
                <div className="Form-Confirmation" data-id="node-release-assign-modal-message">
                    {modalMessage}
                </div>
                <div className="Form-DeleteTitle" data-id="node-release-assign-modal-details">
                    {modalDetails}
                </div>
                <div className="Form-Buttons">
                    <button type="button" data-id="node-release-assign-submit" onClick={submit}>{buttonName}</button>
                    <button type="button" data-id="node-release-assign-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
                </div>
            </form>
        );
    }
    return (
        <form className={className}>
            <div className="Form-Confirmation" data-id="node-release-assign-message">
                {modalMessage}
            </div>
            <div className="row">
                <div className="col-md-3" />
                <Field
                    name="projectUid"
                    dataId="node-release-assign-options"
                    style={{ fontSize: '14px' }}
                    component={renderSelectField}
                >
                    <option />
                    {props.projects.map(project => (
                        <option value={project.uid} key={project.uid}>{project.name}</option>
                    ))}
                </Field>
            </div>
            <div className="Form-Buttons">
                <button type="button" data-id="node-release-assign-submit" onClick={submit}>{buttonName}</button>
                <button type="button" data-id="node-release-assign-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

NodeReleaseAssignForm = reduxForm({
    form: NAME_OF_FORMS.NODE_RELEASE_ASSIGN,
})(NodeReleaseAssignForm);


NodeReleaseAssignForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    projects: PropTypes.arrayOf(PropTypes.shape({
        uid: PropTypes.string,
        string: PropTypes.string,
    })),
    mac_address: PropTypes.string,
    assigned_to: PropTypes.shape({}),
    currentProject: PropTypes.shape({
        uid: PropTypes.string,
    }),
    params: PropTypes.shape({
        project_uid: PropTypes.string,
        company_uid: PropTypes.string,
        project_name: PropTypes.string,
    }),
};


NodeReleaseAssignForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    className: '',
    projects: {
        uid: '',
        string: '',
    },
    mac_address: '',
    assigned_to: {},
    currentProject: {
        uid: '',
    },
    params: {
        project_uid: '',
        company_uid: '',
        project_name: '',
    },
};

const mapStateToForm = state => ({
    initialValues: state.modals.nodeModals.nodeReleaseAssign,
    projects: state.entities.projects,
    currentProject: state.appContext.project,
});
NodeReleaseAssignForm = connect(mapStateToForm)(NodeReleaseAssignForm);


class NodeReleaseAssignModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        projects: PropTypes.arrayOf(PropTypes.shape({
            uid: PropTypes.string,
        })),
        mac_address: PropTypes.string,
        nodeReleaseAssign: PropTypes.shape({
            params: PropTypes.shape({
                project_uid: PropTypes.string,
                company_uid: PropTypes.string,
                project_name: PropTypes.string,
                mac_address: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({}),
        projectScope: PropTypes.shape({
            uid: PropTypes.string,
            name: PropTypes.string,
        }),
        associations: PropTypes.shape(),
    };
    
    
    static defaultProps = {
        dispatch: () => {},
        projects: [],
        mac_address: '',
        nodeReleaseAssign: {
            params: {
                project_uid: '',
                company_uid: '',
                project_name: '',
                mac_address: '',
            },
            open: false,
        },
        style: {
        
        },
        projectScope: {
            uid: '',
            name: '',
        },
        associations: {
            byObjUid: {},
            byMac: {},
        },
    };

    findTrackableObjectAssociation = (mac_address) => {
        if (mac_address) {
            const obj_uid = this.props.associations.byMac[mac_address.toUpperCase()];
            return obj_uid ? this.props.associations.byObjUid[obj_uid] : null;
        }
        return null;
    };
    
    close = () => {
        this.props.dispatch(closeReleaseAssignModal());
    };
    
    submit = (values) => {
        let projectTag;
        const assignReleaseProject = values.params.project_uid && values.params.company_uid ? values.params.project_uid : values.projectUid;
        
        // If project scope get projectTag from appContext else from loaded projects
        if (this.props.projectScope) {
            projectTag = this.props.projectScope;
        } else {
            this.props.projects.forEach((project) => {
                if (project.uid === assignReleaseProject) {
                    projectTag = project;
                }
            });
        }
        
        const params = {
            node: {
                uid: values.params.uid,
                mac_address: values.params.mac_address,
            },
            project: projectTag,
        };
        
        if (this.props.nodeReleaseAssign.params.project_uid && this.props.nodeReleaseAssign.params.company_uid) {
            const trackableObjectAssociation = this.findTrackableObjectAssociation(values.params.mac_address);
            
            // Close trackable object association
            if (trackableObjectAssociation && !this.props.projectScope) {
                // Need project in header to close association in server side
                setProjectUidToHeader(this.props.nodeReleaseAssign.params.project_uid);
                this.props.dispatch(closeTrackableAssociations(trackableObjectAssociation));
                // After closing association project should be removed from header
                removeProjectFromHeader();
            } else if (trackableObjectAssociation) {
                this.props.dispatch(closeTrackableAssociations(trackableObjectAssociation));
            }
            this.props.dispatch(releaseNodes(params));
        } else {
            this.props.dispatch(assignNodes(params));
        }
        this.props.dispatch(closeReleaseAssignModal());
    };
    
    render() {
        let title = '';
        const mac_address = this.props.nodeReleaseAssign.params ? this.props.nodeReleaseAssign.params.mac_address : '';
        const assigned_to = this.findTrackableObjectAssociation(mac_address);
        
        const isProjectScope = this.props.projectScope ? this.props.projectScope.uid : null;
        const isRelease = this.props.nodeReleaseAssign.params && this.props.nodeReleaseAssign.params.project_uid && this.props.nodeReleaseAssign.params.company_uid;
        const isAssigned = !!assigned_to;
        
        if (isRelease && isProjectScope && !isAssigned) {
            title = NODE_RELEASE_ASSIGN_MODAL.TITLE.RELEASE_FROM_PROJECT_FUNC(this.props.projectScope.name);
        } else if (isRelease && isAssigned) {
            title = NODE_RELEASE_ASSIGN_MODAL.TITLE.RELEASE_TAG_FUNC(mac_address);
        } else if (!isRelease && isProjectScope) {
            title = NODE_RELEASE_ASSIGN_MODAL.TITLE.CLAIM_TO_PROJECT_FUNC(this.props.projectScope.name);
        } else if (!isRelease && isProjectScope) {
            title = NODE_RELEASE_ASSIGN_MODAL.TITLE.ASSIGN_TAG_FUNC(mac_address);
        }
        
        const { style } = this.props;
        
        return (
            <Modal
                contentLabel="nodeRelease"
                isOpen={this.props.nodeReleaseAssign.open}
                onRequestClose={this.close}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header" data-id="node-release-assign-title">
                        {title}
                    </span>
                    <NodeReleaseAssignForm
                        className="Modal-Form"
                        onSubmit={this.submit}
                        assigned_to={assigned_to}
                        mac_address={mac_address}
                        close={this.close}
                        params={this.props.nodeReleaseAssign.params}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    projects: sortByField(state.entities.projects, 'name'),
    projectScope: state.appContext.project,
    nodeReleaseAssign: state.modals.nodeModals.nodeReleaseAssign,
    associations: Association.getAllAssociations(state),
});

export default connect(mapStateToProps)(NodeReleaseAssignModal);
