import React from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import _Get from 'lodash/get';
import { NAME_OF_FORMS, NOTIFICATION_TYPE, COMPONENTS, NOTIFICATION_MODAL } from 'common';
import { closeNotificationDetailsModal } from 'stores/actions';
import { formatDbNameToHumanReadable } from 'utils';
import Subscriber from 'stores/classes/Subscriber';
import RenderTooltipHelper from 'components/RenderTooltipHelper';

let NotificationDetailsForm = (props) => {
    const {
        close, params, className, subscribers, intl,
    } = props;
    const isAlarmNotification = params && params.notification_type === NOTIFICATION_TYPE.ALARM;

    return (
        <form className={className}>
            <div className="Form-Line row">
                <div className="Form-Label col-md-5">{NOTIFICATION_MODAL.FORM.LABEL.NOTIFICATION_NAME}</div>
                <div
                    className="Form-Value col-md-7 multiple-lines-ellipsis"
                    data-cy="notification-details-name"
                >
                    {params.name}
                </div>
            </div>
            {isAlarmNotification &&
            <div className="Form-Line row">
                <div className="Form-Label col-md-5">
                    {COMPONENTS.FIELDS.ALARM.ZONE_SEVERITY_LEVEL}
                    <RenderTooltipHelper
                        tooltipText={NOTIFICATION_MODAL.FORM.LABEL.ZONE_SEVERITY_LEVEL_COMMENT_FUNC(intl)}
                    />
                </div>
                <div
                    className="Form-Value col-md-7"
                    data-cy="notification-details-severity"
                >
                    {params.zone_severity_level}
                </div>
            </div>}

            {isAlarmNotification && params.conditions && params.conditions.alarm_type_list && params.conditions.alarm_type_list.length > 0 &&
            <div className="Form-Line row">
                <div className="Form-Label col-md-5">{NOTIFICATION_MODAL.FORM.LABEL.NOTIFICATION_TYPE}</div>
                <div
                    className="Form-Value col-md-7"
                    data-cy="notification-details-alarm-type-list"
                >
                    {params.conditions.alarm_type_list.map(x => formatDbNameToHumanReadable(intl, x)).join(', ')}
                </div>
            </div>}

            {isAlarmNotification &&
            <div className="Form-Line row">
                <div className="Form-Label col-md-5">{NOTIFICATION_MODAL.FORM.LABEL.NOTIFICATION_METHOD}</div>
                <div className="Form-Value col-md-7">
                    {params.email ? <span data-cy="notification-details-email">{COMPONENTS.FIELDS.USER.EMAIL}<br /></span> : ''}
                    {params.phone ? <span data-cy="notification-details-sms">{NOTIFICATION_MODAL.FORM.LABEL.SMS_TEXT}<br /></span> : ''}
                </div>
            </div>}

            <div className="Form-Line row">
                <div className="Form-Label col-md-5">{COMPONENTS.FIELDS.COMMON.STATUS}</div>
                <div
                    className="Form-Value col-md-7"
                    data-cy="notification-details-active"
                >
                    {params.active ? COMPONENTS.FIELDS.COMMON.ACTIVE : COMPONENTS.FIELDS.COMMON.INACTIVE}
                </div>
            </div>
            
            {isAlarmNotification && subscribers.length > 0 &&
            <div className="Form-Line row">
                <div className="Form-Label col-md-5">{COMPONENTS.FIELDS.SUBSCRIBER.SUBSCRIBER_TITLE}</div>
                <div className="Form-Value col-md-7">
                    <div
                        className="RPP-NotificationDetailSubscriberList"
                        data-id="notification-details-subscribers"
                    >
                        {subscribers.map((name, i) => <span key={i}>{i === 0 ? name : `, ${name}`}</span>)}
                    </div>
                </div>
            </div>}
            
            <div className="Form-Buttons OneBtn">
                <button type="button" data-id="notification-details-cancel" onClick={close}>{COMPONENTS.BUTTONS.CLOSE}</button>
            </div>
        </form>
    );
};

NotificationDetailsForm = reduxForm({
    form: NAME_OF_FORMS.NOTIFICATION_DETAILS,
})(injectIntl(NotificationDetailsForm));

NotificationDetailsForm.propTypes = {
    className: PropTypes.string,
    close: PropTypes.func,
    subscribers: PropTypes.arrayOf(PropTypes.string),
    params: PropTypes.shape({}),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

NotificationDetailsForm.defaultProps = {
    close: () => {
    },
    subscribers: [],
    params: {},
    className: '',
    intl: {
        formatMessage: () => {
        },
    },
};

const mapStateToForm = (state) => {
    const params = state.modals.notificationModals.notificationDetails.params || {};
    const subscribers = Subscriber.getAllSubscribers(state)
        .filter(subscriber => subscriber.notifications.some(notification => notification === params.uid))
        .map(subscriber => subscriber.name || '');
    return {
        params,
        subscribers,
    };
};

NotificationDetailsForm = connect(mapStateToForm)(NotificationDetailsForm);

class NotificationDetailsModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        notificationDetails: PropTypes.shape({
            params: PropTypes.shape({
                uid: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({}),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        notificationDetails: {
            params: {
                uid: '',
            },
            open: false,
        },
        style: {},
    };
    
    close = () => {
        this.props.dispatch(closeNotificationDetailsModal());
    };
    
    render() {
        const { style } = this.props;
        
        return (
            <Modal
                contentLabel="notificationDetails"
                isOpen={this.props.notificationDetails.open}
                onRequestClose={this.close}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis">
                        {COMPONENTS.FIELDS.COMMON.VIEW_INFORMATION_FUNC(_Get(this.props, 'notificationDetails.params.name', ''))}
                    </span>
                    <NotificationDetailsForm
                        className="Modal-Form"
                        close={this.close}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    notificationDetails: state.modals.notificationModals.notificationDetails,
});

export default connect(mapStateToProps)(NotificationDetailsModal);
