import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';

import RenderField from 'components/modals/RenderField';
import SelectComponent from 'components/modals/SelectComponent';
import { COMPONENTS, NAME_OF_FORMS, PROJECT_MODALS } from 'common';
import { formatTimeZoneOffset, getTimeZones } from 'utils';
import store from 'stores/LocationStore';
import { closeProjectEditModal, editProject, createProject } from 'stores/actions';

const validate = (values) => {
    const errors = {};
    if (!values.timezone) {
        errors.timezone = PROJECT_MODALS.EDIT_MODAL.TIME_ZONE_ERROR;
    } else if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED;
    } else if (!!store.getProjectByName(values.name) && store.getProjectByName(values.name).uid !== values.uid) {
        errors.name = COMPONENTS.VALIDATION.NAME_EXIST;
    }
    
    return errors;
};

const renderField = RenderField(4, 8);

let ProjectEditForm = (props) => {
    const {
        close, submit, className, invalid, intl, initialValues, submitSucceeded,
    } = props;
    const timezones = getTimeZones();
    const timezoneList = timezones.map(timezone => ({
        value: timezone,
        label: formatTimeZoneOffset(timezone),
    }));
    
    return (
        <form className={className}>
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="name"
                    dataId="project-edit-name"
                    component={renderField}
                    label={`${COMPONENTS.FIELDS.COMMON.NAME_FUNC(intl)} *`}
                    type="text"
                    autoFocus={!(initialValues && initialValues.uid)}
                />
            </div>
    
            <div className="Form-Line row">
                <div className="Form-Label col-md-4">{PROJECT_MODALS.DETAIL_MODAL.FORM.TIME_ZONE} *</div>
                <div className="col-md-8" data-cy="project-edit-timezone-div">
                    <Field
                        className="form-control input-sm "
                        name="timezone"
                        data-id="project-edit-timezone"
                        dataId="project-edit-timezone"
                        style={{ fontSize: '14px' }}
                        simpleValue
                        clearable={false}
                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                        noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                        options={timezoneList}
                        component={SelectComponent}
                    />
                </div>
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="manager"
                    dataId="project-edit-manager"
                    component={renderField}
                    label={PROJECT_MODALS.EDIT_MODAL.FORM.MANAGER_FUNC(intl)}
                    type="text"
                />
            </div>

            <div className="Form-Buttons">
                <button
                    type="button"
                    onClick={submit}
                    disabled={invalid || submitSucceeded}
                    data-id="project-edit-save"
                >
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button type="button" data-id="project-edit-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

ProjectEditForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    className: PropTypes.string,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    initialValues: PropTypes.shape({}),
    submitSucceeded: PropTypes.bool,
};

ProjectEditForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    invalid: false,
    className: '',
    intl: {
        formatMessage: () => {
        },
    },
    initialValues: {},
    submitSucceeded: false,
};

ProjectEditForm = reduxForm({
    form: NAME_OF_FORMS.PROJECT_EDIT,
    validate,
})(ProjectEditForm);

const mapStateToForm = state => ({
    initialValues: state.modals.projectModals.projectEdit.params,
});
ProjectEditForm = connect(mapStateToForm)(injectIntl(ProjectEditForm));


class ProjectEditModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        projectEdit: PropTypes.shape({
            params: PropTypes.shape({
                uid: PropTypes.string,
                name: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({}),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        projectEdit: {
            params: {
                uid: '',
                name: '',
            },
            open: false,
        },
        style: {},
    };
    
    close = () => {
        this.props.dispatch(closeProjectEditModal());
    };
    
    submit = (values) => {
        if (this.props.projectEdit.params && this.props.projectEdit.params.uid) {
            this.props.dispatch(editProject(values));
        } else {
            this.props.dispatch(createProject(values));
        }
        
        this.props.dispatch(closeProjectEditModal());
    };
    
    render() {
        let title = PROJECT_MODALS.EDIT_MODAL.TITLE;
        if (this.props.projectEdit.params && this.props.projectEdit.params.uid) {
            title = this.props.projectEdit.params.name;
        }
        const { style } = this.props;
        
        return (
            <Modal
                contentLabel="projectEdit"
                isOpen={this.props.projectEdit.open}
                onRequestClose={this.close}
                style={style}
            >
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis" data-id="project-edit-title">
                        {title}
                    </span>
                    <ProjectEditForm className="Modal-Form" close={this.close} onSubmit={this.submit} />
                </div>
            </Modal>
        );
    }
}


const mapStateToProps = state => ({
    projectEdit: state.modals.projectModals.projectEdit,
});

export default connect(mapStateToProps)(ProjectEditModal);
