import React from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import { reduxForm } from 'redux-form';
import { TRACKABLE_DELETE_MODAL, TRACKABLE_OR_TRACKABLES_SELECTED } from 'common/ui-labels';
import { NAME_OF_FORMS } from 'common/constants';
import { closeTrackableDeleteModal, deleteTrackable } from 'stores/actions';
import { makeCopy } from 'utils/simpleUtils';
import DeleteButtons from 'components/modals/DeleteButtons';


let TrackableDeleteForm = (props) => {
    const { close, submit, className, initialValues, assigned_to, submitted } = props;
    const isBatchDelete = Array.isArray(initialValues);

    const tobeDeletedCount = initialValues.length;
    const numberOfAssigned = assigned_to.length;

    const isAnyAssigned = numberOfAssigned > 0;
    const isSingleAssigned = !isBatchDelete && numberOfAssigned === 1;

    const tkoName = initialValues.name;
    const assignedNodeMac = isSingleAssigned ? assigned_to[0].node.name : '';

    const singleAssignedMessage = TRACKABLE_DELETE_MODAL.ASSIGN.MESSAGE_FUNC(<b>{tkoName}</b>, <b>{assignedNodeMac}</b>);
    const batchAssignedMessage = isAnyAssigned
        ? TRACKABLE_DELETE_MODAL.BATCH.MESSAGE.ASSIGNED_COMMON(<b>{numberOfAssigned}</b>, <b>{tobeDeletedCount}</b>)
        : TRACKABLE_OR_TRACKABLES_SELECTED(<b>{tobeDeletedCount}</b>);

    const singleDeleteConfirmation = TRACKABLE_DELETE_MODAL.NO_ASSIGN.MESSAGE_FUNC(!isAnyAssigned && tkoName);
    const confirmationMessage = isBatchDelete
        ? TRACKABLE_DELETE_MODAL.BATCH.MESSAGE.NO
        : singleDeleteConfirmation;

    return (
        <form className={className}>
            {
                (isBatchDelete || isSingleAssigned) && <div className="Form-Confirmation info-text" data-id="trackable-delete-message">
                    { isSingleAssigned ? singleAssignedMessage : batchAssignedMessage }
                </div>
            }

            <div className="Form-DeleteTitle confirmation" data-id="trackable-delete-details">
                { confirmationMessage }
            </div>
    
            <DeleteButtons submit={submit} close={close} submitted={submitted} dataId="trackable" />
        </form>
    );
};

TrackableDeleteForm = reduxForm({
    form: NAME_OF_FORMS.TRACKABLE_DELETE,
})(TrackableDeleteForm);

TrackableDeleteForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    submitted: PropTypes.bool,
    initialValues: PropTypes.oneOfType([
        PropTypes.shape({
            uid: PropTypes.string,
            name: PropTypes.string,
        }),
        PropTypes.array,
    ]),
    className: PropTypes.string,
    assigned_to: PropTypes.arrayOf(PropTypes.shape({
        node: PropTypes.shape({}),
        trackable: PropTypes.shape({}),
    })),
};

TrackableDeleteForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    submitted: false,
    initialValues: {
        uid: '',
        name: '',
    },
    className: '',
    assigned_to: [],
};

const mapStateToForm = state => ({
    initialValues: state.modals.trackableModals.trackableDelete.params,
});
TrackableDeleteForm = connect(mapStateToForm)(TrackableDeleteForm);


class TrackableDeleteModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        nodesMap: PropTypes.shape({}),
        associationsMap: PropTypes.shape({}),
        trackableDelete: PropTypes.shape({
            params: PropTypes.oneOfType([
                PropTypes.shape({
                    uid: PropTypes.string,
                    name: PropTypes.string,
                }),
                PropTypes.arrayOf(PropTypes.shape({
                    uid: PropTypes.string,
                })),
            ]),
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({}),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    
    static defaultProps = {
        dispatch: () => { },
        nodesMap: {},
        associationsMap: {},
        trackableDelete: {
            params: {
                uid: '',
                name: '',
            },
            open: false,
        },
        style: {},
        intl: {
            formatMessage: () => {
            },
        },
    };
    
    constructor(props) {
        super(props);
        this.state = {
            submitted: false,
        };
    }
    
    close = () => {
        this.props.dispatch(closeTrackableDeleteModal());
    };
    
    submit = () => {
        this.setState({ submitted: true });
        this.props.dispatch(deleteTrackable({ deleteParams: this.props.trackableDelete.params, intl: this.props.intl }));
    };
    
    render() {
        const { style, nodesMap, associationsMap } = this.props;
        const trackable = this.props.trackableDelete.params ? this.props.trackableDelete.params : {};

        const isBatchDelete = Array.isArray(this.props.trackableDelete.params);
        const assigned_to = [];
        
        if (isBatchDelete) {
            this.props.trackableDelete.params.forEach((to) => {
                const association = associationsMap[to.uid];
                if (association) {
                    assigned_to.push({ node: nodesMap[association.mac_address.toUpperCase()], trackable: to });
                }
            });
        } else if (trackable.uid) {
            const association = associationsMap[trackable.uid];
            if (association) {
                assigned_to.push({ node: nodesMap[association.mac_address.toUpperCase()], trackable });
            }
        }

        const customStyle = makeCopy(style);
        customStyle.content.width = 540;

        return (
            <Modal
                contentLabel="companyDelete"
                isOpen={this.props.trackableDelete.open}
                onRequestClose={this.close}
                shouldCloseOnOverlayClick={false}
                onAfterClose={() => {
                    this.setState({ submitted: false });
                }}
                style={customStyle}
            >
                <div className="RPP-Modal">
                    <TrackableDeleteForm
                        submitted={this.state.submitted}
                        className="Modal-Form"
                        assigned_to={assigned_to}
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    trackableDelete: state.modals.trackableModals.trackableDelete,
    nodesMap: state.entities.nodes.byMac,
    associationsMap: state.entities.associations.byObjUid,
});

export default connect(mapStateToProps)(injectIntl(TrackableDeleteModal));
