import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import _Get from 'lodash/get';
import _Has from 'lodash/has';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import { createSelector } from 'reselect';
import { reduxForm } from 'redux-form';

import { DB_NAMES, TRACKABLE_MODAL, INDICATION, COMPONENTS, GROUP_TRACKABLE_COMMON_FIELD } from 'common/ui-labels';
import { NAME_OF_FORMS, DEFAULT_VALUES, ROUTES, CUSTOM_LOCAL_STORE_KEYS, PWS_PIN } from 'common/constants';
import { getNodeHwCbId, formatInactivity } from 'utils/formatters';
import { valueNaN, macToUpperCase, stringToBoolean, showPwsFields, isPerson } from 'utils/utils';
import {
    formatTextForUnresponsiveTrackable,
    getNodeBatteryLevel,
    getLocalizedRole,
} from 'utils/htmlUtils';
import { makeCopy } from 'utils/simpleUtils';
import { closeTrackableDetailsModal } from 'stores/actions';
import { Trackable, Group, Zone, TrackableTemplate } from 'stores/classes';
import { Node } from 'stores/classes/Selectors';
import { getLocalStorageValue } from 'utils/browserStorage';
import { getNodeInfoByMac } from 'utils/trackableUtils';
import { UserParamsSection } from './userParamsSection';

let TrackableDetailsForm = (props) => {
    const { close, params, className, isPlaybackOpen, isSitePageOpen, maxHeight, intl, trackablesMap, nodes, trackableTemplates } = props;

    const lastHeardText = formatInactivity(params.last_heard, intl);
    const isTrackableResponsive = params.isResponsive;
    const isTrackableAssigned = !!params.node;
    const nodeMacAddress = params.node && params.node.mac_address;
    const positionMacAddress = params.position && params.position.mac_address;
    const showNodeParams = !!(nodeMacAddress || (positionMacAddress && isPlaybackOpen && isSitePageOpen));
    const role = _Get(params, 'tag_config.t2t_cfg.role');
    const hasAlarmBack = _Has(params, 'tag_config.t2t_cfg.alarm_back');
    const alarmBack = _Get(params, 'tag_config.t2t_cfg.alarm_back');
    const safe_distance = _Get(params, 'tag_config.t2t_cfg.safe_distance');
    const distance = _Get(params, 'tag_config.t2t_cfg.distance');
    const alert_distance = _Get(params, 'tag_config.t2t_cfg.alert_distance');
    const pin = _Get(params, 'tag_config.t2t_cfg.pin');
    const delay = _Get(params, 'tag_config.t2t_cfg.delay');
    const haptic = _Get(params, 'tag_config.t2t_cfg.haptic');
    const buzzer = _Get(params, 'tag_config.t2t_cfg.buzzer');
    const isPinOn = pin === PWS_PIN.PULSE;
    const angle = _Get(params, 'tag_config.t2t_cfg.angle');
    const hasStationary = _Has(params, 'tag_config.stationary');
    const stationary = _Get(params, 'tag_config.stationary');
    const custom_info = params.custom_info;
    const noAlarmTrackableObjectNames = params.noAlarmTrackableObjectNames;
    const category = params.category;
    const isCategoryPerson = isPerson(params.category);

    const intlTkoType = params.type && DB_NAMES[params.type.toUpperCase()] && DB_NAMES[params.type.toUpperCase()](intl);
    const intlTkoCategory = params.category && DB_NAMES[params.category.toUpperCase()] && DB_NAMES[params.category.toUpperCase()](intl);
    const pairedWith = params.paired && trackablesMap[params.paired]
        ? trackablesMap[params.paired].name 
        : DB_NAMES.NONE(intl);
    const showUid = stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.SHOW_EXTENDS));

    const {
        showDistance, showAlertDistance, showSafeDistance, showDelay,
        showPin, showAngle, showAlarmBack, showsT2tNoAlarmUid,
    } = showPwsFields(role, category);

    const isHapticBuzzerVisible = pin && showPin && isPinOn && isCategoryPerson;
    const nodeInfo = getNodeInfoByMac(nodeMacAddress, nodes);

    return (
        <form className={className}>
            <div
                className="Modal-Form-Body"
                style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight }}
            >
                <UserParamsSection
                    templateProperties={TrackableTemplate.getTemplatePropertyByCategory(params.category, trackableTemplates)}
                    user_params={params.user_params}
                />

                <Fragment key="node">
                    {showUid &&
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4">UID</div>
                            <div
                                className="Form-Value col-md-8"
                                data-id="trackable-details-uid"
                                data-cy="trackable-details-uid"
                            >
                                {params.uid}
                            </div>
                        </div>
                    }

                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">{TRACKABLE_MODAL.DETAILS.FORM.TAG_ASSOCIATION}</div>
                        <div
                            className="Form-Value col-md-8"
                            data-id="trackable-details-tag-assoc"
                            data-cy="trackable-details-tag-assoc"
                        >
                            {nodeMacAddress ? COMPONENTS.FIELDS.COMMON.YES : COMPONENTS.FIELDS.COMMON.NO}
                        </div>
                    </div>

                    {showNodeParams
                    && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4 grey-text">{COMPONENTS.FIELDS.NODE.MAC_ADDRESS}</div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-id="trackable-details-update-mac"
                                data-cy="trackable-details-tag-mac"
                            >
                                {macToUpperCase(nodeMacAddress || positionMacAddress)}
                            </div>
                        </div>
                    )}

                    {showNodeParams
                    && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.HARDWARE_VERSION}</div>
                            <div className="Form-Value col-md-8 grey-text" data-id="trackable-details-update-mac">
                                {params.hwCbid || '--'}
                            </div>
                        </div>
                    )}

                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.NODE.BATTERY}</div>
                        <div className="Form-Value col-md-8" data-id="trackable-details-update-battery">
                            {getNodeBatteryLevel(nodeInfo, intl)}
                        </div>
                    </div>
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.NODE.LAST_REPORTED}</div>
                        <div className="Form-Value col-md-8" data-id="trackable-details-update-last-reported">
                            {formatTextForUnresponsiveTrackable(lastHeardText, !isTrackableResponsive && isTrackableAssigned)}
                        </div>
                    </div>
                </Fragment>

                <Fragment key="buffer">
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">{TRACKABLE_MODAL.BUFFER}</div>
                        <div
                            className="Form-Value col-md-8"
                            data-id="trackable-details-update-yes-no"
                            data-cy="trackable-details-attached-zone"
                        >
                            {params.attached_zone ? COMPONENTS.FIELDS.COMMON.YES : COMPONENTS.FIELDS.COMMON.NO}
                        </div>
                    </div>

                    {params.attached_zone && params.buffer_radius
                    && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.BUFFER_RADIUS}</div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-id="trackable-details-update-radius"
                                data-cy="trackable-details-buffer-radius"
                            >
                                {params.buffer_radius}
                            </div>
                        </div>
                    )}

                    {params.attached_zone
                    && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.BUFFER_STATUS}</div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-id="trackable-details-update-status-inactive"
                                data-cy="trackable-details-buffer-status"
                            >
                                {params.active_buffer
                                    ? COMPONENTS.FIELDS.COMMON.ACTIVE : COMPONENTS.FIELDS.COMMON.STATUS_INACTIVE
                                }
                            </div>
                        </div>
                    )}

                    {params.attached_zone && params.bufferExceptionLists && params.bufferExceptionLists.length > 0
                    && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.BUFFER_EXCEPTION}</div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-id="trackable-details-update-buffer-exception"
                                data-cy="trackable-details-buffer-exception"
                            >
                                {params.bufferExceptionLists.map(x => x.name).join(', ')}
                            </div>
                        </div>
                    )}
                </Fragment>

                <Fragment key="advanced">
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">{TRACKABLE_MODAL.DETAILS.FORM.ADVANCED}</div>
                        <div className="Form-Value col-md-8" />
                    </div>
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.DEFAULT_HEIGHT}</div>
                        <div
                            className="Form-Value col-md-8 grey-text"
                            data-id="trackable-details-update-default-height"
                        >
                            {params.defaultHeight}
                        </div>
                    </div>
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.TRACKABLE_TYPE}</div>
                        <div
                            className="Form-Value col-md-8 grey-text"
                            data-id="trackable-details-update-type"
                        >
                            {intlTkoType || ''}
                        </div>
                    </div>
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.TRACKABLE_CATEGORY}</div>
                        <div
                            className="Form-Value col-md-8 grey-text"
                            data-id="trackable-details-update-category"
                        >
                            {intlTkoCategory || ''}
                        </div>
                    </div>
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4 grey-text">{TRACKABLE_MODAL.DETAILS.FORM.TRACKABLE_PAIRED_WITH}</div>
                        <div
                            className="Form-Value col-md-8 grey-text"
                            data-id="trackable-details-update-paired-with"
                        >
                            {pairedWith}
                        </div>
                    </div>
                    <div className="Form-Line row">
                        <div
                            className="Form-Label col-md-4 grey-text"
                        >
                            {TRACKABLE_MODAL.DETAILS.FORM.UPDATE_INTERVAL}
                        </div>
                        <div
                            className="Form-Value col-md-8 grey-text"
                            data-id="trackable-details-update-interval"
                        >
                            {valueNaN(params.position_update_interval, '')}
                        </div>
                    </div>
                    {hasStationary && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4 grey-text">
                                {COMPONENTS.FIELDS.NODE.STATIONARY}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-stationary"
                            >
                                {stationary ? COMPONENTS.FIELDS.COMMON.YES : COMPONENTS.FIELDS.COMMON.NO}
                            </div>
                        </div>
                    )}
                </Fragment>

                {params.sensorSpecialCases && Object.keys(params.sensorSpecialCases).length > 0
                && (
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4 grey-text">{COMPONENTS.FIELDS.NODE.ADVANCED_SENSOR_INTEGRATION_PARAMETER}</div>
                        <div className="Form-Value col-md-8 grey-text" data-id="trackable-details-sensor">
                            {Object.keys(params.sensorSpecialCases).map((x, key) => (
                                <span key={key}>
                                    {x}
                                    :
                                    {' '}
                                    { params.sensorSpecialCases[x]
                                        ? GROUP_TRACKABLE_COMMON_FIELD.SENSOR_CONFIG_ENABLED
                                        : GROUP_TRACKABLE_COMMON_FIELD.SENSOR_CONFIG_DISABLED}
                                </span>
                            ))}
                        </div>
                    </div>
                )}

                <Fragment key="zones">
                    {((params.inclusionsForZones && params.inclusionsForZones.length) || (params.exclusionsForZones && params.exclusionsForZones.length))
                        ? (
                            <div className="Form-Line row">
                                <div className="Form-Label col-md-4">{GROUP_TRACKABLE_COMMON_FIELD.ZONE_CONFIGURATION}</div>
                                <div className="Form-Value col-md-8" />
                            </div>
                        ) : ''
                    }
                    {params.inclusionsForZones && params.inclusionsForZones.length
                        ? (
                            <div className="Form-Line row">
                                <div className="Form-Label col-md-4 grey-text">{GROUP_TRACKABLE_COMMON_FIELD.ZONE_INCLUSIONS}</div>
                                <div
                                    className="Form-Value col-md-8 grey-text"
                                    data-cy="trackable-details-inclusions"
                                >
                                    {params.inclusionsForZones.join(', ')}
                                </div>
                            </div>
                        ) : ''
                    }
                    {params.exclusionsForZones && params.exclusionsForZones.length
                        ? (
                            <div className="Form-Line row">
                                <div className="Form-Label col-md-4 grey-text">{GROUP_TRACKABLE_COMMON_FIELD.ZONE_EXCLUSIONS}</div>
                                <div
                                    className="Form-Value col-md-8 grey-text"
                                    data-cy="trackable-details-exclusions"
                                >
                                    {params.exclusionsForZones.join(', ')}
                                </div>
                            </div>
                        ) : ''
                    }
                </Fragment>

                <Fragment key="t2t_config">
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-12">
                            {COMPONENTS.FIELDS.NODE.PWS_TITLE}
                        </div>
                    </div>
                    {role && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3">
                                {TRACKABLE_MODAL.ROLE}
                            </div>
                            <div
                                className="Form-Value col-md-8"
                                data-cy="trackable-details-role"
                            >
                                {getLocalizedRole(role, intl)}
                            </div>
                        </div>
                    )}
                    {hasAlarmBack && showAlarmBack && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.DETAILS.FORM.ALARM_BACK}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-alarm-back"
                            >
                                {alarmBack ? COMPONENTS.FIELDS.COMMON.YES : COMPONENTS.FIELDS.COMMON.NO}
                            </div>
                        </div>
                    )}
                    {!isNaN(safe_distance) && showSafeDistance && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.SAFE_DISTANCE}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-safe-distance"
                            >
                                {safe_distance / 1000} m
                            </div>
                        </div>
                    )}
                    {!isNaN(distance) && showDistance && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.DISTANCE}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-distance"
                            >
                                {distance / 1000} m
                            </div>
                        </div>
                    )}
                    {!isNaN(alert_distance) && showAlertDistance && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.ALERT_DISTANCE}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-alert-distance"
                            >
                                {alert_distance / 1000} m
                            </div>
                        </div>
                    )}
                    {pin && showPin && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.PIN}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text text-capitalize"
                                data-cy="trackable-details-pin"
                            >
                                {isPinOn ? DB_NAMES.ON(intl) : DB_NAMES.OFF(intl)}
                            </div>
                        </div>
                    )}
                    {isHapticBuzzerVisible && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {INDICATION.HAPTIC(intl)}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-haptic"
                            >
                                {haptic ? DB_NAMES.ON(intl) : DB_NAMES.OFF(intl)}
                            </div>
                        </div>
                    )}
                    {isHapticBuzzerVisible && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {INDICATION.BUZZER(intl)}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-buzzer"
                            >
                                {buzzer ? DB_NAMES.ON(intl) : DB_NAMES.OFF(intl)}
                            </div>
                        </div>
                    )}
                    {isPinOn && !isNaN(delay) && showDelay && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.DELAY}
                            </div>
                            <div
                                className="Form-Value col-md-8 grey-text"
                                data-cy="trackable-details-delay"
                            >
                                {delay} s
                            </div>
                        </div>
                    )}

                    {angle && showAngle && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.DETAILS.FORM.ANGLE}
                            </div>
                            <div className="Form-Value col-md-8 grey-text">
                                {angle}°
                            </div>
                        </div>
                    )}
                    {noAlarmTrackableObjectNames && showsT2tNoAlarmUid && (
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-1" />
                            <div className="Form-Label col-md-3 grey-text">
                                {TRACKABLE_MODAL.DETAILS.FORM.T2T_NO_ALARM}
                            </div>
                            <div className="Form-Value col-md-8 grey-text">
                                {noAlarmTrackableObjectNames.join(', ')}
                            </div>
                        </div>
                    )}
                </Fragment>
                {custom_info
                && (
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">
                            {COMPONENTS.FIELDS.COMMON.CUSTOM_INFO}
                        </div>
                        <div className="Form-Value col-md-8">
                            <textarea value={custom_info} readOnly className="RPP-Group-advanced-TextArea" />
                        </div>
                    </div>
                )}
            </div>

            <div className="Form-Buttons OneBtn">
                <button type="button" data-id="trackable-details-close" onClick={close}>{COMPONENTS.BUTTONS.CLOSE}</button>
            </div>
        </form>
    );
};

TrackableDetailsForm.propTypes = {
    close: PropTypes.func,
    className: PropTypes.string,
    isPlaybackOpen: PropTypes.bool,
    isSitePageOpen: PropTypes.bool,
    maxHeight: PropTypes.string,
    params: PropTypes.shape({
        groupList: PropTypes.arrayOf(PropTypes.shape({
            name: PropTypes.string,
        })),
        node: PropTypes.shape({
            mac_address: PropTypes.string,
        }),
        attached_zone: PropTypes.object,
        buffer_radius: PropTypes.string,
        hwCbid: PropTypes.string,
        active_buffer: PropTypes.bool,
        bufferExceptionLists: PropTypes.arrayOf(PropTypes.shape({
            name: PropTypes.string,
        })),
        zones: PropTypes.arrayOf(PropTypes.shape({
            name: PropTypes.string,
        })),
        defaultHeight: PropTypes.number,
        type: PropTypes.string,
        category: PropTypes.string,
        position_update_interval: PropTypes.number,
        user_params: PropTypes.shape({}),
        sensorSpecialCases: PropTypes.arrayOf(PropTypes.shape({})),
    }),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    trackablesMap: PropTypes.shape({}),
    nodes: PropTypes.shape({}),
    trackableTemplates: PropTypes.arrayOf(PropTypes.shape({
        category: PropTypes.string,
        variables: PropTypes.shape({
            label: PropTypes.string,
            name: PropTypes.string,
        }),
    })),
};

TrackableDetailsForm.defaultProps = {
    close: () => {
    },
    className: '',
    isPlaybackOpen: false,
    isSitePageOpen: false,
    maxHeight: '60vh',
    params: {
        groupList: [],
        node: {
            mac_address: '',
        },
        attached_zone: {},
        buffer_radius: '',
        hwCbid: '',
        active_buffer: false,
        bufferExceptionLists: [],
        zones: [],
        defaultHeight: 1,
        type: '',
        category: '',
        position_update_interval: 0,
        user_params: {},
        sensorSpecialCases: [],
    },
    intl: {
        formatMessage: () => {
        },
    },
    trackablesMap: {},
    nodes: {},
    trackableTemplates: [],
};

TrackableDetailsForm = reduxForm({
    form: NAME_OF_FORMS.TRACKABLE_DETAILS,
})(injectIntl(TrackableDetailsForm));

class TrackableDetailsModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        isPlaybackOpen: PropTypes.bool,
        isSitePageOpen: PropTypes.bool,
        trackableDetails: PropTypes.shape({
            params: PropTypes.shape({
                uid: PropTypes.string,
                name: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        trackablesMap: PropTypes.shape({}),
        style: PropTypes.shape({}),
        trackableTemplates: PropTypes.arrayOf(PropTypes.shape({
            category: PropTypes.string,
            variables: PropTypes.shape({
                label: PropTypes.string,
                name: PropTypes.string,
            }),
        })),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        isPlaybackOpen: false,
        isSitePageOpen: false,
        trackableDetails: {
            params: {
                uid: '',
                name: '',
            },
            open: false,
        },
        trackablesMap: {},
        style: {},
        trackableTemplates: [],
    };
    
    close = () => {
        this.props.dispatch(closeTrackableDetailsModal());
    };
    
    render() {
        const { style, trackableDetails, isPlaybackOpen, isSitePageOpen, trackablesMap, nodes, trackableTemplates } = this.props;
        const style2 = makeCopy(style);
        style2.content.width = 750;
        style2.content.top = '50%';

        return (
            <Modal
                contentLabel="trackableDetails"
                isOpen={trackableDetails.open}
                onRequestClose={this.close}
                style={style2}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis">
                        {COMPONENTS.FIELDS.COMMON.VIEW_INFORMATION_FUNC(trackableDetails.params ? trackableDetails.params.name : '')}
                    </span>
                    <TrackableDetailsForm
                        className="Modal-Form"
                        close={this.close}
                        params={trackableDetails.params}
                        nodes={nodes}
                        trackableTemplates={trackableTemplates}
                        trackablesMap={trackablesMap}
                        isPlaybackOpen={isPlaybackOpen}
                        isSitePageOpen={isSitePageOpen}
                    />
                </div>
            </Modal>
        );
    }
}

/**
 * similar to parseTrackableEdit() in TrackableEditModal;
 */
const getTrackableDetails = (state) => state.modals.trackableModals.trackableDetails;
const getPlaybackState = (state) => state.playback.isOpenPlayBackControl;

const getParsedDetails = createSelector(
    [getTrackableDetails, Zone.getAllZones, Group.getAllGroups, Trackable.getAllTrackables, getPlaybackState],
    (trackableDetails, zones, groups, trackables, isPlaybackOpen) => {
        if (trackableDetails.params) {
            const trackable = new Trackable(trackableDetails.params);
            trackableDetails.params.isResponsive = trackable.isResponsive(isPlaybackOpen);

            const { mac_address } = trackableDetails.params.node ? trackableDetails.params.node : {};
            const linkedNode = mac_address ? Node.getNodeByMacAddress(mac_address) : null;
            trackableDetails.params.last_heard = linkedNode ? linkedNode.last_heard : 'UNAVAILABLE';

            trackableDetails.params.hwCbid = getNodeHwCbId(linkedNode);
            // assigning battery from linked node to trackable battery
            trackableDetails.params.battery = linkedNode ? linkedNode.battery : null;

            if (trackableDetails.params.groups) {
                trackableDetails.params.groupList = trackableDetails.params.groups.map(uid => groups[uid]);
            }
        
            if (trackableDetails.params.self_zone_uid) {
                const attachedZone = zones[trackableDetails.params.self_zone_uid];
                if (attachedZone) {
                    trackableDetails.params.attached_zone = attachedZone;
                    trackableDetails.params.buffer_radius = attachedZone.zone_geometry.radius;
                    trackableDetails.params.active_buffer = attachedZone.active;
                    trackableDetails.params.buffer_exceptions = [...attachedZone.zone_exceptions, ...attachedZone.zone_exceptions_groups];
                }
            }
        
            if (trackableDetails.params.buffer_exceptions) {
                trackableDetails.params.bufferExceptionLists = trackableDetails.params.buffer_exceptions.reduce((memo, uid) => {
                    const tko = trackables[uid];
                    if (tko) {
                        memo.push(Object.assign({}, tko, { value: tko.uid, label: tko.name }));
                    }
                    const group = groups[uid];
                    if (group) {
                        memo.push(Object.assign({}, group, { value: group.uid, label: group.name }));
                    }
                    return memo;
                }, []);
            }
            const t2t_no_alarm_uid = _Get(trackableDetails, 'params.t2t_no_alarm_uid') || '';
            if (t2t_no_alarm_uid.length) {
                trackableDetails.params.noAlarmTrackableObjectNames = t2t_no_alarm_uid.split(',').reduce((r, tko_uid) => {
                    const tko = trackables[tko_uid];
                    if (tko && tko.name) {
                        r.push(tko.name);
                    }
                    return r;
                }, []);
            }

            if (trackableDetails.params.default_height) {
                trackableDetails.params.defaultHeight = trackableDetails.params.default_height;
            } else {
                trackableDetails.params.defaultHeight = DEFAULT_VALUES.trackable_default_height;
            }
        }
        return trackableDetails;
    },
);

const mapStateToProps = (state) => ({
    isPlaybackOpen: !!(state.playback && state.playback.isOpenPlayBackControl),
    isSitePageOpen: state.router.location && state.router.location.pathname === ROUTES.SITE_ROUTE,
    nodes: state.entities.nodes.byMac,
    trackableDetails: getParsedDetails(state),
    trackableTemplates: state.entities.trackableTemplates,
    trackablesMap: state.entities.trackables.byUid,
});

export default injectIntl(connect(mapStateToProps)(TrackableDetailsModal));
