import { createSelector } from 'reselect';
import { IntlProvider } from 'react-intl';
import _Get from 'lodash/get';

import {
    DEFAULT_VALUES,
    TRACKABLE_CATEGORY, TRACKABLE_TYPE,
    ZONE_MODE, ZONE_APPLY_TO, STORAGE_KEY,
} from 'common/constants';
import { DB_NAMES } from 'common/ui-labels';
import RenderField from 'components/modals/RenderField';
import RenderPositionCoordinatesField from 'components/modals/RenderPositionCoordinatesField';
import { convertMillimetersToMeters } from 'utils/formatters';
import { parseDefaultAllowed } from 'utils/zoneUtils';
import { lengthToDegree } from 'utils/simpleUtils';
import {
    Location,
    Sublocation,
    Trackable,
    Group,
    Zone,
} from 'stores/classes';
import { Sublocation as SublocationSelector, Trackable as TrackableSelector } from 'stores/classes/Selectors';
import store from 'stores/LocationStore';

import { messages, language } from '../../../../../locales';

export const renderField = RenderField(4, 8);
export const renderFieldMoved = RenderField(5, 7);
export const renderReducedField = RenderField(4, 6);
export const renderPosField = RenderPositionCoordinatesField();

const { intl } = new IntlProvider({ locale: language, messages }, {}).getChildContext();

export const TYPES = [
    { value: TRACKABLE_TYPE.ASSET, label: DB_NAMES.ASSET(intl) },
    { value: TRACKABLE_TYPE.MOBILE, label: DB_NAMES.MOBILE(intl) },
    { value: TRACKABLE_TYPE.NAV, label: DB_NAMES.NAVIGATION(intl) },
];

export const CATEGORIES = [
    { value: TRACKABLE_CATEGORY.PERSON, label: DB_NAMES.PERSON(intl) },
    { value: TRACKABLE_CATEGORY.VEHICLE, label: DB_NAMES.VEHICLE(intl) },
    { value: TRACKABLE_CATEGORY.ASSET, label: DB_NAMES.ASSET(intl) },
    { value: TRACKABLE_CATEGORY.UNSPECIFIED, label: DB_NAMES.UNSPECIFIED(intl) },
];

const OTHER = type => ({
    none: { value: 0, label: DB_NAMES.NONE(intl), type },
    values: [
        { value: 0, label: DB_NAMES.NONE(intl), type },
        { value: 0.25, label: '0.25s', type },
        { value: 0.5, label: '0.5s', type },
        { value: 1, label: '1s', type },
        { value: 2, label: '2s', type },
        { value: 5, label: '5s', type },
        { value: 10, label: '10s', type },
    ],
    default: { value: 1, label: '1s', type },
});

export const getUpdateIntervalValues = (category) => {
    const assetValues = {
        none: {},
        values: [],
        default: { value: 5, label: '5s', type: TRACKABLE_TYPE.ASSET },
    };

    const otherValues = OTHER(TRACKABLE_TYPE.NAV);

    return (category === TRACKABLE_CATEGORY.ASSET) ? assetValues : otherValues;
};

export const ALL_TYPE_VALUES = getUpdateIntervalValues().values;

export const calculateAdvancedOptions = (params) => {
    if (params.default_height !== DEFAULT_VALUES.trackable_default_height) {
        return true;
    }
    // todo: advanced shows when any config exists
    if (params.sensorSpecialCases && Object.keys(params.sensorSpecialCases).length > 0) {
        return true;
    }
    if (!Number.isNaN(params.position_update_interval)) {
        const positionUpdateIntervalValue = _Get(params, 'position_update_interval.value') || params.position_update_interval;

        const positionUpdateIntervalDefault = getUpdateIntervalValues(params.category).default.value;

        return positionUpdateIntervalValue !== positionUpdateIntervalDefault;
    }
    return false;
};

const getTrackableEdit = state => state.modals.trackableModals.trackableEdit;

function getSelectedLocationSublocation(locations) {
    const selectedLocation = store.getFromLocalStorage(STORAGE_KEY.LAST_LOCATION);
    const selectedSublocation = store.getFromLocalStorage(STORAGE_KEY.LAST_SUBLOCATION);
    if (!selectedLocation && !selectedSublocation) {
        const selectedLocationFromState = locations[0];
        const selectedSublocationFromState = SublocationSelector.getSublocationsByLocationUid(selectedLocationFromState ? selectedLocationFromState.uid : '');
        return { selectedLocation: selectedLocationFromState, selectedSublocation: selectedSublocationFromState[0] };
    }
    return { selectedLocation, selectedSublocation };
}

export const getParsedTrackable = createSelector(
    [getTrackableEdit, Zone.getAllZonesArray, Group.getAllGroups, Trackable.getAllTrackables, Location.getAllLocations, Sublocation.getAllSublocations],
    (trackableEdit, zones, groups, trackables, locations) => {
        if (trackableEdit.params) {
            trackableEdit.params.buffer_radius = DEFAULT_VALUES.trackable_buffer_radius;

            const role_tko = _Get(trackableEdit, 'params.low_latency_link_config.role');
            const safe_distance_tko = _Get(trackableEdit, 'params.low_latency_link_config.safe_distance');
            const distance_tko = _Get(trackableEdit, 'params.low_latency_link_config.distance');
            const alert_distance_tko = _Get(trackableEdit, 'params.low_latency_link_config.alert_distance');
            const delay_tko = _Get(trackableEdit, 'params.low_latency_link_config.delay');
            const haptic_tko = _Get(trackableEdit, 'params.low_latency_link_config.haptic');
            const buzzer_tko = _Get(trackableEdit, 'params.low_latency_link_config.buzzer');
            const angle_tko = _Get(trackableEdit, 'params.low_latency_link_config.angle');
            const pin_tko = _Get(trackableEdit, 'params.low_latency_link_config.pin');
            const alarm_back_tko = _Get(trackableEdit, 'params.low_latency_link_config.alarm_back');
            if (trackableEdit.params.groups) {
                const groupList = trackableEdit.params.groups.reduce((r, uid) => {
                    if (groups[uid]) { r.push(groups[uid].addValueLabelToValue()); }
                    return r;
                }, []);
                trackableEdit.params.groupList = groupList;
            }
            trackableEdit.params.role = role_tko || '';
            trackableEdit.params.safe_distance = safe_distance_tko || DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE;
            trackableEdit.params.distance = distance_tko || DEFAULT_VALUES.LLL_COVERAGE_DISTANCE;
            trackableEdit.params.alert_distance = alert_distance_tko || DEFAULT_VALUES.LLL_COVERAGE_ALERT_DISTANCE;
            trackableEdit.params.delay = delay_tko || DEFAULT_VALUES.LLL_DELAY;
            trackableEdit.params.haptic = haptic_tko || false;
            trackableEdit.params.buzzer = buzzer_tko || false;
            trackableEdit.params.angle = angle_tko || DEFAULT_VALUES.LLL_COVERAGE_ANGLE;
            trackableEdit.params.pin = pin_tko || DEFAULT_VALUES.LLL_TRACKABLE_PIN;
            trackableEdit.params.alarm_back = alarm_back_tko || false;

            if (trackableEdit.params.self_zone_uid) {
                // buffer_radius zone radius
                // active_buffer make zone active
                // buffer_exceptions  trackable objects witch can move in attached zone without alarms
                const attachedZone = zones.find(zone => zone.isZoneUidEqualTo(trackableEdit.params.self_zone_uid));
                if (attachedZone) {
                    trackableEdit.params.attached_zone = attachedZone;
                    trackableEdit.params.buffer_radius = attachedZone.zone_geometry.radius;
                    trackableEdit.params.active_buffer = attachedZone.zone_mode === ZONE_MODE.ALWAYS_ACTIVE;
                    trackableEdit.params.buffer_exceptions = attachedZone.zone_exceptions;
                    trackableEdit.params.buffer_exceptions_groups = attachedZone.zone_exceptions_groups;
                }
            }
            
            if (trackableEdit.params.buffer_exceptions || trackableEdit.params.buffer_exceptions_groups) {
                const tkoExceptions = trackableEdit.params.buffer_exceptions.reduce((result, uid) => {
                    const trackable = trackables[uid];
                    if (trackable && trackable instanceof Trackable) {
                        result.push(trackable.addValueLabelToValue());
                    } else {
                        console.error('No trackable or trackable not an class instance for buffer exceptions in edit modal');
                    }
                    return result;
                }, []);
                const groupExceptions = trackableEdit.params.buffer_exceptions_groups.reduce((result, uid) => {
                    const group = groups[uid];
                    if (group && group instanceof Group) {
                        result.push(group.addValueLabelToValue());
                    } else {
                        console.error('No group or group not an class instance for buffer exceptions in edit modal');
                    }
                    return result;
                }, []);
                trackableEdit.params.trackables_and_groups = [...tkoExceptions, ...groupExceptions];
                trackableEdit.params.apply_to = parseDefaultAllowed(trackableEdit.params.attached_zone);
            } else {
                trackableEdit.params.apply_to = ZONE_APPLY_TO.ALL;
            }
            
            if (trackableEdit.params.default_height) {
                trackableEdit.params.defaultHeight = trackableEdit.params.default_height;
            }
            if (!trackableEdit.params.type) {
                const trackableType = TYPES.find(type => type.value === TRACKABLE_TYPE.NAV);
                trackableEdit.params.type = trackableType.value ? trackableType.value : trackableType;
                // todo: while removing type move position_update_interval to category
                trackableEdit.params.position_update_interval = getUpdateIntervalValues(trackableEdit.params.category).default;
            }
            if (!trackableEdit.params.category) {
                const trackableCategory = CATEGORIES.find(category => category.value === TRACKABLE_CATEGORY.UNSPECIFIED);
                trackableEdit.params.category = trackableCategory.value ? trackableCategory.value : trackableCategory;
            }
            const checkboxes = {
                buffer: !!((trackableEdit.params.buffer_radius && trackableEdit.params.buffer_radius !== 5)
                    || trackableEdit.params.active_buffer
                    || (trackableEdit.params.buffer_exceptions && trackableEdit.params.buffer_exceptions.length > 0)),
                // zone_exceptions: (trackableEdit.params.zones_with_exceptions && trackableEdit.params.zones_with_exceptions.length > 0),
                advanced_options: calculateAdvancedOptions(trackableEdit.params),
                stationary: trackableEdit.params.stationary,
            };
            trackableEdit.params.buffer = trackableEdit.params.attached_zone || checkboxes.buffer;
            trackableEdit.params.advanced_options = checkboxes.advanced_options;
            if (trackableEdit.params.stationary) {
                const { position } = TrackableSelector.getTrackableByUid(trackableEdit.params.uid);
                const { isStationaryInitialized } = trackableEdit.params;
                const isPositionValid = position && !Number.isNaN(position.x) && !Number.isNaN(position.y) && !Number.isNaN(position.z);

                if (!isStationaryInitialized && isPositionValid && position.sl_id) {
                    const sublocation = SublocationSelector.getSublocationByUid(position.sl_id);
                    trackableEdit.params.pos_x = convertMillimetersToMeters(position.x - sublocation.offset_x);
                    trackableEdit.params.pos_y = convertMillimetersToMeters(position.y - sublocation.offset_y);
                    trackableEdit.params.defaultHeight = convertMillimetersToMeters(position.z);
                    trackableEdit.params.site = sublocation && sublocation.location_uid;
                    trackableEdit.params.area = position.sl_id;
                    trackableEdit.stationaryPosition = {
                        lng: Number.isNaN(trackableEdit.params.pos_x) ? 0 : lengthToDegree(trackableEdit.params.pos_x) * 10,
                        lat: Number.isNaN(trackableEdit.params.pos_y) ? 0 : lengthToDegree(trackableEdit.params.pos_y) * 10,
                    };
                    trackableEdit.params.isStationaryInitialized = true;
                }
            }
            
            const { selectedLocation, selectedSublocation } = getSelectedLocationSublocation(locations);
            if (!trackableEdit.params.site) {
                if (selectedLocation && selectedLocation.uid) {
                    trackableEdit.params.site = selectedLocation.uid;
                }
            }
            if (!trackableEdit.params.area) {
                if (selectedSublocation && selectedSublocation.uid) {
                    trackableEdit.params.area = selectedSublocation.uid;
                }
            }
        }

        return trackableEdit;
    },
);


/* private */
const _calculateAdvancedOptionsSelected = (checkboxes, formValues) => {
    let isAdvancedOptions;
    if (checkboxes.advanced_options === undefined) {
        isAdvancedOptions = formValues ? !!formValues.advanced_options : false;
    } else if (!checkboxes.advanced_options) {
        let calculatedAdvancedOptions;
        if (formValues && !formValues.advanced_options) {
            calculatedAdvancedOptions = calculateAdvancedOptions(formValues);
        }
        isAdvancedOptions = formValues ? formValues.advanced_options : !!calculatedAdvancedOptions;
    } else {
        isAdvancedOptions = true;
    }

    const isRoleSelected = !!formValues.role;
    const isStationary = formValues.stationary;

    return isAdvancedOptions || isRoleSelected || isStationary;
};

/* private */
const _calculateBufferSelected = (checkboxes, formValues) => {
    let isBufferSelected;
    if (checkboxes.buffer === undefined) {
        isBufferSelected = formValues ? !!formValues.buffer : false;
    } else if (!checkboxes.buffer) {
        let calculatedBuffer;
        if (formValues && !formValues.buffer) {
            calculatedBuffer = (formValues.buffer_radius && formValues.buffer_radius !== 5)
                || formValues.active_buffer
                || (formValues.buffer_exceptions && formValues.zones && formValues.zones.length > 0);
        }
        isBufferSelected = formValues ? formValues.buffer : !!calculatedBuffer;
    } else {
        isBufferSelected = true;
    }
    return isBufferSelected;
};

/* private */
const _calculateStationarySelected = (checkboxes, formValues) => {
    let isStationarySelected;
    if (checkboxes.stationary === undefined) {
        isStationarySelected = formValues ? !!formValues.stationary : false;
    } else if (!checkboxes.stationary) {
        isStationarySelected = formValues ? formValues.stationary : false;
    } else {
        isStationarySelected = true;
    }
    return isStationarySelected;
};

export const isSelectedFields = (checkboxes, formValues) => {
    const isBufferSelected = _calculateBufferSelected(checkboxes, formValues);
    const isStationary = _calculateStationarySelected(checkboxes, formValues);
    const isAdvancedOptions = _calculateAdvancedOptionsSelected(checkboxes, formValues);

    return {
        isBufferSelected,
        isAdvancedOptions,
        isStationary,
    };
};

export const getNoneOptionForUpdateInterval = category => getUpdateIntervalValues(category).none;

export const getFloorplansBySublocationUid = (sublocation_uid, floorplans) => floorplans.filter(floorplan => floorplan.sublocation_uid === sublocation_uid);
