import React from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { reduxForm } from 'redux-form';
import { NAME_OF_FORMS } from 'common/constants';
import { USER_DELETE_MODAL } from 'common/ui-labels';
import { closeUserDeleteModal, deleteUser } from 'stores/actions';
import store from 'stores/LocationStore';
import { userHasPrivileges } from 'utils/utils';
import DeleteButtons from 'components/modals/DeleteButtons';


const isValidRemove = (currentUser, usersToRemove) => {
    if (Array.isArray(usersToRemove)) {
        let isValid = true;
        usersToRemove.forEach((user) => {
            if (user.email === currentUser.email || !userHasPrivileges(currentUser.access.currentRole, user.role)) {
                isValid = false;
            }
        });
        return isValid;
    } else if (currentUser.email === usersToRemove.email) {
        return false;
    }
    return userHasPrivileges(currentUser.access.currentRole, usersToRemove.role);
};

let UserDeleteForm = (props) => {
    const { close, submit, className, message, initialValues, submitted } = props;
    const currentUser = store.getCurrentUser();
    const isBatchDelete = Array.isArray(initialValues);

    return (
        <form className={className}>
            {
                isBatchDelete && <div className="Form-Confirmation info-text" data-id="user-delete-message">
                    { USER_DELETE_MODAL.NUMBER_OF_USERS_SELECTED(<b>{initialValues.length}</b>) }
                </div>
            }
            <div className="Form-DeleteTitle confirmation" data-id="user-delete-confirmation">
                { message }
            </div>
    
            <DeleteButtons submit={submit} close={close} submitted={submitted} disabled={!isValidRemove(currentUser, initialValues)} dataId="user" />
        </form>
    );
};
UserDeleteForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    initialValues: PropTypes.oneOfType([
        PropTypes.shape({
            uid: PropTypes.string,
            name: PropTypes.string,
        }),
        PropTypes.array,
    ]),
    className: PropTypes.string,
    submitted: PropTypes.bool,
    message: PropTypes.oneOfType([
        PropTypes.shape(),
        PropTypes.string,
    ]),
};

UserDeleteForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    initialValues: {
        uid: '',
        name: '',
    },
    className: '',
    submitted: false,
    message: '',
};

UserDeleteForm = reduxForm({
    form: NAME_OF_FORMS.USER_DELETE,
})(UserDeleteForm);

const mapStateToForm = state => ({
    initialValues: state.modals.userModals.userDelete.params,
});
UserDeleteForm = connect(mapStateToForm)(UserDeleteForm);


class UserDeleteModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        userDelete: PropTypes.shape({
            params: PropTypes.oneOfType([
                PropTypes.shape({
                    uid: PropTypes.string,
                    name: PropTypes.string,
                }),
                PropTypes.arrayOf(PropTypes.shape({
                    uid: PropTypes.string,
                })),
            ]),
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({}),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        userDelete: {
            params: {
                uid: '',
                name: '',
            },
            open: false,
        },
        style: {},
    };
    
    constructor(props) {
        super(props);
        this.state = {
            submitted: false,
        };
    }
    
    close = () => {
        this.props.dispatch(closeUserDeleteModal());
    };
    
    submit = () => {
        this.setState({ submitted: true });
        this.props.dispatch(deleteUser(this.props.userDelete.params));
        this.props.dispatch(closeUserDeleteModal());
    };

    message = () => {
        let message = USER_DELETE_MODAL.TITLE;

        if (this.props.userDelete.params && !this.props.userDelete.params.uid) {
            message = USER_DELETE_MODAL.MESSAGE.DELETE_SELECTED_USERS;
        } else if (this.props.userDelete.params && this.props.userDelete.params.name) {
            message = USER_DELETE_MODAL.MESSAGE.DELETE_USER_FUNC(this.props.userDelete.params.name);
        }
        const currentUser = store.getCurrentUser();
        const removedUser = this.props.userDelete.params;

        if (removedUser && removedUser.uid) { // Is single user delete
            if (currentUser.email === removedUser.email) {
                message = USER_DELETE_MODAL.MESSAGE.CANNOT_DELETE_CURRENT_USER;
            } else if (!isValidRemove(currentUser, removedUser)) {
                message = USER_DELETE_MODAL.MESSAGE.HAVE_NO_PERMISSIONS;
            }
        } else if (Array.isArray(removedUser)) { // Is batch users delete
            removedUser.forEach((user) => {
                if (user.email === currentUser.email) {
                    message = USER_DELETE_MODAL.MESSAGE.CANNOT_DELETE_CURRENT_USER;
                } else if (!userHasPrivileges(currentUser.access.currentRole, user.role)) {
                    message = USER_DELETE_MODAL.MESSAGE.HAVE_NO_PERMISSIONS;
                }
            });
        }
        return message;
    }
    
    render() {
        const { style } = this.props;
        const message = this.message();

        return (
            <Modal
                contentLabel="userDelete"
                isOpen={this.props.userDelete.open}
                onRequestClose={this.close}
                shouldCloseOnOverlayClick={false}
                onAfterClose={() => {
                    this.setState({ submitted: false });
                }}
                style={style}
            >
                <div className="RPP-Modal">
                    <UserDeleteForm
                        className="Modal-Form"
                        submitted={this.state.submitted}
                        close={this.close}
                        onSubmit={this.submit}
                        message={message}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    userDelete: state.modals.userModals.userDelete,
});

export default connect(mapStateToProps)(UserDeleteModal);
