import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import * as api from 'api';
import RenderField from 'components/modals/RenderField';
import RenderSelectField from 'components/modals/RenderSelectField';
import RenderDebouncedInput from 'components/modals/RenderDebouncedInput';
import { ProjectUserRoleSelectOptions } from 'components/ProjectUserRoleSelect';
import store from 'stores/LocationStore';
import { USER_EDIT_MODAL, NAME_OF_FORMS, EMAIL_REGEXP, USER_ROLES, COMPONENTS } from 'common';
import { sortByField, userHasPrivileges, getRolesToCreateInProject, getRolesToCreateInCompany } from 'utils/utils';
import { makeCopy } from 'utils/simpleUtils';
import { closeUserEditModal, editUser, createUser, restoreUserPassword } from 'stores/actions';
import { User } from 'stores/classes';

import { getValidatePasswordError } from './utils';

const companyUsersEmails = () => {
    const emails = [];
    store.getCompanyUsers().forEach((user) => {
        emails.push(user.email);
    });
    return emails;
};

const userValidation = (email) => api.validateUser({ email });

const asyncValidate = (values, func, props) => userValidation(values.email).then((result) => {
    if (!result.valid && values.uid !== result.uid) {
        // eslint-disable-next-line
        throw { email: USER_EDIT_MODAL.VALIDATION.USER_EMAIL_EXIST(props.intl) };
    }
});

const validate = (values, props) => {
    let editContext = false;
    if (values.created_at) {
        editContext = true;
    }
    const errors = {};
    if (!values.email) {
        errors.email = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    } else if (!EMAIL_REGEXP.test(values.email)) {
        errors.email = USER_EDIT_MODAL.VALIDATION.USER_EMAIL_INVALID(props.intl);
    } else if (!editContext
        && (companyUsersEmails().indexOf(values.email) !== -1)
        && (props.userEdit.params.email !== values.email)
    ) {
        errors.email = USER_EDIT_MODAL.VALIDATION.USER_EMAIL_EXIST(props.intl);
    }
    if (!values.first_name) {
        errors.first_name = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    if (!values.last_name) {
        errors.last_name = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    if (!values.project_uid && values.role !== USER_ROLES.COMPANY_ADMIN) {
        errors.project_uid = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    if (!values.role) {
        errors.role = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    if (!editContext) {
        const passwordError = getValidatePasswordError(values.password, props.intl);
        if (passwordError) {
            errors.password = passwordError;
        }
    }
    return errors;
};

const renderField = RenderField(3, 8);
const renderDebouncedInput = RenderDebouncedInput(3, 8);

const showPassword = (show) => (show ? 'text' : 'password');

const isSelectedFields = (formValues) => {
    return {
        isUserTypeSelected: !!formValues?.role,
        isProjectSelected: !!formValues?.role && !!formValues?.project_uid,
    };
};

let UserEditForm = (props) => {
    const { close, submit, className, formState, invalid, restorePassword, intl, projects, isProjectScope, asyncValidating } = props;
    const isEditMode = !!(props.userEdit.params && props.userEdit.params.uid);
    const editUserFormState = formState[NAME_OF_FORMS.USER_EDIT] || {};

    const { isUserTypeSelected } = isSelectedFields(editUserFormState.values);
    let { isProjectSelected } = isSelectedFields(editUserFormState.values);
    
    let wasSendNotification = props.sendNotification;
    let sendNotification = true;

    let isPasswordShow = false;
    
    let isCompanyAdminSelected = false;
    let isCurrentUserSelected = false;
    let isUserHasRights = false;

    if (editUserFormState && editUserFormState.values) {
        isCompanyAdminSelected = editUserFormState.values.role === USER_ROLES.COMPANY_ADMIN;
        isProjectSelected = isCompanyAdminSelected ? true : isProjectSelected;
        isPasswordShow = !!editUserFormState.values.showPassword;
        sendNotification = editUserFormState.values.sendNotification;
        isCurrentUserSelected = editUserFormState.values.email === props.currentUserEmail;
        isUserHasRights = userHasPrivileges(props.currentUserRole, editUserFormState.values.role);
    }
    const availableRoles = isProjectScope ? getRolesToCreateInProject(props.currentUserRole) : getRolesToCreateInCompany();

    return (
        <form className={className}>
            <div className="Form-Line row">
                <div htmlFor="name" className="Form-Label col-md-3">{COMPONENTS.FIELDS.USER.USER_TYPE}</div>
                <div className="col-md-8">
                    {!isProjectScope &&
                    <Field
                        className="form-control input-sm "
                        name="role"
                        dataId="user-edit-user-type"
                        style={{ fontSize: '14px' }}
                        props={{ disabled: isCurrentUserSelected }}
                        component={RenderSelectField}
                    >
                        <option />
                        <ProjectUserRoleSelectOptions
                            intl={intl}
                            roles={availableRoles}
                        />
                    </Field>}
                    
                    {isProjectScope &&
                    <Field
                        className="form-control input-sm "
                        name="role"
                        dataId="user-edit-user-type"
                        style={{ fontSize: '14px' }}
                        props={{ disabled: isCurrentUserSelected || (!isUserHasRights && isEditMode) }}
                        component={RenderSelectField}
                    >
                        <option />
                        <ProjectUserRoleSelectOptions
                            intl={intl}
                            roles={availableRoles}
                        />
                    </Field>}
                </div>
            </div>
            
            {!isProjectScope &&
            <div className="Form-Line row">
                <div htmlFor="project" className="Form-Label col-md-3">{COMPONENTS.FIELDS.COMMON.PROJECT}</div>
                <div className="col-md-8">
                    <Field
                        className="form-control input-sm"
                        name="project_uid"
                        dataId="user-edit-project-name"
                        component={RenderSelectField}
                        style={{ fontSize: '14px' }}
                        props={{ disabled: isCompanyAdminSelected || !isUserTypeSelected || !isUserHasRights }}
                    >
                        <option />
                        {projects.map(project => (
                            <option value={project.uid} key={project.uid}>{project.name}</option>
                        ))}
                    </Field>
                </div>
            </div>}
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="email"
                    dataId="user-edit-email"
                    component={renderDebouncedInput}
                    label={`${COMPONENTS.FIELDS.USER.EMAIL_FUNC(intl)} *`}
                    props={{ disabled: !isProjectSelected && (!props.isProjectScope || !isUserTypeSelected || !isUserHasRights) }}
                    type="email"
                />
            </div>
            
            {!isEditMode &&
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="password"
                    dataId="user-edit-password"
                    component={renderField}
                    masked={false}
                    label={`${COMPONENTS.FIELDS.USER.PASSWORD_FUNC(intl)} *`}
                    props={{ disabled: !isProjectSelected && (!props.isProjectScope || !isUserTypeSelected) }}
                    type={showPassword(isPasswordShow)}
                />
            </div>}
            
            {!isEditMode &&
            <div className="Form-Line row">
                <div className="col-md-3" />
                <div className="col-md-1 col-sm-1" data-id="user-edit-show-password">
                    <Field
                        name="showPassword"
                        id="password"
                        component="input"
                        type="checkbox"
                        style={{ marginTop: '0px' }}
                    />
                </div>
                <label className="no-select" htmlFor="password">
                    {COMPONENTS.FIELDS.USER.SHOW_PASSWORD}
                </label>
            </div>}
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="first_name"
                    dataId="user-edit-first-name"
                    component={renderField}
                    label={COMPONENTS.FIELDS.USER.FIRST_NAME_FUNC(intl)}
                    props={{ disabled: !isProjectSelected && (!props.isProjectScope || !isUserTypeSelected || !isUserHasRights) }}
                    type="text"
                />
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="last_name"
                    dataId="user-edit-last-name"
                    component={renderField}
                    label={COMPONENTS.FIELDS.USER.LAST_NAME_FUNC(intl)}
                    props={{ disabled: !isProjectSelected && (!props.isProjectScope || !isUserTypeSelected || !isUserHasRights) }}
                    type="text"
                />
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="phone_number"
                    dataId="user-edit-phone"
                    component={renderField}
                    label={COMPONENTS.FIELDS.USER.PHONE_NUMBER_FUNC(intl)}
                    props={{ disabled: !isProjectSelected && (!props.isProjectScope || !isUserTypeSelected || !isUserHasRights) }}
                    type="text"
                />
            </div>

            {!isEditMode &&
            <div className="Form-Line row">
                <div className="col-md-3" />
                <div className="col-md-1 col-sm-1" data-id="user-edit-send-notification">
                    <Field
                        name="send_notification"
                        id="send_notification"
                        component="input"
                        type="checkbox"
                        checked={sendNotification}
                        style={{ marginTop: 0 }}
                    />
                </div>
                <label className="no-select" htmlFor="send_notification">
                    {USER_EDIT_MODAL.NOTIFICATION_SEND}
                </label>
            </div>}
            
            <div className="Form-Buttons">
                {isEditMode &&
                    <button
                        type="button"
                        onClick={() => {
                            wasSendNotification = true;
                            restorePassword(props.userEdit.params.email);
                        }}
                        disabled={!isUserHasRights}
                        data-id="user-edit-reset-password"
                        className="Form-Reset-Button"
                    >
                        {COMPONENTS.FIELDS.USER.RESET_PASSWORD}
                    </button>
                }
                <button
                    type="button"
                    data-id="user-edit-save"
                    onClick={submit}
                    disabled={invalid || !isUserHasRights || asyncValidating}
                >
                    {asyncValidating && <i className="fa fa-spinner fa-spin" style={{ color: 'grey' }} />}
                    &nbsp;
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button type="button" data-id="user-edit-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
            <div>
                {wasSendNotification && <span className="Notification">{USER_EDIT_MODAL.NOTIFICATION_WAS_SENT}</span>}
            </div>
        </form>
    );
};

UserEditForm.propTypes = {
    userEdit: PropTypes.shape({
        params: PropTypes.shape({
            uid: PropTypes.string,
            email: PropTypes.string,
        }),
    }),
    restorePassword: PropTypes.func,
    currentUserEmail: PropTypes.string,
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    isProjectScope: PropTypes.bool,
    currentUserRole: PropTypes.string,
    sendNotification: PropTypes.bool,
    invalid: PropTypes.bool,
    projects: PropTypes.arrayOf(PropTypes.shape({})),
    formState: PropTypes.shape({}),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    asyncValidating: PropTypes.bool,
};

UserEditForm.defaultProps = {
    userEdit: {
        params: {
            uid: '',
        },
    },
    restorePassword: () => {
    },
    close: () => {
    },
    submit: () => {
    },
    currentUserEmail: '',
    isProjectScope: false,
    invalid: false,
    currentUserRole: '',
    className: '',
    projects: [],
    sendNotification: false,
    formState: {},
    intl: {
        formatMessage: () => {
        },
    },
    asyncValidating: false,
};

UserEditForm = reduxForm({
    form: NAME_OF_FORMS.USER_EDIT,
    validate,
    asyncValidate,
    asyncChangeFields: ['email'],
    shouldAsyncValidate: (params) => {
        if (!params.syncValidationPasses) {
            return false;
        }
        return params.trigger === 'change';
    },
})(injectIntl(UserEditForm));

const mapStateToForm = (state) => {
    const initialValues = state.modals.userModals.userEdit.params;

    return {
        initialValues,
        projects: sortByField(state.entities.projects, 'name'),
        formState: state.form,
        users: User.getAllUsers(state),
        userEdit: state.modals.userModals.userEdit,
        sendNotification: state.modals.userModals.userEdit.sendNotification,
        isProjectScope: !!state.appContext.project,
        currentUserRole: state.user.access.currentRole,
        currentUserEmail: state.user.email,
    };
};

UserEditForm = connect(mapStateToForm)(injectIntl(UserEditForm));

class UserEditModal extends React.Component {
    close = () => {
        this.props.dispatch(closeUserEditModal());
    };
    
    submit = (values) => {
        // set new email of current user to session store
        if (store.getCurrentUser().email === this.props.userEdit.params.email) {
            store.setUserEmail(values.email);
            values.currentUser = true;
        }
        if (values.role && values.role === USER_ROLES.COMPANY_ADMIN) {
            delete values.project_uid;
        }
        if (this.props.userEdit.params && this.props.userEdit.params.uid) {
            this.props.dispatch(editUser(values));
        } else {
            if (this.props.isProjectScope && !values.project_uid) { // create user in project scope with out project selecting
                values.project_uid = this.props.currentProject.uid;
            }
            this.props.dispatch(createUser(values));
        }
        this.props.dispatch(closeUserEditModal());
    };
    
    restorePassword = (email) => {
        this.props.dispatch(restoreUserPassword(email));
    };
    
    render() {
        let title = USER_EDIT_MODAL.TITLE_ADD;
        if (this.props.userEdit.params && this.props.userEdit.params.uid) {
            title = this.props.userEdit.params.name;
        }
        const { style } = this.props;
        const currentStyle = makeCopy(style);
        currentStyle.content.width = '600px';

        return (
            <Modal
                contentLabel="userEdit"
                isOpen={this.props.userEdit.open}
                onRequestClose={this.close}
                style={currentStyle}
            >
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis" data-id="user-edit-title">
                        {title}
                    </span>
                    <UserEditForm
                        className="Modal-Form"
                        close={this.close}
                        restorePassword={this.restorePassword}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

UserEditModal.propTypes = {
    dispatch: PropTypes.func,
    isProjectScope: PropTypes.bool,
    currentProject: PropTypes.shape({}),
    userEdit: PropTypes.shape({
        params: PropTypes.shape({
            email: PropTypes.string,
            name: PropTypes.string,
            uid: PropTypes.string,
        }),
    }),
    style: PropTypes.shape({}),
};
UserEditModal.defaultProps = {
    dispatch: () => {},
    isProjectScope: false,
    currentProject: {},
    userEdit: {
        params: {},
    },
    style: {},
};

const mapStateToProps = (state) => ({
    userEdit: state.modals.userModals.userEdit,
    isProjectScope: !!state.appContext.project,
    currentProject: state.appContext.project,
});

export default connect(mapStateToProps)(UserEditModal);
