import React from 'react';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import SubmitButtons from 'components/modals/forms/SubmitButtons';
import { DEFAULT_VALUES, ERROR_TEXT, NAME_OF_FORMS } from 'common';
import { COMPONENTS, COMMON_LABELS } from 'common/ui-labels';
import { getCronFromToStringValues, getInclusionsExclusionsObjects, getLength, onMouseDownScrollPrevent } from 'utils';
import { getParsedGroups, getParsedTrackables, getParsedStaticZones } from 'stores/getters';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import ActivityScheduleForm from './ActivityScheduleForm';
import { ActivityForm } from './ActivityForm';
import { CustomInfoForm } from './CustomInfoForm';
import { TrackablesGroupsForm } from './TrackablesGroupsForm';


export const validate = (values, props) => {
    const errors = {};
    const resultError = COMMON_LABELS.INVALID_TIME_FUNC(props.intl);
    if (values) {
        const { fromHour, fromMinute, toHour, toMinute, bulk_zones } = values;

        const validateHour = hour => Number(hour) > 23 || Number(hour) < 0;
        const validateMinute = minute => Number(minute) > 59 || Number(minute) < 0;
        const isMidnight = (hour, minute) => Number(hour) === 24 && Number(minute) === 0;

        if (!isMidnight(fromHour, fromMinute)) {
            if (validateHour(fromHour)) {
                errors.fromHour = resultError;
            }
            if (validateMinute(fromMinute)) {
                errors.fromMinute = resultError;
            }
        }
        if (!isMidnight(toHour, toMinute)) {
            if (validateHour(toHour)) {
                errors.toHour = resultError;
            }
            if (validateMinute(toMinute)) {
                errors.toMinute = resultError;
            }
        }
        if (!bulk_zones || bulk_zones.length < 2) {
            errors.bulk_zones = COMMON_LABELS.REQUIRED_SELECT_MULTIPLE_ZONES_FUNC(props.intl);
        }
    }

    if (!values || errors.fromHour || errors.fromMinute || errors.toHour || errors.toMinute) {
        errors.activePeriod = { _error: resultError };
    }

    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }

    if (!values.status_option && !values.custom_info_option && !values.trackables_groups_option && !values.activity_schedule_option) {
        errors.status_option = 'Nothing to change';// not required intl because it is not rendering
    }

    return errors;
};

let ZoneBulkEditSitePageForm = (props) => {
    const { close, submit, className, initialValues, formState, intl, trackables, groups, zones, invalid, maxHeight, dispatch, inProcess } = props;
    initialValues.default_allowed += '';
    const editZoneFormState = formState[NAME_OF_FORMS.ZONE_BULK_EDIT_ADMIN_PAGE] || {};
    const { status, status_option, active24, custom_info, activity_schedule_option, custom_info_option, trackables_groups_option } = editZoneFormState.values || {};

    return (
        <form className={className}>
            <div style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight, paddingRight: '10px' }} onMouseDown={onMouseDownScrollPrevent}>
                <div className="Form-Line row" style={{ textAlign: 'center', marginTop: '15px' }}>
                    <div className="Form-Label col-md-6" style={{ fontSize: '1.2em' }}>{COMMON_LABELS.ZONES}*</div>
                    <div className="col-md-6 text-left" data-id="zone-add-configuration-active">
                        <Field
                            name="bulk_zones"
                            type="text"
                            label="Objective"
                            placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                            noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                            options={zones}
                            scrollMenuIntoView
                            component={MultiSelectComponent}
                        />
                    </div>
                </div>

                <ActivityForm
                    dispatch={dispatch}
                    status_option={status_option}
                    intl={intl}
                />

                <CustomInfoForm
                    intl={intl}
                    custom_info={custom_info}
                    custom_info_option={custom_info_option}
                    dispatch={dispatch}
                />

                <TrackablesGroupsForm
                    intl={intl}
                    trackables={trackables}
                    groups={groups}
                    trackables_groups_option={trackables_groups_option}
                    dispatch={dispatch}
                />

                <ActivityScheduleForm
                    intl={intl}
                    disabled={!status_option || status !== 'active'}
                    dispatch={dispatch}
                    active24={active24}
                    activity_schedule_option={activity_schedule_option}
                />
            </div>

            <SubmitButtons
                className="Form-Buttons"
                submit={submit}
                invalid={invalid}
                close={close}
                inProcess={inProcess}
                saveDataId="zone-bulk-edit-save"
                cancelDataId="zone-bulk-edit-cancel"
            />
        </form>
    );
};

ZoneBulkEditSitePageForm = reduxForm({
    form: NAME_OF_FORMS.ZONE_BULK_EDIT_ADMIN_PAGE,
    validate,
})(ZoneBulkEditSitePageForm);

const mapStateToForm = (state) => {
    const modalParams = state.modals.zoneModals.zoneEditBulkAdminPage.params;
    const trackables = getParsedTrackables(state);
    const groups = getParsedGroups(state);
    const trackables_and_groups = getInclusionsExclusionsObjects([], [], trackables, groups);
    const bulk_zones = modalParams ? modalParams.map(item => ({ value: item.uid, label: item.name })) : [];
    const { toHour, toMinute, fromHour, fromMinute } = getCronFromToStringValues({});
    
    const initialValues = {
        toHour,
        toMinute,
        fromHour,
        fromMinute,
        bulk_zones,
        trackables_and_groups,
        status_option: false,
        custom_info_option: false,
        trackables_groups_option: false,
        activity_schedule_option: false,
    };
    
    return {
        groups,
        trackables,
        initialValues,
        inProcess: state.modals.zoneModals.zoneEditBulkAdminPage.inProcess,
        formState: state.form,
        appContext: state.appContext,
        zones: getParsedStaticZones(state),
    };
};

ZoneBulkEditSitePageForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    inProcess: PropTypes.bool,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    className: PropTypes.string,
    formState: PropTypes.shape({}),
    groups: PropTypes.arrayOf(PropTypes.shape({})),
    trackables: PropTypes.arrayOf(PropTypes.shape({})),
    zones: PropTypes.arrayOf(PropTypes.shape({})),
    initialValues: PropTypes.shape({}),
    maxHeight: PropTypes.string,
};

ZoneBulkEditSitePageForm.defaultProps = {
    intl: {
        formatMessage: () => {
        },
    },
    inProcess: false,
    invalid: false,
    className: '',
    formState: {},
    groups: [],
    trackables: [],
    zones: [],
    close: () => {},
    submit: () => {},
    initialValues: {
        user_params: [],
    },
    maxHeight: '60vh',
};

export default ZoneBulkEditSitePageForm = connect(mapStateToForm)(injectIntl(ZoneBulkEditSitePageForm));
