import React from 'react';
import PropTypes from 'prop-types';
import { Map } from 'react-leaflet';
import _get from 'lodash/get';

import LeafletFloorplan from 'components/leaflet/LeafletFloorplan';
import LeafletScaling from 'components/leaflet/LeafletScaling';
import LeafletZoneLayer from 'components/leaflet/LeafletZoneLayer';
import ZoneDrawingControl from 'components/leaflet/ZoneDrawingControl';
import LeafletEditModeZoneLayer from 'components/leaflet/LeafletEditModeZoneLayer';
import { DEFAULT_VALUES, GEOMETRY_RESTRICTION_TYPES, SITE_MODE, ZONE_DETAILS_MODAL } from 'common';
import {
    saveOptionsZoneCreateModal,
    chooseZoneTypeOnCreate,
    clearCoordinatesRegimeAdminPageModal,
    clearGeometryAdminPageModal,
} from 'stores/actions';
import { MAP_BOUNDS } from 'utils/formatters';


class SelectedFloorBody extends React.Component {
    constructor(props) {
        super(props);
        this.drawControlMounted = this.drawControlMounted.bind(this);
    }
    
    drawControlMounted() {
        this.props.dispatch(chooseZoneTypeOnCreate({ zone_type_name: this.props.zone_type_name }));
    }
    
    render() {
        const {
            leafletClass, unSelectFloorplan, intl, createZoneFormState, dispatch, setLeafletToState, selectedSublocation,
            selectedFloorplan, onMove, zoom, boundsForScale, width, rotation, geometry, isDirectionalGeometry,
            handleChangeWidth, handleChangeGeometry, handleChangeRotation, error, coordinateMode,
        } = this.props;

        const isGeometryReady = _get(geometry, 'zoneGeometryInDegrees.length', 0) > 0;

        return (
            <div className="Zone-Details-Body">
                <div
                    className={leafletClass}
                    title={ZONE_DETAILS_MODAL.DOUBLE_CLICK_COLLAPSE(intl)}
                    onDoubleClick={() => {
                        dispatch(saveOptionsZoneCreateModal(createZoneFormState.values));
                        dispatch(clearCoordinatesRegimeAdminPageModal());
                        dispatch(clearGeometryAdminPageModal());
                        unSelectFloorplan();
                    }}
                >
                    <Map
                        center={[0, 0]}
                        className="zone-modal"
                        ref={leaflet => setLeafletToState(leaflet)}
                        bounds={MAP_BOUNDS(selectedSublocation, selectedFloorplan)}
                        zoomAnimation={false}
                        zoomSnap={0}
                        scrollWheelZoom="center"
                        onMoveend={onMove}
                        doubleClickZoom={false}
                        minZoom={DEFAULT_VALUES.MAP_ZOOM.MIN}
                        maxZoom={DEFAULT_VALUES.MAP_ZOOM.MAX}
                        zoom={DEFAULT_VALUES.MAP_ZOOM.DEFAULT}
                    >
                        <LeafletFloorplan floorplan={selectedFloorplan} />
                        
                        <LeafletScaling
                            zoom={zoom}
                            floorplan={selectedFloorplan}
                            bounds={boundsForScale}
                            isShowingScale
                        />
                        
                        <LeafletZoneLayer sublocation={selectedSublocation} withAlarm={false} />

                        {
                            (isGeometryReady || coordinateMode) && <LeafletEditModeZoneLayer
                                coordinateMode={coordinateMode}
                                error={error}
                                sublocation={selectedSublocation}
                                mode={{
                                    name: SITE_MODE.EDIT_ZONE,
                                    type: isDirectionalGeometry ? GEOMETRY_RESTRICTION_TYPES.DIRECTIONAL : GEOMETRY_RESTRICTION_TYPES.RECTANGLE,
                                }}
                                width={width}
                                rotation={rotation}
                                creatingZone={geometry}
                                changeGeometry={handleChangeGeometry}
                                changeWidth={handleChangeWidth}
                            />
                        }

                        <ZoneDrawingControl
                            drawControlMounted={this.drawControlMounted}
                            sublocation={selectedSublocation}
                            changeGeometry={handleChangeGeometry}
                            changeRotation={handleChangeRotation}
                        />
                    </Map>
                </div>
            </div>
        );
    }
}

SelectedFloorBody.propTypes = {
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    zoom: PropTypes.number,
    error: PropTypes.shape({}),
    rotation: PropTypes.number,
    geometry: PropTypes.shape(),
    width: PropTypes.number,
    onMove: PropTypes.func,
    dispatch: PropTypes.func,
    leafletClass: PropTypes.string,
    boundsForScale: PropTypes.shape(),
    zone_type_name: PropTypes.string,
    unSelectFloorplan: PropTypes.func,
    setLeafletToState: PropTypes.func,
    handleChangeGeometry: PropTypes.func,
    handleChangeRotation: PropTypes.func,
    handleChangeWidth: PropTypes.func,
    selectedFloorplan: PropTypes.shape(),
    createZoneFormState: PropTypes.shape(),
    selectedSublocation: PropTypes.shape(),
    isDirectionalGeometry: PropTypes.bool,
    coordinateMode: PropTypes.bool,
};

SelectedFloorBody.defaultProps = {
    intl: {
        formatMessage: () => {},
    },
    zoom: 19,
    rotation: 0,
    error: {},
    geometry: {
        zoneGeometryInDegrees: [],
        zoneGeometryInMillimeters: [],
    },
    width: 0,
    onMove: () => {
    },
    dispatch: () => {
    },
    handleChangeGeometry: () => {
    
    },
    handleChangeRotation: () => {
    
    },
    handleChangeWidth: () => {
    },
    leafletClass: '',
    boundsForScale: {},
    zone_type_name: PropTypes.string,
    unSelectFloorplan: () => {
    },
    setLeafletToState: () => {
    },
    selectedFloorplan: {},
    createZoneFormState: {},
    selectedSublocation: {},
    isDirectionalGeometry: false,
    coordinateMode: false,
};

export default SelectedFloorBody;
