import React from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { change, formValueSelector, reduxForm } from 'redux-form';
import PropTypes from 'prop-types';
import _Get from 'lodash/get';
import SubmitButtons from 'components/modals/forms/SubmitButtons';
import TrackablesAndGroups from 'components/modals/forms/zones/TrackablesAndGroups';
import SeverityOptions from 'components/modals/forms/zones/SeverityOptions';
import AdvancedOptionsControl from 'components/modals/forms/zones/AdvancedOptionsControl';
import BasicOptions from 'components/modals/forms/zones/BasicOptions';
import CustomZoneOptions from 'components/modals/forms/zones/CustomZoneOptions';
import { ZONE_APPLY_TO, NAME_OF_FORMS, ZONE_TYPE, DEFAULT_VALUES, ERROR_TEXT, COMPONENTS } from 'common';
import { getLength, getValuesFromZoneFormState, getZoneTypeName } from 'utils';
import { Sublocation } from 'stores/classes';
import { getParsedZones, getParsedTrackables, getParsedGroups } from 'stores/getters';
import { ZonePrototype } from 'stores/classes/Selectors';
import ScheduleForm from 'components/modals/forms/zones/ScheduleForm';
import store from 'stores/LocationStore';


const isInfoZone = zone_type_name => zone_type_name === ZONE_TYPE.INFO.name;

const validate = (values, props) => {
    const errors = {};
    const { zones, sublocation_uid } = props;
    const zoneWithTheSameName = zones.find(zone => (zone && zone.sublocation_uid === sublocation_uid && zone.name === values.name));
    
    if (!values.zone_type_name) {
        errors.zone_type_name = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    if (zoneWithTheSameName) {
        errors.name = COMPONENTS.VALIDATION.NAME_EXISTS_FUNC(props.intl);
    }
    if (!values.zone_alarm_level && !isInfoZone(values.zone_type_name)) {
        errors.zone_alarm_level = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }

    return errors;
};


const isSelectedFields = (formValues) => {
    let isZoneTypeSelected = false;
    let isInformationZoneSeverity = false;
    if (formValues) {
        isZoneTypeSelected = !!formValues.zone_type_name;
        isInformationZoneSeverity = formValues.zone_alarm_level ? Number(formValues.zone_alarm_level) === 0 : !formValues.zone_alarm_level;
    }
    
    return {
        isZoneTypeSelected,
        isInformationZoneSeverity,
    };
};

const activityHandler = (props, e) => {
    const activeValue = e.target.value === 'true' ? false : e.target.value === 'false';
    props.dispatch(change(NAME_OF_FORMS.ZONE_CONFIGURATION_ADD, 'active24', activeValue));
};

let ZoneCreateSitePageForm = (props) => {
    const {
        className, close, formState, intl, invalid, submit, zonePrototype,
        zone_type_name, groups, trackables, initialValues, maxHeight,
        customInfoLength, submitSucceeded, zones,
    } = props;
    
    const addZoneFormState = formState[NAME_OF_FORMS.ZONE_CONFIGURATION_ADD] || {};
    
    const { isZoneTypeSelected } = isSelectedFields(addZoneFormState.values);
    
    const zoneTypeName = getZoneTypeName(store.getZonesPrototypes(), zone_type_name, intl);
    const isInformationZone = isInfoZone(zone_type_name);
    const {
        activated,
        location_name,
        sublocation_name,
        areAdvancedOptionsShown,
        active24,
    } = getValuesFromZoneFormState(addZoneFormState);
    
    const isKeepInType = zone_type_name === ZONE_TYPE.KEEP_IN.name;
    const isKeepOutType = zone_type_name === ZONE_TYPE.KEEP_OUT.name;
    const isKeepInOutType = (isKeepInType || isKeepOutType);
    
    return (
        <form className={className}>
            <div
                style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight, paddingRight: '10px' }}
            >
                <BasicOptions
                    activityHandler={(event) => { activityHandler(props, event); }}
                    customInfoLength={customInfoLength}
                    location_name={location_name}
                    sublocation_name={sublocation_name}
                    zoneTypeName={zoneTypeName}
                    isZoneTypeSelected={isZoneTypeSelected}
                    activated={activated}
                    intl={intl}
                />
                <SeverityOptions
                    show={isKeepInOutType}
                    isZoneTypeSelected={isZoneTypeSelected}
                    isInformationZone={isInformationZone}
                    intl={intl}
                />
                <CustomZoneOptions
                    zones={zones}
                    zone_type_name={zone_type_name}
                    zonePrototype={zonePrototype}
                    formState={initialValues}
                />
                <hr style={{ width: '90%', height: '5px', backgroundColor: '#E8E8E8' }} />
                <AdvancedOptionsControl intl={intl} show={areAdvancedOptionsShown} />
                <TrackablesAndGroups
                    show={areAdvancedOptionsShown}
                    trackables={trackables}
                    groups={groups}
                    intl={intl}
                />
                <ScheduleForm
                    show={areAdvancedOptionsShown}
                    active24={active24}
                    activated={activated}
                />
            </div>
            <SubmitButtons
                className="Form-Buttons"
                submit={submit}
                invalid={invalid || submitSucceeded}
                close={close}
                saveDataId="zone-add-configuration-site-page-save"
                cancelDataId="zone-add-configuration-site-page-cancel"
            />
        </form>
    );
};

ZoneCreateSitePageForm.propTypes = {
    close: PropTypes.func,
    className: PropTypes.string,
    zone_type_name: PropTypes.string,
    zonePrototype: PropTypes.shape(),
    formState: PropTypes.oneOfType([
        PropTypes.object,
        PropTypes.shape({
            [NAME_OF_FORMS.ZONE_CONFIGURATION_ADD]: PropTypes.shape({
                values: PropTypes.shape({
                    active: PropTypes.bool,
                    zone_type_name: PropTypes.string,
                    zone_alarm_level: PropTypes.number,
                    groups_allowed_special: PropTypes.bool,
                    trackables_allowed_special: PropTypes.bool,
                }),
            }),
        }),
    ]),
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    trackables: PropTypes.arrayOf(PropTypes.object),
    groups: PropTypes.arrayOf(PropTypes.object),
    initialValues: PropTypes.shape({
        zone: PropTypes.shape({
            sublocation_uid: PropTypes.string,
        }),
    }),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    maxHeight: PropTypes.string,
    customInfoLength: PropTypes.number,
    submitSucceeded: PropTypes.bool,
    zones: PropTypes.arrayOf(PropTypes.shape({})),
};

ZoneCreateSitePageForm.defaultProps = {
    close: () => {
    },
    className: '',
    zone_type_name: '',
    zonePrototype: {},
    formState: {},
    submit: () => {
    },
    invalid: false,
    trackables: [],
    groups: [],
    initialValues: {
        zone: {
            sublocation_uid: '',
        },
        user_params: {},
    },
    intl: {
        formatMessage: () => {
        },
    },
    maxHeight: '60vh',
    customInfoLength: 0,
    submitSucceeded: false,
    zones: [],
};

ZoneCreateSitePageForm = reduxForm({
    form: NAME_OF_FORMS.ZONE_CONFIGURATION_ADD,
    validate,
})(injectIntl(ZoneCreateSitePageForm));


const mapStateToForm = (state) => {
    const currentModal = state.modals.zoneModals.zoneCreateSitePage;
    const location_name = _Get(state, 'siteView.selectedLocation.name', '');
    const { zone_type_name, sublocation_uid, rotation } = currentModal.params || {};
    const sublocations = Sublocation.getAllSublocations(state);

    const selector = formValueSelector(NAME_OF_FORMS.ZONE_CONFIGURATION_ADD);
    const custom_info = selector(state, 'custom_info');

    const sublocation = sublocations.find(_sublocation => sublocation_uid && _sublocation.uid === sublocation_uid);
    const sublocation_name = (sublocation && sublocation.name) || '';
    
    const formDefaults = {
        user_params: {
            direction: rotation,
        },
        location_name,
        sublocation_name,
        zone_type_name,
        zone_alarm_level: 1,
        active24: true,
        activated: false,
        apply_to: ZONE_APPLY_TO.ALL,
    };
    
    const initialValues = {
        ...currentModal,
        ...formDefaults,
    };
    
    const zonePrototype = ZonePrototype.getZonePrototypeByName(zone_type_name);
    
    return {
        initialValues,
        customInfoLength: custom_info ? getLength(custom_info) : 0,
        zonePrototype,
        zone_type_name,
        sublocation_uid,
        trackables: getParsedTrackables(state),
        groups: getParsedGroups(state),
        zones: getParsedZones(state),
        formState: state.form,
    };
};
export default connect(mapStateToForm)(injectIntl(ZoneCreateSitePageForm));
