import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';

import {
    openProjectDetailsModal,
    openProjectEditModal,
    openProjectDeleteModal,
    selectProject,
    userLoginAsProjectSystemAdmin,
} from 'stores/actions';

import InfoTable from 'components/InfoTable';
import expandColumnComponent from 'components/ExpandColumnComponent';
import { trOddEvenClassFormat, formatTimeZoneOffset } from 'utils';
import {
    DEFAULT_VALUES,
    PROJECT_MODALS,
    TABLES,
    COMPONENTS,
    BOOTSTRAP_TABLE_SORT,
    NAMES_OF_TABS,
} from 'common';

class ProjectsTab extends React.Component {
    static propTypes = {
        projects: PropTypes.arrayOf(PropTypes.shape()),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };

    static defaultProps = {
        projects: [],
        dispatch: () => {
        },
        intl: {
            formatMessage: () => {
            },
        },
    };

    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                data-id="projects-actions-info"
                data-cy="project-table-info-button"
                onClick={this.showProjectInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="projects-actions-edit"
                data-cy="project-table-edit-button"
                onClick={this.openEditModal.bind(this, row)}
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="projects-actions-remove"
                data-cy="project-table-remove-button"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    showProjectInfo = row => this.props.dispatch(openProjectDetailsModal(row));

    openEditModal = row => this.props.dispatch(openProjectEditModal(row));

    openAddModal = () => this.props.dispatch(openProjectEditModal());

    openDeleteModal = row => this.props.dispatch(openProjectDeleteModal(row));

    isExpandableRow = () => true;

    expandComponent = (row) => {
        const rowInfo = {
            [PROJECT_MODALS.EDIT_MODAL.FORM.MANAGER_FUNC(this.props.intl)]: row.user_params.manager ? row.user_params.manager : '',
            [PROJECT_MODALS.EDIT_MODAL.FORM.TIME_ZONE_FUNC(this.props.intl)]: row.user_params.timezone ? formatTimeZoneOffset(row.user_params.timezone) : '',
        };
        return <InfoTable rowInfo={rowInfo} />;
    };

    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="projects-plus-button"
                data-cy="project-table-add-button"
                onClick={this.openAddModal}
            />
        </section>
    );

    handleSelectRow = (row, isSelect, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        if (e.target.getAttribute('data-info') === 'project-button') {
            this.props.dispatch(userLoginAsProjectSystemAdmin(row));
            return false;
        }
        this.props.dispatch(selectProject(row));
        return undefined;
    };

    trClassFormat = () => 'Table-Row-CursorPointer';

    createCustomSearchField = intl => (
        <SearchField
            data-id="projects-search"
            data-cy="project-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    render() {
        const { projects } = this.props;
        const options = {
            onlyOneExpanding: true,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            expandBy: 'column',
            expandRowBgColor: 'white',
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            btnGroup: this.createAddDeleteBtn,
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'radio',
            clickToSelect: true,
            hideSelectColumn: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToExpand: true,
            onSelect: this.handleSelectRow,
        };

        return (
            <section className="AppLayout-Admin-Tab tab-projects-projects">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.PROJECTS}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="projects-total-count"
                    data-cy="project-table-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(projects.length)}
                </h4>
                <hr />
                <BootstrapTable
                    trClassName={(row, rowIndex) => (`${trOddEvenClassFormat(row, rowIndex)} ${this.trClassFormat()}`)}
                    tableBodyClass="Table-Body automation-projects-table"
                    data={projects}
                    options={options}
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    expandableRow={this.isExpandableRow}
                    expandComponent={this.expandComponent}
                    bodyContainerClass="Admin-BootstrapTable"
                    expandColumnOptions={{
                        expandColumnVisible: true,
                        expandColumnComponent,
                        columnWidth: 40,
                    }}
                    hover
                    condensed
                    search
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="manager"
                        tdAttr={{ 'data-id': 'projects-table-manager' }}
                        searchable
                        hidden
                    >
                        {PROJECT_MODALS.DETAIL_MODAL.FORM.MANAGER}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        headerAlign="center"
                        columnClassName="Table-Button button-link"
                        tdAttr={{ 'data-id': 'projects-table-name', 'data-info': 'project-button' }}
                        dataSort
                        width="150"
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="85"
                        headerAlign="center"
                        dataAlign="center"
                        dataFormat={this.getActionsIcons.bind(this)}
                        expandable={false}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </section>
        );
    }
}

const mapStateToProps = state => ({
    projects: state.entities.projects.map(project => project.getFormattedProject()),
    companyUid: state.appContext.company && state.appContext.company.uid,
});

export default connect(mapStateToProps)(injectIntl(ProjectsTab));
