import React from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import PropTypes from 'prop-types';
import {
    openUserDetailsModal, openUserEditModal, openUserDeleteModal, editUser, changeSelectedItems, selectItems, unselectItems,
    openUserAccessControlEditCompanyScopeModal,
} from 'stores/actions';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { USER_ROLES, DEFAULT_VALUES, USERS_TAB, TABLES, COMPONENTS, BOOTSTRAP_TABLE_SORT, NAMES_OF_TABS } from 'common';
import User from 'stores/classes/User';
import InfoTable from 'components/InfoTable';
import expandColumnComponent from 'components/ExpandColumnComponent';
import { ProjectUserRoleSelect } from 'components/ProjectUserRoleSelect';

class UsersTab extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        users: PropTypes.arrayOf(PropTypes.shape({})),
        projects: PropTypes.arrayOf(PropTypes.shape({})),
        selectedItems: PropTypes.arrayOf(PropTypes.shape({})),
        currentUser: PropTypes.shape({
            email: PropTypes.string,
        }),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        selectedProject: PropTypes.shape({}),
        totalCount: PropTypes.number,
    };

    static defaultProps = {
        dispatch: () => {},
        users: [],
        projects: [],
        selectedItems: [],
        currentUser: {
            email: '',
        },
        selectedProject: {},
        totalCount: 0,
        intl: {
            formatMessage: () => {
            },
        },
    };

    getUserRoleOptions = (intl, cell, row) => {
        let roles = [];
        if (this.isProjectSelected()) {
            roles = [USER_ROLES.PROJECT_SYSTEM_ADMIN, USER_ROLES.PROJECT_OPERATIONS_ADMIN, USER_ROLES.PROJECT_MAINTAINER, USER_ROLES.PROJECT_USER];
        } else if (row.role === USER_ROLES.COMPANY_ADMIN) {
            roles = [USER_ROLES.COMPANY_ADMIN];
        } else if (row.role) {
            roles = [USER_ROLES.COMPANY_ADMIN, USER_ROLES.PROJECT_SYSTEM_ADMIN, USER_ROLES.PROJECT_OPERATIONS_ADMIN, USER_ROLES.PROJECT_MAINTAINER, USER_ROLES.PROJECT_USER];
        }
        return (
            <ProjectUserRoleSelect
                uid={row.uid}
                onChange={e => this.updateUserType(e, row)}
                intl={intl}
                role={row.role}
                roles={roles}
                disabled={this.props.currentUser.email === row.email || row.role === USER_ROLES.COMPANY_ADMIN}
            />
        )
    };

    getActionsIcons = (cell, row) => (
        <section>
            <i
                data-info="actions-section"
                data-id="users-actions-info"
                data-cy="user-table-info-button"
                onClick={this.showUserInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="users-actions-edit"
                data-cy="user-table-edit-button"
                onClick={this.openEditModal.bind(this, row)}
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="users-actions-remove"
                data-cy="user-table-remove-button"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    handleSelectAll = (isSelected, users) => {
        if (isSelected) {
            this.props.dispatch(selectItems(users));
        } else {
            this.props.dispatch(unselectItems(users));
        }
    };

    showUserInfo = (row) => {
        this.props.dispatch(openUserDetailsModal(row));
    };

    openEditModal = (row) => {
        this.props.dispatch(openUserEditModal(row));
    };

    openAddUserModal = () => {
        this.props.dispatch(openUserEditModal({}));
    };

    openDeleteModal = (row) => {
        this.props.dispatch(openUserDeleteModal(row));
    };

    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const user = this.props.users.find(x => x.uid === item.uid);

            if (user) {
                params.push({
                    name: user.name,
                    uid: user.uid,
                    email: user.email,
                });
            }
        });

        if (params.length > 0) {
            this.props.dispatch(openUserDeleteModal(params));
        }
    };

    updateUserType = (e, row) => {
        row.role = e.target.value;
        this.props.dispatch(editUser(row));
    };

    isProjectSelected = () => this.props.selectedProject;

    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        return true;
    };

    expandComponent = (row) => {
        const rowInfo = {
            // Phone: row.user_details ? row.user_details.phone_number : '',
            [COMPONENTS.FIELDS.USER.PHONE_FUNC(this.props.intl)]: row.user_details ? row.user_details.phone_number : '',
        };
        return <InfoTable rowInfo={rowInfo} />;
    };

    isExpandableRow = () => true;

    openEditUserAccessControl = () => {
        this.props.dispatch(openUserAccessControlEditCompanyScopeModal());
    }

    createAddDeleteBtn = () => (
        <section>
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="users-button-trash"
                data-cy="user-table-batch-remove-button"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="users-button-plus"
                data-cy="user-table-add-button"
                onClick={this.openAddUserModal}
            />
        </section>);

    createCustomSearchField = intl => (
        <SearchField
            data-id="users-search"
            data-cy="user-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    render() {
        const options = {
            expandBy: 'column',
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            expandRowBgColor: 'white',
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            btnGroup: this.createAddDeleteBtn,
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToExpand: true,
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
            columnWidth: '40px',
        };

        const { users } = this.props;
        return (
            <section className="AppLayout-Admin-Tab tab-projects-users">
                <div className="row" style={{ marginBottom: '-10px' }}>
                    <div className="col-md-1" style={{ marginTop: '50px' }}>
                        <i
                            className="fa fa-users fa-lg table-action-icon"
                            data-id="user-edit-access-control-button"
                            data-cy="user-edit-access-control-button"
                            title="Edit Users Access Control Options"
                            onClick={this.openEditUserAccessControl}
                        />
                    </div>
                    <div className="col-md-11">
                        <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.USERS_PROJECT_SCOPE}</h3>
                        <h4 className="text-center AdminTab-Subtitle" data-id="users-total-count">
                            {USERS_TAB.TOTAL_COUNT_FUNC(this.props.totalCount, this.isProjectSelected() ? users.length : 0)}
                        </h4>
                    </div>
                </div>
                <hr />
                <BootstrapTable
                    condensed
                    data={users}
                    options={options}
                    hover
                    selectRow={selectRow}
                    expandableRow={this.isExpandableRow}
                    expandComponent={this.expandComponent}
                    trClassName={trOddEvenClassFormat}
                    tableBodyClass="automation-users-table"
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    expandColumnOptions={{
                        expandColumnVisible: true,
                        expandColumnComponent,
                        columnWidth: '40px',
                    }}
                    search
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        hidden
                        searchable={false}
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="150px"
                        dataField="name"
                        tdAttr={{ 'data-id': 'users-table-name' }}
                        dataAlign="left"
                        dataSort
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="150px"
                        dataField="email"
                        tdAttr={{ 'data-id': 'users-table-email' }}
                        dataAlign="left"
                        dataSort
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.USER.EMAIL}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="150px"
                        dataField="project"
                        tdAttr={{ 'data-id': 'users-table-project' }}
                        dataAlign="center"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.PROJECT}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="200px"
                        dataField="role"
                        dataFormat={(cell, row) => this.getUserRoleOptions(this.props.intl, cell, row)}
                        tdAttr={{ 'data-id': 'users-table-type' }}
                        dataAlign="center"
                        headerAlign="center"
                        expandable={false}
                    >
                        {COMPONENTS.FIELDS.USER.USER_TYPE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="85"
                        headerAlign="center"
                        dataAlign="center"
                        dataFormat={this.getActionsIcons}
                        expandable={false}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </section>
        );
    }
}

const mapStateToProps = (state) => {
    const selectedProject = state.adminPage.projectSelect;
    const projectsMap = state.entities.projects.reduce((memo, project) => {memo[project.uid] = project; return memo;}, {});
    const users = User.getAllUsers(state).reduce((memo, user) => {
        if (!selectedProject || user.belongToProject(selectedProject)) {
            memo.push(user.getFormattedUser(projectsMap[user.project_uid]));
        }
        return memo;
    }, []);

    return {
        users,
        currentUser: state.user,
        totalCount: User.getAllUsers(state).length,
        selectedItems: state.adminPage.selectedItems,
        selectedProject: state.adminPage.projectSelect,
    };
};

export default connect(mapStateToProps)(injectIntl(UsersTab));
