import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import _CloneDeep from 'lodash/cloneDeep';

import { colorFormatter, trOddEvenClassFormat, generateRandomColor } from 'utils/htmlUtils';
import { DEFAULT_VALUES, BOOTSTRAP_TABLE_SORT } from 'common/constants';
import { GROUP_DETAILS_MODAL, TABLES, COMPONENTS, NAMES_OF_TABS } from 'common/ui-labels';
import { changeSelectedItems, selectItems, unselectItems } from 'stores/actions/adminPageActions';
import { openGroupEditModal, openGroupDeleteModal, openGroupDetailsModal } from 'stores/actions/modalActions/groupModalActions';
import { getTrackables, getNodes } from 'stores/getters';
import { Group, Trackable, Zone } from 'stores/classes';

class GroupsTable extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        groups: PropTypes.arrayOf(PropTypes.shape()),
        selectedGroup: PropTypes.shape({
            uid: PropTypes.string,
        }),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        selectedItems: [],
        groups: [],
        selectedGroup: {
            uid: '',
        },
        intl: {
            formatMessage: () => {
            },
        },
    };
    
    shouldComponentUpdate(nextProps) {
        return !isEqual(nextProps, this.props);
    }

    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                data-id="groups-actions-info"
                data-cy="group-table-info-button"
                onClick={this.showInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                style={{ margin: '5px', cursor: 'pointer' }}
                className="fa fa-info-circle"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="groups-actions-edit"
                data-cy="group-table-edit-button"
                onClick={this.openEditModal.bind(this, row)}
                title={COMPONENTS.EDIT(this.props.intl)}
                style={{ margin: '5px', cursor: 'pointer' }}
                className="fa fa-pencil-alt"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="groups-actions-remove"
                data-cy="group-table-remove-button"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                style={{ margin: '5px', cursor: 'pointer' }}
                className="fa fa-times"
                aria-hidden="true"
            />
        </section>
    );
    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const group = this.props.groups.find(_group => _group.uid === item.uid);

            if (group) {
                params.push({
                    name: group.name,
                    uid: group.uid,
                    email: group.email,
                });
            }
        });

        if (params.length > 0) {
            this.props.dispatch(openGroupDeleteModal(params));
        }
    };

    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="groups-button-trash"
                data-cy="group-table-batch-remove-button"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="groups-button-plus"
                data-cy="group-table-add-button"
                onClick={this.openCreateModal}
            />
        </section>);

    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        if (!isSelected) {
            this.forceUpdate();
        }
    };
    handleSelectAll = (isSelected, nodes) => {
        if (isSelected) {
            this.props.dispatch(selectItems(nodes));
        } else {
            this.props.dispatch(unselectItems(nodes));
            this.forceUpdate();
        }
    };
    showInfo = (row) => {
        this.props.dispatch(openGroupDetailsModal(row));
    };
    openCreateModal = () => {
        this.props.dispatch(openGroupEditModal({
            color: generateRandomColor(),
            sensorSpecialCases: {},
            sensorInterface: [],
            zoneExceptionsGroups: [],
            safe_distance: DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE,
            distance: DEFAULT_VALUES.LLL_COVERAGE_DISTANCE,
            alert_distance: DEFAULT_VALUES.LLL_COVERAGE_ALERT_DISTANCE,
            angle: DEFAULT_VALUES.LLL_COVERAGE_ANGLE,
            delay: DEFAULT_VALUES.LLL_DELAY,
            pin: DEFAULT_VALUES.LLL_TRACKABLE_PIN,
        }));
    };
    openEditModal = (row) => {
        const props = _CloneDeep(row);

        this.props.dispatch(openGroupEditModal(props));
    };
    openDeleteModal = (row) => {
        this.props.dispatch(openGroupDeleteModal(row));
    };
    trClassFormat = (row, rowIndex) => {
        if (this.props.selectedGroup && this.props.selectedGroup.uid === row.uid) {
            return 'Table-Assigned-Row-Selected';
        }
        return trOddEvenClassFormat(row, rowIndex);
    };
    createCustomSearchField = intl => (
        <SearchField
            data-id="groups-search"
            data-cy="group-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    render() {
        const groups = this.props.groups;
        
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        
        const selectRow = {
            mode: 'checkbox',
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToSelect: true, // click to select, default is false
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
        };

        return (
            <div className="AppLayout-Admin-Tab tab-for-admin-groups">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.GROUPS}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="groups-total-count"
                    data-cy="group-table-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(groups.length)}
                </h4>
                <hr />
                <BootstrapTable
                    ref="groupsTable"
                    trClassName={this.trClassFormat}
                    tableBodyClass="automation-groups-table"
                    data={groups}
                    options={options}
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    condensed
                    search
                    hover
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'groups-name' }}
                        width="170"
                        headerAlign="center"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="trackable_count"
                        tdAttr={{ 'data-id': 'groups-trackable_count' }}
                        searchable={false}
                        dataSort={false}
                        dataAlign="center"
                        width="120"
                        headerAlign="center"
                    >
                        {GROUP_DETAILS_MODAL.FORM.TRACKABLE_COUNT}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="color"
                        searchable={false}
                        dataAlign="center"
                        dataFormat={colorFormatter}
                        width="60"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.COLOR}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="100"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getSelectedItems = state => state.adminPage.selectedItems;

const getTrackable = state => Trackable.getAllTrackablesArray(state).map(trackable => trackable.formattedTrackablesUidNameGroup());
const getTrackablesMap = state => state.entities.trackables;

// For updating group it is necessary observe trackables and groups
const getFormattedGroups = createDeepEqualSelector(
    [
        Group.getAllGroupsArray,
        getTrackable,
        Zone.getAllZonesArray,
        getTrackablesMap,
    ],
    (groups, trackable, zones, trackablesMap) => (
        groups.map((group) => {
            const _group = new Group(group.getProperties());
            return _group.getFormattedGroup(trackablesMap, zones);
        })
    ),
);

const getSelectedGroup = createDeepEqualSelector(
    [
        getFormattedGroups,
        getSelectedItems,
        getTrackables,
        getNodes,
    ],
    (groups, selectedItems) => {
        const selectedGroups = selectedItems.filter(item => groups.some(group => group.uid === item.uid));
        return selectedGroups.length === 1 ? selectedGroups[0] : null;
    },
);

const mapStateToProps = state => (
    {
        groups: getFormattedGroups(state),
        selectedGroup: getSelectedGroup(state),
        selectedItems: getSelectedItems(state),
    }
);

export default connect(mapStateToProps)(injectIntl(GroupsTable));
