import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import { SortDirection, AutoSizer } from 'react-virtualized';

import TableTitle from './TableTitle';
import VirtualTable from './VirtualTable';

import { nodeFilterNotWin } from '../../../../utils';
import { getTrackables, getNodes } from '../../../../stores/getters';
import { Association } from '../../../../stores/classes';

class NodesProjectScopeReadOnlyTable extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        nodes: PropTypes.shape(),
    };

    static defaultProps = {
        dispatch: () => {},
        nodes: new Map(),
    };

    constructor(props) {
        super(props);

        this.state = {
            containerWidth: 0,
            containerHeight: 0,
        };
    }

    render() {
        const { nodes, dispatch } = this.props;
        const { containerWidth, containerHeight } = this.state;

        const nodesCount = nodes.size;

        return (
            <section className="AppLayout-Admin-Tab tab-admin-tags">
                <AutoSizer
                    onResize={({ height, width }) => {
                        this.setState({
                            containerWidth: width,
                            containerHeight: height,
                        });
                    }}
                >
                    {({ height, width }) => (
                        <Fragment>
                            <TableTitle
                                totalCount={nodesCount}
                                width={containerWidth}
                            />
                            <VirtualTable
                                defaultSortName="trackableName"
                                defaultSortOrder={SortDirection.ASC}
                                tableSize={{
                                    width: containerWidth,
                                    height: containerHeight,
                                    headerHeight: 40,
                                    rowHeight: 40,
                                }}
                                allItems={nodes}
                                totalCount={nodesCount}
                                dispatch={dispatch}
                            />
                        </Fragment>
                    )}
                </AutoSizer>
            </section>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getProject = state => state.appContext.project;

const parseNodes = (nodes, associations, trackables, project) => {
    const nodesMap = new Map();
    if (nodes) {
        nodeFilterNotWin(nodes).forEach((_node) => {
            const node = _node.getProperties();
            if (node.project_uid === project.uid) {
                const association_obj_uid = associations.byMac[node.mac_address.toUpperCase()];
                const trackable = association_obj_uid ? trackables.find(track => track.uid === association_obj_uid) : null;
                nodesMap.set(node.uid, { ...node, trackable, trackableName: trackable ? trackable.name : '' });
            }
        });
    }
    return nodesMap;
};

const getFilteredNodes = createDeepEqualSelector(
    [Association.getAllAssociations, getNodes, getTrackables, getProject],
    (associations, nodes, trackables, project) => parseNodes(nodes, associations, trackables, project),
);

const mapStateToProps = state => ({ nodes: getFilteredNodes(state) });

export default connect(mapStateToProps)(injectIntl(NodesProjectScopeReadOnlyTable));
