import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import isEqual from 'lodash/isEqual';
import { createSelectorCreator, defaultMemoize } from 'reselect';

import { trOddEvenClassFormat } from '../../../utils';
import {
    DEFAULT_VALUES,
    NAMES_OF_TABS,
    TABLES,
    COMPONENTS,
    BOOTSTRAP_TABLE_SORT,
} from './../../../common';

import {
    changeSelectedItems,
    openSubscriberEditModal,
    openSubscriberDeleteModal,
    openSubscriberDetailsModal,
    selectItems,
    unselectItems,
} from '../../../stores/actions';
import { Subscriber, Notification } from '../../../stores/classes';

class SubscribersTable extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        selectedGroup: PropTypes.shape(),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        subscribers: PropTypes.arrayOf(PropTypes.shape()),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    
    static defaultProps = {
        dispatch: PropTypes.func,
        selectedGroup: {},
        selectedItems: [],
        subscribers: [],
        intl: {
            formatMessage: () => {
            },
        },
    };
    
    shouldComponentUpdate(nextProps) {
        return !isEqual(nextProps, this.props);
    }

    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                data-id="subscribers-actions-info"
                onClick={this.showInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="subscribers-actions-edit"
                onClick={this.openEditModal.bind(this, row)}
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="subscribers-actions-remove"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );
    
    showInfo = (row) => {
        this.props.dispatch(openSubscriberDetailsModal(row));
    };
    
    openCreateModal = () => {
        this.props.dispatch(openSubscriberEditModal({}));
    };
    
    openEditModal = (row) => {
        this.props.dispatch(openSubscriberEditModal(row));
    };
    
    openDeleteModal = (row) => {
        this.props.dispatch(openSubscriberDeleteModal(row));
    };
    
    handleSelectAll = (isSelected, nodes) => {
        if (isSelected) {
            this.props.dispatch(selectItems(nodes));
        } else {
            this.props.dispatch(unselectItems(nodes));
            this.forceUpdate();
        }
    };
    
    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="subscribers-button-trash"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="subscribers-button-plus"
                onClick={this.openCreateModal}
            />
        </section>
    );
    
    trClassFormat = (row, rowIndex) => {
        if (this.props.selectedGroup && this.props.selectedGroup.uid === row.uid) {
            return 'Table-Assigned-Row-Selected';
        }
        return trOddEvenClassFormat(row, rowIndex);
    };
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="subscribers-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const subscriber = this.props.subscribers.find(subscriberItem => item.uid === subscriberItem.uid);
            
            if (subscriber) {
                params.push(subscriber);
            }
        });
        
        if (params.length > 0) {
            this.props.dispatch(openSubscriberDeleteModal(params));
        }
    };
    
    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        if (!isSelected) {
            this.forceUpdate();
        }
        return undefined;
    };
    
    render() {
        const { subscribers } = this.props;
        
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        
        const selectRow = {
            mode: 'checkbox',
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToSelect: true, // click to select, default is false
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
        };
        
        return (
            <div className="AppLayout-Admin-Tab tab-admin-subscribers">
                <h3
                    className="text-center AdminTab-Title"
                >
                    {COMPONENTS.FIELDS.SUBSCRIBER.SUBSCRIBER_TITLE}
                </h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="subscribers-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(subscribers.length)}
                </h4>
                <hr />
                <BootstrapTable
                    ref="subscribersTable"
                    trClassName={this.trClassFormat}
                    tableBodyClass="automation-subscribers-table"
                    data={subscribers}
                    options={options}
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    search
                    condensed
                    hover
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'subscriber-name' }}
                        headerAlign="center"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.SUBSCRIBER.HEADER_NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="email"
                        tdAttr={{ 'data-id': 'subscriber-email' }}
                        dataSort={false}
                        dataAlign="center"
                        headerAlign="center"
                        searchable
                    >
                        {COMPONENTS.FIELDS.USER.EMAIL_ADDRESS}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="phone"
                        tdAttr={{ 'data-id': 'subscriber-phone' }}
                        dataAlign="center"
                        headerAlign="center"
                        searchable
                    >
                        {COMPONENTS.FIELDS.USER.PHONE_NUMBER}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="notificationsList"
                        tdAttr={{ 'data-id': 'subscriber-notifications' }}
                        dataAlign="center"
                        headerAlign="center"
                        searchable
                    >
                        {NAMES_OF_TABS.NOTIFICATIONS}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="82"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getSubscribersFormatted = createDeepEqualSelector(
    [
        Notification.getAllNotifications,
        Subscriber.getAllSubscribers,
    ],
    (notifications, subscribers) => subscribers.map(subscriber => subscriber.formattedSubscriber(notifications)),
);

const mapStateToProps = state => ({
    subscribers: getSubscribersFormatted(state),
    selectedItems: state.adminPage.selectedItems,
});

export default connect(mapStateToProps)(injectIntl(SubscribersTable));
