import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import _OrderBy from 'lodash/orderBy';

import { trOddEvenClassFormat, colorFormatter, getCategoryText } from '../../../utils/htmlUtils';
import { filterTrackablesByGroups } from '../../../utils';
import {
    DEFAULT_VALUES,
    TABLES,
    COMPONENTS,
    BOOTSTRAP_TABLE_SORT,
    NAMES_OF_TABS,
} from '../../../common';

import {
    openTrackableDetailsModal,
    openGroupDetachModal,
    selectItems,
    unselectItems,
    changeSelectedItems,
    openBulkEditGroupsModal,
} from '../../../stores/actions';

import { Trackable, Group } from '../../../stores/classes';

class TrackableReadonlyTable extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        selectedTrackables: PropTypes.arrayOf(PropTypes.shape()),
        trackables: PropTypes.arrayOf(PropTypes.shape()),
        selectedGroup: PropTypes.shape(),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        intl: {
            formatMessage: () => {
            },
        },
        selectedTrackables: [],
        trackables: [],
        selectedGroup: [],
    };

    constructor(props) {
        super(props);

        this.state = {
            sortOrder: '',
        };
    }

    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                onClick={this.showInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                data-cy="group-trackable-table-info-button"
                aria-hidden="true"
            />

            {this.props.selectedGroup &&
            <i
                data-info="actions-section"
                onClick={this.openDetachGroupModal.bind(this, row)}
                title="Delete"
                className="fa fa-times inner-table-action"
                data-cy="group-trackable-table-remove-button"
                aria-hidden="true"
            />}
        </section>
    );

    createAddDeleteBtn = () => {
        const { selectedGroup, trackables } = this.props;
        if (selectedGroup) {
            return (
                <section className="Table">
                    {
                        (trackables && trackables.length > 0) && <i
                            className="fas fa-trash-alt fa-lg table-action-icon"
                            data-cy="group-trackable-table-batch-remove-button"
                            onClick={this.openMultiDetachGroupModal}
                        />
                    }
                    <i
                        className="fa fa-plus fa-lg table-action-icon"
                        data-cy="group-trackable-table-batch-add-button"
                        onClick={this._openBulkEditGroupsModal}
                    />
                </section>);
        }
        return (
            <section className="Table" style={{ opacity: 0 }}>
                <i
                    className="fas fa-trash-alt fa-lg table-action-icon"
                />
            </section>);
    };
    
    showInfo = (row) => {
        this.props.dispatch(openTrackableDetailsModal(row));
    };

    _openBulkEditGroupsModal = () => {
        this.props.dispatch(openBulkEditGroupsModal());
    };

    openMultiDetachGroupModal = () => {
        const params = [];
        this.props.selectedTrackables.forEach((item) => {
            const trackable = this.props.trackables.find(t => t.uid === item.uid);
            if (trackable) {
                params.push(trackable);
            }
        });
        if (params.length > 0) {
            this.props.dispatch(openGroupDetachModal(this.props.selectedGroup, params));
        }
    };
    
    openDetachGroupModal = (trackable) => {
        this.props.dispatch(openGroupDetachModal(this.props.selectedGroup, trackable));
    };

    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        if (!isSelected) {
            this.forceUpdate();
        }
        return undefined;
    };
    
    handleSelectAll = (isSelected, nodes) => {
        if (isSelected) {
            this.props.dispatch(selectItems(nodes));
        } else {
            this.props.dispatch(unselectItems(nodes));
            this.forceUpdate();
        }
    };
    
    groupColorFormatter = (groups) => {
        if (this.props.selectedGroup) {
            // move selected group to begin of list for all trackable
            const index = groups.findIndex(x => x.uid === this.props.selectedGroup.uid);
            if (index >= 0) {
                const items = groups.splice(index, 1);
                // console.warn(items);
                groups = items.concat(groups);
            }
        }

        const { sortOrder } = this.state;
        const sortedGroups = sortOrder ? _OrderBy(groups, 'name', sortOrder) : groups;

        return (
            <div className="group-column-text">
                <div className="groups-of-trackable">
                    {sortedGroups.map((x, i, arr) => (
                        <span key={x.uid} className="group-name-wrapper d-flex">
                            {colorFormatter(x.color)}
                            <span className="group-name multiple-lines-ellipsis">{x.name} {i < arr.length - 1 ? ', ' : ''}</span>
                        </span>
                    ))}
                </div>
            </div>);
    };
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="companies-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    categoryFormatter = cell => (getCategoryText(cell, this.props.intl));

    sortGroupNames = (a, b, order) => {
        const { sortOrder } = this.state;
        if (sortOrder !== order) {
            this.setState({ sortOrder: order });
        }
        return (order === 'asc')
            ? a.groupList.localeCompare(b.groupList)
            : b.groupList.localeCompare(a.groupList);
    };

    render() {
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true, // click to select, default is false
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            selected: this.props.selectedTrackables.map(x => x.uid),
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
            columnWidth: '40px',
        };
        const { trackables } = this.props;

        return (
            <section className="AppLayout-Admin-Tab tab-for-admin-trackables">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.TRACKABLE_OBJECTS}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="trackables-readonly-button-plus"
                >
                    {COMPONENTS.FIELDS.NODE.TOTAL_COUNT_FUNC(trackables.length)}
                </h4>
                <hr />
                <BootstrapTable
                    data={trackables}
                    options={options}
                    selectRow={this.props.selectedGroup ? selectRow : {}}
                    trClassName={trOddEvenClassFormat}
                    tableBodyClass="automation-trackable-readonly-table"
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    condensed
                    search
                    hover
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'trackables-table-name' }}
                        dataAlign="left"
                        width="100px"
                        headerAlign="center"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="groupObjectList"
                        tdAttr={{ 'data-id': 'trackables-table-groups' }}
                        dataAlign="left"
                        width="150px"
                        hidden={false}
                        headerAlign="center"
                        dataSort
                        sortFunc={this.sortGroupNames}
                        dataFormat={this.groupColorFormatter}
                        className="groups-col"
                    >
                        {NAMES_OF_TABS.GROUPS}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="category"
                        tdAttr={{ 'data-id': 'trackables-table-category' }}
                        dataAlign="left"
                        width="70px"
                        hidden={false}
                        headerAlign="center"
                        dataSort
                        dataFormat={this.categoryFormatter}
                        columnClassName="capitalizedText"
                    >
                        {COMPONENTS.FIELDS.NODE.CATEGORY}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        headerAlign="center"
                        dataField="groupList"
                        dataAlign="left"
                        hidden
                        dataSort
                    />
                    <TableHeaderColumn
                        width="40px"
                        headerAlign="center"
                        dataAlign="center"
                        dataFormat={this.getActionsIcons.bind(this)}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </section>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getSelectedItems = state => state.adminPage.selectedItems;

const getSelectedGroups = createDeepEqualSelector(
    [Group.getAllGroups, getSelectedItems],
    (groups, selectedItems) => (selectedItems.filter(item => groups[item.uid])),
);

const getTrackables = state => Trackable.getAllTrackablesArray(state).map(trackable => trackable.formattedTrackableWithoutPosition().getProperties());
// todo:  handle other Trackable.getTrackablesWithoutPositions

const getSelectedTrackables = createDeepEqualSelector(
    [getTrackables, getSelectedItems],
    (trackables, selectedItems) => (selectedItems.filter(item => trackables.some(trackable => trackable.uid === item.uid))),
);

const getFilteredTrackables = createDeepEqualSelector(
    [getTrackables, Group.getAllGroups, getSelectedGroups],
    (trackables, groups, selectedGroups) => (
        filterTrackablesByGroups(trackables, selectedGroups)
            .map((trackable) => {
                const groupObjectList = trackable.groups
                    // in table column 'Groups' shows only group which selected on groupTable and not shown other even it exists
                    .filter(uid => selectedGroups.some(selectedGroup => uid === selectedGroup.uid))
                    .map(uid => groups[uid])
                    .filter(group => group);
                trackable.groupObjectList = groupObjectList;
                trackable.groupList = groupObjectList.map(i => i.name).join(', ');
                return trackable;
            })
    ),
);

const mapStateToProps = (state) => {
    const selectedGroups = getSelectedGroups(state);
    return {
        selectedGroup: selectedGroups.length === 1 ? selectedGroups[0] : null,
        selectedGroups,
        selectedTrackables: getSelectedTrackables(state),
        trackables: getFilteredTrackables(state),
    };
};

export default connect(mapStateToProps)(injectIntl(TrackableReadonlyTable));
