import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';

import withSortSelect from 'components/HOC/withSortSelect';
import { COMPONENTS } from 'common/ui-labels';
import { ZONE_SEVERITY } from 'common/constants';
import { openZoneDeleteModal, openZoneDetailsModal, openZoneEditAdminPageModal } from 'stores/actions';
import { isCustomZone, isInfoZone } from 'utils/zoneUtils';
import { formatDbNameToHumanReadable, virtualizedHeaderRenderer } from 'utils/htmlUtils';
import { zoneSeverityColor } from 'utils/simpleUtils';
import TableButtons from './TableButtons';

class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trClassFormat: PropTypes.func,
        trStyleFormat: PropTypes.func,
        checkboxButton: PropTypes.func,
        headerCheckbox: PropTypes.func,
        setFilterItems: PropTypes.func,
        handleSelectRow: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape()),

        // Props from parent
        allItems: PropTypes.shape(),
        totalCount: PropTypes.number,
        tableSize: PropTypes.shape(),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trClassFormat: () => { },
        trStyleFormat: () => { },
        checkboxButton: () => { },
        headerCheckbox: () => { },
        setFilterItems: () => { },
        handleSelectRow: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],

        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        tableSize: {},
        selectedItems: [],
        dispatch: () => { },
        intl: {
            formatMessage: () => { },
        },
    };

    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;

        // HOC Methods
        this.trClassFormat = props.trClassFormat.bind(this);
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.handleSelectRow = props.handleSelectRow.bind(this);
        this.checkboxButton = props.checkboxButton.bind(this);
        this.headerCheckbox = props.headerCheckbox.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }

    getActionsIcons = row => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                key="actions-section-info"
                data-id="zones-actions-info"
                onClick={this.showZoneInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="zones-actions-edit"
                key="zones-actions-edit"
                onClick={this.openEditModal.bind(this, row)}
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="zones-actions-remove"
                key="zones-actions-remove"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    openDeleteModal = (row) => {
        this.props.dispatch(openZoneDeleteModal(row));
    };

    showZoneInfo = (row) => {
        this.props.dispatch(openZoneDetailsModal(row));
    };

    openEditModal = (row) => {
        this.props.dispatch(openZoneEditAdminPageModal(row));
    };

    formatZoneAlarmLevel = (row) => {
        if (isCustomZone(row) || isInfoZone(row)) {
            return null;
        }
        return (
            <div style={{
                backgroundColor: zoneSeverityColor(row.zone_alarm_level),
                textAlign: 'center',
                width: '90%',
                color: row.zone_alarm_level === ZONE_SEVERITY.GUARDED ? 'black' : 'white',
            }}
            >
                {row.zone_alarm_level}
            </div>
        );
    };

    formatIsActive = (row) => {
        const active = COMPONENTS.FIELDS.COMMON.ACTIVE;
        const inActive = COMPONENTS.FIELDS.COMMON.INACTIVE;
        return <div style={{ textAlign: 'center' }}>{row.active ? active : inActive}</div>;
    };

    render() {
        const { sortBy, sortDirection, tableSize, allItems, selectedItems, filteredItems, intl } = this.props;
        const allItemsArray = [...allItems.values()];
        const displayItems = this.sortDisplayItems(filteredItems);
        const displayCount = displayItems.length;

        const allColumnsWidthExceptName = tableSize.width - 890;

        return (
            <section style={{ width: tableSize.width }}>
                <TableButtons
                    items={allItemsArray}
                    width={tableSize.width}
                    setFilterItems={this.setFilterItems}
                    selectedItems={selectedItems}
                    searchFields={[
                        'name',
                        'location_name_display',
                        'sublocation_name_display',
                        'zone_type_name_display',
                        'active_display',
                    ]}
                />
                <Table
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Big"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat(displayItems, { index })}
                    onRowClick={({ event, index, rowData }) => this.handleSelectRow(displayCount, { event, index, rowData })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    <Column
                        label={<span title="">{this.headerCheckbox(displayItems)}</span>}
                        dataKey="checkbox"
                        cellRenderer={({ rowData }) => this.checkboxButton(rowData, displayCount)}
                        width={50}
                        disableSort
                        headerClassName="fixed-width"
                        className="Virtual-Table-Column-Checkbox first-column fixed-width"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.NAME}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="name"
                        id="zones-name"
                        width={allColumnsWidthExceptName >= 200 ? allColumnsWidthExceptName : 200}
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={180}
                        label={<span title="">{COMPONENTS.FIELDS.SITE.SITE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="location_name_display"
                        id="zones-location_name"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={180}
                        label={<span title="">{COMPONENTS.FIELDS.SITE.AREA}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="sublocation_name_display"
                        id="zones-sublocation_name"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={200}
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.TYPE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ rowData, cellData }) => formatDbNameToHumanReadable(intl, rowData.zone_type_name, cellData)}
                        dataKey="zone_type_name_display"
                        id="zones-type"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={150}
                        label={<span title="">{COMPONENTS.FIELDS.ZONE.SEVERITY_LEVEL}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.formatZoneAlarmLevel(rowData)}
                        dataKey="zone_alarm_level"
                        id="zones-severity"
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        width={100}
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.STATUS}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.formatIsActive(rowData)}
                        dataKey="active_display"
                        id="zones-activity"
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        width={110}
                        dataKey="actions"
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.ACTIONS}</span>}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.getActionsIcons(rowData)}
                        id="actions"
                        disableSort
                        className="Virtual-Table-Column-Center"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
