import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { DEFAULT_VALUES, TRACKABLE_CATEGORY } from 'common/constants';
import { TABLES, REPORT_TABLE, COMPONENTS } from 'common/ui-labels';
import { trOddEvenClassFormat, formatDbNameToHumanReadable } from 'utils/htmlUtils';


function AlarmsTable(props) {
    const { data, intl, loading } = props;

    const categoryFormatter = (cell) => {
        if (!cell) {
            return formatDbNameToHumanReadable(intl, TRACKABLE_CATEGORY.UNSPECIFIED, TRACKABLE_CATEGORY.UNSPECIFIED);
        }
        return formatDbNameToHumanReadable(intl, cell, cell);
    };

    const createCustomSearchField = () => (
        <SearchField
            className="custom-search-field"
            data-id="projects-search"
            data-cy="alarm-widget-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );


    const tableOptions = {
        onlyOneExpanding: true,
        noDataText: (loading ? <i className="fa fa-spinner fa-spin" style={{ color: 'grey' }} /> : TABLES.NO_DATA_TO_DISPLAY),
        expandBy: 'column',
        expandRowBgColor: 'white',
        searchField: createCustomSearchField,
        beforeShowError: () => true,
    };
    const headerStyle = {
        marginBottom: '0',
    };
    const selectRow = {
        mode: 'radio',
        hideSelectColumn: true,
        bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
    };

    return (
        <section>
            <BootstrapTable
                trClassName={(row, rowIndex) => (`${trOddEvenClassFormat(row, rowIndex)}`)}
                tableBodyClass="automation-companies-table"
                data={data}
                options={tableOptions}
                selectRow={selectRow}
                headerStyle={headerStyle}
                bodyStyle={{
                    maxHeight: window.innerHeight * 0.45,
                    overflow: 'scroll',
                    overflowX: 'auto',
                    overflowY: 'auto',
                }}
                bodyContainerClass="Admin-BootstrapTable"
                expandColumnOptions={{
                    expandColumnVisible: false,
                    columnWidth: 30,
                }}
                hover
                condensed
                search
                className="Widget-Table middle-align"
            >
                <TableHeaderColumn row="0" hidden />
                <TableHeaderColumn
                    row="0"
                    rowSpan="2"
                    dataField="name"
                    tdAttr={{ 'data-id': 'trackable-name' }}
                    headerAlign="center"
                    dataAlign="letf"
                    columnTitle
                    width="230px"
                >
                    {COMPONENTS.FIELDS.COMMON.NAME}
                </TableHeaderColumn>
                <TableHeaderColumn
                    row="0"
                    rowSpan="2"
                    headerAlign="center"
                    dataField="category"
                    dataAlign="center"
                    tdAttr={{ 'data-id': 'trackable-category' }}
                    dataFormat={categoryFormatter}
                    dataSort
                    width="80"
                >
                    {COMPONENTS.FIELDS.NODE.CATEGORY}
                </TableHeaderColumn>
                <TableHeaderColumn
                    row="0"
                    colSpan="2"
                    headerAlign="center"
                >
                    <span className="capitalizedText">{REPORT_TABLE.ALARMS_TABLE.HEADER_ZONE_VIOLATIONS}</span>
                </TableHeaderColumn>
                <TableHeaderColumn
                    row="0"
                    colSpan="2"
                    headerAlign="center"
                >
                    <span className="capitalizedText">{REPORT_TABLE.ALARMS_TABLE.HEADER_UNCERTIFIED_OPERATIONS}</span>
                </TableHeaderColumn>
                <TableHeaderColumn
                    row="0"
                    colSpan="2"
                    headerAlign="center"
                >
                    <span className="capitalizedText">{REPORT_TABLE.ALARMS_TABLE.HEADER_EMERGENCY_CALLS}</span>
                </TableHeaderColumn>
                <TableHeaderColumn
                    row="1"
                    dataField="uid"
                    searchable={false}
                    hidden
                    isKey
                >
                    Uid
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="zoneViolations"
                    tdAttr={{ 'data-id': 'zone-violations' }}
                    dataSort
                    headerAlign="center"
                    dataAlign="center"
                    searchable={false}
                    row="1"
                    width="70"
                >
                    {COMPONENTS.FIELDS.COMMON.COUNT}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="zoneViolationsDuration"
                    tdAttr={{ 'data-id': 'zone-violations-duration' }}
                    headerAlign="center"
                    dataAlign="center"
                    dataSort
                    searchable={false}
                    row="1"
                    width="90"
                >
                    {COMPONENTS.FIELDS.COMMON.DURATION}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="uncertifiedOps"
                    tdAttr={{ 'data-id': 'uncertified-operations' }}
                    dataSort
                    headerAlign="center"
                    dataAlign="center"
                    searchable={false}
                    row="1"
                    width="70"
                >
                    {COMPONENTS.FIELDS.COMMON.COUNT}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="uncertifiedOpsDuration"
                    tdAttr={{ 'data-id': 'uncertified-operations-duration' }}
                    dataSort
                    headerAlign="center"
                    dataAlign="center"
                    searchable={false}
                    row="1"
                    width="90"
                >
                    {COMPONENTS.FIELDS.COMMON.DURATION}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="emergencyCalls"
                    tdAttr={{ 'data-id': 'emergency-calls' }}
                    dataSort
                    headerAlign="center"
                    dataAlign="center"
                    searchable={false}
                    row="1"
                    width="70"
                >
                    {COMPONENTS.FIELDS.COMMON.COUNT}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="emergencyCallsDuration"
                    tdAttr={{ 'data-id': 'emergency-calls-duration' }}
                    dataSort
                    headerAlign="center"
                    dataAlign="center"
                    searchable={false}
                    row="1"
                    width="90"
                >
                    {COMPONENTS.FIELDS.COMMON.DURATION}
                </TableHeaderColumn>
            </BootstrapTable>
        </section>
    );
}

AlarmsTable.propTypes = {
    data: PropTypes.arrayOf(PropTypes.shape({})),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    loading: PropTypes.bool,
};

AlarmsTable.defaultProps = {
    data: [],
    intl: {
        formatMessage: () => {},
    },
    loading: false,
};

export default injectIntl(AlarmsTable);
