import React, { useEffect } from 'react';
import { connect } from 'react-redux';
import ScrollBar from 'react-scrollbar';
import PropTypes from 'prop-types';
import { TABLES } from 'common/ui-labels';
import { getAlarmsDashboard } from 'stores/actions/dashboardActions';
import { getActiveShiftTimeFrame, getTimeZoneOfProject } from 'utils/timeUtils';
import AlarmsTable from './AlarmsTable';
import Header from './Header';


function AlarmWidget(props) {
    const {
        dashboardAlarms: { data, loading, updatedAt },
        selectedSublocation = {},
        currentActiveShift,
        timezone,
        widget: { isExpanded },
    } = props;
    const sublocationUid = selectedSublocation && selectedSublocation.uid;
    const { startUtc, endUtc } = getActiveShiftTimeFrame(currentActiveShift, timezone);

    useEffect(() => {
        props.dispatch(getAlarmsDashboard({ from: startUtc, to: endUtc, area_uid: sublocationUid }));

        const interval = setInterval(() => {
            props.dispatch(getAlarmsDashboard({ from: startUtc, to: endUtc, area_uid: sublocationUid }));
        }, 5 * 60 * 1000);

        return function cleanup() {
            clearInterval(interval);
        };
    }, [startUtc, endUtc, sublocationUid]);

    if (!currentActiveShift) {
        return (
            <div
                className="text-center"
                data-cy="alarm-no-data"
            >
                {TABLES.NO_DATA_TO_DISPLAY}
            </div>
        );
    }
    return (
        <ScrollBar
            className={`DashboardWidget ${isExpanded ? 'expanded' : ''}`}
            contentClassName="content"
            smoothScrolling
            horizontal={false}
        >
            <Header
                isExpanded={isExpanded}
                updatedAt={updatedAt}
                loading={loading}
                zoneViolations={data && data.zoneViolations}
                uncertifiedOps={data && data.uncertifiedOps}
                emergencyCalls={data && data.emergencyCalls}
            />
            {
                isExpanded && <AlarmsTable
                    loading={loading}
                    data={data ? data.trackables : []}
                />
            }
        </ScrollBar>
    );
}


const mapStateToProps = state => ({
    timezone: getTimeZoneOfProject(state.appContext.project),
    dashboardAlarms: state.dashboard.alarms,
    selectedSublocation: state.siteView.selectedSublocation,
});

AlarmWidget.propTypes = {
    dispatch: PropTypes.func,
    timezone: PropTypes.string,
    currentActiveShift: PropTypes.shape({}),
    dashboardAlarms: PropTypes.shape({
        error: PropTypes.shape({}),
        loading: PropTypes.bool,
        data: PropTypes.arrayOf({}),
        updatedAt: PropTypes.string,
    }),
    selectedSublocation: PropTypes.shape({}),
    widget: PropTypes.shape({
        isExpanded: PropTypes.bool,
    }),
};

AlarmWidget.defaultProps = {
    dispatch: () => {},
    timezone: 'Etc/GMT',
    currentActiveShift: null,
    dashboardAlarms: {
        error: null,
        loading: false,
        data: null,
        updatedAt: '',
    },
    selectedSublocation: null,
    widget: {
        isExpanded: false,
    },
};

export default connect(mapStateToProps)(AlarmWidget);
