import PropTypes from 'prop-types';
import _isEmpty from 'lodash/isEmpty';
import React, { Component } from 'react';
import { BootstrapTable, TableHeaderColumn } from 'react-bootstrap-table';
import { DEFAULT_VALUES, TABLES, BOOTSTRAP_TABLE_SORT, DASHBOARD_WIDGETS, COMPONENTS } from 'common';
import expandColumnComponent from 'components/ExpandColumnComponent';
import ExpandableTable from './ExpandableTable';


class BSTable extends React.Component {
    static propTypes = {
        data: PropTypes.arrayOf(PropTypes.shape()),
    }
    
    static defaultProps = {
        data: [],
    }
    
    expandComponent = row => (<ExpandableTable data={row.expand} />)
    
    trClassFormat = (row, rowIndex) => ((rowIndex % 2 === 0) ? 'tr-odd' : 'tr-even');
    
    render() {
        const { data } = this.props;
        if (!data) return null;
        
        const dataColumnWidth = 100;
        const columnSettings = (width = dataColumnWidth) => ({ dataAlign: 'center', width: `${width}px` });
        
        const options = {
            onlyOneExpanding: true,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            expandBy: 'column',
            expandRowBgColor: 'white',
            expandBodyClass: 'expandable-tr',
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        
        return (
            <BootstrapTable
                data={data}
                options={options}
                expandableRow={row => !!row.expand}
                expandComponent={this.expandComponent}
                expandColumnOptions={{
                    expandColumnVisible: true,
                    expandColumnComponent,
                    columnWidth: 30,
                }}
                bodyStyle={{
                    overflow: 'scroll',
                    overflowX: 'auto',
                    overflowY: 'auto',
                }}
                trClassName={this.trClassFormat}
                hover
                striped
                condensed
            >
                <TableHeaderColumn dataField="sublocation_uid" hidden isKey />
                <TableHeaderColumn dataField="name" width="130px" />
                <TableHeaderColumn dataField="people_active" {...columnSettings()} />
                <TableHeaderColumn dataField="vehicle_active" {...columnSettings()} />
                <TableHeaderColumn dataField="others_actual" {...columnSettings()} />
            </BootstrapTable>
        );
    }
}


class SummaryTable extends Component {
    static propTypes = {
        tableData: PropTypes.arrayOf(PropTypes.shape()),
    };

    static defaultProps = {
        tableData: [],
    };

    expandComponent = row => (<BSTable data={row.expand} />)

    render() {
        const { tableData } = this.props;

        if (_isEmpty(tableData)) {
            return (
                <div
                    data-cy="current-shift-summary-no-data"
                >
                    {TABLES.NO_DATA_TO_DISPLAY}
                </div>
            );
        }

        const options = {
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            expandBy: 'column',
            expandRowBgColor: 'white',
            expandBodyClass: 'expandable-tr',
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };

        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            hideSelectColumn: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToExpand: true,
        };

        const row0ColumnAligning = { headerAlign: 'center' };
        const row1ColumnAligning = { headerAlign: 'center', dataAlign: 'center', width: '50px' };

        return (
            <div className="location-summary-table">
                <BootstrapTable
                    data={tableData}
                    className="Widget-Table middle-align"
                    tableBodyClass="automation-zones-site-table"
                    options={options}
                    selectRow={selectRow}
                    expandableRow={() => true}
                    expandComponent={this.expandComponent}
                    headerStyle={headerStyle}
                    bodyStyle={{
                        overflow: 'scroll',
                        overflowX: 'auto',
                        overflowY: 'auto',
                    }}
                    expandColumnOptions={{
                        expandColumnVisible: true,
                        expandColumnComponent,
                        columnWidth: 30,
                    }}
                    hover
                    striped
                    condensed
                >
                    <TableHeaderColumn row="0" hidden />
                    <TableHeaderColumn row="0" rowSpan="2" dataField="name" width="130px" {...row0ColumnAligning}>
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="0" colSpan="2" {...row0ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.PEOPLE}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="0" colSpan="2" {...row0ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.VEHICLES}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="0" colSpan="2" {...row0ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.OTHERS}
                    </TableHeaderColumn>

                    <TableHeaderColumn row="1" dataField="location_uid" hidden isKey>
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="1" dataField="people_active" {...row1ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.ACTIVE}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="1" dataField="people_expected" {...row1ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.EXPECTED}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="1" dataField="vehicle_active" {...row1ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.ACTIVE}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="1" dataField="vehicle_total" {...row1ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.EXPECTED}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="1" dataField="others_actual" {...row1ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.ACTIVE}
                    </TableHeaderColumn>
                    <TableHeaderColumn row="1" dataField="others_expected" {...row1ColumnAligning}>
                        {DASHBOARD_WIDGETS.CURRENT_SHIFT_WIDGET.EXPECTED}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

export default SummaryTable;
