import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { BootstrapTable, SearchField, TableHeaderColumn } from 'react-bootstrap-table';
import { TABLES, REPORT_TABLE, DASHBOARD_WIDGETS, DB_NAMES, SITE_ZONES_TABLE } from 'common';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import TimeLine from 'components/TimeLine';
import AttendanceScale from './AttendanceScale';
import AttendanceScaleHours from './AttendanceScaleHours';


class AttendanceTable extends React.Component {
    static propTypes = {
        data: PropTypes.arrayOf(PropTypes.shape({})),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        timezone: PropTypes.string,
        from: PropTypes.number,
        to: PropTypes.number,
        currentActiveShift: PropTypes.shape({}),
        parentWrapperHtmlEL: PropTypes.shape({}),
    };
    
    static defaultProps = {
        data: [],
        intl: {
            formatMessage: () => {
            },
        },
        timezone: 'Etc/GMT',
        from: 0,
        to: 0,
        currentActiveShift: undefined,
        parentWrapperHtmlEL: {

        },
    };
    
    constructor(props) {
        super(props);
        this.state = {
            timeLineWidth: 450,
            tableHeight: 100,
        };
        this.myRef = React.createRef();
    }

    componentDidMount() {
        window.addEventListener('resize', this.changeTimelineWidth);
    }
    
    componentDidUpdate() {
        const element = document.querySelector('[data-id=person-widget-timeline]');
        if (element && element.offsetWidth !== this.state.timeLineWidth + 50) {
            this.setState({ timeLineWidth: element.offsetWidth ? element.offsetWidth - 50 : 500 });
        }
        if (this.myRef.current) {
            const position = this.myRef.current.getBoundingClientRect();
            const tableHeight = window.innerHeight - position.top - 130;
            if (this.state.tableHeight !== tableHeight) {
                this.setState({ tableHeight });
            }
        }
    }
    
    componentWillUnmount() {
        window.removeEventListener('resize', this.changeTimelineWidth);
    }

    createCustomSearchField = intl => (
        <SearchField
            className="custom-search-field"
            data-id="projects-search"
            data-cy="attendance-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    changeTimelineWidth = () => {
        const element = document.querySelector('[data-id=person-widget-timeline]');
        this.setState({ timeLineWidth: element.offsetWidth ? element.offsetWidth - 50 : 500 });
    };

    timelineComponent = (cell, row) => {
        const { from, to, timezone, parentWrapperHtmlEL } = this.props;
        const { timeLineWidth } = this.state;

        return (
            <Fragment>
                <AttendanceScale start={from} end={to} width={timeLineWidth} />
                <TimeLine
                    width={timeLineWidth}
                    timeline={row.timeline}
                    from={from.valueOf()}
                    to={to.valueOf()}
                    name={row.person_name}
                    timezone={timezone}
                    parentWrapperHtmlEL={parentWrapperHtmlEL}
                />
            </Fragment>
        );
    };
    
    render() {
        const { data, from, to, intl, timezone, currentActiveShift } = this.props;
        const { timeLineWidth, tableHeight } = this.state;

        if (!currentActiveShift) {
            return '';
        }

        const options = {
            onlyOneExpanding: false,
            noDataText: TABLES.NO_DATA_TO_DISPLAY,
            expandBy: 'column',
            expandRowBgColor: 'white',
            searchField: this.createCustomSearchField.bind(this, intl),
            beforeShowError: () => true,
        };
        const headerStyle = {
            marginBottom: '0',
        };

        const columnSettings = (fieldName, _width = 90, rowSpan = 2) => ({
            dataField: fieldName,
            tdAttr: { 'data-id': `person-widget-${fieldName}` },
            dataSort: true,
            searchable: true,
            headerAlign: 'center',
            dataAlign: 'center',
            row: 0,
            rowSpan,
            width: `${_width}px`,
        });

        return (
            <div ref={this.myRef} className="personnel-activity-table">
                <BootstrapTable
                    trClassName={(row, rowIndex) => (`${trOddEvenClassFormat(row, rowIndex)}`)}
                    tableBodyClass="automation-companies-table"
                    data={data}
                    options={options}
                    headerStyle={headerStyle}
                    bodyStyle={{
                        maxHeight: tableHeight,
                        overflow: 'scroll',
                        overflowX: 'auto',
                        overflowY: 'auto',
                    }}
                    bodyContainerClass="Admin-BootstrapTable"
                    hover
                    condensed
                    search
                    className="Widget-Table middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        Uid
                    </TableHeaderColumn>
                    <TableHeaderColumn {...columnSettings('person_name', 200)}>
                        {DB_NAMES.PERSON(intl)}
                    </TableHeaderColumn>
                    <TableHeaderColumn {...columnSettings('checkin_time')}>
                        {DASHBOARD_WIDGETS.PERSON_ACTIVITY_WIDGET.CHECK_IN(intl)}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        {...columnSettings('status')}
                        columnClassName="text-capitalize"
                    >
                        {SITE_ZONES_TABLE.TEXT.STATUS_FUNC(intl)}
                    </TableHeaderColumn>
                    <TableHeaderColumn {...columnSettings('active_time')}>
                        {DASHBOARD_WIDGETS.PERSON_ACTIVITY_WIDGET.ACTIVE_TIME(intl)}
                    </TableHeaderColumn>
                    <TableHeaderColumn {...columnSettings('vehicle')}>
                        {DB_NAMES.VEHICLE(intl)}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        columnClassName="no-background pos-r no-border-t-b padding-0"
                        dataFormat={this.timelineComponent}
                        {...columnSettings('timeline', 560, 1)}
                    >
                        {REPORT_TABLE.ATTENDANCE_TABLE.HEADER_TIMELINE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        row="1"
                        dataField="timeline"
                        className="no-border-b padding-0 va-bottom"
                        thStyle={{ height: '15px' }}
                        columnClassName="no-background pos-r no-border-t-b padding-0"
                        tdAttr={{ 'data-id': 'person-widget-timeline' }}
                        dataFormat={this.timelineComponent}
                        headerAlign="left"
                        dataAlign="center"
                        width="560"
                    >
                        <AttendanceScaleHours
                            start={from}
                            end={to}
                            width={timeLineWidth}
                            timezone={timezone}
                        />
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

export default injectIntl(AttendanceTable);
