import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import ScrollBar from 'react-scrollbar';
import { injectIntl } from 'react-intl';
import moment from 'moment';
import _isEqual from 'lodash/isEqual';
import { getAttendanceDashboardTimeline, minToMs, getTimeZoneOfProject, getActiveShiftTimeFrame } from 'utils/timeUtils';
import * as api from 'api/index';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import { TABLES } from 'common/ui-labels';
import AttendanceTable from './AttendanceTable';
import Header from './Header';


class PersonActivityWidget extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        timezone: PropTypes.string,
        trackables: PropTypes.shape({}),
        selectedLocation: PropTypes.shape({}),
        associations: PropTypes.shape({}),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        widget: PropTypes.shape({
            isExpanded: PropTypes.bool,
        }),
        currentActiveShift: PropTypes.shape({}),
    };

    static defaultProps = {
        dispatch: () => {},
        timezone: '',
        trackables: {},
        selectedLocation: {},
        associations: {},
        intl: {
            formatMessage: () => {
            },
        },
        widget: {
            isExpanded: false,
        },
        currentActiveShift: null,
    };

    constructor() {
        super();
        this.state = {
            shiftActivities: null,
            updatedAt: null,
            loading: false,
        };
        this.interval = null;
        this.now = Date.now();
        this.parentWrapperHtmlEL = React.createRef();
    }

    componentDidMount() {
        this.getActivities();

        clearInterval(this.interval);
        this.interval = setInterval(() => {
            this.getActivities();
            this.now = Date.now();
        }, minToMs(5));
    }

    shouldComponentUpdate(nextProps, nextState) {
        return !_isEqual(this.props.selectedLocation, nextProps.selectedLocation)
            || !_isEqual(this.props.trackables, nextProps.trackables)
            || !_isEqual(this.props.widget, nextProps.widget)
            || !_isEqual(this.props.currentActiveShift, nextProps.currentActiveShift)
            || !_isEqual(this.state.shiftActivities, nextState.shiftActivities);
    }

    componentWillUnmount() {
        clearInterval(this.interval);
    }
    
    getActivities = () => {
        const { selectedLocation } = this.props;
        if (!selectedLocation) return;

        this.setState({ loading: true });
        api.getLocationShiftActivities(selectedLocation.uid)
            .then((shiftActivities) => {
                this.setState({ shiftActivities, updatedAt: new Date().toISOString(), loading: false });
            })
            .catch((err) => {
                this.setState({ loading: false });
                errorHandlerGetRequest(err, this.props.dispatch);
            });
    };

    render() {
        const {
            timezone,
            trackables,
            associations,
            selectedLocation,
            intl,
            currentActiveShift,
            widget: { isExpanded },
        } = this.props;

        const { shiftActivities, loading, updatedAt } = this.state;
        const { startUtc, endUtc } = getActiveShiftTimeFrame(currentActiveShift, timezone);
        const showAttendance = shiftActivities && selectedLocation && currentActiveShift;

        const {
            timelines,
            drivers,
            nonDrivers,
        } = getAttendanceDashboardTimeline(shiftActivities, trackables, associations, moment(startUtc), this.now, timezone, intl);

        const noDataToDisplay = !currentActiveShift || !showAttendance;

        return (
            <ScrollBar
                ref={ref => this.parentWrapperHtmlEL = ref}
                className={`DashboardWidget ${isExpanded ? 'expanded' : ''}`}
                contentClassName={`content ${noDataToDisplay ? 'd-flex va-middle ha-center' : ''}`}
                smoothScrolling
                horizontal={false}
            >
                {
                    noDataToDisplay ?
                        <div
                            className="text-center"
                            data-cy="personal-activity-no-data"
                        >
                            {TABLES.NO_DATA_TO_DISPLAY}
                        </div> :
                        <Fragment>
                            <Header
                                updatedAt={updatedAt}
                                drivers={drivers}
                                nonDrivers={nonDrivers}
                                loading={loading}
                                isExpanded={isExpanded}
                            />
                            {
                                showAttendance && isExpanded && <AttendanceTable
                                    currentActiveShift={currentActiveShift}
                                    timezone={timezone}
                                    data={timelines}
                                    from={startUtc}
                                    to={endUtc}
                                    parentWrapperHtmlEL={this.parentWrapperHtmlEL.content}
                                />
                            }
                        </Fragment>
                }
            </ScrollBar>
        );
    }
}

const mapStateToProps = state => ({
    selectedLocation: state.siteView.selectedLocation,
    trackables: state.entities.trackables.byUid,
    associations: state.entities.associations.byObjUid,
    timezone: getTimeZoneOfProject(state.appContext.project),
});

export default connect(mapStateToProps)(injectIntl(PersonActivityWidget));
