import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, SearchField, TableHeaderColumn } from 'react-bootstrap-table';
import { TABLES, REPORT_TABLE, TRACKABLE_STATUSES, TIMELINE_ACTIONS, COMPONENTS, NAMES_OF_TABS } from 'common';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { getTimeZoneOfProject } from 'utils/timeUtils';
import TimeLine from 'components/TimeLine';


class AttendanceTable extends React.Component {
    static propTypes = {
        data: PropTypes.arrayOf(PropTypes.shape({})),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        status: PropTypes.number,
        timezone: PropTypes.string,
        reportLoading: PropTypes.bool,
        report: PropTypes.shape({
            from: PropTypes.number,
            to: PropTypes.number,
            shift_name: PropTypes.string,
        }),
    };
    
    static defaultProps = {
        data: [],
        intl: {
            formatMessage: () => {
            },
        },
        status: 0,
        timezone: 'Etc/GMT',
        reportLoading: false,
        report: null,
    };
    
    constructor(props) {
        super(props);
        this.state = {
            width: 400,
            tableHeight: 100,
            htmlTableRef: undefined,
        };
        this.myRef = React.createRef();
    }

    componentDidMount() {
        window.addEventListener('resize', this.changeTimelineWidth);

        if (this.myRef !== null) {
            this.setState({ htmlTableRef: this.myRef });
        }
    }

    componentDidUpdate() {
        const element = document.querySelector('[data-id=attendance-table-timeline]');
        if (element && element.offsetWidth !== this.state.width) {
            this.setState({ width: element.offsetWidth || 400 });
        }
        if (this.myRef) {
            const position = this.myRef.getBoundingClientRect();
            const tableHeight = window.innerHeight - position.top - 130;
            if (this.state.tableHeight !== tableHeight) {
                this.setState({ tableHeight });
            }
        }
    }
    
    componentWillUnmount() {
        window.removeEventListener('resize', this.changeTimelineWidth);
    }
    
    createAddDeleteBtn = () => {
        if (this.props.status === 0) return <section />;
        return (
            <div className="row" style={{ marginLeft: '5px' }}>
                <div className="row text-row-small">
                    <div className="col-md-6"><b>{NAMES_OF_TABS.TRACKABLE_OBJECTS}:</b> {this.props.data.length}</div>
                </div>
            </div>);
    };
    
    createCustomSearchField = intl => (
        <Fragment>
            <div className="search-bar-wrapper">
                <SearchField
                    data-id="projects-search"
                    data-cy="attendance-table-search"
                    placeholder={TABLES.SEARCH(intl)}
                />
                <div className="timeline-legend-wrapper col-md-12 col-lg-12 col-sm-12">
                    <div className="timeline-legend">
                        {
                            Object.values(TRACKABLE_STATUSES).map((status, index) => (
                                <div key={index} className="status-wrapper">
                                    <div className={`status-color status-${status.toLowerCase()}`} />
                                    <div className="status-text">- {TIMELINE_ACTIONS[status]}</div>
                                </div>
                            ))
                        }
                    </div>
                </div>
            </div>

        </Fragment>
    );
    
    changeTimelineWidth = () => {
        const element = document.querySelector('[data-id=attendance-table-timeline]');
        this.setState({ width: element ? element.offsetWidth : 400 });
    }
    
    
    timelineComponent = (cell, row) => (
        <TimeLine
            width={this.state.width}
            timeline={row.timeline}
            from={row.from}
            to={row.to}
            name={row.name}
            timezone={this.props.timezone}
            parentWrapperHtmlEL={this.state.htmlTableRef}
        />
    );
    
    render() {
        const { data } = this.props;

        const options = {
            onlyOneExpanding: false,
            noDataText: (this.props.reportLoading ? <i className="fa fa-spinner fa-spin" style={{ color: 'grey' }} /> : TABLES.NO_DATA_TO_DISPLAY),
            expandBy: 'column',
            expandRowBgColor: 'white',
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            btnGroup: this.createAddDeleteBtn,
            beforeShowError: () => true,
        };
        const headerStyle = {
            marginBottom: '0',
        };

        return (
            <div ref={ref => this.myRef = ref}>
                <BootstrapTable
                    trClassName={(row, rowIndex) => (`${trOddEvenClassFormat(row, rowIndex)}`)}
                    tableBodyClass="automation-companies-table"
                    data={data}
                    options={options}
                    headerStyle={headerStyle}
                    bodyStyle={{
                        maxHeight: this.state.tableHeight,
                        overflow: 'scroll',
                        overflowX: 'auto',
                        overflowY: 'auto',
                    }}
                    bodyContainerClass="Admin-BootstrapTable"
                    hover
                    condensed
                    search
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        Uid
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'contact-trackable-name' }}
                        headerAlign="center"
                        dataSort
                        columnTitle
                        dataAlign="center"
                        width="250"
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="formattedTotalOnSite"
                        tdAttr={{ 'data-id': 'contact-trackable-name' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="100"
                        searchable
                    >
                        {REPORT_TABLE.ATTENDANCE_TABLE.HEADER_ON_SITE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="formattedTotalUnresponsive"
                        tdAttr={{ 'data-id': 'contact-trackable-name' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="120"
                        searchable
                    >
                        <span className="text-capitalize">{COMPONENTS.FIELDS.COMMON.UNRESPONSIVE}</span>
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="formattedTotalIdle"
                        tdAttr={{ 'data-id': 'contact-trackable-name' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="100"
                        searchable
                    >
                        <span className="text-capitalize">{REPORT_TABLE.ATTENDANCE_TABLE.HEADER_IDLE}</span>
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="formattedTotalActive"
                        columnClassName="Table-Button button-link"
                        tdAttr={{ 'data-id': 'company-table-name' }}
                        headerAlign="center"
                        dataSort
                        dataAlign="center"
                        width="100"
                    >
                        <span className="text-capitalize">{REPORT_TABLE.ATTENDANCE_TABLE.HEADER_ACTIVE}</span>
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="timeline"
                        tdAttr={{ 'data-id': 'attendance-table-timeline' }}
                        dataFormat={this.timelineComponent}
                        headerAlign="center"
                        dataAlign="center"
                        width="500"
                        searchable={false}
                    >
                        {REPORT_TABLE.ATTENDANCE_TABLE.HEADER_TIMELINE}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const mapStateToProps = state => ({
    report: state.reports.attendanceReport.report,
    data: state.reports.attendanceReport.tableData,
    status: state.reports.attendanceReport.status,
    reportLoading: state.reports.attendanceReport.reportLoading,
    timezone: getTimeZoneOfProject(state.appContext.project),
});

export default connect(mapStateToProps)(injectIntl(AttendanceTable));
