import React from 'react';
import _isEqual from 'lodash/isEqual';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { getReportsList, getReport, clearReport, loadReportList, loadReport } from 'stores/actions/reportsActions/generalReportActions';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { getTimeByFormatAndTimezone } from 'utils/timeUtils';
import { BootstrapTable, TableHeaderColumn } from 'react-bootstrap-table';
import { TABLES, SELECT_A_FUNC, COMPONENTS } from 'common/ui-labels';
import { DEFAULT_VALUES } from 'common/constants';


class ReportsList extends React.Component {
    static propTypes = {
        options: PropTypes.shape({
            from: PropTypes.string,
            to: PropTypes.string,
            locations: PropTypes.arrayOf(PropTypes.string),
        }),
        reports: PropTypes.arrayOf(PropTypes.shape({})),
        dispatch: PropTypes.func,
        timezone: PropTypes.string,
        reportListLoading: PropTypes.bool,
        type: PropTypes.string,
        report_uid: PropTypes.string,
    }
    
    static defaultProps = {
        options: {
            from: new Date().toISOString(),
            to: new Date().toISOString(),
            locations: [],
        },
        reports: [],
        dispatch: () => { },
        timezone: 'Etc/GMT',
        reportListLoading: false,
        type: '',
        report_uid: '',
    }
    
    componentDidUpdate(prevProps) {
        const { from, to, locations } = this.props.options;
        const fromChanged = prevProps.options.from !== this.props.options.from;
        const toChanged = prevProps.options.to !== this.props.options.to;
        const locationsChanged = !_isEqual(prevProps.options.locations, this.props.options.locations);
        
        if ((from && to && locations) && (fromChanged || toChanged || locationsChanged)) {
            const params = {
                from,
                to,
                location_uid: locations[0],
            };
            this.props.dispatch(loadReportList(this.props.type));
            this.props.dispatch(getReportsList(params, this.props.type));
        }
    }
    
    handleSelectRow = (row, isSelected) => {
        if (isSelected) {
            this.props.dispatch(loadReport(this.props.type));
            this.props.dispatch(getReport(row.uid, this.props.type));
        } else {
            this.props.dispatch(clearReport(this.props.type));
        }
        return undefined;
    };
    
    trClassFormat = () => 'Table-Row-CursorPointer';
    
    shiftFormatter = cell => cell.name || 'Auto-created';
    
    timeFormatter = cell => getTimeByFormatAndTimezone(cell, 'MM/DD/YYYY HH:mm', this.props.timezone)
    
    render() {
        const options = {
            onlyOneExpanding: false,
            noDataText: (this.props.reportListLoading ? <i className="fa fa-spinner fa-spin" style={{ color: 'grey' }} /> : TABLES.NO_AVAILABLE_REPORTS),
            expandBy: 'column',
            expandRowBgColor: 'white',
            btnGroup: this.createAddDeleteBtn,
            beforeShowError: () => true,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'radio',
            clickToSelect: true,
            hideSelectColumn: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToExpand: true,
            onSelect: this.handleSelectRow,
            selected: [this.props.report_uid],
        };
        
        return (
            <div className="row row-margin-bottom">
                <div className="col-sm-12 col-md-6 col-lg-5">
                    <h4 className="text-center row-head-box">
                        {SELECT_A_FUNC(COMPONENTS.FIELDS.SHIFT.SHIFT)}
                    </h4>
                    <BootstrapTable
                        className="middle-align"
                        trClassName={(row, rowIndex) => (`${trOddEvenClassFormat(row, rowIndex)} ${this.trClassFormat()}`)}
                        tableBodyClass="automation-companies-table"
                        data={this.props.reports}
                        options={options}
                        selectRow={selectRow}
                        headerStyle={headerStyle}
                        bodyStyle={{
                            maxHeight: 150,
                            overflow: 'scroll',
                            overflowX: 'auto',
                            overflowY: 'auto',
                        }}
                        hover
                        condensed
                    >
                        <TableHeaderColumn
                            dataField="uid"
                            hidden
                            isKey
                        >
                            Uid
                        </TableHeaderColumn>
                        <TableHeaderColumn
                            dataField="shift"
                            tdAttr={{ 'data-id': 'company-table-name' }}
                            dataSort
                            headerAlign="center"
                            dataAlign="center"
                            width="150"
                            dataFormat={this.shiftFormatter}
                            expandable={false}
                        >
                            {COMPONENTS.FIELDS.SHIFT.SHIFT}
                        </TableHeaderColumn>
                        <TableHeaderColumn
                            dataField="start"
                            dataFormat={this.timeFormatter}
                            tdAttr={{ 'data-id': 'contact-trackable-name' }}
                            headerAlign="center"
                            dataAlign="center"
                            width="100"
                            dataSort
                        >
                            {COMPONENTS.FIELDS.SHIFT.START}
                        </TableHeaderColumn>
                        <TableHeaderColumn
                            dataField="end"
                            dataFormat={this.timeFormatter}
                            tdAttr={{ 'data-id': 'contact-trackable-name' }}
                            headerAlign="center"
                            dataAlign="center"
                            width="100"
                        >
                            {COMPONENTS.FIELDS.SHIFT.END}
                        </TableHeaderColumn>
                    </BootstrapTable>
                </div>
            </div>
        );
    }
}

export default connect()(ReportsList);
