import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import uniq from 'lodash/uniq';

import store from 'stores/LocationStore';
import { DEFAULT_VALUES, SEND_NEW_ROUTE } from 'common/constants';
import { RIGHT_SIDE_BAR_MESSAGING, COMPONENTS } from 'common/ui-labels';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import { sortByField, formatToLabelValueList, filterSelectBoxOptions } from 'utils/utils';

import { Trackable, Group } from 'stores/classes';

import {
    sendMessages,
    saveMessage,
    selectTrackablesAndGroups,
    sendMessaging,
} from 'stores/actions';

class RightSideBarMessaging extends Component {
    static propTypes = {
        groups: PropTypes.arrayOf(PropTypes.shape()),
        trackables: PropTypes.arrayOf(PropTypes.shape()),
        message: PropTypes.string,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        dispatch: PropTypes.func,
        selectedItems: PropTypes.arrayOf(PropTypes.shape({
            uid: PropTypes.string,
        })),
    };
    
    static defaultProps = {
        groups: [],
        trackables: [],
        selectedItems: [],
        message: '',
        intl: {
            formatMessage: () => {
            },
        },
        dispatch: () => {
        },
    };
    
    onSendMessage = () => {
        if (!this.props.message) {
            return;
        }
        const recipients = [];
        this.props.selectedItems.forEach((item) => {
            if (item.label === RIGHT_SIDE_BAR_MESSAGING.BUTTONS.ALL_FUNC(this.props.intl)) {
                this.props.trackables.forEach((tko) => {
                    recipients.push(tko.node.mac_address);
                });
            } else if (item.label.startsWith(RIGHT_SIDE_BAR_MESSAGING.BUTTONS.TRACKABLE_START(this.props.intl))) {
                recipients.push(item.node.mac_address);
            } else if (item.label.startsWith(RIGHT_SIDE_BAR_MESSAGING.BUTTONS.GROUP_START(this.props.intl))) {
                this.props.trackables.forEach((tko) => {
                    if (tko.groups && tko.groups.find(group => group === item.uid)) {
                        recipients.push(tko.node.mac_address);
                    }
                });
            }
        });
        this.props.dispatch(sendMessages(uniq(recipients), this.props.message));
    };
    
    onSendMessaging = () => {
        // const currentUser = store.getCurrentUser();
        // const companyUsers = store.getCompanyUsers().companyUsers;
        if (!this.props.message) {
            return;
        }
        const from = store.getCurrentUserData();
        const recipients = [];
        this.props.selectedItems.forEach((item) => {
            const { label, value } = item;
            if (label === RIGHT_SIDE_BAR_MESSAGING.BUTTONS.ALL_FUNC(this.props.intl)) {
                this.props.trackables.forEach((tko) => {
                    recipients.push(tko.uid);
                });
            } else if (label.startsWith(RIGHT_SIDE_BAR_MESSAGING.BUTTONS.TRACKABLE_START(this.props.intl))) {
                recipients.push(value);
            } else if (label.startsWith(RIGHT_SIDE_BAR_MESSAGING.BUTTONS.GROUP_START(this.props.intl))) {
                this.props.trackables.forEach((tko) => {
                    if (tko.groups && tko.groups.includes(value)) {
                        recipients.push(tko.uid);
                    }
                });
            }
        });
        this.props.dispatch(sendMessaging(from.uid, uniq(recipients), this.props.message));
    };
    
    onSaveMessage = (message) => {
        this.props.dispatch(saveMessage(message));
    };
    
    onSelectTrackablesChange = (values) => {
        this.props.dispatch(selectTrackablesAndGroups(values));
    };
    
    render() {
        const { intl, trackables, groups } = this.props;
        const trackablesAndGroups = sortByField([...trackables, ...groups], 'label');
        // add all to trackablesAndGroups
        trackablesAndGroups.unshift({ label: RIGHT_SIDE_BAR_MESSAGING.BUTTONS.ALL_FUNC(intl) });
        const trackablesAndGroupsOptions = formatToLabelValueList(trackablesAndGroups, 'label');

        return (
            <div>
                <div>
                    <textarea
                        data-cy="messaging-message"
                        className="form-control RPP-Right-Site-Bar-TextArea"
                        rows="5"
                        maxLength={DEFAULT_VALUES.MESSAGE_MAX_LENGTH}
                        value={this.props.message}
                        onKeyPress={(e) => {
                            if (e.key === 'Enter') {
                                e.preventDefault();
                            }
                        }}
                        onChange={(e) => {
                            this.onSaveMessage(e.target.value);
                        }}
                        placeholder={RIGHT_SIDE_BAR_MESSAGING.TITLES.PLACEHOLDER_FUNC(intl)}
                    />
                </div>
                <div className="RPP-Right-Site-Bar-Row">
                    <div
                        className="RPP-Right-Site-Bar-Controls"
                        data-cy="messaging-node-address-div"
                    >
                        <Field
                            name="trackables_and_groups"
                            type="text"
                            label="Objective"
                            component={MultiSelectComponent}
                            placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                            noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                            options={trackablesAndGroupsOptions}
                            filterOptions={filterSelectBoxOptions}
                            onChange={this.onSelectTrackablesChange}
                        />
                    </div>
                    {SEND_NEW_ROUTE
                        ? (
                            <button
                                data-cy="messaging-send-button"
                                className="RPP-RightSiteBar-Envelope"
                                onClick={this.onSendMessaging}
                            >
                                {RIGHT_SIDE_BAR_MESSAGING.SEND}
                            </button> // new messaging
                        ) : (
                            <button
                                data-cy="messaging-send-button"
                                className="RPP-RightSiteBar-Envelope"
                                onClick={this.onSendMessage}
                            >
                                {RIGHT_SIDE_BAR_MESSAGING.SEND}
                            </button> // old message
                        )
                    }
                </div>
            </div>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getTrackable = (state, intl) => Trackable.getAllTrackablesArray(state).map(trackable => 
    trackable.formattedTrackableUidNameNode(intl)).filter(x => x.node); // filter out the unassign trackables

const getTrackables = createDeepEqualSelector(
    [getTrackable],
    trackables => trackables,
);

const getGroup = (state, intl) => Group.getAllGroupsArray(state).map(group => group.getFormattedGroupForSendingMessage(intl));

const getGroups = createDeepEqualSelector(
    [getGroup],
    groups => groups,
);

const mapStateToProps = (state, { intl }) => (
    {
        message: state.entities.messages.message,
        selectedItems: state.entities.messages.selectedItems,
        trackables: getTrackables(state, intl),
        groups: getGroups(state, intl),
    }
);

export default injectIntl(reduxForm({ form: 'send_message' })(connect(mapStateToProps)(injectIntl(RightSideBarMessaging))));
