import React, { Component } from 'react';
import { connect } from 'react-redux';
import moment from 'moment';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import _matchesProperty from 'lodash/matchesProperty';

import { getTimeZoneOfProject, getFullDateByTz } from 'utils';
import { SITE_ANALYTICS_LABELS, SITE_MODE, NOT_AVAILABLE } from 'common';
import { changeHeatmapParams, changeMode, clearHeatmap, siteViewAnalyticsModalUpdateState } from 'stores/actions';

const timeFrameArray = [0.5, 2, 8, 24]; // in hours
const NoClose = 'NoClose';
const RECENT_HEATMAPS_LIMIT = 5;

class SiteAnalyticsDropDown extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        heatmap: PropTypes.shape({
            completedHeatmaps: PropTypes.arrayOf(PropTypes.shape({})),
            currentHeatmaps: PropTypes.arrayOf(PropTypes.shape({})),
        }),
        sublocation: PropTypes.shape({}),
        lastMode: PropTypes.shape({
            func: PropTypes.func,
            args: PropTypes.arrayOf(PropTypes.shape({})),
        }),
        timeZone: PropTypes.string,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        isAnalyticsDisabled: PropTypes.bool,
    };
    
    static defaultProps = {
        dispatch: () => {},
        heatmap: {
            completedHeatmaps: [],
            currentHeatmaps: [],
        },
        sublocation: {},
        lastMode: null,
        timeZone: 'Etc/Gmt',
        intl: {
            formatMessage: () => {},
        },
        isAnalyticsDisabled: false,
    };
    
    constructor() {
        super();
        
        this.state = {
            isSiteAnalyticsListShowed: false,
            isOpenRecentHeatmaps: false,
        };
        
        this.dropDown = React.createRef();
    }
    
    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutsideDropDown);
    }
    
    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutsideDropDown);
    }
    
    handleClickOutsideDropDown = (event) => {
        const dropDown = this.dropDown.current; // from ref

        if (dropDown && !dropDown.contains(event.target)) {
            this.hideAnalyticsDropDown();
        }
    };

    selectTimeFrame = (selectedTime) => {
        const now = moment();
        const params = {
            from: Date.parse(now.clone().add(-selectedTime, 'h')),
            to: Date.parse(now),
            isPreselectedTimePeriod: true,
            selectedTime, // needed in ViewSiteAnalyticsModal
        };

        this.handleSubmit(params);
    };

    handleSubmit = (params) => {
        this.openSiteViewAnalyticsModal(params);
        this.showHideAnalyticsDropDown();
    };

    openSiteViewAnalyticsModal = (params) => {
        this.props.dispatch(siteViewAnalyticsModalUpdateState({
            isModalOpened: true,
            ...params,
        }));
    };

    showHideAnalyticsDropDown = () => {
        this.setState({ isSiteAnalyticsListShowed: !this.state.isSiteAnalyticsListShowed });
    };
    
    hideAnalyticsDropDown = () => {
        this.setState({ isSiteAnalyticsListShowed: false });
    };

    handleButtonClick = () => {
        this.showHideAnalyticsDropDown();
    };

    getSiteAnalyticsDropdownOptions = () => timeFrameArray.map((time, index) => (
        <div
            key={index}
            className="siteAnalytics-list-option"
            onClick={this.selectTimeFrame.bind(this, time)}
            data-cy={`analytic-last-${time}-dropdown`}
        >
            {SITE_ANALYTICS_LABELS[`LAST_${time}_HOURS`]}
        </div>
    ));

    handleCustomIntervalClick = () => {
        const params = {
            isCustomInterval: true,
        };
        this.handleSubmit(params);
    };

    getCustomIntervalOption = () => (
        <div
            className="siteAnalytics-list-option"
            onClick={this.handleCustomIntervalClick}
            data-cy="analytic-custom-dropdown"
        >
            {SITE_ANALYTICS_LABELS.CUSTOM_INTERVAL}
        </div>
    );

    openRecentHeatmaps = () => {
        this.setState({
            isOpenRecentHeatmaps: true,
        });
    };

    closeRecentHeatmaps = (event) => {
        if (!event.relatedTarget || event.relatedTarget.getAttribute('data-info') !== NoClose) {
            this.setState({
                isOpenRecentHeatmaps: false,
            });
        }
    };

    changeCurrentHeatmaps = (event) => {
        let currentHeatmaps = [];
        if (event.target.checked && event.target.name) {
            currentHeatmaps = [
                ...this.props.heatmap.currentHeatmaps,
                this.props.heatmap.completedHeatmaps.find((heatmap) => heatmap.uid === event.target.name),
            ];
            
            this.props.dispatch(changeHeatmapParams({
                currentHeatmaps,
            }));
        } else if (!event.target.checked && event.target.name) {
            currentHeatmaps = [...this.props.heatmap.currentHeatmaps.filter((heatmap) => heatmap.uid !== event.target.name)];
            this.props.dispatch(changeHeatmapParams({
                currentHeatmaps,
            }));
        }
        
        if (currentHeatmaps.length === 1) {
            const onReturnClearHeatmap = () => {
                this.props.dispatch(clearHeatmap());
            };
            this.props.dispatch(changeMode({
                name: SITE_MODE.HEATMAP,
                onReturn: onReturnClearHeatmap,
            }));
        } else if (currentHeatmaps.length === 0) {
            if (this.props.lastMode) {
                this.props.dispatch(this.props.lastMode.func.call(this, ...this.props.lastMode.args));
            } else {
                this.props.dispatch(clearHeatmap());
            }
        }
    };

    removeHeatmap = (heatmap) => {
        const currentHeatmaps = [
            ...this.props.heatmap.currentHeatmaps.filter((currentHeatmap) => (currentHeatmap.id !== heatmap.id)),
        ];
        const completedHeatmaps = [
            ...this.props.heatmap.completedHeatmaps.filter((completedHeatmap) => (completedHeatmap.id !== heatmap.id)),
        ];
        
        this.props.dispatch(changeHeatmapParams({
            completedHeatmaps,
            currentHeatmaps,
        }));
        
        if (currentHeatmaps.length === 0) {
            this.props.dispatch(clearHeatmap());
        }
    };

    getRecentOption = () => (
        <div
            className="siteAnalytics-list-option"
            onMouseOver={this.openRecentHeatmaps}
            onMouseOut={this.closeRecentHeatmaps}
            data-cy="analytic-time-recent-dropdown"
        >
            {SITE_ANALYTICS_LABELS.RECENT_HEATMAPS(this.props.intl)}
        </div>
    );

    getRecentHeatmaps = (completedHeatmaps) => {
        const duplicateCounts = [];
        const lastRecentHeatmaps = completedHeatmaps.slice(-RECENT_HEATMAPS_LIMIT).map((heatmap) => {
            const duplicates = completedHeatmaps.filter(_matchesProperty('name', heatmap.name));
            duplicateCounts[heatmap.name] = duplicates.length;

            return heatmap;
        });

        const lastRecentModifiedHeatmaps = lastRecentHeatmaps.slice(0).reverse().map((heatmap) => { // creating
            // a new copy of [lastRecentHeatmaps] Array with reversed order

            const { input: { start, end }, id, uid, name } = heatmap;

            const displayName = (duplicateCounts[name] > 1) ? `${name} (${--duplicateCounts[name]})` : name;

            return {
                id,
                uid,
                name,
                displayName,
                start,
                end,
            };
        });

        return lastRecentModifiedHeatmaps;
    };

    render() {
        const { timeZone, sublocation, heatmap, isAnalyticsDisabled, intl } = this.props;
        const completedHeatmaps = heatmap.completedHeatmaps.filter((_heatmap) => _heatmap.relatedSublocationUid === sublocation.uid);
        const recentHeatmaps = this.getRecentHeatmaps(completedHeatmaps);

        return (
            <div
                ref={this.dropDown}
                className="RPP-SiteAnalyticsDropDown"
            >
                <button
                    disabled={isAnalyticsDisabled}
                    className="SiteView-Control-Button btn"
                    onClick={this.handleButtonClick}
                    title={NOT_AVAILABLE(intl)}
                >
                    <span className="small-row-block">
                        <i className="fas fa-chart-line" />
                    </span>
                    <span className="collapse-sidebar-text">
                        {SITE_ANALYTICS_LABELS.BUTTON}
                    </span>
                    <span className="arrow-down">
                        <i className="fas fa-sort-down" />
                    </span>
                </button>

                {
                    this.state.isSiteAnalyticsListShowed
                        ? (
                            <div
                                className="siteAnalytics-dropdown"
                            >
                                {this.getSiteAnalyticsDropdownOptions()}
                                {this.getCustomIntervalOption()}
                                {this.getRecentOption()}
                            </div>
                        )
                        : ''
                }

                {
                    this.state.isOpenRecentHeatmaps ?
                        <div
                            className="RecentHeatmaps-List"
                            data-info={NoClose}
                            onMouseOut={this.closeRecentHeatmaps}
                        >
                            {
                                recentHeatmaps.length ? recentHeatmaps
                                    .map((completed_heatmap) => {
                                        const { start, end, uid, displayName } = completed_heatmap;

                                        const fromTime = getFullDateByTz(start * 1000, timeZone);
                                        const toTime = getFullDateByTz(end * 1000, timeZone);

                                        const isCurrentHeatmap = heatmap.currentHeatmaps.find((hm) => hm.uid === uid);

                                        return (
                                            <div className="RecentHeatmaps" data-info={NoClose} key={uid}>
                                                <div className="RecentHeatmaps-Head" data-info={NoClose}>
                                                    <div className="d-flex">
                                                        <input
                                                            type="checkbox"
                                                            checked={!!isCurrentHeatmap}
                                                            name={uid}
                                                            onChange={this.changeCurrentHeatmaps}
                                                            data-info={NoClose}
                                                        />

                                                        <label
                                                            htmlFor={uid}
                                                            className="RecentHeatmaps-Label"
                                                            onClick={this.changeCurrentHeatmaps.bind(this, {
                                                                target: {
                                                                    name: uid,
                                                                    checked: !isCurrentHeatmap,
                                                                },
                                                            })}
                                                            data-info={NoClose}
                                                        >
                                                            { displayName }
                                                        </label>
                                                    </div>

                                                    <div
                                                        className="RecentHeatmaps-RemoveBtn"
                                                        onClick={this.removeHeatmap.bind(this, completed_heatmap)}
                                                        data-info={NoClose}
                                                    >
                                                        <i className="fa fa-times" data-info={NoClose} />
                                                    </div>
                                                </div>

                                                <div className="RecentHeatmaps-date-period" data-info={NoClose}>
                                                    {`${fromTime} - ${toTime}`}
                                                </div>
                                            </div>);
                                    })
                                    : (<div className="RecentHeatmaps">{SITE_ANALYTICS_LABELS.NO_RECENT_HEAT_MAP}</div>)
                            }
                        </div>
                        :
                        ''
                }
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    const { project } = state.appContext;
    const timeZone = getTimeZoneOfProject(project);

    return {
        timeZone,
        floorplan: state.siteView.selectedFloorplan,
        sublocation: state.siteView.selectedSublocation,
        lastMode: state.siteView.lastMode,
        heatmap: state.heatmap,
    };
};

export default connect(mapStateToProps)(injectIntl(SiteAnalyticsDropDown));
