import React, { Component } from 'react';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import store from 'stores/LocationStore';
import { STORAGE_KEY } from 'common/constants';
import { getTimeZoneOfProject } from 'utils/timeUtils';
import { getSiteViewClassName } from 'utils/htmlUtils';
import { isPlaybackMode, isZoneEditing } from 'utils/simpleUtils';
import { getVisibleTrackables } from 'utils/trackableUtils';
import { getVisibleZones } from 'utils/zoneUtils';

import SiteViewHeader from './SiteViewHeader';
import SiteViewLeaflet from './SiteViewLeaflet';
import SiteViewFooter from './SiteViewFooter';
import SiteViewMode from './SiteViewMode';


class SiteView extends Component {
    static propTypes = {
        isOpenSideBar: PropTypes.bool,
        coordinateMode: PropTypes.bool,
        isModeLoading: PropTypes.bool,
        isAnalyticsDisabled: PropTypes.bool,
        sublocation: PropTypes.shape(),
        floorplan: PropTypes.shape(),
        zoom: PropTypes.number,
        filter: PropTypes.shape(),
        center: PropTypes.shape(),
        mode: PropTypes.shape({
            name: PropTypes.string,
            onReturn: PropTypes.func,
        }),
        rotation: PropTypes.number,
        width: PropTypes.number,
        creatingZone: PropTypes.shape(),
        currentRole: PropTypes.string,
        timeZone: PropTypes.string,
        handleSiteViewZoneSelection: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        spaghetti: PropTypes.shape(),
        nearMissReport: PropTypes.shape(),
        hiddenTkoUnresponsiveCount: PropTypes.number,
        visibleTrackables: PropTypes.arrayOf(PropTypes.shape({})),
        visibleZones: PropTypes.arrayOf(PropTypes.shape({})),
        lastMode: PropTypes.shape({}),
    };

    static defaultProps = {
        isOpenSideBar: true,
        coordinateMode: false,
        isModeLoading: false,
        isAnalyticsDisabled: false,
        sublocation: null,
        floorplan: null,
        zoom: 18,
        filter: {},
        center: {},
        mode: {
            name: '',
            onReturn: () => {
            },
        },
        rotation: 0,
        width: 0,
        creatingZone: {},
        currentRole: '',
        timeZone: 'Etc/GMT',
        handleSiteViewZoneSelection: () => {},
        intl: {
            formatMessage: () => {
            },
        },
        spaghetti: {},
        nearMissReport: {},
        hiddenTkoUnresponsiveCount: 0,
        visibleTrackables: [],
        visibleZones: [],
        lastMode: null,
    };

    render() {
        const {
            intl,
            isOpenSideBar,
            sublocation,
            floorplan,
            zoom,
            filter,
            center,
            mode,
            rotation,
            width,
            creatingZone,
            currentRole,
            handleSiteViewZoneSelection,
            spaghetti,
            timeZone,
            visibleTrackables,
            visibleZones,
            nearMissReport,
            hiddenTkoUnresponsiveCount,
            coordinateMode,
            isModeLoading,
            isAnalyticsDisabled,
            lastMode,
        } = this.props;

        return (
            <div className={getSiteViewClassName(isOpenSideBar, mode)}>
                <SiteViewHeader
                    intl={intl}
                    lastMode={lastMode}
                    mode={mode}
                    isAnalyticsDisabled={isAnalyticsDisabled}
                    sublocation_name={sublocation && sublocation.name}
                />
                <SiteViewLeaflet
                    visibleTrackables={visibleTrackables}
                    visibleZones={visibleZones}
                    sublocation={sublocation}
                    floorplan={floorplan}
                    timeZone={timeZone}
                    zoom={zoom}
                    center={center}
                    mode={mode}
                    rotation={rotation}
                    width={width}
                    creatingZone={creatingZone}
                    currentRole={currentRole}
                    isOpenGroupFilter={filter.isOpenGroupFilter}
                    handleSiteViewZoneSelection={handleSiteViewZoneSelection}
                    spaghetti={spaghetti}
                    nearMissReport={nearMissReport}
                    coordinateMode={coordinateMode}
                />
                <SiteViewMode
                    mode={mode}
                    isModeLoading={isModeLoading}
                />
                <SiteViewFooter hiddenTkoUnresponsiveCount={hiddenTkoUnresponsiveCount} mode={mode} />
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    const { visibleTrackables, hiddenTkoUnresponsiveCount } = getVisibleTrackables(state);
    const visibleZones = getVisibleZones(state.siteView.selectedSublocation, state);
    
    const { byUid, reportUid } = state.nearMissReport;
    const isNearMissLoading = byUid[reportUid] && byUid[reportUid].loading;
    const { spaghettiId, byId } = state.spaghetti;
    const isSpaghettiLoading = byId[spaghettiId] && byId[spaghettiId].loading;
    const isHeatmapLoading = state.heatmap && state.heatmap.isHeatmapLoading;
    
    const { mode } = state.siteView;
    const isPlaybackLoading = isPlaybackMode(mode && mode.name) && !state.playback.isOpenPlayBackControl;
    const isModeLoading = isNearMissLoading || isSpaghettiLoading || isHeatmapLoading || isPlaybackLoading;
    const isNotAvailableFloorplan = !state.siteView.selectedFloorplan;
    const isAnalyticsDisabled = isModeLoading || isZoneEditing(mode && mode.name) || isNotAvailableFloorplan;
    
    return ({
        visibleTrackables,
        visibleZones,
        hiddenTkoUnresponsiveCount,
        timeZone: getTimeZoneOfProject(state.appContext.project),
        filter: state.siteViewFilter.filter,
        isOpenSideBar: state.siteView.isOpenSideBar,
        sublocation: state.siteView.selectedSublocation,
        floorplan: state.siteView.selectedFloorplan,
        zoom: state.siteViewFilter.leaflet.zoom,
        center: state.siteViewFilter.leaflet.center,
        mode: state.siteView.mode,
        rotation: state.siteView.rotation,
        coordinateMode: state.siteViewZoneEditCoordinates.active,
        width: store.getFromLocalStorage(STORAGE_KEY.LAST_DIRECTIONAL_ZONE_WIDTH) || state.siteView.width,
        creatingZone: state.siteView.geometry,
        currentRole: state.user.access.currentRole,
        spaghetti: state.spaghetti,
        nearMissReport: state.nearMissReport,
        isAnalyticsDisabled,
        isModeLoading,
        lastMode: state.siteView.lastMode,
    });
};

export default connect(mapStateToProps)(injectIntl(SiteView));
