import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { TimeMarker } from 'react-player-controls';
import Paper from '@material-ui/core/Paper';
import { PLAYBACK } from 'worker';
import ProgressBarWithTime from 'components/ProgressBarWithTime';
import { changePlaybackParams, getHistoryPos, generatePlayback } from 'stores/actions';
import { getTimeZoneOffsetByProject } from 'utils';


class SiteViewPlaybackControl extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        floorplan: PropTypes.shape(),
        timeZoneOffset: PropTypes.number,
        timezone: PropTypes.string,
        playback: PropTypes.shape({
            isOpenPlayBackControl: PropTypes.bool,
            pending_request: PropTypes.bool,
            playSpeed: PropTypes.number,
            playSpeeds: PropTypes.arrayOf(PropTypes.number),
            currentTs: PropTypes.number,
            finishTs: PropTypes.number,
            isPlaying: PropTypes.bool,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        floorplan: {},
        timeZoneOffset: 0,
        timezone: 'Etc/GMT',
        playback: {
            isOpenPlayBackControl: false,
            pending_request: false,
            playSpeed: 1,
            playSpeeds: [1, 2],
            currentTs: 0,
            finishTs: 0,
            isPlaying: false,
        },
    };
    
    constructor(props) {
        super(props);
    
        this.playbackInterval = null;
    }
    
    componentDidUpdate() {
        if (!this.props.playback.isPlaying && this.playbackInterval) {
            clearInterval(this.playbackInterval);
        }
    }
    
    componentWillUnmount() {
        this.handlePause();
    }
    
    getHistoricalData = () => {
        const playBackSetting = this.props.playback;
        const newTs = playBackSetting.currentTs;
        
        if (playBackSetting.isReadyToPlay) {
            this.props.dispatch(getHistoryPos({
                ts: newTs,
            }));
        }
        
        if ((newTs + PLAYBACK.DELTA_TIME) >= playBackSetting.finishTs) {
            this.handlePause();
        }
    };

    resetToStart = () => {
        const { playback } = this.props;

        this.props.dispatch(changePlaybackParams({
            currentTs: playback.setting.playFrom,
            isReadyToPlay: false,
        }));

        const setting = { ...playback.setting, currentTs: playback.setting.playFrom };
        const trackables = playback.trackables.map(trackable => trackable.uid);
        this.props.dispatch(generatePlayback({ setting, trackables }));
    };

    handlePause = () => {
        clearInterval(this.playbackInterval);
        this.props.dispatch(changePlaybackParams({
            isPlaying: false,
        }));
    };

    handlePlay = () => {
        if ((this.props.playback.currentTs + PLAYBACK.DELTA_TIME) >= this.props.playback.finishTs) {
            this.resetToStart();
            return;
        }

        // need to first download the playback data because setInterval will execute only after delay
        this.getHistoricalData();

        clearInterval(this.playbackInterval);
        this.playbackInterval = setInterval(this.getHistoricalData, PLAYBACK.DELTA_TIME / this.props.playback.playSpeed);

        this.props.dispatch(changePlaybackParams({ isPlaying: true }));
    };

    handleSpeed = (e) => {
        while (!e.target.name) {
            e.target = e.target.parentNode;
        }

        const { playSpeeds, playSpeed, isPlaying } = this.props.playback;

        const speedIndex = playSpeeds.findIndex(element => playSpeed === element);

        let newSpeed;

        if (e.target.name === 'playSpeedUp') {
            if (speedIndex === 0) {
                return;
            }

            newSpeed = playSpeeds[speedIndex - 1];
        } else {
            if (speedIndex === playSpeeds.length - 1) {
                return;
            }

            newSpeed = playSpeeds[speedIndex + 1];
        }

        this.props.dispatch(changePlaybackParams({
            playSpeed: newSpeed,
        }));


        if (isPlaying) {
            clearInterval(this.playbackInterval);

            let interval = null;
            new Promise((resolve) => {
                interval = setInterval(() => {
                    if (this.props.playback.playSpeed === newSpeed) {
                        resolve();
                    }
                }, 1);
            }).then(() => {
                clearInterval(interval);
                this.handlePlay();
            });
        }
    };

    render() {
        const { playback, floorplan, timeZoneOffset, timezone } = this.props;

        if (!playback.isOpenPlayBackControl || !floorplan) {
            return null;
        }
        const isPending = playback.pending_request || !playback.isReadyToPlay;
        return (
            <Paper className="RPP-PlayBackControl" zdepth={1}>
                <ProgressBarWithTime
                    isPending={isPending}
                    playback={playback}
                    timeZoneOffset={timeZoneOffset}
                    timezone={timezone}
                    dispatch={this.props.dispatch}
                />

                <div className="Processes-Control-Btns">
                    <button
                        onClick={this.handleSpeed}
                        name="playSpeedDown"
                        disabled={isPending}
                        className="btn btn-default btn-xs Processes-Control-Btn"
                    >
                        <i className="fa fa-backward" />
                    </button>

                    {
                        playback.isPlaying ?
                            <button
                                onClick={this.handlePause}
                                disabled={isPending}
                                name="playSpeedUp"
                                className="btn btn-default btn-xs Processes-Control-Btn"
                            >
                                {isPending ?
                                    <div className="Playback-Loader">
                                        <div /><div /><div /><div /><div /><div /><div /><div /><div /><div /><div /><div />
                                    </div> : <i className="fa fa-pause" />}
                            </button> :
                            <button
                                onClick={this.handlePlay}
                                name="playSpeedUp"
                                className="btn btn-default btn-xs Processes-Control-Btn"
                            >
                                <i className="fa fa-play" />
                            </button>

                    }

                    <button
                        onClick={this.handleSpeed}
                        name="playSpeedUp"
                        disabled={isPending}
                        className="btn btn-default btn-xs Processes-Control-Btn"
                    >
                        <i className="fa fa-forward" />
                    </button>

                    <div className="Processes-Control-Total-Time">
                        <TimeMarker
                            totalTime={(playback.finishTs - playback.startTs) / 1000}
                            currentTime={(playback.currentTs - playback.startTs) / 1000}
                            markerSeparator=" / "
                        />
                    </div>
                </div>
            </Paper>
        );
    }
}

const mapStateToProps = (state) => {
    const project = state.appContext.project;
    const timeZoneOffset = getTimeZoneOffsetByProject(project);
    const timezone = state.appContext.project.user_params.timezone;
    return {
        timeZoneOffset,
        timezone,
        playback: state.playback,
        floorplan: state.siteView.selectedFloorplan,
    };
};

export default connect(mapStateToProps)(SiteViewPlaybackControl);
