import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { BootstrapTable, TableHeaderColumn } from 'react-bootstrap-table';
import { injectIntl } from 'react-intl';
import { BOOTSTRAP_TABLE_SORT, SITE_ANALYTICS_LABELS, TABLES } from 'common';
import { millisecondsToHumanReadable, getTimeByFormatAndTimezone } from 'utils';


const dateFormat = 'MM/DD/YYYY HH:mm';

class SpaghettiSummaryInfo extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            summaryShowed: true,
        };
    }

    toggleSpaghettiSummaryInfo = () => this.setState({ summaryShowed: !this.state.summaryShowed });
    
    formatNameColumn = (cell, row) => (<div title={row.trackableName}>{cell}</div>);

    getSummaryData = trackables => Object.values(trackables).map(to => ({
        uid: to.uid,
        trackableName: to.name,
        totalDistance: to.totalDistance,
        totalIdleTime: millisecondsToHumanReadable(to.totalIdleTime),
    }));

    getSummarySubTitleHtml = (timeframe) => {
        const start = getTimeByFormatAndTimezone(timeframe.spaghettiFrom, dateFormat, this.props.timeZone);
        const end = getTimeByFormatAndTimezone(timeframe.spaghettiTo, dateFormat, this.props.timeZone);
        return (
            <h6 className="text-center AdminTab-Subtitle">
                {SITE_ANALYTICS_LABELS.PERIOD}: {start} - {end}
            </h6>
        );
    };

    render() {
        const { trackables, timeframe } = this.props;
        const { summaryShowed } = this.state;

        const options = {
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            defaultSortName: 'trackableName',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };

        return (
            <div className="spaghetti-summary-info">
                <button
                    className="spaghetti-summary-info-toggle btn btn-link"
                    onClick={this.toggleSpaghettiSummaryInfo}
                >
                    <h4 className="spaghetti-summary-title text-center text-capitalize">{SITE_ANALYTICS_LABELS.TRACKABLES_SUMMARY}</h4>
                    <span className="arrow-down">
                        <i className={`fas fa-sort-down ${summaryShowed ? 'rotate180deg' : ''}`} />
                    </span>
                </button>
                {
                    summaryShowed &&
                    <div className="spaghetti-summary-info-body">
                        {this.getSummarySubTitleHtml(timeframe)}
                        <BootstrapTable
                            data={this.getSummaryData(trackables)}
                            options={options}
                            className="Admin-BootstrapTable spaghetti-summary-info-table"
                            condensed
                            hover
                        >
                            <TableHeaderColumn
                                dataField="uid"
                                searchable={false}
                                hidden
                                isKey
                            />
                            <TableHeaderColumn
                                dataField="trackableName"
                                tdAttr={{ 'data-id': 'trackable-name' }}
                                dataFormat={this.formatNameColumn}
                                headerAlign="center"
                                dataSort
                            >
                                {SITE_ANALYTICS_LABELS.TRACKABLE_NAME}
                            </TableHeaderColumn>
                            <TableHeaderColumn
                                dataField="totalDistance"
                                tdAttr={{ 'data-id': 'total-distance' }}
                                width="100px"
                                headerAlign="center"
                                dataAlign="right"
                                dataSort
                            >
                                {SITE_ANALYTICS_LABELS.TOTAL_DISTANCE}
                            </TableHeaderColumn>
                            <TableHeaderColumn
                                dataField="totalIdleTime"
                                tdAttr={{ 'data-id': 'total-idle-time' }}
                                headerAlign="center"
                                dataAlign="right"
                                width="100px"
                                dataSort
                            >
                                {SITE_ANALYTICS_LABELS.TOTAL_IDLE_TIME}
                            </TableHeaderColumn>
                        </BootstrapTable>
                    </div>
                }
            </div>
        );
    }
}

SpaghettiSummaryInfo.propTypes = {
    trackables: PropTypes.arrayOf(PropTypes.shape({})),
    timeframe: PropTypes.shape({
        spaghettiFrom: PropTypes.number,
        spaghettiTo: PropTypes.number,
    }),
    timeZone: PropTypes.string,
};

SpaghettiSummaryInfo.defaultProps = {
    trackables: [],
    timeframe: {
        spaghettiFrom: 0,
        spaghettiTo: 0,
    },
    timeZone: 'Etc/GMT',
};

export default connect()(injectIntl(SpaghettiSummaryInfo));
