import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { BOOTSTRAP_TABLE_SORT, DEFAULT_VALUES } from 'common/constants';
import { SITE_VIEW_MODE, TABLES, NAMES_OF_TABS } from 'common/ui-labels';
import { openNearMissEventPopup, getNearMissEventsCsv } from 'stores/actions/nearMissEventsActions';


class NearMissEventsTable extends React.Component {
    static propTypes = {
        events: PropTypes.arrayOf(PropTypes.shape({})),
        project: PropTypes.arrayOf(PropTypes.shape({
            name: PropTypes.string,
        })),
        nearMissReport: PropTypes.arrayOf(PropTypes.shape({
            displayFrom: PropTypes.number,
            displayTo: PropTypes.number,
        })),
        company_name: PropTypes.string,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        dispatch: PropTypes.func,
        selectedNearMissMarker: PropTypes.number,
        height: PropTypes.number,
    };
    
    static defaultProps = {
        events: [],
        project: null,
        nearMissReport: null,
        company_name: '',
        intl: {
            formatMessage: () => {
            },
        },
        dispatch: () => {},
        selectedNearMissMarker: null,
        height: 0,
    };
    
    constructor(props) {
        super(props);
        this.state = { clickFlag: false };
        this.refsList = {};
    }
    
    componentDidUpdate(prevProps, prevState) {
        const markerClicked = this.props.selectedNearMissMarker && prevProps.selectedNearMissMarker !== this.props.selectedNearMissMarker;
        const clickOutsideTable = this.state.clickFlag === prevState.clickFlag;

        if (markerClicked && clickOutsideTable) {
            this.refsList[this.props.selectedNearMissMarker].current.scrollIntoView(true);
        }
    }
    
    eventTimeFormatter = (cell, row) => (<div ref={row.id} title={row.time}>{row.time}</div>);
    
    rowRefFormatter = (cell, row) => (<div className="multiline-text" ref={this.refsList[row.id]}>{cell}</div>);
    
    handleSelectRow = (row, isSelect, e) => {
        if (e.target.getAttribute('data-info') === 'not-selectable') {
            return false;
        }
        if (isSelect) {
            this.setState({ clickFlag: !this.state.clickFlag });
            this.props.dispatch(openNearMissEventPopup({ ...row }));
        } else {
            this.props.dispatch(openNearMissEventPopup(null));
        }
        return undefined;
    };
    
    tableBtnGroup = () => {
        const { events, nearMissReport, project, company_name, intl } = this.props;
        return (
            <section>
                <button
                    className="site-view-table-button"
                    onClick={() => {
                        getNearMissEventsCsv(events, nearMissReport, project, company_name);
                    }}
                    data-id="site-view-tab-events-export-button"
                    data-cy="zones-table-export-button"
                    data-info="actions-section"
                    disabled={!events.length}
                    title={SITE_VIEW_MODE.TITLES.EXPORT_FUNC(intl)}
                >
                    <i className="fa fa-file-download fa-lg table-action-icon" />
                </button>
            </section>
        );
    }
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="trackables-search"
            data-cy="near-miss-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    render() {
        const { events, intl, height } = this.props;
        this.refsList = events.reduce((memo, event) => {
            memo[event.id] = React.createRef();
            return memo;
        }, {});

        const options = {
            btnGroup: this.tableBtnGroup,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, intl),
            beforeShowError: () => true,
            defaultSortName: 'timestampUtc',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };

        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            hideSelectColumn: true,
            selected: [this.props.selectedNearMissMarker],
            onSelect: this.handleSelectRow,
        };
        
        return (
            <BootstrapTable
                data={events}
                height={height}
                options={options}
                selectRow={selectRow}
                headerStyle={headerStyle}
                tableBodyClass="automation-trackables-site-table"
                bodyStyle={{
                    overflow: 'scroll',
                    overflowX: 'auto',
                    overflowY: 'auto',
                }}
                hover
                condensed
                search
            >
                <TableHeaderColumn
                    dataField="id"
                    searchable={false}
                    hidden
                    isKey
                >
                    ID
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="name"
                    tdAttr={{ 'data-id': 'near-miss-table-name' }}
                    headerAlign="center"
                    dataFormat={this.rowRefFormatter}
                    dataSort
                >
                    {NAMES_OF_TABS.TRACKABLE_OBJECTS}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="timestampUtc"
                    width="130"
                    tdAttr={{ 'data-id': 'near-miss-table-time' }}
                    dataFormat={this.eventTimeFormatter}
                    dataAlign="center"
                    headerAlign="center"
                    dataSort
                >
                    {SITE_VIEW_MODE.NEAR_MISS_TABLE_HEADER_TIME}
                </TableHeaderColumn>
            </BootstrapTable>
        );
    }
}


const mapStateToProps = (state) => {
    const { nearMissReport: { byUid, reportUid } } = state;
    const events = byUid[reportUid] ? byUid[reportUid].events : [];
    
    return {
        events,
        height: state.siteView.height,
        company_name: state.appContext.company.name,
        project: state.appContext.project,
        nearMissReport: state.nearMissReport,
        selectedNearMissMarker: state.nearMissReport.selectedNearMissMarker,
    };
};

export default connect(mapStateToProps)(injectIntl(NearMissEventsTable));
