import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table, defaultTableRowRenderer } from 'react-virtualized';
import 'react-virtualized/styles.css';
import withSortSelect from 'components/HOC/withSortSelect';
import { COMPONENTS, SITE_VIEW_TABLE, TABLES } from 'common/ui-labels';
import { openAlarmAcknowledgeModal, openAlarmInfoModal, siteViewChangeSelectedItems } from 'stores/actions';
import { virtualizedHeaderRenderer } from 'utils';
import ExpandColumnComponent from 'components/ExpandColumnComponent';
import TableButtons from './TableButtons';
import AlarmExpandedTable from './AlarmExpandedTable';

class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        setFilterItems: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape()),
        
        // Props from parent
        allItems: PropTypes.shape(),
        totalCount: PropTypes.number,
        tableSize: PropTypes.shape(),
        selectedItems: PropTypes.arrayOf(PropTypes.string),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({ formatMessage: PropTypes.func }),
        isUserProjectAdmin: PropTypes.bool,
        timeZone: PropTypes.string,
    };
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        setFilterItems: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],
    
        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        tableSize: {},
        selectedItems: [],
        dispatch: () => { },
        intl: { formatMessage: () => { } },
        isUserProjectAdmin: true,
        timeZone: 'Etc/GMT',
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        this.state = { expandedRows: [] };
        this.tableRef = React.createRef();
        
        // HOC Methods
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }

    componentDidUpdate(prevProps, prevState) {
        if (this.state.expandedRows.length !== prevState.expandedRows.length) {
            this.tableRef.current && this.tableRef.current.recomputeRowHeights();
        }
    }

    // todo: find the way to change index to uid
    getRowHeight = ({ index }) => (this.state.expandedRows.includes(index) ? 75 : this.props.tableSize.rowHeight);

    getActionsIcons = (row) => {
        const { isUserProjectAdmin, intl } = this.props;
        return (
            <section data-info="actions-section">
                {isUserProjectAdmin &&
                <i
                    data-info="actions-section"
                    onClick={this.acknowledgeAlarms.bind(this, row)}
                    title={SITE_VIEW_TABLE.ALARM_ACKNOWLEDGE(intl)}
                    style={{ color: 'red' }}
                    className="fa fa-thumbs-up inner-table-action"
                    aria-hidden="true"
                />}
                <i
                    data-info="actions-section"
                    onClick={this.showAlarmInfo.bind(this, row)}
                    title={COMPONENTS.SHOW_DETAILS(intl)}
                    className="fa fa-info-circle inner-table-action"
                    aria-hidden="true"
                />
            </section>
        );
    }
    
    checkboxClick = (rowIndex) => {
        const expandedRows = [...this.state.expandedRows];
        const indexInExpanded = expandedRows.indexOf(rowIndex);
        const isExpanded = indexInExpanded !== -1;
        if (!isExpanded) {
            expandedRows.push(rowIndex);
        } else {
            expandedRows.splice(indexInExpanded, 1);
        }
        this.setState({ expandedRows });
    }

    checkboxButton = (rowIndex) => {
        const isExpanded = this.state.expandedRows.includes(rowIndex);
        return (
            <div
                style={{ cursor: 'pointer' }}
                onClick={() => this.checkboxClick(rowIndex)}
            >
                <ExpandColumnComponent
                    isExpandableRow
                    isExpanded={isExpanded}
                />
            </div>
        );
    };

    handleSelectRow = (selectedItems, { event, rowData }) => {
        const target = event.target.getAttribute('data-info');
        if (target === 'actions-section' || target === 'not-selectable') {
            return false;
        }
        this.props.dispatch(siteViewChangeSelectedItems(rowData));
        return undefined;
    };

    acknowledgeAlarms = (row) => {
        this.props.dispatch(openAlarmAcknowledgeModal(row));
    };

    showAlarmInfo = (row) => {
        this.props.dispatch(openAlarmInfoModal(row || {}));
    };

    _noRowsRenderer = () => (
        <table className="table table-bordered table-hover table-condensed table-no-data">
            <tbody>
                <tr>
                    <td className="react-bs-table-no-data">
                        <span>{TABLES.NO_DATA_TO_DISPLAY}</span>
                    </td>
                </tr>
            </tbody>
        </table>
    );

    rowRenderer = (props) => {
        const { index, style, className, key, rowData } = props;
        const { timeZone, intl } = this.props;
        if (this.state.expandedRows.includes(index)) {
            return (
                <div
                    style={{ ...style, display: 'flow-root' }}
                    className={className}
                    key={key}
                >
                    {defaultTableRowRenderer({ ...props, style: { ...style, position: 'static', height: '32px' } })}
                    <AlarmExpandedTable alarm={rowData} timeZone={timeZone} intl={intl} />
                </div>
            );
        }
        return defaultTableRowRenderer(props);
    };
    
    render() {
        const { sortBy, sortDirection, tableSize, allItems, selectedItems, filteredItems } = this.props;
        const allItemsArray = [...allItems.values()];
        const displayFilteredItems = this.sortDisplayItems(filteredItems);
        const displayFilteredCount = displayFilteredItems.length;
        return (
            <section style={{ width: tableSize.width }}>
                <TableButtons
                    items={allItemsArray}
                    width={tableSize.width}
                    setFilterItems={this.setFilterItems}
                    searchFields={['trackableName']}
                />
                <Table
                    noRowsRenderer={this._noRowsRenderer}
                    sortBy={sortBy}
                    sort={this.sort}
                    ref={this.tableRef}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Medium"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={this.getRowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header"
                    headerStyle={{ height: tableSize.headerHeight, borderTop: '1px solid #d3d3d3' }}
                    rowStyle={({ index }) => this.trStyleFormat(displayFilteredItems, { index })}
                    onRowClick={({ event, index, rowData }) => this.handleSelectRow(selectedItems, { event, index, rowData })}
                    rowCount={displayFilteredCount} // required
                    rowGetter={({ index }) => displayFilteredItems[index]} // required
                    rowRenderer={this.rowRenderer}
                >
                    <Column
                        label=""
                        dataKey="checkbox"
                        width={30}
                        cellRenderer={({ rowIndex }) => this.checkboxButton(rowIndex)}
                        disableSort
                        className="Virtual-Table-Column-Checkbox first-column"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.TRACKABLE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        width={200}
                        cellRenderer={({ cellData, rowData }) => (<div title={rowData.alarmInfo}>{cellData}</div>)}
                        dataKey="trackableName"
                        id="site-vew-alarms-table-trackables"
                        disableSort
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={75}
                        label=""
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.getActionsIcons(rowData)}
                        dataKey="actions"
                        disableSort
                        className="Virtual-Table-Column-Center"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
