import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';

import 'react-virtualized/styles.css';
import withSortSelect from 'components/HOC/withSortSelect';
import { COMPONENTS } from 'common/ui-labels';
import { macToUpperCase } from 'utils/utils';
import { virtualizedHeaderRenderer, formatDbNameToHumanReadable } from 'utils/htmlUtils';
import { getTimeByFormatAndTimezone } from 'utils/timeUtils';

class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        timeZone: PropTypes.string,
        trStyleFormat: PropTypes.func,
        trClassFormat: PropTypes.func,
        setFilterItems: PropTypes.func,
        handleSelectRow: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape()),

        // Props from parent
        allItems: PropTypes.shape(),
        totalCount: PropTypes.number,
        tableSize: PropTypes.shape(),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        timeZone: '',
        trStyleFormat: () => { },
        trClassFormat: () => { },
        setFilterItems: () => { },
        handleSelectRow: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],
    
        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        tableSize: {},
        dispatch: () => { },
        intl: {
            formatMessage: () => { },
        },
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        
        // HOC Methods
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.trClassFormat = props.trClassFormat.bind(this);
        this.handleSelectRow = props.handleSelectRow.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }

    formatLastHeard = (row, dataKey) => {
        const style = (row.node && row.node.responsive) ? {} : { color: 'red' };

        return (
            <span style={style}>
                {getTimeByFormatAndTimezone(row[dataKey], 'MM/DD/YYYY HH:mm:ss', this.props.timeZone)}
            </span>);
    };

    formatStatus = (row) => {
        if (row.node && row.node.responsive) {
            return (
                <span style={{ color: 'green', fontSize: '1.5em' }}>
                    <i
                        className="fa fa-check"
                    />
                </span>);
        }
        return (
            <span style={{ color: 'red', fontSize: '1.5em' }}>
                <i
                    className="fa fa-exclamation-triangle"
                />
            </span>);
    };

    render() {
        const { sortBy, sortDirection, tableSize, filteredItems, intl } = this.props;
        const displayItems = this.sortDisplayItems(filteredItems);
        const displayCount = displayItems.length;

        const allColumnsWidthExceptName = tableSize.width - 1030;

        return (
            <section style={{ width: tableSize.width }}>
                <Table
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Big"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat(displayItems, { index })}
                    // onRowClick={({ event, index, rowData }) => this.handleSelectRow(displayCount, { event, index, rowData })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.NAME}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => <div className="text-ellipsis">{cellData}</div>}
                        dataKey="name"
                        width={allColumnsWidthExceptName >= 225 ? allColumnsWidthExceptName : 225}
                        id="shm-anchors-name"
                        dataAlign="center"
                        className="Virtual-Table-Column-Left first-column"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.MAC_ADDRESS}</span>}
                        width={200}
                        dataKey="mac_address"
                        id="shm-anchors-mac_address"
                        cellRenderer={({ cellData }) => macToUpperCase(cellData)}
                        headerRenderer={virtualizedHeaderRenderer}
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.TYPE}</span>}
                        width={130}
                        dataKey="type"
                        id="shm-anchors-type"
                        dataAlign="center"
                        cellRenderer={({ cellData }) => (formatDbNameToHumanReadable(intl, cellData))}
                        headerRenderer={virtualizedHeaderRenderer}
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.SITE.AREA}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => <div className="text-ellipsis">{cellData}</div>}
                        width={200}
                        dataKey="sublocation_name"
                        id="shm-anchors-sublocation_name"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.LAST_REPORTED_COLUMN}</span>}
                        width={200}
                        dataKey="last_heard"
                        id="shm-anchors-last_heard"
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ /* columnData, */dataKey, rowData }) => (this.formatLastHeard(rowData, dataKey))}
                        headerRenderer={virtualizedHeaderRenderer}
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.STATUS}</span>}
                        width={130}
                        dataKey="last_heard"
                        id="shm-anchors-last_heard_status"
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ /* columnData, */dataKey, rowData }) => this.formatStatus(rowData, dataKey)}
                        headerRenderer={virtualizedHeaderRenderer}
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.FIRMWARE_VERSION}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => <div className="text-ellipsis">{cellData}</div>}
                        width={200}
                        dataKey="firmware_version"
                        id="shm-anchors-fw"
                        className="Virtual-Table-Column-Left"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
