import * as api from 'api';
import { ACTIONS, MODAL_ACTIONS, DEFAULT_VALUES } from 'common/constants';
import { NODE_RELEASE_ASSIGN_MODAL, NODE_DELETE_MODAL } from 'common/ui-labels';
import { openStatusNotificationPopup, closeStatusNotificationPopup } from 'stores/actions/modalActions/commonModalActions';
import { errorHandler, errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';


export const loadNodes = () => async dispatch => new Promise(async (resolve) => {
    try {
        const nodes = await api.getNodes();

        dispatch({
            type: ACTIONS.NODES_SET,
            params: nodes,
        });
        resolve();
    } catch (err) {
        resolve();
        console.error(err);
        errorHandlerGetRequest(err, dispatch);
    }
});

export const assignNodes = params => async (dispatch) => {
    try {
        const assignedNode = await api.assignNode(params);

        dispatch({
            type: ACTIONS.NODE_ASSIGN,
            params: assignedNode,
        });
    } catch (err) {
        console.error(err);
        errorHandlerGetRequest(err, dispatch);
    }
};

export const releaseNodes = params => async (dispatch) => {
    try {
        const releasedNode = await api.releaseNode(params);
        dispatch({
            type: ACTIONS.NODE_RELEASE,
            params: releasedNode,
        });

        const message = NODE_RELEASE_ASSIGN_MODAL.MESSAGE.SUCCESSFULLY_RELEASED;
        const title = NODE_RELEASE_ASSIGN_MODAL.TITLE.RELEASE_TAG_FUNC(params.node.mac_address);
        const status = DEFAULT_VALUES.SUCCESS_STATUS;

        dispatch(openStatusNotificationPopup({ title, message, status }));

        setTimeout(() => {
            dispatch(closeStatusNotificationPopup());
        }, 3000);
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

export const addNode = params => async (dispatch, getState) => {
    const { appContext: { company, project } } = getState();

    try {
        const node = await api.createNode(params);
        node.company_uid = company ? company.uid : null;
        node.project_uid = project ? project.uid : null;
        node.project_name = project ? project.name : null;

        await dispatch({
            type: ACTIONS.NODE_CREATE,
            params: node,
        });

        return undefined;
    } catch (err) {
        errorHandler(err, dispatch);
        return err;
    }
};

export const deleteNodes = params => async (dispatch) => {
    try {
        if (params.uid) {
            const removedNode = await api.deleteNode(params.uid);

            dispatch({
                type: ACTIONS.NODE_DELETE,
                params: removedNode,
            });

            const message = NODE_DELETE_MODAL.MESSAGE.SINGLE.SUCCESSFULLY_DELETED;
            const title = NODE_DELETE_MODAL.TITLE.SINGLE_FUNC(params.mac_address);
            const status = DEFAULT_VALUES.SUCCESS_STATUS;

            dispatch(openStatusNotificationPopup({ title, message, status }));

            setTimeout(() => {
                dispatch(closeStatusNotificationPopup());
            }, 3000);
        } else {
            let nodesMacAddresses = '';

            for (const node of params) {
                nodesMacAddresses += ` ${node.mac_address}`;
                const removedNode = await api.deleteNode(node.uid);

                dispatch({
                    type: ACTIONS.NODE_DELETE,
                    params: removedNode,
                });
            }

            const message = NODE_DELETE_MODAL.MESSAGE.MULTI.SUCCESSFULLY_DELETED_FUNC(nodesMacAddresses);
            const title = NODE_DELETE_MODAL.TITLE.MULTI;
            const status = DEFAULT_VALUES.SUCCESS_STATUS;

            dispatch(openStatusNotificationPopup({ title, message, status }));

            setTimeout(() => {
                dispatch(closeStatusNotificationPopup());
            }, 3000);
        }
    } catch (err) {
        errorHandler(err, dispatch);
    }
};

export const createNodeInCSV = params => async (dispatch, getState) => {
    try {
        const { appContext: { company, project } } = getState();
        const node = await api.createNode(params);

        dispatch({
            type: ACTIONS.NODE_CREATE,
            params: {
                ...node,
                company_uid: company ? company.uid : null,
                project_uid: project ? project.uid : null,
                project_name: project ? project.name : null,
            },
        });
    } catch (err) {
        errorHandler(err, dispatch);
    }
};

export const validateCsvMacAddresses = mac_addresses => async (dispatch) => {
    try {
        const data = await api.validateNodes(mac_addresses);

        if (data.invalid.length || data.duplicate.length) {
            await dispatch({
                type: MODAL_ACTIONS.MODAL_OPEN_NODE_IMPORT_ERROR,
                params: data,
            });
        } else if (data.valid.length > 0) {
            for (const mac_address of data.valid) {
                await dispatch(createNodeInCSV({ mac_address }));
            }

            await dispatch({
                type: MODAL_ACTIONS.MODAL_OPEN_NODE_IMPORT_SUCCESS,
                params: data,
            });
        }
    } catch (err) {
        errorHandler(err, dispatch);
    }
};

export const importValidMacAddresses = mac_addresses => async (dispatch) => {
    for (const mac_address of mac_addresses) {
        await dispatch(createNodeInCSV({ mac_address }));
    }

    await dispatch({
        type: MODAL_ACTIONS.MODAL_OPEN_NODE_IMPORT_SUCCESS,
        params: { valid: mac_addresses },
    });
};
