import { getReportData, getReportsList, getAlarmsCSV } from 'api';
import { ACTIONS } from 'common/constants';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import { getTimeByFormatAndTimezone, getTimeZoneOfProject, durationHumanize } from 'utils/timeUtils';
import _isEmpty from 'lodash/isEmpty';
import _Get from 'lodash/get';
import { download } from 'utils/htmlUtils';


export const getAlarmsReportsListAction = (params, type) => async (dispatch) => {
    try {
        const { reports } = await getReportsList(params, type);
        
        dispatch({
            type: ACTIONS.SET_ALARMS_REPORT_LIST,
            params: reports,
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_ALARMS_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

function alarmsCount(alarms) {
    return alarms.reduce((res, item) => (res + item.count), 0);
}

function alarmsDuration(alarms) {
    return alarms.reduce((res, item) => (res + item.accumulated_duration), 0);
}

const formatData = data => data.map((element) => {
    const zoneViolationCount = alarmsCount(element.buffer_violations) + alarmsCount(element.zone_violations);
    const uncertifiedOpCount = _Get(element, 'uncertified_operation.count', 0);
    const emergencyCallCount = _Get(element, 'emergency_button.count', 0);
    const zoneViolationDuration = alarmsDuration(element.buffer_violations) + alarmsDuration(element.zone_violations);
    const uncertifiedOpDuration = _Get(element, 'uncertified_operation.accumulated_duration', 0);
    const emergencyCallDuration = _Get(element, 'emergency_button.accumulated_duration', 0);

    return {
        uid: element.uid,
        name: element.name,
        category: element.category,
        zoneViolations: zoneViolationCount || '-',
        uncertifiedOps: uncertifiedOpCount || '-',
        emergencyCalls: emergencyCallCount || '-',
        zoneViolationsDuration: durationHumanize(zoneViolationDuration),
        uncertifiedOpsDuration: durationHumanize(uncertifiedOpDuration),
        emergencyCallsDuration: durationHumanize(emergencyCallDuration),
    };
});

export const getAlarmsReport = params => async (dispatch, getState) => {
    try {
        const { appContext: { project } } = getState();
        const timezone = getTimeZoneOfProject(project);
        const report = await getReportData(params, false);
        if (_isEmpty(report)) return;
        
        const from = getTimeByFormatAndTimezone(report.start, 'MM/DD/YYYY HH:mm', timezone);
        const to = getTimeByFormatAndTimezone(report.end, 'MM/DD/YYYY HH:mm', timezone);
        
        dispatch({
            type: ACTIONS.SET_ALARMS_REPORT,
            params: {
                status: 100,
                data: formatData(report.data),
                from,
                to,
                from_iso: report.start,
                to_iso: report.end,
                shift_name: report.shift && report.shift.name,
                uid: report.uid,
                site_name: report.site_name,
                site_uid: report.site_uid,
            },
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_ALARMS_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

function getDocumentHeader(report, project, companyName, userEmail, timezone) {
    const reportName = '#Alarms report\n';
    const companyRow = `#Company: ${companyName}\n`;
    const projectRow = `#Project: ${project.name}\n`;
    const siteRow = `#Site: ${report.site_name}\n`;
    const shiftName = `#Shift Name: ${report.shift_name || 'Auto-created'}\n`;
    const shiftStart = `#Shift Start: ${getTimeByFormatAndTimezone(report.from_iso, 'YYYY-MM-DD HH:mm z', timezone)}\n`;
    const shiftEnd = `#Shift End: ${getTimeByFormatAndTimezone(report.to_iso, 'YYYY-MM-DD HH:mm z', timezone)}\n`;
    const downloadedBy = `#Downloaded by ${userEmail}\n`;

    return `${reportName}${companyRow}${projectRow}${siteRow}${shiftName}${shiftStart}${shiftEnd}${downloadedBy}\n\n`;
}

export const getAlarmsReportCsv = (report, project, companyName, userEmail) => {
    const timezone = getTimeZoneOfProject(project);
    const fileName = `${report.shift_name || 'day'}_${getTimeByFormatAndTimezone(report.from_iso, 'YYYY-MM-DD', timezone)}_alarms`;
    const documentHeader = getDocumentHeader(report, project, companyName, userEmail, timezone);
    getAlarmsCSV(report.from_iso, report.to_iso, report.site_uid).then((res) => {
        download(`${fileName}.csv`, `${documentHeader}${res.data}`);
    });
};
