import _isEmpty from 'lodash/isEmpty';

import { getReportData, getReportsList } from 'api';
import { ACTIONS } from 'common/constants';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import { roundTo } from 'utils';
import { getTimeByFormatAndTimezone, getTimeZoneOfProject } from 'utils/timeUtils';
import { download } from 'utils/htmlUtils';
import { humanizeDistance } from 'utils/formatters';

export const getDistanceReportsListAction = (params, type) => async (dispatch) => {
    try {
        const { reports } = await getReportsList(params, type);
        
        dispatch({
            type: ACTIONS.SET_DISTANCE_REPORT_LIST,
            params: reports,
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_DISTANCE_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

const formatDistance = (cell) => {
    if (!cell) return 0;
        
    return humanizeDistance(cell, 'm', 'ft');
};

const formatData = (data) => {
    data.trackables.forEach((element) => {
        element.formattedTotalDistance = formatDistance(element.totalDistance);
    });
    
    return data;
};

export const getDistanceReport = params => async (dispatch, getState) => {
    try {
        const { appContext: { project } } = getState();
        const timezone = getTimeZoneOfProject(project);
        const report = await getReportData(params, false);
        if (_isEmpty(report)) return;
        
        const from = getTimeByFormatAndTimezone(report.start, 'MM/DD/YYYY HH:mm', timezone);
        const to = getTimeByFormatAndTimezone(report.end, 'MM/DD/YYYY HH:mm', timezone);
        
        dispatch({
            type: ACTIONS.SET_DISTANCE_REPORT,
            params: {
                status: 100,
                data: formatData(report.data),
                from,
                to,
                from_iso: report.start,
                to_iso: report.end,
                shift_name: report.shift && report.shift.name,
                uid: report.uid,
                site_name: report.site_name,
                site_uid: report.site_uid,
                project_name: report.project_name,
            },
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_DISTANCE_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};


function getDocumentHeader(report, project, companyName, userEmail) {
    const reportName = '#Travel Distance report\n';
    const companyRow = `#Company: ${companyName}\n`;
    const projectRow = `#Project: ${project.name}\n`;
    const siteRow = `#Site: ${report.site_name}\n`;
    const shiftName = `#Shift Name: ${report.shift_name || 'Auto-created'}\n`;
    const shiftStart = `#Shift Start: ${report.from_iso}\n`;
    const shiftEnd = `#Shift End: ${report.to_iso}\n`;
    const downloadedBy = `#Downloaded by ${userEmail}\n`;

    return `${reportName}${companyRow}${projectRow}${siteRow}${shiftName}${shiftStart}${shiftEnd}${downloadedBy}\n\n`;
}

export const getDistanceReportCsv = (data, report, project, companyName, userEmail) => {
    const timezone = getTimeZoneOfProject(project);
    let result = getDocumentHeader(report, project, companyName, userEmail);
    function getCategory(category) {
        if (!category) return 'UNSPECIFIED';
        return category;
    }
    
    result += 'Name,Time,Distance (mm),Category\n';
    
    for (let i = 0; i < data.length; i += 1) {
        const trackable = data[i];
        result += report.ranges.reduce((memo, range, index) => {
            const rounded = roundTo(trackable.distances[index], 2);
            memo += `${trackable.name},${new Date(range).toISOString()},${rounded},${getCategory(trackable.category)}\n`;
            return memo;
        }, '');
    }
    const start = getTimeByFormatAndTimezone(report.from, 'YYYY-MM-DD', timezone);
    const file_name = `${report.shift_name || 'day'}_${start}_travel_distance.csv`;
    
    download(file_name, result);
};
