import { downloadPws, getPwsData, loadTrackablesExistedInPeriod } from 'api';
import moment from 'moment';
import { ACTIONS } from 'common/constants';
import { getTimeByFormatAndTimezone, durationHumanize } from 'utils/timeUtils';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';


function getPwsReportLink(params) {
    return `/api/v2/external_events?event_type[]=pws&from=${params.from}&to=${params.to}&track_uid[]=${params.trackable_uid}&archive=true&report=true`;
}

function getReportOptions(params) {
    const isDay = moment(params.to).diff(params.from, 'days') === 0;
    const format = isDay ? 'MM/DD/YYYY HH:mm:ss' : 'MM/DD/YYYY';

    return {
        trackable_name: params.trackable_name,
        start: getTimeByFormatAndTimezone(params.from, format, params.timezone),
        end: getTimeByFormatAndTimezone(params.to, format, params.timezone),
    };
}

const getPwsTableData = async (params) => {
    const { external_events, totalCount } = await getPwsData(params);
    let objectName = '';
    
    const tableData = external_events.map((row) => {
        if (!objectName) objectName = row.object_name;
        const event_info = JSON.parse(row.event_info);
        return ({
            date: getTimeByFormatAndTimezone(row.ts, 'ddd MM/DD/YYYY HH:mm:ss', params.timezone),
            contact_uid: event_info.contact_uid,
            contact_name: event_info.contact_name,
            severity_level: event_info.severity_level,
            duration: durationHumanize(event_info.duration * 1000),
            raw_duration: event_info.duration,
            distance: Math.round(event_info.minimal_distance / 100) / 10,
        });
    });
    
    return {
        objectName,
        tableData,
        totalCount,
    };
};

function handleTableDate(report, data) {
    data.forEach((row) => {
        if (row.contact_name && row.contact_uid) {
            if (!report[row.contact_uid]) {
                report[row.contact_uid] = {
                    contactNumber: 0,
                    contact_name: row.contact_name,
                    duration: 0,
                    uid: row.contact_uid,
                    count: 0,
                    subTable: [],
                };
                if (!report.contactNames[row.contact_name]) {
                    report.contactNames[row.contact_name] = 1;
                } else {
                    report.contactNames[row.contact_name] += 1;
                }
            }
            report.totalContacts += 1;
            report.totalDuration += row.raw_duration * 1000;
            report[row.contact_uid].duration += row.raw_duration * 1000;
            report[row.contact_uid].count += 1;
            report[row.contact_uid].subTable.push({
                number: report[row.contact_uid].count,
                date: row.date,
                severity_level: row.severity_level,
                duration: row.duration,
                distance: row.distance,
            });
        }
    });
}

const setReportError = err => ({
    type: ACTIONS.SET_PWS_REPORT_ERROR,
    params: { message: err.message || 'Error during generation of Event History report' },
});

const setReportStatus = status => ({
    type: ACTIONS.SET_PWS_REPORT,
    params: { status },
});

const setReportData = ({ reportData, link, options, objectName }) => {
    const totalContacts = reportData.totalContacts;
    const totalDuration = reportData.totalDuration;
    const contactNames = reportData.contactNames;
    delete reportData.totalContacts;
    delete reportData.totalDuration;
    delete reportData.contactNames;
    
    return {
        type: ACTIONS.SET_PWS_TABLE_DATA,
        params: {
            tableData: Object.values(reportData).map((row, index) => ({ 
                ...row, formattedDuration: durationHumanize(row.duration), id: index + 1, 
            })),
            report: {
                link,
                contactNames,
                totalContacts,
                totalDuration,
                objectName,
                start: options.start,
                end: options.end,
            },
        },
    };
};

export const createPwsReport = params => async (dispatch) => {
    try {
        const reportData = { totalDuration: 0, totalContacts: 0, contactNames: {} };
        const page = 0;
        const limit = 1000;
        
        const resultOne = await getPwsTableData({ ...params, limit, page });
    
        if (resultOne.totalCount) {
            const pages = Math.ceil(resultOne.totalCount / limit);
            handleTableDate(reportData, resultOne.tableData);
            for (let i = 1; i < pages; i += 1) {
                // eslint-disable-next-line no-await-in-loop
                const result = await getPwsTableData({ ...params, limit, page: page + i });
                const status = Math.round((i / pages) * 100);
                
                handleTableDate(reportData, result.tableData);
                dispatch(setReportStatus(status));
            }
        }
        const link = getPwsReportLink(params);
        const options = getReportOptions(params);
        dispatch(setReportData({ reportData, link, options, objectName: resultOne.objectName }));
    } catch (err) {
        console.error(err);
        dispatch(setReportError(err));
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

export const loadingTrackables = () => ({
    type: ACTIONS.SET_PWS_REPORT_LOADING_TRACKABLES,
});

export const getPwsTrackablesForPeriod = params => async (dispatch) => {
    try {
        const trackables = (await loadTrackablesExistedInPeriod(params.start, params.end)).map(tko => ({ value: tko.uid, label: tko.name }));
    
        dispatch({
            type: ACTIONS.SET_PWS_REPORT_FILTER_TRACKABLES,
            params: trackables,
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_PWS_REPORT_FILTER_TRACKABLES,
            params: null,
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

export const getPwsReportCsv = params => async (dispatch) => {
    downloadPws(params.link).catch((err) => {
        errorHandlerGetRequest(err, dispatch);
    });
};
