import { getReportData, getReportsList, getZoneEvents, getAlarmsByUrl } from 'api';
import { ACTIONS } from 'common/constants';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import { getTimeByFormatAndTimezone, getTimeZoneOfProject, durationHumanize } from 'utils/timeUtils';
import _isEmpty from 'lodash/isEmpty';
import { download } from 'utils/htmlUtils';


export const getZoneActivityReportsListAction = (params, type) => async (dispatch) => {
    try {
        const { reports } = await getReportsList(params, type);
        
        dispatch({
            type: ACTIONS.SET_ZONE_ACTIVITY_REPORT_LIST,
            params: reports,
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_ZONE_ACTIVITY_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

const formatData = (data) => {
    data.trackables.forEach((element) => {
        element.formattedDuration = durationHumanize(element.duration);
    });

    return data;
};

export const getZoneActivityReport = params => async (dispatch, getState) => {
    try {
        const { appContext: { project } } = getState();
        const timezone = getTimeZoneOfProject(project);
        const report = await getReportData(params, false);
        if (_isEmpty(report)) return;
        
        const from = getTimeByFormatAndTimezone(report.start, 'MM/DD/YYYY HH:mm', timezone);
        const to = getTimeByFormatAndTimezone(report.end, 'MM/DD/YYYY HH:mm', timezone);
        
        dispatch({
            type: ACTIONS.SET_ZONE_ACTIVITY_REPORT,
            params: {
                status: 100,
                data: formatData(report.data),
                from,
                to,
                from_iso: report.start,
                to_iso: report.end,
                shift_name: report.shift && report.shift.name,
                uid: report.uid,
                site_name: report.site_name,
                site_uid: report.site_uid,
            },
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_ZONE_ACTIVITY_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

async function loadZoneActivityFile(url, file_name) {
    const page = url.match(/page=[0-9]+/g);
    return getAlarmsByUrl(url).then((res) => {
        download(`${file_name}_${page}.csv`, res.data);
        const nextPage = res.headers['content-next-page'];
        if (nextPage && nextPage !== 'null') {
            loadZoneActivityFile(nextPage, file_name);
        }
    });
}

function getDocumentHeader(report, project, companyName, userEmail, timezone) {
    const reportName = '#Info Zone Activity report\n';
    const companyRow = `#Company: ${companyName}\n`;
    const projectRow = `#Project: ${project.name}\n`;
    const siteRow = `#Site: ${report.site_name}\n`;
    const shiftName = `#Shift Name: ${report.shift_name || 'Auto-created'}\n`;
    const shiftStart = `#Shift Start: ${getTimeByFormatAndTimezone(report.from_iso, 'YYYY-MM-DD HH:mm z', timezone)}\n`;
    const shiftEnd = `#Shift End: ${getTimeByFormatAndTimezone(report.to_iso, 'YYYY-MM-DD HH:mm z', timezone)}\n`;
    const downloadedBy = `#Downloaded by ${userEmail}\n`;

    return `${reportName}${companyRow}${projectRow}${siteRow}${shiftName}${shiftStart}${shiftEnd}${downloadedBy}\n\n`;
}

export const getZoneActivityReportCsv = (report, project, companyName, userEmail) => {
    const timezone = getTimeZoneOfProject(project);
    const file_name = `${report.shift_name || 'day'}_${getTimeByFormatAndTimezone(report.from_iso, 'YYYY-MM-DD', timezone)}_zone_activity`;
    const documentHeader = getDocumentHeader(report, project, companyName, userEmail, timezone);
    getZoneEvents(report.from_iso, report.to_iso, report.area_uids).then((res) => {
        download(`${file_name}.csv`, `${documentHeader}${res.data}`);
        const nextPage = res.headers['content-next-page'];
        if (nextPage && nextPage !== 'null') {
            loadZoneActivityFile(nextPage, file_name);
        }
    });
};
