import _Get from 'lodash/get';
import _isEmpty from 'lodash/isEmpty';

export default class CurrentShiftSummaryTableData {
    constructor(locations, sublocations, trackables, zones) {
        this.locations = locations.reduce((r, location) => {
            r[location.uid] = location;
            return r;
        }, {}); // array

        this.sublocations = sublocations.reduce((r, sublocation) => {
            r[sublocation.uid] = sublocation;
            return r;
        }, {}); // array

        this.trackables = trackables; // object
        this.zones = zones; // object
    }

    _createSummaryTableStructureObject() {
        const locations = Object.keys(this.locations).reduce((r, locationKey) => {
            const location = this.locations[locationKey];
            r[locationKey] = {
                name: location.name,
                people_active: 0,
                people_expected: 0,
                vehicle_active: 0,
                vehicle_total: 0,
                others_actual: 0,
                others_expected: 0,
                location_uid: locationKey,
                expand: { },
            };
            return r;
        }, {});
        Object.keys(this.sublocations).forEach((sublocationKey) => {
            const sublocation = this.sublocations[sublocationKey];
            const zoneObject = Object.keys(this.zones).reduce((r, zoneKey) => {
                const zone = this.zones[zoneKey];

                if (zone && zone.active && zone.sublocation_uid === sublocationKey) {
                    r[zoneKey] = {
                        name: zone.name,
                        people_active: 0,
                        people_expected: 0,
                        vehicle_active: 0,
                        vehicle_total: 0,
                        others_actual: 0,
                        others_expected: 0,
                        zone_uid: zoneKey,
                    };
                }
                return r;
            }, {});
            locations[sublocation.location_uid].expand[sublocationKey] = {
                name: sublocation.name,
                people_active: 0,
                people_expected: 0,
                vehicle_active: 0,
                vehicle_total: 0,
                others_actual: 0,
                others_expected: 0,
                sublocation_uid: sublocationKey,
                ...!_isEmpty(zoneObject) && { expand: zoneObject },
            };
        });
        return locations;
    }

    _tkoInsideZone(zone, uid) {
        const { trackables_in_zone } = zone;
        if (Array.isArray(trackables_in_zone) && trackables_in_zone.length) {
            return !!trackables_in_zone.find(tko => (tko.uid === uid));
        }
        return false;
    }

    _getZonesInclusion(sublocation_uid, tko_uid) {
        return Object.keys(this.zones).reduce((r, zoneKey) => {
            const zone = this.zones[zoneKey];
            if (zone.sublocation_uid === sublocation_uid && this._tkoInsideZone(zone, tko_uid)) {
                r.push(zone.uid);
            }
            return r;
        }, []);
    }

    _getPropertyName(responsive, category) {
        if (!responsive) {
            return null;
        }
        if (category === 'PERSON') {
            return 'people_active';
        }
        if (category === 'VEHICLE') {
            return 'vehicle_active';
        }
        return 'others_actual';
    }

    /**
     *
     * @param emptyLocations
     * @returns {{
        [location_uid1]: {
            name, location_uid,
            people_active, people_expected,
            vehicle_active, vehicle_total,
            others_actual, others_expected,
            location_uid: location_uid1,
            expand: {
                [sublocation_uid3]: {
                    name, sublocation_uid,
                    people_active, people_expected,
                    vehicle_active, vehicle_total,
                    others_actual, others_expected,
                    expand: {
                        [zone_uid1]: {
                            name, zone_uid,
                            people_active, people_expected,
                            vehicle_active, vehicle_total,
                            others_actual, others_expected
                        },
                    },
                },
            }
        }
     }}
     * @private
     */
    _fillSummaryTableStructureObject(emptyLocations) {
        Object.keys(this.trackables).forEach((tkoKey) => {
            const tko = this.trackables[tkoKey];

            const sl_uid = _Get(tko, 'position.sl_uid');
            const sl_id = _Get(tko, 'position.sl_id');
            const responsive = _Get(tko, 'node.responsive');
            const category = _Get(tko, 'category');

            const sublocation_uid = sl_uid || sl_id;
            const location_uid = (this.sublocations[sublocation_uid] || {}).location_uid;
            const propertyName = this._getPropertyName(responsive, category);
            const zones_uid = this._getZonesInclusion(sublocation_uid, tko.uid);

            const hasLocation = location_uid && emptyLocations[location_uid];
            const hasSublocation = sublocation_uid
                && emptyLocations[location_uid]
                && emptyLocations[location_uid].expand
                && emptyLocations[location_uid].expand[sublocation_uid];

            if (propertyName && hasLocation) {
                emptyLocations[location_uid][propertyName] += 1;

                if (hasSublocation) {
                    emptyLocations[location_uid].expand[sublocation_uid][propertyName] += 1;

                    zones_uid.forEach((zone_uid) => {
                        const hasZone = zone_uid
                            && emptyLocations[location_uid].expand[sublocation_uid].expand
                            && emptyLocations[location_uid].expand[sublocation_uid].expand[zone_uid];

                        if (hasZone) {
                            emptyLocations[location_uid].expand[sublocation_uid].expand[zone_uid][propertyName] += 1;
                        }
                    });
                }
            }
        });
        return emptyLocations;
    }

    /**
     *
     * @param filledLocations
     * @returns [{
            name, location_uid,
            people_active, people_expected,
            vehicle_active, vehicle_total,
            others_actual, others_expected,
            expand: [{
                name, sublocation_uid,
                people_active, people_expected,
                vehicle_active, vehicle_total,
                others_actual, others_expected,
                expand: [{
                    name, zone_uid,
                    people_active, people_expected
                    vehicle_active:, vehicle_total
                    others_actual, others_expected,
                }, ...],
            }, ...]
         }, ...]
     * @private
     */
    _convertArrayStructure(filledLocations) {
        return Object.values(filledLocations).reduce((memo, location) => {
            location.expand = Object.values(location.expand).map((sublocation) => {
                if (sublocation.expand) {
                    sublocation.expand = Object.values(sublocation.expand);
                }
                return sublocation;
            });
            memo[location.location_uid] = location;
            return memo;
        }, {});
    }

    getSummaryTableDataArray() {
        const emptyLocations = this._createSummaryTableStructureObject();
        const filledLocations = this._fillSummaryTableStructureObject(emptyLocations);
        return this._convertArrayStructure(filledLocations);
    }
}
