import { getZones } from 'utils/simpleUtils';
import store from 'stores/LocationStore';
import { macToUpperCase, nodeFilterNotWin } from 'utils/utils';


function getLocationByUid(uid) {
    return store.getLocations().find(location => location.uid === uid);
}

function getAnchorsBySublocation(sublocation) {
    if (!sublocation) return [];
    return store.getAnchors().filter(anchor =>
        anchor.position && anchor.position.sublocation_uid === sublocation.uid);
}

function getZonePrototypeById(zone_type_id) {
    return store.getZonesPrototypes().find(zone_prototype => zone_prototype.isZoneTypeIdEqualTo(zone_type_id));
}

function getZonePrototypeByName(zone_type_name) {
    return store.getZonesPrototypes().find(zone_prototype => zone_prototype.isZoneTypeNameEqualTo(zone_type_name));
}

function getSublocationByUid(uid) {
    return store.getSublocations().find(sublocation => sublocation.uid === uid);
}

function getSublocationsByLocationUid(location_uid) {
    return store.getSublocations().filter(sublocation => sublocation.location_uid === location_uid);
}

function getAlarm(uid) {
    return store.getAlarms().find(x => x.uid === uid);
}

function getZoneByUid(uid) {
    const playback = store.getPlayback();
    const isPlaybackOpen = (playback && playback.isOpenPlayBackControl);
    let zone = store.getZones()[uid];

    if (!zone && isPlaybackOpen) {
        zone = playback.zones.find(z => z.uid === uid);
    }

    return zone;
}

function getNodeAssociationByTrackUid(obj_uid) {
    const playback = store.getPlayback();
    const isPlaybackOpen = playback && playback.isOpenPlayBackControl;

    if (!isPlaybackOpen) {
        return store.getAssociations().byObjUid[obj_uid];
    }
    return playback.associations.find(assoc => assoc.trob_uid === obj_uid);
}

/**
 *
 * @param uid
 * @returns {Trackable}
 */
function getTrackableByUid(uid) {
    let trackable;
    const playback = store.getPlayback();
    const isPlaybackOpen = playback && playback.isOpenPlayBackControl;

    if (!isPlaybackOpen) {
        trackable = store.getTrackablesObject()[uid];
    } else {
        trackable = playback.trackables.find(to => to.uid === uid);
    }

    return trackable;
}

function getBuffersBySublocation(sublocation, isPlayback = false) {
    const zones = getZones(store.getZonesArray(), store.getPlayback(), sublocation, isPlayback);
    return zones.reduce((memo, zone) => {
        const to = getTrackableByUid(zone.trackable_object_uid);
        const assoc = to && getNodeAssociationByTrackUid(to.uid);
        if (zone.dynamic && assoc && to.position && to.position.sl_id === sublocation.uid) {
            memo.push({
                ...zone,
                position: to.position,
            });
        }
        return memo;
    }, []);
}

function getNodeByMacAddress(mac_address) {
    if (mac_address) {
        return store.getNodesObject()[mac_address.toUpperCase()];
    }
    return null;
}

function getTrackables() {
    return store.getTrackables();
}

function getNodesFromStoreByMac(mac) {
    if (mac) {
        const storeNodes = store.getNodesObject();
        return storeNodes[mac.toUpperCase()];
    }
    return null;
}

function getNotAssociatedNode(state, project) {
    if (project) {
        const nodes = Object.values(state.entities.nodes.byMac);
        const associations = store.getAssociations(state);

        const parsedNodes = nodeFilterNotWin(nodes)
            .filter(node => (node.project_uid === project.uid && !associations.byMac[node.mac_address.toUpperCase()]));

        return parsedNodes.map(x => (
            {
                mac_address: x.mac_address,
                label: macToUpperCase(x.mac_address),
                value: macToUpperCase(x.mac_address),

            }));
    }
    return [];
}

function getZoneArray(state) {
    return Object.values(state.entities.zones.byUid);
}

function getAssociations(state) {
    return state.entities.associations;
}

function getNodesMap(state) {
    return state.entities.nodes.byMac;
}

function getGroupsMap(state) {
    return state.entities.groups.byUid;
}


export const Location = {
    getLocationByUid,
};
export const WinAnchor = {
    getAnchorsBySublocation,
};
export const ZonePrototype = {
    getZonePrototypeById,
    getZonePrototypeByName,
};
export const Sublocation = {
    getSublocationByUid,
    getSublocationsByLocationUid,
};
export const Alarm = {
    getAlarm,
};
export const Zone = {
    getZoneByUid,
    getZoneArray,
    getBuffersBySublocation,
};
export const Association = {
    getAssociations,
};
export const Trackable = {
    getTrackableByUid,
    getTrackables,
};
export const Node = {
    getNodeByMacAddress,
    getNodesFromStoreByMac,
    getNotAssociatedNode,
    getNodesMap,
};
export const Group = {
    getGroupsMap,
};
