import { getZoneTypeName, parseDefaultAllowed, parseZoneMode, zoneFullActive, adjustScheduleFromServer } from 'utils/zoneUtils';
import { ZONE_MODE, ZONE_TYPE } from 'common/constants';
import { SITE_ZONES_TABLE, DB_NAMES } from 'common/ui-labels';


export default class Zone {
    constructor(zone) {
        this.active = zone.active || false;
        this.apply_to = zone.apply_to;
        this.created_at = zone.created_at;
        this.default_allowed = zone.default_allowed;
        this.depth = zone.depth || null;
        this.dynamic = zone.dynamic;
        this.location_uid = zone.location_uid || null;
        this.name = zone.name || '';
        this.priority = zone.priority;
        this.project_uid = zone.project_uid;
        this.removed_at = zone.removed_at;
        this.schedule = adjustScheduleFromServer(zone.schedule) || { start_time: null, stop_time: null };
        this.shape = zone.shape || null;
        this.sublocation_id = zone.sublocation_id || null;
        this.sublocation_uid = zone.sublocation_uid || null;
        this.trackable_object_uid = zone.trackable_object_uid || null;
        this.uid = zone.uid;
        this.updated_at = zone.updated_at;
        this.user_params = zone.user_params || {};
        this.zone_alarm_level = zone.zone_alarm_level;
        this.zone_exceptions = zone.zone_exceptions || [];
        this.zone_exceptions_groups = zone.zone_exceptions_groups || [];
        this.zone_mode = zone.zone_mode || ZONE_MODE.ALWAYS_ACTIVE;
        this.zone_type = zone.zone_type || null;
        this.custom_info = zone.custom_info;
        this.zone_type_id = zone.zone_type_id || null;
        this.zone_id = zone.zone_id || null;
        this.trackables_in_zone = zone.trackables_in_zone || [];
        this.zone_type_name = zone.zone_type_name || null;
        if (zone.zone_geometry && zone.zone_geometry.radius) {
            this.zone_geometry = {
                ...zone.zone_geometry,
                radius: zone.zone_geometry.radius / 1000,
            };
        } else {
            this.zone_geometry = zone.zone_geometry;
        }
    }
    
    isZoneUidEqualTo(uid) {
        if ((typeof this.uid === 'string') && (typeof uid === 'string')) {
            return this.uid === uid;
        }
        return undefined;
    }
    
    getProperties() {
        return { ...this };
    }
    
    getZoneSpecialCases(uid) {
        const item = this.zone_exceptions.find(x => uid === x) || this.zone_exceptions_groups.find(x => uid === x);
        
        if (item) {
            return this.default_allowed;
        }
        return null;
    }
    
    zoneHasSameSublocationName(sublocation_uid, name) {
        return this.sublocation_uid === sublocation_uid && this.name === name;
    }

    formattedZone({ locations, sublocations, intl, zonesPrototypes }) {
        const props = this.getProperties();
        props.schedule_start_time = this.schedule.start_time;
        props.schedule_stop_time = this.schedule.stop_time;
        props.apply_to = parseDefaultAllowed(props);
        const { activated, active24 } = parseZoneMode(props);
        props.activated = activated;
        props.active24 = active24;
        props.active_display = this.active ? 'Active' : 'Inactive';
        props.zone_type_name_display = getZoneTypeName(zonesPrototypes, this.zone_type_name, intl);
        if (locations && sublocations) {
            const zoneSublocation = this.sublocation_uid ? sublocations.find(sublocation => sublocation.uid === this.sublocation_uid) : null;
            const zoneLocation = this.location_uid ? locations.find(location => location.isLocationUidEqualTo(this.location_uid)) : null;
            props.sublocation_name_display = zoneSublocation ? zoneSublocation.name : '';
            props.location_name_display = zoneLocation ? zoneLocation.name : '';
        }
        if (intl) {
            props.title = this.getZoneTitle(zonesPrototypes, intl);
            props.zoneInfo = this.getZoneInfo(zonesPrototypes, intl);
        }
        return props;
    }
    
    
    getZoneInfo(zonePrototypes, intl) {
        const result = {};
        result[SITE_ZONES_TABLE.TEXT.TYPE_FUNC(intl)] = getZoneTypeName(zonePrototypes, this.zone_type_name, intl);

        if (this.zone_alarm_level && this.zone_type_name.toLowerCase() !== ZONE_TYPE.INFO.name) {
            result[SITE_ZONES_TABLE.TEXT.SEVERITY_FUNC(intl)] = this.zone_alarm_level;
        }
        result[SITE_ZONES_TABLE.TEXT.STATUS_FUNC(intl)] = zoneFullActive(this) ?
            DB_NAMES.ACTIVE(intl) : SITE_ZONES_TABLE.TEXT.STATUS_INACTIVE_FUNC(intl);
        return result;
    }
    
    getZoneTitle(zonesPrototypes, intl) {
        let title = `${this.name}\n`;
        if (this.zone_type_name) {
            title += `${SITE_ZONES_TABLE.TEXT.TYPE_FUNC(intl)}: ${getZoneTypeName(zonesPrototypes, this.zone_type_name, intl)}\n`;
        }
        if (this.zone_alarm_level) {
            title += `${SITE_ZONES_TABLE.TEXT.SEVERITY_FUNC(intl)}: ${this.zone_alarm_level}\n`;
        }
        title += `${SITE_ZONES_TABLE.TEXT.STATUS_FUNC(intl)}: ${zoneFullActive(this) ? DB_NAMES.ACTIVE(intl) : SITE_ZONES_TABLE.TEXT.STATUS_INACTIVE_FUNC(intl)}\n`;
        return title;
    }
    
    static getAllZones(state) {
        return state.entities.zones.byUid;
    }
    
    static getAllZonesArray(state) {
        return Object.values(Zone.getAllZones(state));
    }
}
