import { NOTIFY_ACTIONS, WS_ACTIONS } from 'worker';
import { ACTIONS, NODE_TYPE } from 'common/constants';
import _Get from 'lodash/get';
import Node from 'stores/classes/Node';
import { getNotifyParams } from 'utils/storeUtils';


const initialState = {
    byMac: {},
    byUid: {},
};

function makeStateCopy(state) {
    return {
        byMac: { ...state.byMac },
        byUid: { ...state.byUid },
    };
}


function getSwVersion(new_sw_version = {}, old_sw_version = {}) {
    const sw_str = _Get(new_sw_version, 'sw_str', null) || _Get(old_sw_version, 'sw_str', null);
    const git = _Get(new_sw_version, 'git', null) || _Get(old_sw_version, 'git', null);
    
    return {
        ...old_sw_version,
        sw_str,
        git,
    };
}

const nodes = (state = initialState, action) => {
    switch (action.type) {
        case ACTIONS.NODES_SET: {
            const newState = { byMac: {}, byUid: {} };
            action.params.forEach((node) => {
                node.last_heard = new Date(node.last_heard);
                if (node.mac_address) {
                    newState.byMac[node.mac_address.toUpperCase()] = new Node(node);
                    newState.byUid[node.uid] = node.mac_address.toUpperCase();
                }
            });
            return newState;
        }
        case ACTIONS.NODES_CLEAR:
            return { ...initialState };
        case NOTIFY_ACTIONS.TRACKABLE_OBJECT_ASSOCIATION_CREATED: {
            const newState = makeStateCopy(state);
            const params = getNotifyParams(action);
            params.forEach((param) => {
                const association = param[0] || param;
                if (association.mac_address) {
                    const node = newState.byMac[association.mac_address.toUpperCase()];
                    if (node) {
                        const nodeProperties = node.getProperties();
                        nodeProperties.node_type = NODE_TYPE.TRACKABLE;
                        newState.byMac[association.mac_address.toUpperCase()] = new Node(nodeProperties);
                    }
                }
            });
            return newState;
        }
        case NOTIFY_ACTIONS.NODE_ASSIGN:
        case ACTIONS.NODE_ASSIGN: {
            const newState = makeStateCopy(state);
            const params = getNotifyParams(action);
            params.forEach((param) => {
                const assignedNode = param[0] || param;
                const mac_address = newState.byUid[assignedNode.uid];
                if (mac_address) {
                    newState.byMac[mac_address] = new Node(assignedNode);
                }
            });
            return newState;
        }
        case NOTIFY_ACTIONS.NODE_RELEASE:
        case ACTIONS.NODE_RELEASE: {
            const newState = makeStateCopy(state);
            const params = getNotifyParams(action);
            params.forEach((param) => {
                const releasedNode = param[0] || param;
                const mac_address = newState.byUid[releasedNode.uid];

                if (mac_address) {
                    const node = newState.byMac[mac_address];
                    const nodeProperties = node.getProperties();
                    delete nodeProperties.project_uid;
                    delete nodeProperties.project_name;
                    newState.byMac[mac_address] = new Node(nodeProperties);
                }
            });
            return newState;
        }
        case NOTIFY_ACTIONS.NODE_CREATED:
        case ACTIONS.NODE_CREATE: {
            const newState = makeStateCopy(state);
            const params = getNotifyParams(action);
            params.forEach((param) => {
                newState.byMac[param.mac_address.toUpperCase()] = new Node(param);
                newState.byUid[param.uid] = param.mac_address.toUpperCase();
            });
            return newState;
        }
        case WS_ACTIONS.POSITIONS:
        case NOTIFY_ACTIONS.HBS: {
            if (action.params.length > 0) {
                const newState = makeStateCopy(state);
                action.params.forEach((elem) => {
                    const node = elem.mac ? newState.byMac[elem.mac] : null;
                    if (node) {
                        const ms = elem.ms ? elem.ms : 0;
                        const ts = elem.ts ? elem.ts : 0;
                        const lastHeard = new Date((ts * 1000) + ms);
                        const nodeProperties = node.getProperties();
                        
                        nodeProperties.last_heard = lastHeard;
                        if ('charging' in elem) {
                            nodeProperties.charging = elem.charging;
                        }
                        
                        newState.byMac[elem.mac] = new Node(nodeProperties);
                    }
                });
                return newState;
            }
            return state;
        }
        case NOTIFY_ACTIONS.NODE_UPDATED:
        case ACTIONS.NODE_UPDATE: {
            const newState = makeStateCopy(state);
            const params = getNotifyParams(action);

            params.forEach((param) => {
                const mac_address = newState.byUid[param.uid];

                if (mac_address) {
                    const node = newState.byMac[mac_address];
                    const nodeProps = node.getProperties();
                    const nodeLastHeard = new Date(node.last_heard);
                    const updatedLastHeard = new Date(param.last_heard);

                    nodeProps.last_heard = nodeLastHeard > updatedLastHeard ? nodeLastHeard : updatedLastHeard;
                    nodeProps.battery = param.battery;
                    nodeProps.hw_info = param.hw_info;
                    nodeProps.sw_version = getSwVersion(param.sw_version, nodeProps.sw_version);
                    newState.byMac[mac_address] = new Node(nodeProps);
                }
            });
            return newState;
        }
        case NOTIFY_ACTIONS.NODE_DELETED:
        case ACTIONS.NODE_DELETE: {
            const newState = makeStateCopy(state);
            const params = getNotifyParams(action);

            params.forEach((param) => {
                const mac_address = newState.byUid[param.uid];
                if (mac_address) {
                    delete newState.byUid[param.uid];
                    delete newState.byMac[mac_address];
                }
            });
            return newState;
        }
        default:
            return state;
    }
};

export default nodes;
