import { takeEvery, all, fork, select, call, put } from 'redux-saga/effects';
import * as api from 'api';
import { ACTIONS, SAGA_ACTIONS, ZONE_MODE } from 'common/constants';
import errorHandler from 'stores/sagas/commonSaga';
import { changeScaleZoneRadius, isDirectionalRestriction } from 'utils/zoneUtils';
import { changeParamsOnSiteView } from 'stores/actions/siteViewActions';
import { loadTrackables } from 'stores/actions/entityActions/trackablesActions';
import { trackableByUid, zonePrototypeById, zoneByUid } from 'stores/sagas/selectors';


export function* deleteSingleZone(uid) {
    try {
        yield call(api.deleteZone, uid);
        yield put({ type: ACTIONS.ZONE_DELETE, params: { uid } });
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* handleZoneTrackable(zone_uid, trackable_uid) {
    try {
        const to = yield select(trackableByUid, trackable_uid);
        if (to) {
            const trackable = {
                ...to,
                self_zone_uid: zone_uid,
                default_height: to.default_height * 1000,
            };

            yield call(api.updateTrackable, trackable, to.uid);
            yield put(loadTrackables());
        }
    } catch (err) {
        yield call(errorHandler, err);
    }
}

export function* deleteStaticDynamicZone({ uid }) {
    try {
        const zone = select(zoneByUid, uid);
        if (zone && zone.dynamic) {
            yield call(handleZoneTrackable, null, zone.trackable_object_uid);
        }
        yield call(deleteSingleZone, uid);
    } catch (err) {
        yield call(errorHandler, err);
    }
}

export function* loadZones() {
    try {
        const zones = yield call(api.getZones);
        yield put({ type: ACTIONS.ZONES_SET, params: zones });
    } catch (err) {
        yield call(errorHandler, err);
    }
}

export function* loadZonesPrototypes() {
    try {
        const customZonesPrototypes = yield call(api.getCustomZonesPrototypes);
        yield put({ type: ACTIONS.ZONES_PROTOTYPES_SET, params: customZonesPrototypes });
    } catch (err) {
        yield call(errorHandler, err);
    }
}

export function* createZone(zone) {
    try {
        if (zone.dynamic) {
            yield call(handleZoneTrackable, zone.uid, zone.trackable_object_uid);
        }
        const zoneCopy = yield call(changeScaleZoneRadius, zone);
        const currentZonePrototype = yield select(zonePrototypeById, zoneCopy.zone_type_id);
        const isDirectionalGeometry = yield call(isDirectionalRestriction, currentZonePrototype && currentZonePrototype.zone_geometry_restriction);
    
        if (!isDirectionalGeometry) {
            delete zoneCopy.user_params.direction;
        }
    
        const zoneData = yield call(api.createZone, zoneCopy);
        yield put({ type: ACTIONS.ZONE_CREATE, params: zoneData });
    } catch (err) {
        yield call(errorHandler, err);
    }
}

export function* editZone(uid, zone) {
    try {
        const oldZone = yield select(zoneByUid, uid);
        if (oldZone.dynamic) {
            if (zone.trackable_object_uid !== undefined && oldZone.trackable_object_uid !== zone.trackable_object_uid) {
                yield call(handleZoneTrackable(null, oldZone.trackable_object_uid));
            
                if (zone.trackable_object_uid !== null) {
                    yield call(handleZoneTrackable, zone.uid, zone.trackable_object_uid);
                }
            }
        }
        const zoneCopy = yield call(changeScaleZoneRadius, zone);
        const zoneData = yield call(api.updateZone, uid, zoneCopy);
        yield put({ type: ACTIONS.ZONE_UPDATE, params: zoneData });
        if (zoneData.zone_mode !== ZONE_MODE.DEACTIVATED) {
            yield put(changeParamsOnSiteView({ isAlarmsDismissed: false }));
        }
    } catch (err) {
        yield call(errorHandler, err);
    }
}


function* watchLoadZone() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_LOAD, loadZones);
}

function* watchLoadZonesPrototypes() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_PROTOTYPES_LOAD, loadZonesPrototypes);
}

export function* commonZonesSagas() {
    return yield all([
        fork(watchLoadZone),
        fork(watchLoadZonesPrototypes),
    ]);
}
