import moment from 'moment-timezone';
import _Get from 'lodash/get';
import { ZONE_SEVERITY, DURATION_SORT_RANGES } from 'common/constants';
import { DATE_FORMATTER, WEEKDAYS_SHORT_TXT, COMPONENTS } from 'common/ui-labels';
import Position from 'stores/classes/Position';
import Cron from 'cron-converter';

import { zoneSeverityColor, lengthToDegree } from './simpleUtils';


// minute (0 - 59)
// hour (0 - 23)
// day of month (1 - 31)
// month (1 - 12)
// day of week (0 - 6) (Sunday to Saturday; 7 is also Sunday on some systems)
// 00 00 10 * * *
export function cronFormatter(minute = '*', hour = '*', dayOfMonth = '*', month = '*', dayOfWeek = '*') {
    const minutes = minute.length < 2 ? `0${minute}` : minute;
    const hours = hour.length < 2 ? `0${hour}` : hour;
    return `${minutes} ${hours} ${dayOfMonth} ${month} ${dayOfWeek}`;
}

export function cronInterpreter(cronString) {
    if (!cronString) {
        return undefined;
    }
    const dates = cronString.split(' ');
    const minute = dates[0];
    const hour = dates[1];
    const dayOfMonth = dates[2];
    const month = dates[3];
    const dayOfWeek = dates[4];
    return {
        minute, hour, dayOfMonth, month, dayOfWeek,
    };
}

export const getCronFromToStringValues = ({ from, to }) => {
    const cronFrom = cronInterpreter(from);
    const cronTo = cronInterpreter(to);
    
    const fromHour = cronFrom ? cronFrom.hour : '00';
    const fromMinute = cronFrom ? cronFrom.minute : '00';
    const toHour = cronTo ? cronTo.hour : '24';
    const toMinute = cronTo ? cronTo.minute : '00';
    
    return {
        toHour,
        toMinute,
        fromHour,
        fromMinute,
    };
};

function normalizeTimeZoneOffsetString(timezoneOffset) {
    let hours = Math.floor(Math.abs(timezoneOffset) / 60);
    let minutes = (Math.abs(timezoneOffset) - (hours * 60));
    hours = hours < 10 ? `0${hours}` : hours;
    minutes = minutes < 10 ? `0${minutes}` : minutes;
    
    let formattedTZ = '(';
    if (timezoneOffset >= 0) {
        formattedTZ += '+';
    } else {
        formattedTZ += '-';
    }
    formattedTZ += `${hours}:${minutes}`;
    formattedTZ += ' UTC)';
    
    return formattedTZ;
}

export function formatTimeZoneOffset(timezone) {
    const timezoneOffset = -1 * moment.tz.zone(timezone).parse(new Date());
    return `${normalizeTimeZoneOffsetString(timezoneOffset)} ${timezone}`;
}

export const getFloorplanUrl = (floorplan, host) => {
    if (floorplan.url
        && floorplan.url.endsWith('.pdf')
        && floorplan.alt_image_urls) {
        const image = floorplan.alt_image_urls.find(found_image => (found_image.type.toLowerCase() === 'svg'));
        // console.log("[getFloorplanUrl] image:",image);
        if (image) {
            return host + image.url;
        }
    }
    return host + floorplan.url;
};

export function zonePositions(zone = {}, sublocation = {}) {
    const zone_points = zone.zone_geometry ? zone.zone_geometry.positions : undefined;
    if (zone_points && zone_points.length !== 0) {
        return zone_points.map(point => new Position(point, sublocation).latlng);
    }
    return [];
}

export function heatmapBoundsTranslate(bounds = [], sublocation = {}) {
    return bounds.map(point => new Position(point, sublocation).latlng);
}

export function pixelsToLength(pixels, z) {
    const maxScale = 156412;
    const coefficientNormalScale = 10;
    
    return (((maxScale / Math.pow(2, (z - 1))) * pixels) / coefficientNormalScale);
}

export function calcNumberLabels(widthInPixels, z, step) {
    const numberOfLabels = Math.ceil(widthInPixels / z / step);
    return (step > 5) ? numberOfLabels * 2 : numberOfLabels;
}

export const degreeToLength = d => (d * (40075161.2 / 360));

export const toRadians = degrees => ((degrees * Math.PI) / 180);

export const toDegrees = radians => ((radians * 180) / Math.PI);

export const calculateDefaultBounds = (lat, lng, radiusM) => {
    const R = 6371; // earth radius in km
    const radius = radiusM / 1000; // in km
    
    const x1 = lng - toDegrees(radius / R / Math.cos(toRadians(lat)));
    
    const x2 = lng + toDegrees(radius / R / Math.cos(toRadians(lat)));
    
    const y1 = lat + toDegrees(radius / R);
    
    const y2 = lat - toDegrees(radius / R);
    return [[x1, y1], [x2, y2]];
};

export const translateDegreesToMillimeters = (pos, sublocation) => ({
    x: (degreeToLength(pos.lng) * 100) + sublocation.offset_x,
    y: (degreeToLength(pos.lat) * 100) + sublocation.offset_y,
});

export const positionWithSublocationOffset = (pos, sublocation) => {
    const offset_x = _Get(sublocation, 'offset_x', 0);
    const offset_y = _Get(sublocation, 'offset_y', 0);
    
    return {
        x: pos.x - offset_x,
        y: pos.y - offset_y,
    };
};

export const positionOffset = (point, sublocation) => {
    if (!point || !sublocation) {
        return null;
    }
    return new Position(point, sublocation).latlngObj;
};

export const createGrid = (zoom, step, mapWidth, mapHeight) => {
    if (!zoom || !mapWidth || !mapHeight || !step) return;

    const windowWidth = mapWidth;
    const windowHeight = mapHeight;
    const gridHeight = 10 * lengthToDegree(pixelsToLength(windowHeight, zoom));
    const gridWidth = 10 * lengthToDegree(pixelsToLength(windowWidth, zoom));
    const x = Math.floor(pixelsToLength(windowWidth, zoom) / step);
    const y = Math.floor(pixelsToLength(windowHeight, zoom) / step);
    
    const rectsHeight = new Array(y);
    rectsHeight.fill(null);
    
    const rectsHeightComplete = rectsHeight.map((mass, index) => {
        const SW = {
            lng: (gridHeight),
            lat: (-lengthToDegree((step * index * 10))),
        };
        const NE = {
            lng: (-gridHeight),
            lat: (-lengthToDegree((step * index * 10))),
        };
        
        return [[SW.lat, SW.lng], [NE.lat, NE.lng]];
    });
    
    const rectsHeightRewers = new Array(y);
    rectsHeightRewers.fill(null);
    
    const rectsHeightRewersComplete = rectsHeightRewers.map((mass, index) => {
        const SW = {
            lng: (gridHeight),
            lat: (lengthToDegree((step * (index + 1) * 10))),
        };
        
        const NE = {
            lng: (-gridHeight),
            lat: (lengthToDegree((step * (index + 1) * 10))),
        };
        
        return [[SW.lat, SW.lng], [NE.lat, NE.lng]];
    });
    
    const rectsWidth = new Array(x);
    rectsWidth.fill(null);
    
    const rectsWidthComplete = rectsWidth.map((mass, index) => {
        const SW = {
            lng: (-lengthToDegree((step * index * 10))),
            lat: (-gridWidth),
        };
        
        const NE = {
            lng: (-lengthToDegree((step * index * 10))),
            lat: (gridWidth),
        };
        
        return [[SW.lat, SW.lng], [NE.lat, NE.lng]];
    });
    
    const rectsWidthRewers = new Array(x);
    rectsWidthRewers.fill(null);
    
    const rectsWidthRewersComplete = rectsWidthRewers.map((mass, index) => {
        const SW = {
            lng: (lengthToDegree((step * (index + 1) * 10))),
            lat: (-gridWidth),
        };
        
        const NE = {
            lng: (lengthToDegree((step * (index + 1) * 10))),
            lat: (gridWidth),
        };
        
        return [[SW.lat, SW.lng], [NE.lat, NE.lng]];
    });
    
    const rectsAllComplete = rectsHeightComplete.concat(rectsWidthComplete);
    const rectsAllRewersComplete = rectsHeightRewersComplete.concat(rectsWidthRewersComplete);
    return rectsAllComplete.concat(rectsAllRewersComplete);
};

export const createLabels = (zoom, step, mapWidth, mapHeight, imageScale = 1) => {
    if (!zoom || !mapWidth || !mapHeight) {
        return;
    }

    const x = calcNumberLabels(mapWidth * imageScale, zoom, step);
    const labelPositionsNegative = [];
    const labelPositionsPositive = [];

    for (let i = 0; i <= x; i += 1) {
        labelPositionsNegative.push(-lengthToDegree(step * i * 10));
        labelPositionsPositive.push(lengthToDegree(step * (i + 1) * 10));
    }

    return labelPositionsNegative.concat(labelPositionsPositive);
};

export const calcSublocationBoundsAngular = (sublocation, anchors) => {
    if (anchors.length === 1) {
        const position = new Position(anchors[0].position);
        return calculateDefaultBounds(position.lat, position.lng, 0.3);
    }
    
    const { xList, yList } = Object.values(anchors).reduce((memo, anchor) => {
        memo.xList.push(anchor.position.x);
        memo.yList.push(anchor.position.y);
        return memo;
    }, { xList: [], yList: [] });

    const maxX = Math.max(...xList);
    const minX = Math.min(...xList);
    const maxY = Math.max(...yList);
    const minY = Math.min(...yList);
    
    const minPosition = new Position({ x: minX, y: minY }, sublocation);
    const maxPosition = new Position({ x: maxX, y: maxY }, sublocation);
    const deltaX = Math.abs(maxX - minX);
    const deltaY = Math.abs(maxY - minY);
    
    if (deltaX < 1000 && deltaY < 1000) {
        return calculateDefaultBounds(maxPosition.lat, maxPosition.lng, 5);
    }
    if (deltaX < 1000) { // 1 meter
        return calculateDefaultBounds(minPosition.lat, minPosition.lng, deltaY);
    }
    if (deltaY < 1000) { // 1 meter
        return calculateDefaultBounds(minPosition.lat, minPosition.lat, deltaX);
    }
    
    return [minPosition.latlng, maxPosition.latlng];
};

export const MAP_BOUNDS = (sublocation, floorplan, anchors) => {
    const K_FOR_OFFSET = 6;

    if (!anchors || anchors.length <= 2) {
        const r = floorplan.width > floorplan.height
            ? (floorplan.width * floorplan.image_scale) / 2 : (floorplan.height * floorplan.image_scale) / 2;
        
        return calculateDefaultBounds(lengthToDegree(floorplan.image_offset_x / K_FOR_OFFSET), lengthToDegree(floorplan.image_offset_y / K_FOR_OFFSET), r);
    }
    return calcSublocationBoundsAngular(sublocation, anchors);
};

export const alarmLevelStyle = zone_alarm_level => ({
    fontSize: '14px',
    backgroundColor: zoneSeverityColor(zone_alarm_level),
    textAlign: 'center',
    color: parseInt(zone_alarm_level) === ZONE_SEVERITY.GUARDED ? 'black' : 'white',
});

export const durationFormatting = (duration, intl) => {
    if (!Number.isNaN(duration)) {
        if (duration < 60 * 1000) { return { text: DATE_FORMATTER.LESS_A_MIN(intl), duration, range: DURATION_SORT_RANGES.LESS_A_MIN }; }
        if (duration < 5 * 60 * 1000) { return { text: DATE_FORMATTER.LESS_5_MIN(intl), duration, range: DURATION_SORT_RANGES.LESS_5_MIN }; }
        if (duration < 30 * 60 * 1000) { return { text: DATE_FORMATTER.MORE_5_MIN(intl), duration, range: DURATION_SORT_RANGES.MORE_5_MIN }; }
        if (duration < 60 * 60 * 1000) { return { text: DATE_FORMATTER.MORE_30_MIN(intl), duration, range: DURATION_SORT_RANGES.MORE_30_MIN }; }
        if (duration < 6 * 60 * 60 * 1000) { return { text: DATE_FORMATTER.MORE_HOURE(intl), duration, range: DURATION_SORT_RANGES.MORE_HOURE }; }
        if (duration < 24 * 60 * 60 * 1000) { return { text: DATE_FORMATTER.MORE_6_HOURS(intl), duration, range: DURATION_SORT_RANGES.MORE_6_HOURS }; }
        if (duration < 168 * 60 * 60 * 1000) { return { text: DATE_FORMATTER.MORE_DAY(intl), duration, range: DURATION_SORT_RANGES.MORE_DAY }; }
        return { text: DATE_FORMATTER.MORE_WEEK(intl), duration, range: DURATION_SORT_RANGES.MORE_WEEK };
    }
    return { text: COMPONENTS.FIELDS.COMMON.NEVER_FUNC(intl), duration, range: DURATION_SORT_RANGES.NEVER };
};

export const formatInactivity = (lastHeard, intl) => {
    const lastHeardDate = new Date(lastHeard);
    if (Number.isNaN(lastHeardDate.getTime()) || new Date(0).getTime() === lastHeardDate.getTime()) {
        return '--';
    }
    const duration = new Date() - lastHeardDate;
    
    return durationFormatting(duration, intl).text;
};

export const convertMillimetersToMeters = mm => mm / 1000;
export const convertMetersToMillimeters = m => m * 1000;

export const getNodeHwCbId = node => (node && node.hw_info ? node.hw_info.cbid : '--');
export const getNodeSwVersion = node => (node.sw_version && node.sw_version.sw_str ? node.sw_version.sw_str : '--');

export const convertNumbersToWeekDays = (days, intl) => days.map((day) => {
    const dayShort = (moment().day(day).format('dd') || '').toUpperCase();
    if (dayShort && WEEKDAYS_SHORT_TXT[dayShort] && intl) {
        return WEEKDAYS_SHORT_TXT[dayShort](intl);
    }
    return '';
});

export const makeCronTimeString = (time, weekdaysArr = [1, 2, 3, 4, 5]) => {
    const cronInstance = new Cron();
    const hoursMins = time.split(':');
    const hoursArr = [parseInt(hoursMins[0])];
    const minsArr = [parseInt(hoursMins[1])];

    const allDaysArr = [...Array(31 + 1).keys()].slice(1);
    const allMonthsArr = [...Array(12 + 1).keys()].slice(1);

    cronInstance.fromArray([minsArr, hoursArr, allDaysArr, allMonthsArr, weekdaysArr]);

    return cronInstance.toString();
};

export const humanizeDistance = (distanceInMm, format = 'kms', imperialFormat = '') => {
    const showKm = format.toLowerCase().includes('k');
    const showM = format.toLowerCase().includes('m');
    const showS = format.toLowerCase().includes('s');
    const imperialShowMl = imperialFormat.toLowerCase().includes('ml');
    const imperialShowYd = imperialFormat.toLowerCase().includes('yd');
    const imperialShowFt = imperialFormat.toLowerCase().includes('ft');
    
    if (!distanceInMm) return distanceInMm;
    if (distanceInMm < 1000) return '< 1 m';
    let result = '';
    
    const kilometers = showKm ? Math.floor(distanceInMm / 1000000) : 0;
    const meters = showM ? Math.floor(distanceInMm / 1000) - (kilometers * 1000) : 0;
    const santimeters = showS ? Math.floor(distanceInMm / 10) - (kilometers * 100000) - (meters * 100) : 0;
    
    if (showKm && kilometers > 0) result += `${kilometers} km`;
    if (showM && meters > 0) result += ` ${meters} m`;
    if (showS && santimeters > 0) result += ` ${santimeters} sm`;
    let imperial = '';
    
    if (imperialShowMl || imperialShowYd || imperialShowFt) {
        const totalFts = imperialShowFt ? Math.floor((distanceInMm / 1000) / 0.3048) : 0; // 1 foot equal to 0.3048 meters
        const totalYds = imperialShowYd ? Math.floor(totalFts / 3) : 0; // 1 yard equal to 3 foot
        const miles = imperialShowMl ? Math.floor(totalFts / 5280) : 0; // 1 mile equal to 5280 foot
        const yds = totalYds - (miles * 1760); // 1 mile equal to 1760 yards
        const fts = totalFts - (yds * 3) - (miles * 5280);
        if (imperialShowMl && miles > 0) imperial += `${miles} mi `;
        if (imperialShowYd && yds > 0) imperial += `${yds} yd `;
        if (imperialShowFt && fts > 0) imperial += `${fts} ft`;
        
        return `${result}(${imperial})`;
    }
    
    return result;
};

export const capitalize = (text = '') => (text.charAt(0).toUpperCase() + text.slice(1).toLowerCase());
