import { DEFAULT_VALUES, T2T_TYPE, PWS_PIN } from 'common/constants';
import { TRACKABLE_MODAL, UNITS, COMPONENTS } from 'common/ui-labels';
import { getValue, isDivisible } from 'utils/simpleUtils';
import { showPwsFields, isRoleMonitor, isRoleAnnouncer, isVehicle, isPerson } from 'utils/utils';

/**
 * Create pws configuration values for sending `tag_config.t2t_cfg` on create/update trackable
 * @param pws_config object
 * @property pws_config.role string ['Monitors for announcements', 'Announces presence', '']
 * @property pws_config.alarm_back bool for 'Monitors for announcements'
 * @property pws_config.safe_distance for 'Announces presence'
 * @property pws_config.distance for 'Announces presence'
 * @property pws_config.alert_distance for 'Announces presence'
 * @property pws_config.delay for 'Announces presence'
 * @property pws_config.pin for 'Announces presence'     OFF|SOLID|PULSE
 * @property pws_config.haptic for both
 * @property pws_config.buzzer for both
 * @property pws_config.angle for 'Announces presence'
 * @param category
 */
export function formatPwsConfig(pws_config, category) {
    let t2t_cfg = {};
    if (pws_config) {
        if (isRoleAnnouncer(pws_config.role) || isRoleMonitor(pws_config.role)) {
            const {
                showDistance, showAlertDistance, showSafeDistance,
                showDelay, showPin, showAngle, showAlarmBack,
            } = showPwsFields(pws_config.role, category);

            t2t_cfg.role = pws_config.role;

            if (showSafeDistance) {
                const safe_distance = getValue(pws_config.safe_distance);
                t2t_cfg.safe_distance = safe_distance ? safe_distance * 1000 : 0;
            }
            if (showAlertDistance) {
                const alert_distance = getValue(pws_config.alert_distance);
                t2t_cfg.alert_distance = alert_distance ? alert_distance * 1000 : 0;
            }
            if (showDistance) {
                const distance = getValue(pws_config.distance);
                t2t_cfg.distance = distance ? distance * 1000 : 0;
            }
            if (showPin) {
                t2t_cfg.pin = pws_config.pin || DEFAULT_VALUES.LLL_TRACKABLE_PIN;
                const categoryPerson = isPerson(category);
                if (categoryPerson) {
                    t2t_cfg.haptic = (pws_config.pin === PWS_PIN.PULSE) ? (pws_config.haptic || false) : false;
                    t2t_cfg.buzzer = (pws_config.pin === PWS_PIN.PULSE) ? (pws_config.buzzer || false) : false;
                }
            }
            if (showAngle) {
                t2t_cfg.angle = pws_config.angle || 0;
            }
            if (showAlarmBack) {
                t2t_cfg.alarm_back = pws_config.alarm_back !== undefined ? pws_config.alarm_back : true;
            }
            if (showDelay) {
                const delay = getValue(pws_config.delay);
                t2t_cfg.delay = (delay && Number(delay)) || 0;
            }
            if (isVehicle(category) && isRoleAnnouncer(pws_config.role)) {
                t2t_cfg.alarm_back = true;
            }
            if (isPerson(category) && isRoleAnnouncer(pws_config.role)) {
                t2t_cfg.alarm_back = true;
                t2t_cfg.angle = 360;
            }
        } else {
            t2t_cfg = null;
        }
        return t2t_cfg;
    }
    return undefined;
}

function _handlePwsConfig(t2t_cfg) {
    const low_latency_link_config = {};
    if (t2t_cfg.role === T2T_TYPE.MONITOR) {
        low_latency_link_config.role = T2T_TYPE.MONITOR;
        low_latency_link_config.alarm_back = t2t_cfg.alarm_back || false;
        low_latency_link_config.pin = t2t_cfg.pin || DEFAULT_VALUES.LLL_TRACKABLE_PIN;
        const delay = getValue(t2t_cfg.delay);
        const haptic = getValue(t2t_cfg.haptic);
        const buzzer = getValue(t2t_cfg.buzzer);
        low_latency_link_config.delay = delay || 0;
        low_latency_link_config.haptic = haptic || false;
        low_latency_link_config.buzzer = buzzer || false;
    } else if (t2t_cfg.role === T2T_TYPE.ANNOUNCER) {
        low_latency_link_config.role = T2T_TYPE.ANNOUNCER;
        low_latency_link_config.pin = t2t_cfg.pin || DEFAULT_VALUES.LLL_TRACKABLE_PIN;

        const delay = getValue(t2t_cfg.delay);
        const haptic = getValue(t2t_cfg.haptic);
        const buzzer = getValue(t2t_cfg.buzzer);
        low_latency_link_config.delay = delay || 0;
        low_latency_link_config.haptic = haptic || false;
        low_latency_link_config.buzzer = buzzer || false;

        const safe_distance = getValue(t2t_cfg.safe_distance);
        low_latency_link_config.safe_distance = safe_distance ? safe_distance / 1000 : 0;

        const distance = getValue(t2t_cfg.distance);
        low_latency_link_config.distance = distance ? distance / 1000 : 0;

        const alert_distance = getValue(t2t_cfg.alert_distance);
        low_latency_link_config.alert_distance = alert_distance ? alert_distance / 1000 : 0;
        low_latency_link_config.angle = t2t_cfg.angle || 0;
    }
    return low_latency_link_config;
}

/**
 * @param t2t_cfg
 * @returns {{}}
 */
export function translateT2TLowLatencyConfigForTrackable(t2t_cfg) {
    if (t2t_cfg) {
        if (t2t_cfg.role === T2T_TYPE.DISABLED) {
            return { role: T2T_TYPE.DISABLED };
        }
        return _handlePwsConfig(t2t_cfg);
    }
    return {};
}

const validationRequired = (value) => (value && isNaN(Number(value)) && value.length === 0 ? COMPONENTS.VALIDATION.REQUIRED : undefined);
const validationNumber = (value) => (value && isNaN(Number(value)) ? TRACKABLE_MODAL.EDIT.ERROR.MUST_BE_NUMBER_REQUIRE : undefined);
const minValue = (min) => (value, unit = '') => (value && (isNaN(Number(value)) || !isNaN(Number(value))) && value < min
    ? TRACKABLE_MODAL.EDIT.ERROR.MIN_REQUIRE(min, unit)
    : undefined);
const maxValue = (max) => (value, unit = '') => (value && value > max ? TRACKABLE_MODAL.EDIT.ERROR.MAX_REQUIRE(max, unit) : undefined);
const stepValue = (step) => (value, unit = '') => (value && !isDivisible(value, step) ? TRACKABLE_MODAL.EDIT.ERROR.STEP_REQUIRE(step, unit) : undefined);
const validatePositive = (value) => (value && (isNaN(Number(value)) || !isNaN(Number(value))) && Number(value) < 0
    ? TRACKABLE_MODAL.EDIT.ERROR.MUST_BE_POSITIVE_NUMBER
    : undefined);

const validationMin0m = (value, intl) => minValue(0)(value, UNITS.METER(intl));
const validationMin05m = (value, intl) => minValue(0.5)(value, UNITS.METER(intl));
const validationMax25m = (value, intl) => maxValue(25)(value, UNITS.METER(intl));

const validationMin0s = (value, intl) => minValue(0)(value, UNITS.SEC(intl));
const validationMax10m = (value, intl) => maxValue(600)(value, UNITS.SEC(intl));

const validationStep01m = (value, intl) => stepValue(0.1)(value, UNITS.METER(intl));

const validationStep1s = (value, intl) => stepValue(1)(value, UNITS.SEC(intl));

function validateSafeDistances(values, errors, intl) {
    errors.safe_distance = validationRequired(values.safe_distance) || validatePositive(values.safe_distance)
        || validationNumber(values.safe_distance) || validationMin0m(values.safe_distance, intl)
        || validationMax25m(values.safe_distance, intl) || validationStep01m(values.safe_distance, intl);
}

function validateDistance(values, errors, intl) {
    errors.distance = validationRequired(values.distance) || validatePositive(values.distance)
        || validationNumber(values.distance) || validationMin05m(values.distance, intl)
        || validationMax25m(values.distance, intl) || validationStep01m(values.distance, intl);
}

function validateAlertDistance(values, errors, intl) {
    errors.alert_distance = validationRequired(values.alert_distance) || validatePositive(values.alert_distance)
        || validationNumber(values.alert_distance) || validationMin05m(values.alert_distance, intl)
        || validationMax25m(values.alert_distance, intl) || validationStep01m(values.alert_distance, intl);
}

function isValidationNeeded(values) {
    return true;
}

function validateDelay(values, errors, intl) {
    if (isValidationNeeded(values)) {
        errors.delay = validationRequired(values.delay) || validatePositive(values.delay) || validationNumber(values.delay)
            || validationMin0s(values.delay, intl) || validationMax10m(values.delay, intl) || validationStep1s(values.delay, intl);
    }
}
function distancesInterdependency(values, errors) {
    if (!errors.safe_distance && !errors.distance && !errors.alert_distance) {
        if (values.safe_distance && values.distance && Number(values.safe_distance) > Number(values.distance)) {
            errors.safe_distance = TRACKABLE_MODAL.EDIT.FORM.ERROR_SAFE_LESS_WARNING;
        } else if (values.safe_distance && values.alert_distance && Number(values.safe_distance) > Number(values.alert_distance)) {
            errors.safe_distance = TRACKABLE_MODAL.EDIT.FORM.ERROR_SAFE_LESS_ALERT;
        } else if (values.distance && values.alert_distance && Number(values.distance) > Number(values.alert_distance)) {
            errors.distance = TRACKABLE_MODAL.EDIT.FORM.ERROR_WARNING_LESS_ALERT;
        }
    }
}

export function validatePWS(values, errors, intl) {
    validateSafeDistances(values, errors, intl);
    validateDistance(values, errors, intl);
    validateAlertDistance(values, errors, intl);
    validateDelay(values, errors, intl);

    distancesInterdependency(values, errors);
}
