import React from 'react';
import _Has from 'lodash/has';
import _Get from 'lodash/get';
import { ZONE_APPLY_TO, ZONE_MODE, ZONE_TYPE, ZONE_SEVERITY, ZONES_CONSTANTS } from 'common/constants';
import { DB_NAMES } from 'common/ui-labels';
import { makeCopy, getUUID, getZones, lengthToDegree } from 'utils/simpleUtils';
import { getAlarms } from 'utils/alarmUtils';
import Position from 'stores/classes/Position';
import { cronFormatter, degreeToLength, getCronFromToStringValues } from './formatters';


export const compareZoneType = (zone, zoneType) => (zone && zoneType && zone.zone_type_name === zoneType.name);

export const zoneFullActive = zone => zone.active;

export const isInfoZone = zone => (compareZoneType(zone, ZONE_TYPE.INFO));

export const isKeepOutZone = zone => (compareZoneType(zone, ZONE_TYPE.KEEP_OUT));

export const isKeepInZone = zone => (compareZoneType(zone, ZONE_TYPE.KEEP_IN));

export const isCustomZone = zone => (zone && zone.zone_type && zone.zone_type.id >= 16);

export const isStopZone = zone => (compareZoneType(zone, { name: 'stop_zone' }));

export const isAlarmZone = zone => (isKeepInZone(zone) || isKeepOutZone(zone));

export const isZoneVisible = (zone, layerFilter) => {
    const {
        isShowingActiveAlarmZones,
        isShowingInactiveAlarmZones,
        isShowingActiveInfoZones,
        isShowingInactiveInfoZones,
    } = layerFilter;
    
    const isActiveZone = zoneFullActive(zone);
    
    if (isAlarmZone(zone)) {
        return isActiveZone ? isShowingActiveAlarmZones : isShowingInactiveAlarmZones;
    } else if (isInfoZone(zone)) {
        return isActiveZone ? isShowingActiveInfoZones : isShowingInactiveInfoZones;
    } else if (isCustomZone(zone)) {
        const zoneTypeName = zone.zone_type_name;
        
        return isActiveZone
            ? layerFilter[`isShowingActive_${zoneTypeName}`]
            : layerFilter[`isShowingInactive_${zoneTypeName}`];
    }
    return false;
};

function isBufferVisible(buffer, layerFilter) {
    const { isShowingActiveAlarmZones, isShowingInactiveAlarmZones } = layerFilter;
    
    const isBufferActive = zoneFullActive(buffer);
    
    return (isBufferActive) ? isShowingActiveAlarmZones : isShowingInactiveAlarmZones;
}

export function showBuffer(buffer, siteViewFilter) {
    const { layerFilter, objectFilter: { selectedTrackables } } = siteViewFilter;
    
    return isBufferVisible(buffer, layerFilter) && selectedTrackables.includes(buffer.trackable_object_uid);
}

export function isBufferSelected(selectedItems, buffer) {
    return selectedItems ? !!selectedItems.find(x => x.uid === buffer.uid) : false;
}

export function isBufferAlarmed(alarms, buffer) {
    return alarms ? !!alarms.find(alarm => buffer.uid === alarm.zone_uid) : false;
}

const getZoneTypes = (zonePrototypes) => {
    const zoneTypes = [];
    Object.values(ZONE_TYPE).forEach((zoneType) => {
        zoneTypes.push({
            display_name: zoneType.display_name,
            name: zoneType.name,
            id: zoneType.id,
        });
    });
    
    if (zonePrototypes && Array.isArray(zonePrototypes)) {
        zonePrototypes.forEach((zonePrototype) => {
            zoneTypes.push({
                display_name: zonePrototype.zone_type && zonePrototype.zone_type.display_name,
                name: zonePrototype.zone_type && zonePrototype.zone_type.name,
                id: zonePrototype.zone_type && zonePrototype.zone_type.id,
            });
        });
    }

    return zoneTypes;
};

export function zoneSeverityTextColor(zone_alarm_level_id) {
    switch (parseInt(zone_alarm_level_id)) {
        case ZONE_SEVERITY.GUARDED:
            return 'black';
        default:
            return 'white';
    }
}

export const isCustomZoneType = (zonePrototype, zone_type_name) => (zonePrototype.find(zone_prototype => zone_prototype.zone_type.name === zone_type_name));

export const getZoneTypeOptions = (zonePrototype, intl) => (
    getZoneTypes(zonePrototype).map((zoneType) => {
        if (zoneType.name && DB_NAMES[zoneType.name.toUpperCase()]) {
            const name = DB_NAMES[zoneType.name.toUpperCase()](intl);
            return <option key={getUUID()} value={zoneType.name}>{name}</option>;
        }
        return <option key={getUUID()} value={zoneType.name}>{zoneType.display_name}</option>;
    })
);

export const isRectangleRestriction = (restrictions = []) => (restrictions.length > 0 && restrictions.indexOf('rectangle') !== -1);

export const isDirectionalRestriction = (restrictions = []) => (restrictions.length > 0 && restrictions.indexOf('directional') !== -1);

export const calculateAllowDefault = zone => (zone.apply_to === ZONE_APPLY_TO.LIST);

export const parseDefaultAllowed = zone => (zone.default_allowed ? ZONE_APPLY_TO.LIST : ZONE_APPLY_TO.ALL);

export const getZoneTypeName = (zonePrototypes, zone_type_name, intl) => {
    const currentZoneType = getZoneTypes(zonePrototypes).find(zoneType => zoneType.name === zone_type_name);
    const nameUpper = currentZoneType && currentZoneType.name && currentZoneType.name.toUpperCase();
    if (nameUpper && intl && DB_NAMES[nameUpper]) {
        return DB_NAMES[nameUpper](intl);
    }
    return currentZoneType ? currentZoneType.display_name : '';
};

export const showActivePeriod = (mode, schedule_start_time, schedule_stop_time) => {
    if (mode === ZONE_MODE.ALWAYS_ACTIVE) {
        return <span data-cy="zone-details-site-active24">24 Hour</span>;
    }
    const {
        toHour,
        toMinute,
        fromHour,
        fromMinute,
    } = getCronFromToStringValues({ from: schedule_start_time, to: schedule_stop_time });
    
    return (
        <div className="col-md-12 text-left">
            <div className="Zone-Edit-Schedule row">
                <span className="col-md-5 Schedule-Mask" data-cy="zone-details-site-schedule-start">
                    {fromHour}:{fromMinute}
                </span>
                <span className="col-md-2 Zone-Edit-Center">-</span>
                <span className="col-md-5 Schedule-Mask" data-cy="zone-details-site-schedule-end">
                    {toHour}:{toMinute}
                </span>
            </div>
        </div>
    );
};

export const geometryFromDegreesToMillimeters = (latlngs) => {
    if (latlngs && latlngs.length > 0 && _Has(latlngs[0], 'lng')) {
        return latlngs.map(point => ({
            x: parseInt(((degreeToLength(point.lng)) * 100)),
            y: parseInt(((degreeToLength(point.lat)) * 100)),
        }));
        // console.log('***pointsForZone', pointsForZone);
    } else if (latlngs && latlngs.length > 0 && Array.isArray(latlngs[0])) {
        return latlngs.map(point => ({
            x: parseInt(((degreeToLength(point[1])) * 100)),
            y: parseInt(((degreeToLength(point[0])) * 100)),
        }));
    }
    return false;
};

export const rotatePoint = (point, rotationCenter, rotationAngle) => {
    const newPoint = [];
    const x = point[0];
    const y = point[1];
    const x0 = rotationCenter[0];
    const y0 = rotationCenter[1];
    
    newPoint[0] = x0 + (((x - x0) * Math.cos(rotationAngle * (Math.PI / 180))) - ((y - y0) * Math.sin(rotationAngle * (Math.PI / 180))));
    newPoint[1] = y0 + (((y - y0) * Math.cos(rotationAngle * (Math.PI / 180))) + ((x - x0) * Math.sin(rotationAngle * (Math.PI / 180))));
    
    return newPoint;
};

export const getCentroid = points => (
    points.reduce((x, y) => ([x[0] + (y[0] / points.length), x[1] + (y[1] / points.length)]), [0, 0])
);

export const rotateGeometry = (geometry, rotation, center) => {
    let zoneGeometryInDegrees = geometry.zoneGeometryInDegrees ? geometry.zoneGeometryInDegrees : [];
    if (zoneGeometryInDegrees.length > 0) {
        const rotationCenter = center || getCentroid(zoneGeometryInDegrees);
        zoneGeometryInDegrees = zoneGeometryInDegrees.map(point => (rotatePoint(point, rotationCenter, rotation)));
    }
    return zoneGeometryInDegrees;
};

export const calculateAngleByTwoPoints = (startX, endX, startY, endY) => {
    let delta = 0; // first quoter of circle
    let atan = (startX - endX) / (startY - endY);
    
    if (startY > endY && startX < endX) {
        delta = 90;
        atan = (startY - endY) / (startX - endX);
    } // second quoter of circle
    if (startY > endY && startX > endX) {
        delta = 180;
        atan = (startX - endX) / (startY - endY);
    } // third quoter of circle
    if (startY < endY && startX > endX) {
        delta = 270;
        atan = (startY - endY) / (startX - endX);
    } // fourth quoter of circle
    
    return Math.floor((Math.abs(180 * (Math.atan(atan) / Math.PI)) + delta) * 100) / 100;
};

export const rotateLineByCenter = (line) => {
    const newLine = [...line];
    const startX = line[0].lng;
    const startY = line[0].lat;
    const endX = line[1].lng;
    const endY = line[1].lat;
    const center = [(startX + endX) / 2, (startY + endY) / 2];
    
    const angle = calculateAngleByTwoPoints(startX, endX, startY, endY);
    const pointOne = rotatePoint([startX, startY], center, angle);
    const pointTwo = rotatePoint([endX, endY], center, angle);
    
    newLine[0].lat = pointOne[1];
    newLine[0].lng = pointOne[0];
    newLine[1].lat = pointTwo[1];
    newLine[1].lng = pointTwo[0];
    
    return { newLine, angle };
};

export const recalculatePolygon = ({ vertexDrag, vertexPrev, vertexOpposite }) => {
    const startY = vertexPrev.lat;
    const endY = vertexOpposite.lat;
    const startX = vertexPrev.lng;
    const endX = vertexOpposite.lng;
    
    const rotationAngle = calculateAngleByTwoPoints(startX, endX, startY, endY);
    
    const rotatedDraggedMarker = rotatePoint([vertexDrag.lat, vertexDrag.lng], [vertexOpposite.lat, vertexOpposite.lng], -rotationAngle);
    
    const rectangle = {
        zoneGeometryInDegrees: [
            [vertexOpposite.lat, vertexOpposite.lng],
            [rotatedDraggedMarker[0], vertexOpposite.lng],
            [...rotatedDraggedMarker],
            [vertexOpposite.lat, rotatedDraggedMarker[1]],
        ],
    };
    
    return rotateGeometry(rectangle, rotationAngle, [vertexOpposite.lat, vertexOpposite.lng]).map(point => ({
        lat: point[0],
        lng: point[1],
    }));
};

export const getRectangleZoneCenter = (geometry) => {
    if (!geometry || !geometry.length) {
        return null;
    }
    const n = geometry.length;
    let latSum = 0;
    let lngSum = 0;
    for (let i = 0; i < geometry.length; i += 1) {
        latSum += geometry[i][0];
        lngSum += geometry[i][1];
    }
    return [latSum / n, lngSum / n];
};

const getRotationAngleCenter = ({ zoneGeometryInDegrees }) => {
    const startX = zoneGeometryInDegrees[0][1];
    const startY = zoneGeometryInDegrees[0][0];
    const endX = zoneGeometryInDegrees[1][1];
    const endY = zoneGeometryInDegrees[1][0];
    const zoneCenter = getRectangleZoneCenter(zoneGeometryInDegrees);
    
    let horizontalCatet = endX - startX;
    let verticalCatet = endY - startY;
    const length = Math.sqrt((horizontalCatet **= 2) + (verticalCatet **= 2));
    const center = startY < endY ? zoneGeometryInDegrees[0] : zoneGeometryInDegrees[1];
    return {
        center,
        length,
        zoneCenter,
    };
};

const calculateVerticalRectangle = ({ center, width, length }) => {
    if (!center || !Array.isArray(center) || isNaN(width) || isNaN(length)) {
        return [];
    }
    return [
        [center[0], center[1] - (width / 2)],
        [center[0], center[1] + (width / 2)],
        [center[0] + length, center[1] + (width / 2)],
        [center[0] + length, center[1] - (width / 2)],
    ];
};

export const getDirectionGeometry = (vertexStart) => {
    if (!vertexStart || !Array.isArray(vertexStart)) {
        return [];
    }
    return [
        [vertexStart[0] - 0.0001, vertexStart[1]],
        [vertexStart[0] - 0.0002, vertexStart[1] - 0.00004],
        [vertexStart[0] - 0.0002, vertexStart[1] + 0.00004],
    ];
};

export const getStopSignGeometry = (vertexStart) => {
    if (!vertexStart || !Array.isArray(vertexStart)) {
        return [];
    }
    vertexStart[1] -= 0.00006;
    return [
        [...vertexStart],
        [vertexStart[0] + 0.000025, vertexStart[1]],
        [vertexStart[0] + 0.00006, vertexStart[1] + 0.000035],
        [vertexStart[0] + 0.00006, vertexStart[1] + 0.000085],
        [vertexStart[0] + 0.000025, vertexStart[1] + 0.00012],
        [vertexStart[0] - 0.000025, vertexStart[1] + 0.00012],
        [vertexStart[0] - 0.00006, vertexStart[1] + 0.000085],
        [vertexStart[0] - 0.00006, vertexStart[1] + 0.000035],
        [vertexStart[0] - 0.000025, vertexStart[1]],
        [...vertexStart],
    ];
};

export const getStopSignHandGeometry = (vertexStart) => {
    if (!vertexStart || !Array.isArray(vertexStart)) {
        return [];
    }
    vertexStart[1] += 0.00005;
    return [
        [vertexStart[0] - 0.00003, vertexStart[1] - 0.000008],
        [vertexStart[0] - 0.00003, vertexStart[1] + 0.000023],
        [vertexStart[0] - 0.0000025, vertexStart[1] + 0.000046],
        [vertexStart[0] + 0.0000015, vertexStart[1] + 0.00004],
        [vertexStart[0] - 0.00001, vertexStart[1] + 0.000026],
        [vertexStart[0] + 0.000035, vertexStart[1] + 0.000028],
        [vertexStart[0] + 0.000035, vertexStart[1] + 0.000021],
        [vertexStart[0] + 0.000007, vertexStart[1] + 0.000017],
        [vertexStart[0] + 0.00004, vertexStart[1] + 0.000016],
        [vertexStart[0] + 0.00004, vertexStart[1] + 0.000009],
        [vertexStart[0] + 0.0000065, vertexStart[1] + 0.000006],
        [vertexStart[0] + 0.000035, vertexStart[1] + 0.0000037],
        [vertexStart[0] + 0.000035, vertexStart[1] - 0.0000036],
        [vertexStart[0] + 0.000004, vertexStart[1] - 0.0000036],
        [vertexStart[0] + 0.000023, vertexStart[1] - 0.0000069],
        [vertexStart[0] + 0.000023, vertexStart[1] - 0.0000126],
    ];
};

export const setWidthToGeometry = (geometry, widthInMeters) => {
    const width = lengthToDegree(widthInMeters * 10);
    const zoneGeometryInDegrees = geometry.zoneGeometryInDegrees ? geometry.zoneGeometryInDegrees : [];
    
    const result = {
        geometry: [],
    };
    if (zoneGeometryInDegrees.length > 0) {
        const { center, zoneCenter, length } = getRotationAngleCenter({ zoneGeometryInDegrees });
        result.geometry.push(...calculateVerticalRectangle({ center, width, length }));
        result.zoneCenter = zoneCenter;
    }
    return result;
};

export const changeScaleZoneRadius = (item) => {
    if (item.zone_geometry && item.zone_geometry.radius) {
        return {
            ...item,
            zone_geometry: {
                ...item.zone_geometry,
                radius: item.zone_geometry.radius * 1000,
            },
        };
    }
    
    return item;
};

export const getZoneDetailsAndZoneTypeName = (details, zonesPrototypes) => {
    if (details.params) {
        const createdZonePrototype = zonesPrototypes.find(zp => zp.isZoneTypeIdEqualTo(details.params.zone_type_id));
        const zoneTypeName = createdZonePrototype
            ? createdZonePrototype.zone_type.display_name
            : details.params.zone_type_name;
        
        return {
            details,
            zoneTypeName,
        };
    }
    return {
        details,
    };
};

export const formatZoneGeometry = (zoneInLatLng) => {
    const zoneGeometryInMillimeters = geometryFromDegreesToMillimeters(zoneInLatLng);
    const zoneGeometryInDegrees = zoneInLatLng.map(point => [point.lat, point.lng]);
    return {
        zoneGeometryInDegrees,
        zoneGeometryInMillimeters,
    };
};
const isActive24 = (zone = {}) => {
    const { fromHour, fromMinute, toHour, toMinute } = zone;
    return ((Number(fromHour) === 0) && (Number(fromMinute) === 0) && (Number(toHour) === 24) && (Number(toMinute) === 0));
};

const calculateZoneMode = (zone) => {
    if (zone.activated) {
        if (zone.active24 || isActive24(zone)) {
            return ZONE_MODE.ALWAYS_ACTIVE;
        }
        return ZONE_MODE.ON_SCHEDULE;
    }
    return ZONE_MODE.DEACTIVATED;
};

export const parseZoneMode = (zone) => {
    switch (zone.zone_mode) {
        case ZONE_MODE.ALWAYS_ACTIVE:
            return {
                activated: true,
                active24: true,
            };
        case ZONE_MODE.ON_SCHEDULE:
            return {
                activated: true,
                active24: false,
            };
        case ZONE_MODE.DEACTIVATED:
        default:
            return {
                activated: false,
                active24: false,
            };
    }
};

export const separateTrackableAndGroup = (trackables_groups_uids, storeTrackables, storeGroups) => {
    const trackables = [];
    const groups = [];
    if (trackables_groups_uids) {
        trackables_groups_uids.forEach((uid) => {
            const trackable = storeTrackables[uid];
            const group = storeGroups[uid];
            if (trackable) {
                trackables.push(trackable.uid);
            }
            if (group) {
                groups.push(group.uid);
            }
        });
    }
    return { trackables, groups };
};

/**
 * Separate exceptions for trackablesExceptions and groupsExceptions
 * @param trackables_and_groups {Array(Trackable|Group)}
 * @param storeTrackables
 * @param storeGroups
 * @returns {{trackablesExceptions: Array, groupsExceptions: Array}}
 */
export const separateTrackableAndGroupExceptions = (trackables_and_groups, storeTrackables, storeGroups) => {
    if (!trackables_and_groups) {
        return { trackablesExceptions: [], groupsExceptions: [] };
    }

    const uids = trackables_and_groups.map(trackable_group => trackable_group.uid);
    const { trackables, groups } = separateTrackableAndGroup(uids, storeTrackables, storeGroups);
    return { trackablesExceptions: trackables, groupsExceptions: groups };
};

function adjustScheduleToServer(schedule) {
    if (schedule && schedule.start_time && schedule.stop_time
        && schedule.start_time !== '00 00 * * *' && schedule.stop_time === '00 24 * * *') {
        schedule.stop_time = '00 00 * * *';
    }
    return schedule;
}

export function adjustScheduleFromServer(schedule) {
    if (schedule && schedule.start_time && schedule.stop_time
        && schedule.start_time !== '00 00 * * *' && schedule.stop_time === '00 00 * * *') {
        schedule.stop_time = '00 24 * * *';
    }
    return schedule;
}

function calculateSchedule(zone) {
    const schedule = {};
    const zoneProperties = Object.keys(zone);
    if (zoneProperties.indexOf('toMinute') !== -1 && zoneProperties.indexOf('toHour') !== -1
        && zoneProperties.indexOf('fromMinute') !== -1 && zoneProperties.indexOf('fromHour') !== -1) {
        schedule.start_time = cronFormatter(zone.fromMinute, zone.fromHour);
        schedule.stop_time = cronFormatter(zone.toMinute, zone.toHour);
    }
    return adjustScheduleToServer(schedule);
}

// todo: in q4 mostly same as formatCreatedZone
export const formatEditedZone = (value, trackables, groups, zonesPrototypes) => {
    const zone = makeCopy(value);
    
    const customType = isCustomZoneType(zonesPrototypes, zone.zone_type_name);
    
    const { trackablesExceptions, groupsExceptions } = separateTrackableAndGroupExceptions(zone.trackables_and_groups, trackables, groups);
    
    const zone_exceptions = trackablesExceptions;
    const zone_exceptions_groups = groupsExceptions;
    
    const schedule = calculateSchedule(zone);
    const zone_mode = calculateZoneMode(zone);
    
    return {
        zone_mode: zone_mode === ZONE_MODE.ON_SCHEDULE && !schedule.start_time && !schedule.stop_time
            ? ZONE_MODE.ALWAYS_ACTIVE
            : zone_mode,
        schedule,
        zone_exceptions,
        name: zone.name,
        custom_info: zone.custom_info,
        shape: 'polygon',
        zone_exceptions_groups,
        priority: zone.priority || 0,
        sublocation_uid: zone.sublocation_uid,
        apply_to: zone.apply_to,
        default_allowed: calculateAllowDefault(zone),
        user_params: zone.user_params ? zone.user_params : {},
        zone_type_id: zone.zone_type_id,
        zone_type_name: zone.zone_type_name,
        zone_alarm_level: (isInfoZone(zone) || customType) ? 0 : Number(zone.zone_alarm_level),
    };
};

export const formatBulkEditingZone = (storeZone, params, trackables, groups) => {
    const changedCustomInfo = params.custom_info_option;
    const customInfoValue = params.custom_info;
    const changedStatus = params.status_option;
    const statusValue = params.status === 'inactive' ? ZONE_MODE.DEACTIVATED : (params.active24 ? ZONE_MODE.ALWAYS_ACTIVE : ZONE_MODE.ON_SCHEDULE);
    const scheduleValue = changedStatus ? calculateSchedule(params) : storeZone.schedule;
    const changedTrackablesGroups = params.trackables_groups_option;

    const { trackablesExceptions, groupsExceptions } = separateTrackableAndGroupExceptions(params.trackables_and_groups, trackables, groups);

    return {
        name: storeZone.name,
        custom_info: changedCustomInfo ? (customInfoValue || '') : storeZone.custom_info,
        shape: storeZone.shape,
        sublocation_uid: storeZone.sublocation_uid,
        priority: storeZone.priority || 0,
        user_params: storeZone.user_params,
        zone_type_id: storeZone.zone_type_id,
        zone_type_name: storeZone.zone_type_name,
        zone_alarm_level: storeZone.zone_alarm_level,
        zone_mode: changedStatus ? statusValue : storeZone.zone_mode,
        schedule: changedStatus ? scheduleValue : storeZone.schedule,
        zone_exceptions: changedTrackablesGroups ? trackablesExceptions : storeZone.zone_exceptions,
        zone_exceptions_groups: changedTrackablesGroups ? groupsExceptions : storeZone.zone_exceptions_groups,
        apply_to: changedTrackablesGroups ? params.apply_to : storeZone.apply_to,
        default_allowed: changedTrackablesGroups ? calculateAllowDefault(params) : storeZone.default_allowed,
    };
};

// todo: in q4 mostly same as formatEditedZone
export const formatCreatedZone = (zonePrototype, value, additives, trackables, groups) => {
    const zone = makeCopy(value);
    zone.user_params.direction = Number.isNaN(Number(zone.rotation)) ? 0 : Number(zone.rotation);
    const currentZoneType = getZoneTypes(zonePrototype).find(zoneType => zoneType.name === zone.zone_type_name);
    const customType = isCustomZoneType(zonePrototype, zone.zone_type_name);
    
    const { trackablesExceptions, groupsExceptions } = separateTrackableAndGroupExceptions(zone.trackables_and_groups, trackables, groups);
    const zone_exceptions = trackablesExceptions;
    const zone_exceptions_groups = groupsExceptions;
    
    const schedule = calculateSchedule(zone);
    const zone_mode = calculateZoneMode(zone);
    
    return {
        ...additives,
        zone_mode: zone_mode === ZONE_MODE.ON_SCHEDULE && !schedule.start_time && !schedule.stop_time
            ? ZONE_MODE.ALWAYS_ACTIVE
            : zone_mode,
        schedule,
        zone_exceptions,
        name: zone.name,
        custom_info: zone.custom_info,
        shape: 'polygon',
        zone_exceptions_groups,
        priority: zone.priority || 0,
        sublocation_uid: zone.sublocation_uid,
        apply_to: zone.apply_to,
        default_allowed: calculateAllowDefault(zone),
        user_params: zone.user_params ? zone.user_params : {},
        zone_type_id: currentZoneType.id,
        zone_type_name: currentZoneType.name,
        zone_alarm_level: (isInfoZone(zone) || customType) ? 0 : Number(zone.zone_alarm_level),
    };
};

export const getValuesFromZoneFormState = (editZoneFormState) => {
    if (editZoneFormState && editZoneFormState.values) {
        const active24 = editZoneFormState.values.active24;
        const activated = editZoneFormState.values.activated;
        const location_name = editZoneFormState.values.location_name;
        const sublocation_name = editZoneFormState.values.sublocation_name;
        const areGeometryOptionsShown = editZoneFormState.values.geometry_options;
        const areAdvancedOptionsShown = editZoneFormState.values.advanced_options;
        const geometry = editZoneFormState.values?.zone_geometry?.positions || [];

        return {
            activated,
            location_name,
            sublocation_name,
            areAdvancedOptionsShown,
            areGeometryOptionsShown,
            active24,
            geometry
        };
    }
    return {
        activated: false,
        location_name: '',
        sublocation_name: '',
        areAdvancedOptionsShown: true,
        areGeometryOptionsShown: true,
        active24: false,
        geometry: [],
    };
};

export const getVerticesLimit = project => _Get(project, 'settings.zone_vertices_limit') || ZONES_CONSTANTS.MAX_VERTICES;

function getStaticZonesBySublocation(sublocation, isPlayback = false, state) {
    const zonesArray = Object.values(state.entities.zones.byUid);
    const playback = state.playback;
    
    const zones = getZones(zonesArray, playback, sublocation, isPlayback);
    
    return zones.reduce((memo, zone) => {
        if (!zone.dynamic && zone.sublocation_uid === sublocation.uid) {
            memo.push(zone);
        }
        return memo;
    }, []);
}

export const getVisibleZones = (sublocation, state) => {
    const { siteView: { selectedItems }, playback, siteViewFilter: { layerFilter }, entities } = state;
    const hasPrototypes = entities.zonesPrototypes.length > 0;
    const alarms = getAlarms(state).filter(alarm => !alarm.acknowledged);
    
    return getStaticZonesBySublocation(sublocation, (playback && playback.isOpenPlayBackControl), state)
        .reduce((memo, zone) => {
            if (isZoneVisible(zone, layerFilter, hasPrototypes ? layerFilter.isShowingCustomZones : false)) {
                const formattedZone = zone.formattedZone({ zonesPrototypes: entities.zonesPrototypes });
                memo.push({
                    ...formattedZone,
                    isSelected: selectedItems ? !!selectedItems.find(x => x.uid === zone.uid) : false,
                    isAlarmed: alarms ? !!alarms.find(alarm => zone.uid === alarm.zone_uid) : false,
                });
            }
            return memo;
        }, []);
};

export function pointInPolygon(point, polygon) {
    const pointX = point[0];
    const pointY = point[1];
    let inside = false;
    const len = polygon.length;

    for (let i = 0, j = len - 1; i < len; j = i++) {
        const xi = polygon[i][0];
        const yi = polygon[i][1];
        const xj = polygon[j][0];
        const yj = polygon[j][1];

        const intersect = ((yi > pointY) !== (yj > pointY)) && (pointX < (xj - xi) * (pointY - yi) / (yj - yi) + xi);
        if (intersect) inside = !inside;
    }
    return inside;
}

/**
 * @function
 * @param {Array<Object>} zonesList - list of zones
 * @param {Object} sublocation
 * @param {Object} leafletPosition
 * @return {Array<Object>}
* */
export function getZonesByPoint(zonesList, sublocation, leafletPosition) {
    const result = [];
    const position = new Position(leafletPosition, sublocation);
    const positionXY = position.xy;

    for (const zone of zonesList) {
        const polygonPositions = zone?.zone_geometry?.positions.map(pos => ([ pos.x, pos.y ]));
        if (polygonPositions && pointInPolygon(positionXY, polygonPositions)) {
            result.push(zone);
        }
    }
    return result;
}


export function isGeometryChanged(vertexesOne, vertexesTwo) {
    if (vertexesOne.length !== vertexesTwo.length) return true;
    if (vertexesOne.length === 0) return false;

    const hashMap = {};
    const getKey = (idx, vertex) => `${idx}-${vertex.x}-${vertex.y}`;

    for (const idx in vertexesOne) {
        hashMap[getKey(idx, vertexesOne[idx])] = vertexesOne[idx];
    }

    for (const idx in vertexesTwo) {
        if (!hashMap[getKey(idx, vertexesTwo[idx])]) return true;
    }

    return false;
}
