import moment from 'moment';
import { HTTP_STATUS } from '../../js/common/constants';
import NearMissError from '../../js/stores/classes/NearMissError';
import NearMissEvent from './NearMissEvent';
import { NEAR_MISS_WORKER_METHODS } from './common/constants';


class NearMissNotifyMethods {
    sendDataToClient = (reportUid) => {
        if (!this.nearMissEvents[reportUid]) {
            this.notifyClientAboutError(reportUid, { message: 'Not found' });
        } else {
            const { events, timeLineMarkers } = this.nearMissEvents[reportUid];
            this.sendEventsToClient(reportUid, { events, timeLineMarkers });
        }
    };
    
    notifyClientAboutError = (reportUid, error) => {
        self.postMessage({
            type: NEAR_MISS_WORKER_METHODS.NEAR_MISS_ERROR,
            params: { reportUid, error },
        });
    };

    sendEventsToClient = (reportUid, params) => {
        self.postMessage({
            type: NEAR_MISS_WORKER_METHODS.SET_NEAR_MISS_EVENTS,
            params: {
                reportUid,
                ...params,
            },
        });
    };
    
    sendNearMissParamsToClient = (reportUid, options) => {
        self.postMessage({
            type: NEAR_MISS_WORKER_METHODS.SET_NEAR_MISS_PARAMS,
            params: { reportUid, options },
        });
    };
}

export default class NearMissEventHandler extends NearMissNotifyMethods {
    initTimeout = (params, reportUid) => {
        this.timeout[reportUid] = setTimeout(() => {
            const nearMissReport = this.nearMissEvents[reportUid] || {};
            
            if (nearMissReport.status < 100 && nearMissReport.error === null) {
                this.getReportStatus(params, reportUid);
            } else if (nearMissReport.status === 100) {
                this.getReportData(params, reportUid);
            } else {
                this.notifyClientAboutError(reportUid, nearMissReport.error);
                clearTimeout(this.timeout[reportUid]);
            }
        }, this.statusRequestTimeout);
    };
    
    requestServerData = (params) => {
        const { token, email, project_uid, hostname, dateFrom, dateTo, distance, timeZone } = params;
        const reportParams = {
            from: dateFrom,
            to: dateTo,
            distance,
            project_uid,
            sublocation_uid: params.sublocation.uid,
        };
        const req = new XMLHttpRequest();
        req.open('POST', `${hostname}/api/v2/near_miss`);
        req.setRequestHeader('X-User-Token', token);
        req.setRequestHeader('X-User-Email', email);
        req.setRequestHeader('X-User-Project', project_uid);
        req.setRequestHeader('Content-Type', 'application/json');
        req.send(JSON.stringify(reportParams));
        
        req.onprogress = () => { };
        req.onerror = (error) => {
            this.notifyClientAboutError('', new NearMissError(error));
        };
        
        req.onloadend = (data) => {
            if (data.target && data.target.status === HTTP_STATUS.OK) {
                const { reportUid } = JSON.parse(data.target.response);
                this.reportUids.push(reportUid);
                this.nearMissEvents[reportUid] = { status: 0, error: null, events: [], sublocation: params.sublocation, timeZone };
                this.sendNearMissParamsToClient(reportUid, { timeframe: { dateFrom, dateTo } });
                this.initTimeout(params, reportUid);
            } else {
                const error = new NearMissError({ status: data.target.status, message: data.target.responseText });
                this.notifyClientAboutError('', error);
            }
        };
    };
    
    getReportStatus = (params, reportUid) => {
        const { token, email, project_uid, hostname } = params;
        
        const req = new XMLHttpRequest();
        req.open('GET', `${hostname}/api/v2/near_miss/status/${reportUid}`);
        req.setRequestHeader('X-User-Token', token);
        req.setRequestHeader('X-User-Email', email);
        req.setRequestHeader('X-User-Project', project_uid);
        req.setRequestHeader('Content-Type', 'application/json');
        req.send();
    
        req.onprogress = () => { };
    
        req.onerror = (err) => {
            clearTimeout(this.timeout[reportUid]);
            this.notifyClientAboutError(reportUid, new NearMissError(err));
        };
    
        req.onloadend = (data) => {
            if (data.target && data.target.status === HTTP_STATUS.OK) {
                const { status, error } = JSON.parse(data.target.response);
                this.nearMissEvents[reportUid].status = status;
                this.nearMissEvents[reportUid].error = error;
    
                this.initTimeout(params, reportUid);
            } else if (!data.target || data.target.status !== HTTP_STATUS.TOO_MANY_REQUESTS) {
                clearInterval(this.interval[reportUid]);

                const error = new NearMissError({ });
                this.notifyClientAboutError('', error);
            } else {
                // if status === HTTP_STATUS.TOO_MANY_REQUESTS so requests by timeout should continue
                this.initTimeout(params, reportUid);
            }
        };
    };
    
    getReportData = (params, reportUid) => {
        const { token, email, project_uid, hostname } = params;
    
        const req = new XMLHttpRequest();
        req.open('GET', `${hostname}/api/v2/near_miss/${reportUid}`);
        req.setRequestHeader('X-User-Token', token);
        req.setRequestHeader('X-User-Email', email);
        req.setRequestHeader('X-User-Project', project_uid);
        req.setRequestHeader('Content-Type', 'application/json');
        req.send();
    
        req.onprogress = () => { };
        
        req.onerror = (err) => {
            clearTimeout(this.timeout[reportUid]);
            this.notifyClientAboutError(reportUid, new NearMissError(err));
        };
    
        req.onloadend = (data) => {
            if (data.target && data.target.status === HTTP_STATUS.OK) {
                const response = JSON.parse(data.target.response);
                const { events = [], trackableObjects = [] } = response;
                const { sublocation, timeZone } = this.nearMissEvents[reportUid];
                const trackableMap = trackableObjects.reduce((memo, trackable) => {
                    memo[trackable.uid] = trackable;
                    return memo;
                }, {});
    
                const { nearMissEvents, timeLineMarkers } = events.reduce((memo, event, i) => {
                    const nearMissEvent = new NearMissEvent(event, sublocation, timeZone, i);
                    nearMissEvent.addTrackablesInfo(trackableMap);
        
                    memo.nearMissEvents.push({ ...nearMissEvent.data });
                    memo.timeLineMarkers.push({ ...nearMissEvent.marker });
                    return memo;
                }, { nearMissEvents: [], timeLineMarkers: [] });
    
                this.nearMissEvents[reportUid].events = nearMissEvents;
                this.nearMissEvents[reportUid].timeLineMarkers = timeLineMarkers;
                
                this.sendDataToClient(reportUid);
            } else if (!data.target || data.target.status !== HTTP_STATUS.TOO_MANY_REQUESTS) {
                const error = new NearMissError({ });
                this.notifyClientAboutError('', error);
            } else {
                // if status === HTTP_STATUS.TOO_MANY_REQUESTS so requests by timeout should continue
                this.initTimeout(params, reportUid);
            }
        };
    };
    
    getDataForTimeFrame = (params) => {
        const { dateFromUtc, dateToUtc, reportUid } = params;
        const from = moment(dateFromUtc);
        const to = moment(dateToUtc);
        const events = [];

        if (this.nearMissEvents[reportUid]) {
            events.push(...this.nearMissEvents[reportUid].events.filter(event => moment(event.timestampUtc).isBetween(from, to)));
        }
        
        this.sendEventsToClient(reportUid, { events });
    };
    
    generateNearMiss = (params) => {
        this.clearReports();
        this.requestServerData(params);
    };
    
    clearReports = () => {
        this.reportUids.forEach((reportUid) => {
            clearTimeout(this.timeout[reportUid]);
            delete this.nearMissEvents[reportUid];
        });
        this.reportUids = [];
    };
    
    constructor() {
        super();
        this.nearMissEvents = {};
        this.statusRequestTimeout = 5000;
        this.reportUids = [];
        this.timeout = {};
    }
}
